unit fEditNotificationLists;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Buttons,
  oInterfaces,
  oCPTypes,
  ExtCtrls;

type
  TfrmEditNotificationLists = class(TForm)
    cbxLists: TComboBox;
    lbxMembers: TListBox;
    bbtnAdd: TBitBtn;
    bbtnRemove: TBitBtn;
    bbtnUpdate: TBitBtn;
    btnOK: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Shape1: TShape;
    lblGeneral: TLabel;
    bbtnEdit: TBitBtn;
    bbtnTest: TBitBtn;
    Shape2: TShape;
    procedure cbxListsChange(Sender: TObject);
    procedure bbtnUpdateClick(Sender: TObject);
    procedure bbtnAddClick(Sender: TObject);
    procedure bbtnRemoveClick(Sender: TObject);
    procedure lbxMembersClick(Sender: TObject);
    procedure bbtnEditClick(Sender: TObject);
    procedure bbtnTestClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    fCPParameters: ICPParameters;
    fCPService: ICPService;
    fChanges: boolean;
    procedure EnableDisable;
  public
    //
  end;

procedure EditNotificationLists(aCPService: ICPService; aCPParameters: ICPParameters);

implementation

uses
  u508Dialogs;

{$R *.dfm}

procedure EditNotificationLists(aCPService: ICPService; aCPParameters: ICPParameters);
var
  aList: TCPNotificationList;
begin
  with TfrmEditNotificationLists.Create(Application) do
    try
      fCPService := aCPService;
      fCPParameters := aCPParameters;
      for aList := Low(TCPNotificationList) to High(TCPNotificationList) do
        cbxLists.Items.Add(CP_NOTIFICATION_LIST[aList]);
      lbxMembers.Items.Delimiter := ';';
      lblGeneral.Font.Style := [fsBold];
      EnableDisable;
      cbxLists.ItemIndex := 0;
      cbxListsChange(nil);
      fChanges := False;
      ShowModal;
    finally
      Free;
    end;
end;

procedure TfrmEditNotificationLists.bbtnAddClick(Sender: TObject);
var
  aNewMember: string;
begin
  aNewMember := 'Someone@Somewhere';
  if InputQuery('Add Member to Notification List', 'Member:', aNewMember) then
    if aNewMember <> '' then
      if (Length(lbxMembers.Items.DelimitedText) + Length(aNewMember)) < 250 then
        begin
          lbxMembers.Items.Add(aNewMember);
          fChanges := True;
        end
      else
        MessageDlg508('Sorry, this list cannot hold anymore recipients.', mtInformation, [mbOK], 0);
  EnableDisable;
end;

procedure TfrmEditNotificationLists.bbtnEditClick(Sender: TObject);
var
  aMember: string;
begin
  aMember := lbxMembers.Items[lbxMembers.ItemIndex];
  if InputQuery('Edit Member of List', 'Member:', aMember) then
    if aMember <> '' then
      begin
        lbxMembers.Items[lbxMembers.ItemIndex] := aMember;
        fChanges := True;
      end;
  EnableDisable;
end;

procedure TfrmEditNotificationLists.bbtnRemoveClick(Sender: TObject);
begin
  lbxMembers.DeleteSelected;
  fChanges := True;
  EnableDisable;
end;

procedure TfrmEditNotificationLists.bbtnTestClick(Sender: TObject);
begin
  if fChanges then
    if MessageDlg508('Save Changes First?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      bbtnUpdateClick(Sender);
  with fCPService.NewCPEmailMessage do
    try
      SetSubject('Test Message From CP Notification List Editor');
      AddRecipient(
        fCPParameters.getNotificationList(TCPNotificationList(cbxLists.ItemIndex)));
      AddText('This is a test message generated from the CP Notification List Editor');
      AddText(' ');
      AddText('     Please verify that the following members of ''' +
        CP_NOTIFICATION_LIST[TCPNotificationList(cbxLists.ItemIndex)] + '''.');
      AddText('     have successfully received this message.');
      AddText(' ');
      AddList(lbxMembers.Items);
      Send;
      MessageDlg508('Test message has been sent.', mtInformation, [mbOk], 0);
    except
      on E: Exception do
        MessageDlg508('Error encountered sending E-Mail.' + #13 + E.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfrmEditNotificationLists.bbtnUpdateClick(Sender: TObject);
begin
  fCPParameters.setNotificationList(
    TCPNotificationList(cbxLists.ItemIndex),
    lbxMembers.Items.DelimitedText);

  fCPParameters.SaveList(
    TCPNotificationList(cbxLists.ItemIndex));

  fChanges := False;
  EnableDisable;
end;

procedure TfrmEditNotificationLists.cbxListsChange(Sender: TObject);
begin
  if fChanges then
    if MessageDlg508('Save Changes First?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      bbtnUpdateClick(Sender);

  lbxMembers.Items.DelimitedText :=
    fCPParameters.getNotificationList(
    TCPNotificationList(cbxLists.ItemIndex));
  fChanges := False;
  EnableDisable;
end;

procedure TfrmEditNotificationLists.EnableDisable;
begin
  bbtnAdd.Enabled := (cbxLists.ItemIndex > -1);
  bbtnUpdate.Enabled := (cbxLists.ItemIndex > -1);
  bbtnRemove.Enabled := (lbxMembers.SelCount > 0);
  bbtnEdit.Enabled := (lbxMembers.SelCount = 1);
  bbtnTest.Enabled := (lbxMembers.Count > 0);
  bbtnUpdate.Enabled := fChanges;
end;

procedure TfrmEditNotificationLists.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := True;
  if fChanges then
    case MessageDlg508('Save Changes Before Closing?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes: bbtnUpdateClick(nil);
      mrCancel: CanClose := False;
    end;
end;

procedure TfrmEditNotificationLists.lbxMembersClick(Sender: TObject);
begin
  EnableDisable;
end;

end.

