unit fEditViewTerm;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  oInterfaces,
  oCPTypes,
  oCliOTypes,
  mEditObservationViewFilter;

type
  TfrmEditViewTerm = class(TForm)
    pnlMain: TPanel;
    cbxTerm: TComboBox;
    Label1: TLabel;
    edtDisplayName: TEdit;
    lblDisplayName: TLabel;
    edtDisplayWidth: TEdit;
    lblDisplayWidth: TLabel;
    udDisplayWidth: TUpDown;
    gbxCalculations: TGroupBox;
    chkbxTotal: TCheckBox;
    chkbxAverage: TCheckBox;
    chkbxCount: TCheckBox;
    Shape1: TShape;
    lblTerm: TLabel;
    lblFilters: TLabel;
    Shape2: TShape;
    ScrollBox1: TScrollBox;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    cbxUnit: TComboBox;
    lblUnit: TLabel;
    pnlBottom: TPanel;
    Shape3: TShape;
    btnOk: TButton;
    btnCancel: TButton;
    cbxColumns: TComboBox;
    lblColumns: TLabel;
    chkbxInCellTotal: TCheckBox;
    chkbxDisplayOnly: TCheckBox;
    chkbxMinValue: TCheckBox;
    chkbxMaxValue: TCheckBox;
    chkbxRequiredValue: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    cbxDropdown: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure cbxTermChange(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure cbxTermDropDown(Sender: TObject);
    procedure cbxUnitDropDown(Sender: TObject);
  protected
    fViewTerm: ICPFlowsheetViewTerm;
    fTerm: ITerm;
    fFilters: array[TTermType] of TfraEditObservationViewFilter;
    fTerminology: ITerminology;
  public
    //function getViewTerm: ICPFlowsheetViewTerm;
    //procedure setViewTerm(aViewTerm: ICPFlowsheetViewTerm);
  end;

function NewEditViewTermForm(aViewTerm: ICPFLowsheetViewTerm): TfrmEditViewTerm;

implementation

{$R *.dfm}

uses
  fMain,
  u508Dialogs,
  uCP_GUI;

function NewEditViewTermForm(aViewTerm: ICPFLowsheetViewTerm): TfrmEditViewTerm;
var
  aTerm: ITerm;
  aUnit: ITerm;
  tt: TTermType;
  i: integer;
begin
  Result := TfrmEditViewTerm.Create(Application.MainForm);
  with Result do
    begin
      MyTerminology.LoadList(cbxTerm.Items, ttObservation);
      fViewTerm := aViewTerm;
      Caption := 'View Terminology Editor';
      cbxTerm.ItemIndex := -1;
      fViewTerm := aViewTerm;
      for i := 0 to cbxTerm.Items.Count - 1 do
        if cbxTerm.Items.Objects[i].GetInterface(ITerm, aTerm) then
          if aTerm.getID = fViewTerm.getAttributeID(ttObservation) then
            begin
              cbxTerm.ItemIndex := i;
              cbxTermChange(nil); {Loads the form up with default stuff for this term}
              Break;
            end;

      if aViewTerm.getAttributeID(ttUnit) <> '' then
        for i := 0 to cbxUnit.Items.Count - 1 do
          if cbxUnit.Items.Objects[i].GetInterface(ITerm, aUnit) then
            if aUnit.getID = fViewTerm.getAttributeID(ttUnit) then
              cbxUnit.ItemIndex := i;

      edtDisplayName.Text := fViewTerm.getDisplayName;
      udDisplayWidth.Position := fViewTerm.getDisplayWidth;
      chkbxDisplayOnly.Checked := fViewTerm.getDisplayOnly;
      chkbxTotal.Checked := fViewTerm.getDisplayTotal;
      chkbxAverage.Checked := fViewTerm.getDisplayAverage;
      chkbxCount.Checked := fViewTerm.getDisplayCount;
      chkbxInCellTotal.Checked := fViewTerm.getDisplayInCellTotal;
      chkbxMinValue.Checked := fViewTerm.getDisplayMinValue;
      chkbxMaxValue.Checked := fViewTerm.getDisplayMaxValue;
      chkbxRequiredValue.Checked := fViewTerm.getRequiredTerm;
      cbxColumns.ItemIndex := fViewTerm.getDisplayColumns;
      cbxDropdown.Checked := fViewTerm.getDisplayInDropdown;

      for tt := ttMethod to ttNullFlavor do
        if fFilters[tt].Visible then
          begin
            fFilters[tt].chkbxAsk.Checked := fViewTerm.getAttributeAllowed(tt);
            fFilters[tt].cbxFilter.ItemIndex := -1;
            for i := 0 to fFilters[tt].cbxFilter.Items.Count - 1 do
              if fFilters[tt].cbxFilter.Items.Objects[i].GetInterface(ITerm, aTerm) then
                if aTerm.getID = fViewTerm.getAttributeID(tt) then
                  fFilters[tt].cbxFilter.ItemIndex := i;

            if fFilters[tt].cbxFilter.ItemIndex > -1 then
              begin
                fFilters[tt].chkbxMandatory.Enabled := True;
                fFilters[tt].chkbxMandatory.Checked := fViewTerm.getAttributeMandatory(tt);
              end
            else
              fFilters[tt].chkbxMandatory.Enabled := False;
          end;
    end;
end;

{ TfrmEditViewTerm }

procedure TfrmEditViewTerm.FormCreate(Sender: TObject);
var
  tt: TTermType;
begin
  Font := Application.MainForm.Font;
  lblTerm.Font.Style := [fsBold];
  lblFilters.Font.Style := [fsBold];
  for tt := ttMethod to ttNullFlavor do
    begin
      fFilters[tt] := NewEditObservationViewFilterFrame(ScrollBox1);
      fFilters[tt].setFilterType(tt);
      fFilters[tt].Top := 0;
      fFilters[tt].Align := alNone;
      fFilters[tt].Visible := False;
    end;
end;

procedure TfrmEditViewTerm.btnOkClick(Sender: TObject);
var
  aTerm: ITerm;
  tt: TTermType;
begin
  with TStringList.Create do
    try
      if cbxTerm.ItemIndex < 0 then
        Add('No term selected.');

      if cbxUnit.Items.Count > 0 then
        if cbxUnit.ItemIndex < 0 then
          Add('Default unit is required.');

      if edtDisplayName.Text = '' then
        Add('Display name is required.');

      if cbxColumns.Enabled then
        if cbxColumns.ItemIndex < 0 then
          Add('Display columns is required.');

      if Count > 0 then
        MessageDlg508('Errors found.' + #13 + Text, mtError, [mbOk], 0)
      else
        begin
          {Clear every filter}
          for tt := Low(TTermType) to High(TTermType) do
            fViewTerm.setAttribute(tt, False, False, '');

          {Set the observation filter}
          if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, aTerm) then
            fViewTerm.setAttribute(ttObservation, True, True, aTerm.getID);

          {Set the Unit Filter if any}
          if cbxUnit.ItemIndex > -1 then
            if cbxUnit.Items.Objects[cbxUnit.ItemIndex].GetInterface(ITerm, aTerm) then
              fViewTerm.setAttribute(ttUnit, True, False, aTerm.getID);

          {Set the rest of the stuff}
          fViewTerm.setDisplayName(edtDisplayName.Text);
          fViewTerm.setDisplayWidth(udDisplayWidth.Position);
          fViewTerm.setDisplayTotal(chkbxTotal.Checked);
          fViewTerm.setDisplayAverage(chkbxAverage.Checked);
          fViewTerm.setDisplayCount(chkbxCount.Checked);
          fViewTerm.setDisplayOnly(chkbxDisplayOnly.Checked);
          fViewTerm.setDisplayInCellTotal(chkbxInCellTotal.Checked);
          fViewTerm.setDisplayMinValue(chkbxMinValue.Checked);
          fViewTerm.setDisplayMaxValue(chkbxMaxValue.Checked);
          fViewTerm.setRequiredTerm(chkbxRequiredValue.Checked);
          fViewTerm.setDisplayInDropdown(cbxDropdown.Checked);

          if cbxColumns.ItemIndex > -1 then
            fViewTerm.setDisplayColumns(cbxColumns.ItemIndex)
          else
            fViewTerm.setDisplayColumns(0);

          {Now do the individual filters from the frame collection}
          for tt := Low(TTermType) to High(TTermType) do
            if Assigned(fFilters[tt]) then
              if fFilters[tt].Visible then
                with fFilters[tt] do
                  if cbxFilter.ItemIndex > -1 then
                    begin
                      cbxFilter.Items.Objects[cbxFilter.ItemIndex].GetInterface(ITerm, aTerm);
                      fViewTerm.setAttribute(tt, chkbxAsk.Checked, chkbxMandatory.Checked, aTerm.getID);
                    end
                  else
                    fViewTerm.setAttribute(tt, chkbxAsk.Checked, chkbxMandatory.Checked, '');

          ModalResult := mrOk;
        end;
    finally
      Free;
    end;
end;

procedure TfrmEditViewTerm.cbxTermChange(Sender: TObject);
var
  tt: TTermType;
begin
  if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, fTerm) then
    begin
      if not fTerm.getActive then
        MessageDlg508('Notice: This term is marked inactive',mtInformation,[mbOk],0);
      for tt := ttMethod to ttNullFlavor do
        begin
          fFilters[tt].Align := alNone;
          fFilters[tt].Top := 0;
          fFilters[tt].Visible := False;
        end;

      for tt := ttMethod to ttNullFlavor do
        if fTerm.getHasQualifiers(tt) then
          begin
            fTerm.LoadListOfPairs(tt, fFilters[tt].cbxFilter.Items);
            fFilters[tt].chkbxAsk.Enabled := True;
            fFilters[tt].chkbxAsk.Checked := False;
            fFilters[tt].cbxFilter.ItemIndex := -1;
            fFilters[tt].cbxFilter.Enabled := False;
            fFilters[tt].chkbxMandatory.Checked := False;
            fFilters[tt].chkbxMandatory.Enabled := False;
            fFilters[tt].Top := Ord(tt); // * fFilters[tt].Height;
            fFilters[tt].Align := alTop;
            fFilters[tt].Visible := True;
          end;

      ScrollBox1.VertScrollBar.Position := 0;

      fTerm.LoadListOfPairs(ttUnit, cbxUnit.Items);

      cbxUnit.Enabled := (cbxUnit.Items.Count > 0);
      lblUnit.Enabled := (cbxUnit.Items.Count > 0);

      lblDisplayName.Enabled := True;
      edtDisplayName.Enabled := True;

      lblDisplayWidth.Enabled := True;
      edtDisplayWidth.Enabled := True;
      udDisplayWidth.Enabled := True;

      cbxColumns.ItemIndex := 0;
      cbxColumns.Enabled := (fTerm.getDataType = dtPicklist);
      lblColumns.Enabled := (fTerm.getDataType = dtPicklist);

      cbxDropdown.Checked := False;
      cbxDropDown.Enabled := (fTerm.getDataType = dtPickList) and (not fTerm.getMultiSelectPickList);

      chkbxTotal.Checked := False;
      chkbxTotal.Enabled := (fTerm.getDataType = dtNumeric);

      chkbxAverage.Checked := False;
      chkbxAverage.Enabled := (fTerm.getDataType = dtNumeric);

      chkbxCount.Checked := False;
      chkbxCount.Enabled := True;

      chkbxInCellTotal.Checked := False;
      chkbxInCellTotal.Enabled := (fTerm.getDataType = dtNumeric);

      chkbxDisplayOnly.Checked := False;
      chkbxDisplayOnly.Enabled := True;

      edtDisplayName.Text := fTerm.getDisplayName;

      udDisplayWidth.Position := 120;
    end;
end;

procedure TfrmEditViewTerm.cbxTermDropDown(Sender: TObject);
begin
  SetDropDownWIdth(cbxTerm);
end;

procedure TfrmEditViewTerm.cbxUnitDropDown(Sender: TObject);
begin
  {
  Note: We may have some troubles here with cbxUnit.  It's close enough to the
    right hand side of the form that the drop-down list is likely to be clipped by the parent form.
  }
  SetDropDownWidth(cbxUnit);
end;

(*
function TfrmEditViewTerm.getViewTerm: ICPFlowsheetViewTerm;
begin
  Result := fViewTerm;
end;

procedure TfrmEditViewTerm.setViewTerm(aViewTerm: IObservationViewTerm);
var
  aTerm: ITerm;
  aUnit: ITerm;
  tt: TTermType;
  i: integer;
begin
  cbxTerm.ItemIndex := -1;
  fViewTerm := aViewTerm;
  for i := 0 to cbxTerm.Items.Count - 1 do
    if cbxTerm.Items.Objects[i].GetInterface(ITerm, aTerm) then
      if aTerm.getID = fViewTerm.getAttributeID(ttObservation) then
        cbxTerm.ItemIndex := i;

  cbxTermChange(nil); {Loads the form up with default stuff for this term}

  if aViewTerm.getAttributeID(ttUnit) <> '' then
    for i := 0 to cbxUnit.Items.Count - 1 do
      if cbxUnit.Items.Objects[i].GetInterface(ITerm, aUnit) then
        if aUnit.getID = fViewTerm.getAttributeID(ttUnit) then
          cbxUnit.ItemIndex := i;

  edtDisplayName.Text := fViewTerm.getDisplayName;
  udDisplayWidth.Position := fViewTerm.getDisplayWidth;
  chkbxDisplayOnly.Checked := fViewTerm.getDisplayOnly;
  chkbxTotal.Checked := fViewTerm.getDisplayTotal;
  chkbxAverage.Checked := fViewTerm.getDisplayAverage;
  chkbxCount.Checked := fViewTerm.getDisplayCount;
  chkbxInCellTotal.Checked := fViewTerm.getDisplayInCellTotal;
  cbxColumns.ItemIndex := (fViewTerm.getDisplayColumns - 1);

  for tt := ttMethod to ttNullFlavor do
    if fFilters[tt].Visible then
      begin
        fFilters[tt].chkbxAsk.Checked := fViewTerm.getAttributeAllowed(tt);
        fFilters[tt].cbxFilter.ItemIndex := -1;
        for i := 0 to fFilters[tt].cbxFilter.Items.Count - 1 do
          if fFilters[tt].cbxFilter.Items.Objects[i].GetInterface(ITerm, aTerm) then
            if aTerm.getID = fViewTerm.getAttributeID(tt) then
              fFilters[tt].cbxFilter.ItemIndex := i;

        if fFilters[tt].cbxFilter.ItemIndex > -1 then
          begin
            fFilters[tt].chkbxMandatory.Enabled := True;
            fFilters[tt].chkbxMandatory.Checked := fViewTerm.getAttributeMandatory(tt);
          end
        else
          fFilters[tt].chkbxMandatory.Enabled := False;
      end;
end;
*)

end.

