unit fMain;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 64 $  $Modtime: 12/08/11 2:55p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary UI for CPConsole.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/CP Console/fMain.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ActnList,
  StdCtrls,
  ComCtrls,
  ToolWin,
  ExtCtrls,
  ImgList,
  Menus,
  XMLDoc,
  XMLIntf,
  XPMan,
  Buttons,
  mSearch,
  XPStyleActnCtrls,
  ActnMan,
  ActnCtrls,
  ActnMenus,
  StdStyleActnCtrls,
  CommCtrl,
  HelpIntfs,
  HTMLHelpViewer,
  oCPTypes,
  oCliO,
  oCliOTypes,
  oInterfaces,
  VA508AccessibilityManager,
  VA508ImageListLabeler, System.Actions, System.ImageList;

type
  TfrmMain = class(TForm)
    scbx: TScrollBox;
    sbar: TStatusBar;
    ImageList1: TImageList;
    tv: TTreeView;
    pnlEditor: TPanel;
    pnlHeader: TPanel;
    pnlFooter: TPanel;
    Shape2: TShape;
    Bevel3: TBevel;
    BitBtn1: TBitBtn;
    Bevel2: TBevel;
    Bevel4: TBevel;
    CoolBar1: TCoolBar;
    ActionMainMenuBar1: TActionMainMenuBar;
    ToolBar1: TToolBar;
    ToolButton2: TToolButton;
    ToolButton4: TToolButton;
    ToolButton7: TToolButton;
    Splitter1: TSplitter;
    Bevel8: TBevel;
    ToolButton3: TToolButton;
    ActionManager1: TActionManager;
    acDeleteConsoleItem: TAction;
    acSaveItem: TAction;
    acExit: TAction;
    acSearchForItem: TAction;
    acExportToXML: TAction;
    acImportFromXML: TAction;
    acAbout: TAction;
    acHelpContents: TAction;
    acInitDisplay: TAction;
    acLoadConsoleItems: TAction;
    acNewFlowsheet: TAction;
    acNewMappingTable: TAction;
    acRenameConsoleItem: TAction;
    acNewProcedure: TAction;
    acNewInstrument: TAction;
    acNewFlowsheetView: TAction;
    PopupMenu1: TPopupMenu;
    Instrument1: TMenuItem;
    Procedure1: TMenuItem;
    View1: TMenuItem;
    Bevel1: TBevel;
    Bevel5: TBevel;
    Bevel6: TBevel;
    acViewICPConsoleItem: TAction;
    acNewShift: TAction;
    mnNewFlowsheet: TMenuItem;
    acNewFlowsheetTotal: TAction;
    ToolButton1: TToolButton;
    acRefreshRoles: TAction;
    acRefreshPermissions: TAction;
    acNewSchedule: TAction;
    Shift1: TMenuItem;
    FlowsheetTotal1: TMenuItem;
    lblName: TLabel;
    lblType: TLabel;
    lblAccessLevel: TLabel;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    edtItemName: TEdit;
    edtItemType: TEdit;
    edtItemAccessLevel: TEdit;
    VA508ImageListLabeler1: TVA508ImageListLabeler;
    VA508ComponentAccessibility1: TVA508ComponentAccessibility;
    tbCopyItem: TToolButton;
    acMakeCopy: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure acInitDisplayExecute(Sender: TObject);
    procedure acLoadConsoleItemsExecute(Sender: TObject);
    procedure acDeleteConsoleItemExecute(Sender: TObject);
    procedure acExitExecute(Sender: TObject);
    procedure acSaveItemExecute(Sender: TObject);
    procedure acImportFromXMLExecute(Sender: TObject);
    procedure acExportToXMLExecute(Sender: TObject);
    procedure acSearchForItemExecute(Sender: TObject);
    procedure acAboutExecute(Sender: TObject);
    procedure acHelpContentsExecute(Sender: TObject);
    procedure acNewFlowsheetExecute(Sender: TObject);
    procedure acNewFlowsheetTotalExecute(Sender: TObject);
    procedure acNewFlowsheetViewExecute(Sender: TObject);
    procedure acRenameConsoleItemExecute(Sender: TObject);
    procedure acNewProcedureExecute(Sender: TObject);
    procedure acNewInstrumentExecute(Sender: TObject);
    procedure acNewShiftExecute(Sender: TObject);
    procedure acViewICPConsoleItemExecute(Sender: TObject);
    procedure tvChange(Sender: TObject; Node: TTreeNode);
    procedure tvGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure acRefreshRolesExecute(Sender: TObject);
    procedure acRefreshPermissionsExecute(Sender: TObject);
    procedure acNewScheduleExecute(Sender: TObject);
    procedure acPrintTerminologyExecute(Sender: TObject);
    procedure VA508ComponentAccessibility1ItemQuery(Sender: TObject;
      var Item: TObject);
    procedure VA508ComponentAccessibility1ValueQuery(Sender: TObject;
      var Text: string);
    procedure acMakeCopyExecute(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    fCPGatewayConfig: ICPGatewayConfig;
    fCPADTConfig: ICPProtocolLocation;
    fCurrentItem: ICPConsoleItem;
    fCurrentFrame: ICPEditor;
    fItemTypeNode: array[TCPItemType] of TTreeNode;
    fSearchFrame: TfraSearch;
    fTimeout: boolean;

    procedure ItemSelected(aItem: ICPConsoleItem);
    procedure CreateNewConsoleItem(aItemType: TCPItemType);
    function GetNewItemName(aItemType: TCPItemType; var aNewItemName: string): boolean;
    procedure Shutdown;
  end;

procedure DoHelp(aFile: string);
procedure DoLookup(aQueryName, aPartial: string; aCount: integer; aList: TStrings);
procedure LoadItemList(aType: TCPItemType; aList: TStrings);

var
  frmMain: TfrmMain;
  MyUser: IUser;
  MyCPParameters: ICPParameters;
  MyTerminology: ITerminology;
  MyCPService: ICPService;
  MyPersonService: IPersonService;
  MyVistALogin: IDBConnection;
  MyDBConnection: IDBConnection;
  //MyServerOffset: Int64;

implementation

uses
  DateUtils,
  uCP_GUI,
  uCPXML,
  oCP_Exceptions,
  fAboutDlg,
  fSplash,
  fTester,
  fXMLExport,
  fXMLImport,
  mEditInstrument,
  mEditProcedure,
  mEditFlowsheet,
  mEditFlowsheetView,
  mEditMappingTable,
  mEditBackgroundTask,
  mEditGatewayConfig,
  mEditCPParameters,
  mEditADTConfig,
  mEditShift,
  mLookup,
  mEditFlowsheetTotal,
  fAccessControlList,
  mEditSchedule,
  fTimeoutManager,
  uWordDoc,
  VA508AccessibilityRouter,
  u508Dialogs,
  U_HelpMGR;

{$R *.dfm}

procedure DoHelp(aFile: string);
begin
end;

procedure DoLookup(aQueryName, aPartial: string; aCount: integer; aList: TStrings);
{Return collection as InternalValue^ExternalValue}
var
//  aItem: TFetchItem;
  aNode: IXMLNode;
  aXMLDoc: IXMLDocument;
begin
  aList.Clear;
  aXMLDoc := NewXMLDocument;
  try
    MyDBConnection.ExecQuery(aQueryName, [aPartial], aXMLDoc);
    aXMLDoc.Active := True;
    aNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('RECORD');
    while aNode <> nil do
      try
        AddItem(XMLValueAsStr(aNode, 'ID'), XMLValueAsStr(aNode, 'NAME'), aList);
      finally
        aNode := aNode.NextSibling;
      end;
  except
    aList.Clear;
  end;
end;

procedure LoadItemList(aType: TCPItemType; aList: TStrings);
var
  aNode: TTreeNode;
begin
  with frmMain do
    if fItemTypeNode[aType] <> nil then
      begin
        aNode := fItemTypeNode[aType].getFirstChild;
        while aNode <> nil do
          begin
            aList.AddObject(aNode.Text, TObject(aNode.Data));
            aNode := aNode.getNextSibling;
          end;
      end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  ShowSplashScreen(nil);
  Application.ProcessMessages;
  //Application.HelpFile := ChangeFileExt(Application.ExeName, '.hlp');
  SetApplicationHelpFile;
  Font.Name := Screen.IconFont.Name;

  fSearchFrame := NewSearchFrame(pnlEditor);

  acInitDisplay.Execute;

  {Init timout form from IUser.getTimeOut}
  InitTimeOut(Self.ShutDown);
  UpdateTimeOutInterval(MyUser.getTimeOut);
  fTimeout := False;

  HideSplashScreen;
  FreeSplashScreen;
  edtItemName.TabStop := ScreenReaderSystemActive;
  edtItemType.TabStop := ScreenReaderSystemActive;
  edtItemAccessLevel.TabStop := ScreenReaderSystemActive;
end;

procedure TfrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//var
//  HelpContext: Integer;
begin
  {
  if Key = VK_F1 then
    acHelpContents.Execute;
  }
  if (Key = VK_F1) then
  begin
    Application.HelpCommand(HELP_CONTENTS, 0);
  end;
end;

procedure TfrmMain.acInitDisplayExecute(Sender: TObject);
//var
//  i: integer;
//  aLoadTimer: TDateTime;
begin
  Font := Screen.IconFont;
  lblName.Font.Style := [fsBold];
  lblType.Font.Style := [fsBold];
  lblAccessLevel.Font.Style := [fsBold];

  try
    fCPGatewayConfig := NewCPGatewayConfig(MyDBConnection);
  except
    on e: Exception do
      MessageDlg508(e.Message, mtError, [mbOk], 0);
  end;

  try
    fCPADTConfig := NewCPProtocolLocation(MyDBConnection);
  except
    on e: Exception do
      MessageDlg508(e.Message, mtError, [mbOk], 0);
  end;


  NewTerminology.QueryInterface(ITerminology, MyTerminology);
  MyTerminology.Initialize('CP_CONSOLE', MyDBConnection);
  MyTerminology.SetCaching(MyCPParameters.getTerminologyCacheConnectionString);
//  aLoadTimer := Now;
  MyTerminology.Load;

  MyPersonService := NewPersonService(MyDBConnection);

  MyCPService := NewCPService(MyTerminology, MyPersonService);
  MyCPService.Initialize('CP_CONSOLE', MyDBConnection);
  MyCPService.Load;
  (*
  MyCPService.LoadACL; {This call is not needed in CP Flowsheets}
  *)
  acRefreshPermissions.Execute;

  ItemSelected(nil);
  acLoadConsoleItems.Execute;
end;

procedure TfrmMain.acLoadConsoleItemsExecute(Sender: TObject);
var
  aRootNode: TTreeNode;
//  aXMLNode: IXMLNode;
//  i: integer;
  aConsoleItem: ICPConsoleItem;
  aConsoleItemType: TCPItemType;
  AccessList: ICPParameterFile;
  IsAdmin: boolean;
begin
  tv.Items.Clear;

  aRootNode := tv.Items.AddChild(nil, 'CP Console');

  {
  blj  30 September 2011 - CQ 680.  Need to check visibility of everything in the
  treeview before we load it.
  }

  AccessList := MyCPService.getAccessParameters;
  IsAdmin := MyUser.HasRole('MD ADMINISTRATOR');

  for aConsoleItemType := Low(TCPItemType) to High(TCPItemType) do
  begin
    if (not AccessList.ReadBoolean(pkUser, CONSOLE_ITEM_HEADER[aConsoleItemType], false)) and
       (not IsAdmin) then
      continue;

    // note: we don't have a list to load for Parameters.
    //if aConsoleItemType = ciParameters then
    //  continue;


    if aConsoleItemType in [ciProcedure, ciInstrument, ciParameters, ciFlowsheetView,
        ciFlowsheet, ciBackgroundTask, ciShift, ciFlowsheetTotal] then
    begin
      fItemTypeNode[aConsoleItemType] := tv.Items.AddChild(aRootNode, CONSOLE_ITEM_HEADER[aConsoleItemType]);
      MyCPService.LoadList(fItemTypeNode[aConsoleItemType], aConsoleItemType, False);
    end;
  end;

  (*
  MyCPService.LoadList(fItemTypeNode[ciProcedure], ciProcedure, False);
  MyCPService.LoadList(fItemTypeNode[ciInstrument], ciInstrument, False);
  MyCPService.LoadList(fItemTypeNode[ciBackgroundTask], ciBackgroundTask, False);
  MyCPService.LoadList(fItemTypeNode[ciFlowsheet], ciFlowsheet, False);
  MyCPService.LoadList(fItemTypeNode[ciFlowsheetTotal], ciFlowsheetTotal, False);
  MyCPService.LoadList(fItemTypeNode[ciFlowsheetView], ciFlowsheetView, False);
  MyCPService.LoadList(fItemTypeNode[ciShift], ciShift, False);
  *)
  //MyCPService.LoadList(fItemTypeNode[ciSchedule], ciSchedule, False);


  if MyCPParameters.QueryInterface(ICPConsoleItem, aConsoleItem) = 0 then
  begin
    if AccessList.ReadBoolean(pkUser, aConsoleItem.getName, false) or IsAdmin then
      aConsoleItem.AddToTreeView(tv, fItemTypeNode[ciParameters]);
  end;

  if fCPGatewayConfig.QueryInterface(ICPConsoleItem, aConsoleItem) = 0 then
  begin
    if AccessList.ReadBoolean(pkUser, aConsoleItem.getName, false) or IsAdmin then
      aConsoleItem.AddToTreeView(tv, fItemTypeNode[ciParameters]);
  end;

  if fCPADTConfig.QueryInterface(ICPConsoleItem, aConsoleItem) = 0 then
  begin
    if AccessList.ReadBoolean(pkUser, aConsoleItem.getName, false) or IsAdmin then
      aConsoleItem.AddToTreeView(tv, fItemTypeNode[ciParameters]);
  end;
  
  tv.AlphaSort;
end;

procedure TfrmMain.acMakeCopyExecute(Sender: TObject);
var
//  aNewItem: ICPFlowsheetView;
  aNewItem: ICPConsoleCopy;
begin
  if MessageDlg508('Create a copy of this item?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
//    if MyCPService.NewConsoleItem('Copy of ' + Copy(fCurrentItem.getName, 1, 42), fCurrentItem.getItemType, MyUser).QueryInterface(ICPFlowsheetView, aNewItem) = 0 then
    if MyCPService.NewConsoleItem('Copy of ' + Copy(fCurrentItem.getName, 1, 42), fCurrentItem.getItemType, MyUser).QueryInterface(ICPConsoleCopy, aNewItem) = 0 then
      try
        aNewItem.CopyFrom(fCurrentItem.getID);
        aNewItem.Save;
        tv.Selected := aNewItem.AddToTreeView(tv, fItemTypeNode[fCurrentItem.getItemType]);
        fItemTypeNode[ciFlowsheetView].AlphaSort(True);
      except
        on E: ECPConsoleCreate do
          MessageDlg508(E.Message, mtError, [mbOK], 0);
      else
        raise;
      end;
end;

procedure TfrmMain.ItemSelected(aItem: ICPConsoleItem);
var
  aCPParameters: ICPParameters;
  aCPGatewayConfig: ICPGatewayConfig;
  aCPProtocolLocation: ICPProtocolLocation;
  i: integer;
//  aTreeNode: TTreeNode;
//  tt: TTermType;
begin
  fSearchFrame.Visible := False;

  fCurrentItem := nil;
  fCurrentFrame := nil;

  {Clear out all existing frames}
  for i := scbx.ComponentCount - 1 downto 0 do
    scbx.Components[i].Free;

  fCurrentItem := aItem;

  scbx.Visible := Assigned(fCurrentItem);
  pnlFooter.Visible := Assigned(fCurrentItem);
  acDeleteConsoleItem.Enabled := False;
  acRenameConsoleItem.Enabled := False;
  tv.ReadOnly := True;

  if Assigned(fCurrentItem) then
    try
      fCurrentItem.Load;
      edtItemName.Text := fCurrentItem.getName;
      edtItemType.Text := CONSOLE_ITEM_HEADER[fCurrentItem.getItemType];
      //edtItemAccessLevel.Text := CP_ACCESS_LEVEL[MyCPService.ACLAccessLevel(fCurrentItem, MyUser as IPerson)];

      acDeleteConsoleItem.Enabled :=
        fCurrentItem.getCanDelete;
        //and (MyCPService.ACLAccessLevel(fCurrentItem, MyUser as IPerson) in [alReadWriteDelete, alFullControl]);

      acRenameConsoleItem.Enabled :=
        fCurrentItem.getCanRename;
        //and (MyCPService.ACLAccessLevel(fCurrentItem, MyUser as IPerson) in [alReadWrite, alReadWriteDelete, alFullControl]);


      case aItem.getItemType of

        ciInstrument:
          with NewEditInstrumentFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              fCurrentFrame.Initialize(fCurrentItem);

        ciProcedure:
          with NewEditProcedureFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              begin
                MyCPService.LoadList(cklstbxInstruments.Items, ciInstrument, False);
                fCurrentFrame.Initialize(fCurrentItem);
              end;

        ciFlowsheet:
          with NewEditFlowsheetFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              fCurrentFrame.Initialize(fCurrentItem);

        ciFlowsheetTotal:
          with NewEditFlowsheetTotalFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              begin
                MyTerminology.LoadList(cbxDefaultUnit.Items, ttUnit);
                MyTerminology.LoadList(lbxTermAll.Items, ttObservation);
                fCurrentFrame.Initialize(fCurrentItem);
              end;

        ciFlowsheetView:
          with NewEditFlowsheetViewFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              fCurrentFrame.Initialize(fCurrentItem);

        ciBackgroundTask:
          with NewEditBackgroundTaskFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              fCurrentFrame.Initialize(fCurrentItem);

        ciParameters:
          begin
            if fCurrentItem.QueryInterface(ICPParameters, aCPParameters) = 0 then
              with NewEditCPParametersFrame(scbx) do
                if GetInterface(ICPEditor, fCurrentFrame) then
                  fCurrentFrame.Initialize(fCurrentItem);

            if fCurrentItem.QueryInterface(ICPGatewayConfig, aCPGatewayConfig) = 0 then
              with NewGatewayAdminFrame(scbx) do
                if GetInterface(ICPEditor, fCurrentFrame) then
                  fCurrentFrame.Initialize(fCurrentItem);

            if fCurrentItem.QueryInterface(ICPProtocolLocation, aCPProtocolLocation) = 0 then
              with NewADTConfigFrame(scbx) do
                if GetInterface(ICPEditor, fCurrentFrame) then
                  fCurrentFrame.Initialize(fCurrentItem);
          end;

        ciShift:
          with NewEditShiftFrame(scbx) do
            if GetInterface(ICPEditor, fCurrentFrame) then
              fCurrentFrame.Initialize(fCurrentItem);

//        ciSchedule:
//          with NewEditScheduleFrame(scbx) do
//            if GetInterface(ICPEditor, fCurrentFrame) then
//              fCurrentFrame.Initialize(fCurrentItem);
      end;
    except
      on e: ECPLoadFailure do
        begin
          MessageDlg508(e.Message, mtError, [mbOk], 0);

          for i := scbx.ComponentCount - 1 downto 0 do
            scbx.Components[i].Free;

          fSearchFrame.Visible := False;

          fCurrentFrame := nil;
          fCurrentItem := nil;

          scbx.Visible := False;
          pnlFooter.Visible := False;
        end;
    else
      raise;
    end
  else
    begin
      edtItemName.Text := '<No Item Selected>';
      edtItemType.Text := '';
      edtItemAccessLevel.Text := '';
    end;

  {Do we allow access?}
  if Assigned(fCurrentFrame) then
    begin
//      if (MyCPService.ACLAccessLevel(fCurrentItem, MyUser as IPerson) <> alReadOnly) then
//        begin
//          fCurrentFrame.SetReadOnly(False);
//          acSaveItem.Enabled := True;
//        end
//      else
//        begin
//          fCurrentFrame.SetReadOnly(True);
//          acSaveItem.Enabled := False;
//        end;

      fCurrentFrame.SetReadOnly(False);
      acSaveItem.Enabled := True;


      {One thing that might overrule everything else}
      if (fCurrentItem.getItemType = ciFlowsheetView) then
        if (fCurrentItem as ICPFLowsheetView).getIsNationalTitle then
          fCurrentFrame.SetReadOnly(True);
    end;
end;

procedure TfrmMain.Shutdown;
begin
  fTimeout := True;
  Close;
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if not fTimeOut then
    CanClose := (MessageDlg508('Okay to close the CP Console Application?', mtConfirmation, [mbYes, mbNo], 0) = mrYes)
  else
    CanClose := True;

  if CanClose then
    begin
      ItemSelected(nil);
      MyCPService := nil;
      MyPersonService := nil;
      MyTerminology := nil;
      fCPADTConfig := nil;
      fCPGatewayConfig := nil;
      if MyDBConnection.IsConnected then
        MyDBConnection.Disconnect;
    end;
end;

procedure TfrmMain.acExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.tvChange(Sender: TObject; Node: TTreeNode);
var
  aItem: ICPConsoleItem;
  aCopyItem: ICPConsoleCopy;
begin
  ItemSelected(nil);
  acMakeCopy.Enabled := false;


  if Assigned(Node.Data) then
    begin
      if TObject(Node.Data).GetInterface(ICPConsoleItem, aItem) then
      begin
        ItemSelected(aItem);
//        acMakeCopy.Enabled := aItem.getItemType = ciFlowsheetView;
        acMakeCopy.Enabled := aItem.QueryInterface(ICPConsoleCopy,aCopyItem)=0;  // zzzzzzandria 2011-10-03
      end
      else
        begin
          acDeleteConsoleItem.Enabled := False;
          acRenameConsoleItem.Enabled := False;
          tv.ReadOnly := True;
        end;
    end
  else
    begin
      acDeleteConsoleItem.Enabled := False;
      acRenameConsoleItem.Enabled := False;
      tv.ReadOnly := True;
    end;
end;

procedure TfrmMain.acSaveItemExecute(Sender: TObject);
begin
  if Assigned(fCurrentFrame) then
    try
      fCurrentFrame.Save;
    except
      on E: Exception do
        MessageDlg508('Error Saving Item' + #13 + E.Message, mtError, [mbOk], 0);
    end;
  tv.Invalidate; {Updates the icons}
end;

procedure TfrmMain.acImportFromXMLExecute(Sender: TObject);
begin
  if ExecuteXMLImport then
    begin
      MessageDlg508('CP Console needs to be restarted now to apply these' + #13 +
        'changes. Click OK to close the application.',
        mtInformation, [mbOk], 0);
      Application.Terminate;
      Exit;
    end;
end;

procedure TfrmMain.acExportToXMLExecute(Sender: TObject);
begin
  ExecuteXMLExport;
end;

procedure TfrmMain.tvGetImageIndex(Sender: TObject; Node: TTreeNode);
var
  aItem: ICPConsoleItem;
  aView: ICPFlowsheetView;
begin
  if Node.Data = nil then
    begin
      if (Node.Expanded) or (Node.Selected) then
        Node.ImageIndex := 0
      else
        Node.ImageIndex := 1;
    end
  else
    if TObject(Node.Data).GetInterface(ICPConsoleItem, aItem) then
      case aItem.getItemType of
        ciInstrument:
          begin
            if aItem.getActive then
              Node.ImageIndex := 8
            else
              Node.ImageIndex := 7;
          end;
        ciFlowsheetView:
          begin
            if aItem.QueryInterface(ICPFlowsheetView, aView) = 0 then
              if aView.getIsNationalTitle then
                Node.ImageIndex := 27
              else
                if aItem.getActive then
                  Node.ImageIndex := 9
                else
                  Node.ImageIndex := 18;
          end;
        ciBackgroundTask:
          begin
            Node.ImageIndex := 17;
          end;
        ciFlowsheet:
          begin
            if aItem.getActive then
              Node.ImageIndex := 10
            else
              Node.ImageIndex := 11;
          end;
        ciMappingTable:
          begin
            if aItem.getActive then
              Node.ImageIndex := 12
            else
              Node.ImageIndex := 13;
          end;
        ciProcedure:
          begin
            if aItem.getActive then
              Node.ImageIndex := 6
            else
              Node.ImageIndex := 16;
          end;
        ciParameters:
          begin
            Node.ImageIndex := 19;
          end;
        ciShift:
          begin
            if aItem.getActive then
              Node.ImageIndex := 24
            else
              Node.ImageIndex := 32;
          end;
        ciFlowsheetTotal:
          begin
            if aItem.getActive then
              Node.ImageIndex := 25
            else
              Node.ImageIndex := 26;
          end;
//        ciSchedule:
//          begin
//            if aItem.getActive then
//              Node.ImageIndex := 30
//            else
//              Node.ImageIndex := 31;
//          end;
      else
        Node.ImageIndex := -1;
      end
    else
      Node.ImageIndex := -1;
  Node.SelectedIndex := Node.ImageIndex;
end;

procedure TfrmMain.VA508ComponentAccessibility1ItemQuery(Sender: TObject;
  var Item: TObject);
begin
  Item := tv.Selected;
end;

procedure TfrmMain.VA508ComponentAccessibility1ValueQuery(Sender: TObject;
  var Text: string);
var
  aItem: ICPConsoleItem;
  aView: ICPFlowsheetView;
begin
  {
    blj 25 June 2009 - 508: Had to stick this routine in here as, for some reason,
    JAWS wasn't seeing the freakin' VA508ImageListLabeller component.  Grrrr.
  }

  if Assigned(tv.Selected) then
    begin
      Text := tv.Selected.Text;

      if not assigned(tv.Selected.Data) then
        Exit;

      if TObject(tv.Selected.Data).GetInterface(ICPConsoleItem, aItem) then
        begin
          //if aItem.getItemType in [ciProcedure, ciInstrument, ciFlowsheetView, ciFlowsheet, ciShift, ciFlowsheetTotal, ciSchedule] then
          if aItem.getItemType in [ciProcedure, ciInstrument, ciFlowsheetView, ciFlowsheet, ciShift, ciFlowsheetTotal] then
            if aItem.getActive then
              Text := Text + ' Active '
            else
              Text := Text + ' Inactive ';

          if aItem.QueryInterface(ICPFlowsheetView, aView) = 0 then
            if aView.getIsNationalTitle then
              Text := Text + ' Locked ';
        end;
    end;
end;

procedure TfrmMain.acSearchForItemExecute(Sender: TObject);
begin
  ItemSelected(nil);
  with fSearchFrame do
    begin
      edtSearchFor.Text := '';
      lvSearch.Items.Clear;
      Visible := True;
      edtSearchFor.SetFocus;
    end;
end;

procedure TfrmMain.acViewICPConsoleItemExecute(Sender: TObject);
var
  aItem: ICPConsoleItem;
begin
  if Assigned(tv.Selected) then
    if Assigned(tv.Selected.Data) then
      if TObject(tv.Selected.Data).GetInterface(ICPConsoleItem, aItem) then
        MessageDlg508(
          'ID:' + #9 + #9 + aItem.getID + #13 +
          'Name:' + #9 + #9 + aItem.getName + #13 +
          'Type:' + #9 + #9 + CONSOLE_ITEM_NAME[aItem.getItemType] + #13 +
          'Active:' + #9 + #9 + CP_YN_TEXT[aItem.getActive] + #13 +
          'Display: ' + #9 + aItem.getDisplayName + #13 +
          'Comment: ' + #9 + aItem.getComment,
          mtInformation, [mbOk], 0);
end;

procedure TfrmMain.CreateNewConsoleItem(aItemType: TCPItemType);
var
  aName: string;
begin
  ItemSelected(nil);
  if GetNewItemName(aItemType, aName) then
    try
      with MyCPService.NewConsoleItem(aName, aItemType, MyUser) do
        begin
          tv.Selected := AddToTreeView(tv, fItemTypeNode[aItemType]);
          fItemTypeNode[aItemType].AlphaSort(True);
        end;
    except
      on E: ECPConsoleCreate do
        MessageDlg508(E.Message, mtError, [mbOK], 0);
    else
      raise;
    end;
end;

procedure TfrmMain.acNewFlowsheetExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciFlowsheet);
end;

procedure TfrmMain.acNewFlowsheetTotalExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciFlowsheetTotal);
end;

procedure TfrmMain.acNewFlowsheetViewExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciFlowsheetView);
end;

procedure TfrmMain.acNewScheduleExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciSchedule);
end;

procedure TfrmMain.acNewShiftExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciShift);
end;

procedure TfrmMain.acNewInstrumentExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciInstrument);
end;

procedure TfrmMain.acNewProcedureExecute(Sender: TObject);
begin
  CreateNewConsoleItem(ciProcedure);
end;

procedure TfrmMain.acPrintTerminologyExecute(Sender: TObject);
var
  aTerm: ITerm;
//  aChildTerm: ITerm;
  aList: TStringList;
  i, j: integer;
  atmpList: TStringList;
//  aQual: TTermType;
  aFileName: string;

  procedure AddQualifiers(aQualType: TTermType);
  var
    i: integer;
    aQualifier: ITerm;
  begin
    aTerm.LoadListOfPairs(aQualType, atmpList);
    if atmpList.Count > 0 then
      begin
        for i := 0 to atmpList.Count - 1 do
          if atmpList.Objects[i].GetInterface(ITerm, aQualifier) then
            atmpList[i] := aQualifier.getTerm + #9 + aQualifier.getID;
        LineFeed;
        ClearTabs;
        DrawSubHeaderA(TERMTYPE[aQualType]);
        SetTabs(1.0, taLeftJustify);
        NumberedList(atmpList);
      end;
  end;

begin
  if MessageDlg508('This will create a MS-Word document with all the terminology' + #13 +
    'in it. Are you sure you want to continue?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
    Exit;

  with TOpenDialog.Create(Application) do
    try
      if Execute then
        begin
          aFileName := Filename;
          if FileExists(aFileName) then
            DeleteFile(aFileName);
        end
      else
        Exit;
    finally
      Free;
    end;

  InitWordDoc;
  aList := TStringList.Create;
  atmpList := TStringList.Create;
  with MyTerminology do
    begin
      LoadList(aList, ttObservation);
      aList.Sorted := True;
      for i := 0 to aList.Count - 1 do
        if aList.Objects[i].GetInterface(ITerm, aTerm) then
          if aTerm.getActive then
            begin
              atmpList.Clear;
              atmpList.Add('ID: ' + #9 + aTerm.getID);
              atmpList.Add('Name: ' + #9 + aTerm.getTerm);

              Caption := 'Processing ''' + aTerm.getTerm + '''';
              Application.ProcessMessages;
              DrawHeader(atmpList);
              //AddIndex(aTerm.getTerm);
              LineFeed;

              atmpList.Clear;
              atmpList.Add('Display Name:' + #9 + aTerm.getDisplayName);
              atmpList.Add('Abbreviation:' + #9 + aTerm.getAbbreviation);
              atmpList.Add('Description:' + #9 + aTerm.getDescription);
              atmpList.Add('Data Type:' + #9 + TERMDATATYPE[aTerm.getDataType]);

              case aTerm.getDataType of
                dtPickList:
                  begin
                    atmpList.Add('     Multi-Select PickList:' + #9 + CP_YN_TEXT[aTerm.getMultiSelectPickList]);
                  end;
                dtBoolean:
                  begin
                    atmpList.Add('     Boolean Value True:' + #9 + aTerm.getBooleanValueTrue);
                    atmpList.Add('     Boolean Value False:' + #9 + aTerm.getBooleanValueFalse);
                  end;
                dtComplex:
                  begin
                    for j := 0 to aTerm.getChildCount - 1 do
                      aTmpList.Add(
                        Format('     Child Term %d:', [j + 1]) + #9 +
                        aTerm.getChildTerm(j).getTerm);
                  end;
              end;

              atmpList.Add('Help Text:' + #9 + aTerm.getHelpText);

              ClearTabs;
              ClearHangingIndent;
              SetHangingIndent(3.0, True);
              DefaultList(atmpList);
              ClearTabs;
              ClearHangingIndent;

              AddQualifiers(ttValue);
              AddQualifiers(ttMethod);
              AddQualifiers(ttPosition);
              AddQualifiers(ttLocation);
              AddQualifiers(ttQuality);
              AddQualifiers(ttProduct);
              AddQualifiers(ttNullFlavor);

              PageFeed;
            end;
      uWordDoc.SaveFile(aFileName);
    end;
end;

procedure TfrmMain.acRefreshPermissionsExecute(Sender: TObject);
  procedure EnableActionByPermission(aAction: TAction; aUser: IUser; aFolderName: string);
  var
    AccessList: ICPParameterFile;
    Enabled: Boolean;
  begin
    AccessList := MyCPService.getAccessParameters;
    {
    result := AccessList.ReadBoolean(pkUser, CONSOLE_ITEM_HEADER[ciFlowsheetView], false) or
       AccessList.ReadBoolean(pkUser, CONSOLE_ITEM_HEADER[ciProcedure], false) or
       AccessList.ReadBoolean(pkUser, CONSOLE_ITEM_HEADER[ciInstrument], false);

    'Procedure', 'Instrument', 'Parameters', 'Mapping Table', 'Flowsheet View',
    'Flowsheet', 'Background Task', 'Shift', 'Flowsheet Total', 'Schedule'
    }
    Enabled := AccessList.ReadBoolean(pkUser, aFolderName, false);

    Enabled := Enabled or MyUser.HasRole('MD ADMINISTRATOR');

    aAction.Enabled := Enabled;

    AccessList := nil;
  end;
begin
  InitStatusBar(sbar, MyUser, MyDBConnection);
  (*
  EnableActionByRole(acExportToXML, MyUser, 'MD ADMINISTRATOR');
  EnableActionByRole(acImportFromXML, MyUser, 'MD ADMINISTRATOR');
  *)

  EnableActionByRole(acDeleteConsoleItem, MyUser, 'MD ADMIN');
  EnableActionByRole(acRenameConsoleItem, MyUser, 'MD ADMINISTRATOR');
  EnableActionByRole(acSaveItem, MyUser, 'MD ADMINISTRATOR');
  EnableActionByPermission(acNewFlowsheet, MyUser, CONSOLE_ITEM_HEADER[ciFlowsheet]);
  EnableActionByRole(acNewMappingTable, MyUser, 'MD ADMINISTRATOR');
  EnableActionByPermission(acNewProcedure, MyUser, CONSOLE_ITEM_HEADER[ciProcedure]);
  EnableActionByPermission(acNewInstrument, MyUser, CONSOLE_ITEM_HEADER[ciInstrument]);
  EnableActionByPermission(acNewFlowsheetView, MyUser, CONSOLE_ITEM_HEADER[ciFlowsheetView]);
  EnableActionByPermission(acNewShift, MyUser, CONSOLE_ITEM_HEADER[ciShift]);
  EnableActionByPermission(acNewFlowsheetTotal, MyUser, CONSOLE_ITEM_HEADER[ciFlowsheetTotal]);

  if MyUser.HasRole('MD ADMINISTRATOR') then
    Caption := Application.Title
  else
    Caption := Format('%s - [Restricted Access]', [Application.Title])
end;

procedure TfrmMain.acRefreshRolesExecute(Sender: TObject);
var
  x: string;

const
  ASSIGNED: array[Boolean] of string = ('Not Assigned', 'Assigned');

  procedure AddRole(aRoleName: string);
  begin
    x := x + #13 + Format('%s: %s', [aRoleName, ASSIGNED[MyUser.HasRole(aRoleName)]]);
  end;

begin
  MyUser.RefreshAccess;
  acRefreshPermissions.Execute;
  ItemSelected(fCurrentItem);
  x := 'User Roles refreshed ' + DateTimeToStr(Now) + '.' + #13;
  AddRole('MD ADMINISTRATOR');
  AddRole('MD MANAGER');
  AddRole('MD HL7 MANAGER');
  AddRole('MD READ-ONLY');
  AddRole('MD TRAINEE');
  MessageDlg508(x, mtInformation, [mbOk], 0);
end;

procedure TfrmMain.acDeleteConsoleItemExecute(Sender: TObject);
begin
  if Assigned(fCurrentItem) then
    if fCurrentItem.getCanDelete then
      if MessageDlg508('Are you sure you want to delete' + #13 + CONSOLE_ITEM_HEADER[fCurrentItem.getItemType] + ': ''' + fCurrentItem.getName + '''?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        try
          MyCPService.DeleteConsoleItem(fCurrentItem);
          fCurrentItem := nil;
          tv.Items.Delete(tv.Selected);
        except
          on E: Exception do
            MessageDlg508(e.Message, mtError, [mbOk], 0);
        end;
end;

procedure TfrmMain.acRenameConsoleItemExecute(Sender: TObject);
var
  aItem: ICPConsoleItem;
  aName: string;

  function GetNewName(var aName: string): boolean;
  begin
    Result := False;
    if InputQuery(Format('Rename %s', [CONSOLE_ITEM_HEADER[aItem.getItemType]]), 'New Name:', aName) then
      if (Length(aName) > 2) and (Length(aName) < 51) then
        if (Pos('^', aName) < 1) and (Pos('?', aName) < 1) then
          Result := True
        else
          MessageDlg508('Invalid characters in name.', mtError, [mbOk], 0)
      else
        MessageDlg508('Name must be 3-50 characters in length.', mtError, [mbOk], 0)
  end;

begin
  if Assigned(tv.Selected) then
    try
      if TObject(tv.Selected.Data).GetInterface(ICPConsoleItem, aItem) then
        begin
          aName := tv.Selected.Text;
          if GetNewName(aName) then
            begin
              MyCPService.RenameConsoleItem(aItem, aName);
              tv.Selected.Text := aName;
              edtItemName.Text := aName;
            end;
        end;
    except
      on E: ECPRenameFailure do
        MessageDlg508('Error renaming item.' + #13 + E.Message, mtError, [mbOk], 0);
    else
      raise;
    end;
end;

function TfrmMain.GetNewItemName(aItemType: TCPItemType; var aNewItemName: string): boolean;
begin
  Result := False;
  if InputQuery(Format('Create New %s', [CONSOLE_ITEM_HEADER[aItemType]]), 'Name:', aNewItemName) then
    if (Length(aNewItemName) > 2) and (Length(aNewItemName) < 51) then
      if (Pos('^', aNewItemName) < 1) and (Pos('?', aNewItemName) < 1) then
        Result := True
      else
        MessageDlg508('Invalid characters in name.', mtError, [mbOk], 0)
    else
      MessageDlg508('Name must be 3-50 characters in length.', mtError, [mbOk], 0)
end;

procedure TfrmMain.acAboutExecute(Sender: TObject);
begin
  AboutDlg.Execute;
end;

procedure TfrmMain.acHelpContentsExecute(Sender: TObject);
begin
  try
    //check for network help and download if applicable (non-threaded)
    LoadHelpFile(Application.HelpFile);
    Application.HelpCommand(HELP_CONTENTS, 0);
  except
    on E: Exception do
      MessageDlg508(E.Message + #13#10 +
        '"' + Application.HelpFile + '"', mtError, [mbOK], 0);
  end;
end;

initialization
  SpecifyFormIsNotADialog(TfrmMain);

end.

