unit fUpdateTerminology;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  XMLIntf,
  XMLDoc,
  oInterfaces,
  uCP_GUI,
  StdCtrls,
  ExtCtrls,
  ActnList,
  ExtActns,
  ComCtrls, System.Actions;

type
  TfrmUpdateTerminology = class(TForm)
    ActionList1: TActionList;
    WWW: TDownLoadURL;
    btnGetUpdate: TButton;
    lblStatus: TLabel;
    pBar: TProgressBar;
    acGetHeader: TAction;
    acGetUpdate: TAction;
    SaveDialog1: TSaveDialog;
    acInstallUpdate: TAction;
    pnlHeader: TPanel;
    Image1: TImage;
    Bevel4: TBevel;
    gbxInstalled: TGroupBox;
    lblCurrentDescription: TLabel;
    lblCurrentFramework: TLabel;
    lblCurrentVersion: TLabel;
    gbxWeb: TGroupBox;
    Label10: TLabel;
    Label7: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lblWebVersion: TLabel;
    lblWebDescription: TLabel;
    Bevel1: TBevel;
    btnCancel: TButton;
    Label4: TLabel;
    lblWebFramework: TLabel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    acTestLocalInstall: TAction;
    btnTestInstall: TButton;
    procedure FormCreate(Sender: TObject);
    procedure WWWDownloadProgress(Sender: TDownLoadURL; Progress,
      ProgressMax: Cardinal; StatusCode: TURLDownloadStatus; StatusText: string;
      var Cancel: Boolean);
    procedure acGetHeaderExecute(Sender: TObject);
    procedure acGetUpdateExecute(Sender: TObject);
    procedure acInstallUpdateExecute(Sender: TObject);
    procedure UpdateStatusMessage(const aMsg: string; aProgress, aProgressMax: integer);
    procedure acTestLocalInstallExecute(Sender: TObject);
  protected
    fFramework: string;
    fVersion: integer;
    fUpdateFile: string;
    Description: string;
    fSourceFile: string;
    fTerminologyEditor: ITerminologyEditor;
    fCPParameters: ICPParameters;
  end;

function UpdateTerminology(aTerminology: ITerminology; aCPParameters: ICPParameters): boolean;

implementation

uses
  fMain,
  u508Dialogs;

{$R *.dfm}

function UpdateTerminology(aTerminology: ITerminology; aCPParameters: ICPParameters): boolean;
begin
  with TfrmUpdateTerminology.Create(Application) do
    try
      try
        if aTerminology.QueryInterface(ITerminologyEditor, fTerminologyEditor) <> 0 then
          MessageDlg508('Your Terminology Service does not support importing.', mtInformation, [mbOk], 0)
        else
          begin
            fCPParameters := aCPParameters;
            acGetHeader.Execute;
            btnTestInstall.Visible := MyUser.HasRole('XUPROGMODE');
            ShowModal;
          end
      finally
        Free;
      end;
    except
      on E: Exception do
        begin
          MessageDlg508('Error: ' + E.Message, mtError, [mbOk], 0);
        end
    end;
  result := true;
end;

procedure TfrmUpdateTerminology.FormCreate(Sender: TObject);
begin
  lblCurrentFramework.Font.Style := [fsBold];
  lblCurrentDescription.Font.Style := [fsBold];
  lblCurrentVersion.Font.Style := [fsBold];
  lblWebFramework.Font.Style := [fsBold];
  lblWebVersion.Font.Style := [fsBold];
  lblWebDescription.Font.Style := [fsBold];

  lblStatus.Visible := False;
  pBar.Visible := False;
end;

procedure TfrmUpdateTerminology.UpdateStatusMessage(const aMsg: string; aProgress, aProgressMax: integer);
begin
  lblStatus.Caption := Format('%s ... %d of %d', [aMsg, aProgress, aProgressMax]);

  if aProgressMax > 0 then
    begin
      pBar.Visible := True;
      pbar.Position := trunc(frac(aProgress / aProgressMax) * 100);
    end
  else
    pBar.Visible := False;

  Application.ProcessMessages;
end;

procedure TfrmUpdateTerminology.acGetHeaderExecute(Sender: TObject);
var
  aStream: TMemoryStream;
begin
  aStream := TMemoryStream.Create;
  with NewXMLDocument do
    try
      lblCurrentFramework.Caption := fCPParameters.getUpdateFramework;
      lblCurrentVersion.Caption := IntToStr(fCPParameters.getTerminologyVersion);
      lblCurrentDescription.Caption := fCPParameters.getTerminologyDescription;

      GetUpdateFile(fCPParameters.getUpdateMaster, aStream);
      LoadFromStream(aStream);

      fFramework := DocumentElement.ChildValues['FRAMEWORK'];
      fSourceFile := DocumentElement.ChildValues['SOURCE'];
      lblWebFramework.Caption := fFramework;
      lblWebVersion.Caption := DocumentElement.ChildValues['VERSION'];
      lblWebDescription.Caption := DocumentElement.ChildValues['DESCRIPTION'];
    except
      on E: Exception do
        raise Exception.CreateFmt('Error Accessing CP Terminology Server' + #13 + 'Error: %s', [e.Message]);
    end;
  FreeAndNil(aStream);
end;

procedure TfrmUpdateTerminology.acGetUpdateExecute(Sender: TObject);
begin
  with TSaveDialog.Create(Application) do
    try
      Filter := 'XML Documents|*.xml';
      FilterIndex := 1;
      Options := [ofOverwritePrompt, ofHideReadOnly, ofPathMustExist, ofEnableSizing];
      Title := 'Save Terminology Import File';
      DefaultExt := '.xml';
      if Execute then
        begin
          acGetUpdate.Enabled := False;
          btnCancel.Enabled := False;
          Self.Enabled := False;
          fUpdateFile := FileName;
          WWW.URL := Format(fCPParameters.getUpdateSource, [fSourceFile]);
          WWW.Filename := fUpdateFile;
          WWW.ExecuteTarget(nil);
          pBar.Visible := False;
          UpdateStatusMessage('Installing new terminology', 0, 1);
          acInstallUpdate.Execute;
          if FileExists(fUpdateFile) then
            DeleteFile(fUpdateFile);
          fCPParameters.setTerminologyVersion(StrToInt(lblWebVersion.Caption));
          fCPParameters.setTerminologyDescription(lblWebDescription.Caption);
          fCPParameters.Save;
          MessageDlg508('Terminology Files Updated', mtInformation, [mbOK], 0);
        end;
    finally
      //acGetUpdate.Enabled := False;
      //btnCancel.Enabled := False;
      //Enabled := False;
      Free;
    end;
  ModalResult := mrOK;
end;

procedure TfrmUpdateTerminology.acInstallUpdateExecute(Sender: TObject);
var
  aXMLDoc: IXMLDocument;
begin
  try
    aXMLDoc := NewXMLDocument;
    aXMLDoc.LoadFromFile(fUpdateFile);
    fTerminologyEditor.ImportTerminology(aXMLDoc, UpdateStatusMessage);
  except
    on E: Exception do
      MessageDlg508('Error Updating Terminology' + #13 + E.Message, mtError, [mbOk], 0);
  end;
end;

procedure TfrmUpdateTerminology.acTestLocalInstallExecute(Sender: TObject);
var
  aHeaderNode: IXMLNode;
  aFileName: string;
begin
  with TOpenDialog.Create(Application) do
    try
      Filter := 'XML Documents|*.xml';
      FilterIndex := 1;
      Options := [ofOverwritePrompt, ofHideReadOnly, ofPathMustExist, ofEnableSizing];
      Title := 'Open Test Terminology Import File';
      DefaultExt := '.xml';
      if Execute then
        begin
          aFileName := FileName;
          with NewXMLDocument do
            begin
              Active := True;
              LoadFromFile(aFileName);
              aHeaderNode := DocumentElement.ChildNodes.FindNode('HEADER');
              if not Assigned(aHeaderNode) then
                Exception.Create('Invalid Terminology File');
              lblWebVersion.Caption := aHeaderNode.ChildValues['VERSION'];
              lblWebDescription.Caption := aHeaderNode.ChildValues['DESCRIPTION'];
              lblWebFramework.Caption := aHeaderNode.ChildValues['FRAMEWORK'];
            end;
          acGetUpdate.Enabled := False;
          btnCancel.Enabled := False;
          btnTestInstall.Enabled := False;
          Self.Enabled := False;
          fUpdateFile := FileName;
          pBar.Visible := False;
          UpdateStatusMessage('Installing new terminology', 0, 1);
          acInstallUpdate.Execute;
          fCPParameters.setTerminologyVersion(StrToInt(lblWebVersion.Caption));
          fCPParameters.setTerminologyDescription(lblWebDescription.Caption);
          fCPParameters.Save;
          MessageDlg508('Terminology Files Updated', mtInformation, [mbOK], 0);
        end;
    finally
      //acGetUpdate.Enabled := False;
      //btnCancel.Enabled := False;
      //Enabled := False;
      Free;
    end;
  ModalResult := mrOK;
end;

procedure TfrmUpdateTerminology.WWWDownloadProgress(
  Sender: TDownLoadURL; Progress, ProgressMax: Cardinal;
  StatusCode: TURLDownloadStatus; StatusText: string; var Cancel: Boolean);
begin
  lblStatus.Caption := Format('Downloading %s', [fUpdateFile]);
  Cancel := False;
  if progressMax > 0 then
    pBar.Position := trunc((Progress / ProgressMax) * 100)
  else
    pBar.Position := 0;
  lblStatus.Visible := True;
  pBar.Visible := True;
  Application.ProcessMessages;
end;

end.

