unit mEditADTConfig;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  oInterfaces,
  StdCtrls,
  ComCtrls, VA508AccessibilityManager;

type
  TfraEditADTConfig = class(TForm, ICPEditor)
    lvTargets: TListView;
    lblTargets: TLabel;
    btnNewTgt: TButton;
    btnDeleteSelectedTgt: TButton;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    VA508ComponentAccessibility1: TVA508ComponentAccessibility;
    procedure lvTargetsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure btnDeleteSelectedTgtClick(Sender: TObject);
    procedure btnNewTgtClick(Sender: TObject);
    procedure VA508ComponentAccessibility1CaptionQuery(Sender: TObject;
      var Text: string);
    procedure VA508ComponentAccessibility1ItemQuery(Sender: TObject;
      var Item: TObject);
    procedure VA508ComponentAccessibility1ValueQuery(Sender: TObject;
      var Text: string);
  private
    { Private declarations }
    FProtocolLocation: ICPProtocolLocation;
  public
    { Public declarations }
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Save; stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  end;

function NewADTConfigFrame(aParent: TWinControl): TfraEditADTConfig;

implementation

uses
  oCPProtocolLocation,
  fMain,
  fAddADTTarget,
  u508Dialogs;

{$R *.dfm}

function NewADTConfigFrame(aParent: TWinControl): TfraEditADTConfig;
begin
  Result := TfraEditADTConfig.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      Visible := true;
    end;
end;

{ TfraEditADTConfig }

procedure TfraEditADTConfig.btnDeleteSelectedTgtClick(Sender: TObject);
var
  Targets: ICPADTTargetList;
begin
  if lvTargets.Selected <> nil then
    if MessageDlg508('Are you sure you want to delete this item?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
        Targets := FProtocolLocation.getADTTargetList as ICPADTTargetList;
        Targets.DeleteItem(ICPADTTarget(lvTargets.Selected.Data));
        lvTargets.Items.Delete(lvTargets.ItemIndex);
        lvTargets.ItemIndex := -1;
        Initialize(FProtocolLocation);
      end;
end;

procedure TfraEditADTConfig.btnNewTgtClick(Sender: TObject);
var
  AddTgtFrm: TFrmAddADTTarget;
  Protocols: ICPProtocolList;
  Divisions: ICPDivisionList;
  Target: ICPADTTarget;
  Intf: IInterface;
  Division: ICPDivision;
  Ward: ICPWard;
begin
  AddTgtFrm := TFrmAddADTTarget.Create(nil);
  try
    Protocols := FProtocolLocation.getProtocolList as ICPProtocolList;
    Divisions := FProtocolLocation.getDivisionList as ICPDivisionList;
    AddTgtFrm.Initialize(Protocols, Divisions);
    if AddTgtFrm.ShowModal = mrOK then
      begin
        Ward := nil;
        Intf := IInterface(AddTgtFrm.tvDivisions.Selected.Data);
        if Intf.QueryInterface(ICPWard, Ward) = 0 then
          Intf := IInterface(AddTgtFrm.tvDivisions.Selected.Parent.Data);

        Intf.QueryInterface(ICPDivision, Division);

        Target := (FProtocolLocation.getADTTargetList as ICPADTTargetList).NewTarget(
          ICPProtocol(AddTgtFrm.lvProtocols.Selected.Data),
          Division,
          Ward,
          AddTgtFrm.edtADTTargetName.Text,
          AddTgtFrm.cbMessageType.Text,
          AddTgtFrm.cbEventType.Text);
        (FProtocolLocation.getADTTargetList as ICPADTTargetList).Add(Target);
      end;
  finally
    FreeAndNil(AddTgtFrm);
  end;
  Initialize(FProtocolLocation);
end;

procedure TfraEditADTConfig.Initialize(aItem: IInterface);
var
  I: Integer;
  Item: TListItem;
  Targets: ICPADTTargetList;
  Target: ICPADTTarget;
begin
  lvTargets.Items.BeginUpdate;
  try
    lvTargets.Items.Clear;
    FProtocolLocation := aItem as ICPProtocolLocation;
    Targets := FProtocolLocation.getADTTargetList as ICPADTTargetList;
    for I := 0 to Targets.Count - 1 do
      begin
        Target := Targets[I] as ICPADTTarget;

        Item := lvTargets.Items.Add;
        Item.Caption := Target.getName;
        Item.SubItems.Add(Target.getSubscriber.getName);
        Item.SubItems.Add(Target.getDivision.getName);
        if Target.getWard <> nil then
          Item.SubItems.Add(Target.getWard.getWard)
        else
          Item.SubItems.Add('');
        Item.SubItems.Add(Target.getMessageType);
        Item.SubItems.Add(Target.getEventType);
        Item.Data := Pointer(Target);
      end;
  finally
    lvTargets.Items.EndUpdate;
  end;
  Visible := true;
end;

procedure TfraEditADTConfig.lvTargetsSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  btnDeleteSelectedTgt.Enabled := Item <> nil;
end;

procedure TfraEditADTConfig.Save;
var
  ADTTargetList: ICPADTTargetList;
begin
  if FProtocolLocation.getADTTargetList.QueryInterface(ICPADTTargetList, ADTTargetList) = 0 then
    ADTTargetList.Save;
end;

procedure TfraEditADTConfig.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditADTConfig.VA508ComponentAccessibility1CaptionQuery(
  Sender: TObject; var Text: string);
begin
  Text := lblTargets.Caption;
end;

procedure TfraEditADTConfig.VA508ComponentAccessibility1ItemQuery(
  Sender: TObject; var Item: TObject);
begin
  Item := lvTargets.Selected;
end;

procedure TfraEditADTConfig.VA508ComponentAccessibility1ValueQuery(
  Sender: TObject; var Text: string);
var
  I: Integer;
  Result: String;
begin
  Result := lvTargets.Columns[0].Caption + ' ' + lvTargets.Selected.Caption;
  for I := 1 to lvTargets.Columns.Count - 1 do
  begin
    Text := Text + lvTargets.Columns[I].Caption + ' ' +
            lvTargets.Selected.SubItems[I - 1];
  end;
  Text := Result + Text;
end;

procedure TfraEditADTConfig.Validate(var aErrorLog: TStringList);
begin

end;

end.

