unit mEditBackgroundTask;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  XMLIntf,
  XMLDoc,
  oCliO,
  oCPTypes,
  oInterfaces,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  Buttons, VA508AccessibilityManager;

type
  TfraEditBackgroundTask = class(TForm, ICPEditor)
    Label1: TLabel;
    Label3: TLabel;
    Label6: TLabel;
    Label8: TLabel;
    dtpTime: TDateTimePicker;
    dtpDate: TDateTimePicker;
    pnlSchedule: TPanel;
    btnChange: TBitBtn;
    rgRecurrence: TRadioGroup;
    Label2: TLabel;
    Label4: TLabel;
    lblSettings: TLabel;
    Shape1: TShape;
    Shape2: TShape;
    btnNow: TBitBtn;
    Timer1: TTimer;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    Panel1: TPanel;
    lblHeader: TLabel;
    edtID: TEdit;
    edtName: TEdit;
    edtStatus: TEdit;
    edtScheduled: TEdit;
    procedure btnChangeClick(Sender: TObject);
    procedure ChangeMade(Sender: TObject);
    procedure btnNowClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    fBackgroundTask: ICPBackgroundTask;
    fScheduledAt: TDateTime;
    fNewSchedule: Boolean;
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Save; stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditBackgroundTaskFrame(aParent: TWinControl): TfraEditBackgroundTask;

implementation

uses
  DateUtils,
  fMain,
  u508Dialogs;

{$R *.dfm}

function NewEditBackgroundTaskFrame(aParent: TWinControl): TfraEditBackgroundTask;
begin
  Result := TfraEditBackgroundTask.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblHeader.Font.Style := [fsBold];
      lblSettings.Font.Style := [fsBold];
    end;
end;

{ TfraEditBackgroundTask }

procedure TfraEditBackgroundTask.btnChangeClick(Sender: TObject);
begin
  pnlSchedule.Visible := (not pnlSchedule.Visible);
end;

procedure TfraEditBackgroundTask.btnNowClick(Sender: TObject);
begin
  dtpDate.DateTime := Now;
  dtpTime.DateTime := Now;
  ChangeMade(Sender);
end;

procedure TfraEditBackgroundTask.ChangeMade(Sender: TObject);
begin
  fNewSchedule := True;
end;

procedure TfraEditBackgroundTask.Initialize(aItem: IInterface);
//var
//  aXMLDoc: IXMLDocument;
//  aXMLRec: IXMLNode;
begin
  if aItem.QueryInterface(ICPBackgroundTask, fBackgroundTask) = 0 then
    begin
      edtID.Text := fBackgroundTask.getTaskID;
      if StrToIntDef(fBackgroundTask.getSystemID, 0) > 0 then
        edtID.Text := Format('%s (ZTSK: %s)', [edtID.Text, fBackgroundTask.getSystemID]);
      edtName.Text := fBackgroundTask.getTaskName;
      edtStatus.Text := fBackgroundTask.getTaskStatus;
      rgRecurrence.ItemIndex := fBackgroundTask.getTaskRecurrence;
      if fBackgroundTask.getIsScheduled then
        begin
          fScheduledAt := fBackgroundTask.getScheduledAt;
          edtScheduled.Text := FormatDateTime('mm/dd/yyyy hh:mm', fScheduledAt);
        end
      else
        begin
          edtScheduled.Text := 'Not Scheduled';
          fScheduledAt := Now;
        end;
      dtpDate.DateTime := fScheduledAt;
      dtpTime.DateTime := fScheduledAt;
    end
  else
    ShowMessage('Error');
  Timer1.Enabled := True;
  fNewSchedule := False;
  Visible := true;
end;

procedure TfraEditBackgroundTask.Save;
begin
  if fNewSchedule then
    if (Trunc(dtpDate.DateTime) + Frac(dtpTime.DateTime)) < IncMinute(Now, -1) then
      MessageDlg508('Task''s cannot be scheduled in the past.', mtError, [mbOk], 0)
    else
      begin
        fScheduledAt := Trunc(dtpDate.DateTime) + Frac(dtpTime.DateTime);
        if MessageDlg508(
          'Queue Task at: ' + FormatDateTime('YYYY-MM-DD hh:mm', fScheduledAt) + #13 + #13 +
          'Recurrence interval: ' + CP_BACKGROUND_TASK_RECURRENCE[TCPBackgroundTaskRecurrence(rgRecurrence.ItemIndex)],
          mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          begin
            MyDBConnection.ExecCommand('StartBackgroundTask', [fBackgroundTask.getTaskID, fScheduledAt, rgRecurrence.ItemIndex]);
            fBackgroundTask.Load;
            Initialize(fBackgroundTask);
            pnlSchedule.Visible := False;
          end;
      end;
end;

procedure TfraEditBackgroundTask.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditBackgroundTask.Timer1Timer(Sender: TObject);
begin
  if (Trunc(dtpDate.DateTime) + Frac(dtpTime.DateTime)) < Now then
    begin
      dtpDate.DateTime := Now;
      dtpTime.DateTime := Now;
    end;
end;

procedure TfraEditBackgroundTask.Validate(var aErrorLog: TStringList);
begin
  //
end;

end.

