unit mEditCPParameters;

{$WARN UNIT_PLATFORM OFF}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  XMLDoc,
  XMLIntf,
  oInterfaces,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  Buttons, VA508AccessibilityManager;

type
  TfraEditCPParameters = class(TForm, ICPEditor)
    edtCPHomePage: TEdit;
    chkbxOnline: TCheckBox;
    edtOfflineMessage: TEdit;
    chkbxExternalAttachments: TCheckBox;
    Label1: TLabel;
    lblOfflineMessage: TLabel;
    chkbxBypassCRCCheck: TCheckBox;
    lblHeader: TLabel;
    Shape2: TShape;
    Label3: TLabel;
    udLoadSize: TUpDown;
    chkbxCachedQueries: TCheckBox;
    Label4: TLabel;
    udUnverifedPurge: TUpDown;
    memLoadSize: TMemo;
    memUnverifiedPurge: TMemo;
    Label5: TLabel;
    udInstrumentDataRetention: TUpDown;
    memInstrumentDataRetention: TMemo;
    lblImgDir: TLabel;
    edtImagingNetworkDirectory: TEdit;
    btnSelImagingNetworkDir: TButton;
    btnNotificationLists: TButton;
    Label2: TLabel;
    edtCachePath: TEdit;
    btnSelTermCachePath: TButton;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure chkbxOnlineClick(Sender: TObject);
    procedure btnCheckForUpdateClick(Sender: TObject);
    procedure btnSelImagingNetworkDirClick(Sender: TObject);
    procedure btnNotificationListsClick(Sender: TObject);
    procedure chkbxCachedQueriesClick(Sender: TObject);
    procedure edtCPHomePageChange(Sender: TObject);
  private
    fCPParameters: ICPParameters;
    //procedure getLatestVersion(var aFramework: string; var aVersion: integer; var aUpdateFile: string; var aDescription: string);
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Save; stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditCPParametersFrame(aParent: TWinControl): TfraEditCPParameters;

implementation

uses
  FileCtrl,
  fMain,
  uCP_GUI,
  fUpdateTerminology,
  fEditNotificationLists,
  u508Dialogs;


{$R *.dfm}

function NewEditCPParametersFrame(aParent: TWinControl): TfraEditCPParameters;
begin
  Result := TfraEditCPParameters.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblHeader.Font.Style := [fsBold];
      //gbxTerminology.Visible := MyUser.HasRole('MD ADMINISTRATOR');
    end;
end;

{ TfraEditCPParameters }

procedure TfraEditCPParameters.btnCheckForUpdateClick(Sender: TObject);
begin
  UpdateTerminology(MyTerminology, fCPParameters);
end;

procedure TfraEditCPParameters.btnNotificationListsClick(Sender: TObject);
begin
  EditNotificationLists(MyCPService, MyCPParameters);
end;

procedure TfraEditCPParameters.btnSelImagingNetworkDirClick(Sender: TObject);
var
  aDirectory: string;
begin
  if SelectDirectory(lblImgDir.Caption, '', aDirectory) then //, [sdNewFolder, sdShowShares, sdNewUI, sdValidateDir], Self) then
    begin
      aDirectory := ExpandUNCFileName(aDirectory);
      if Sender = btnSelImagingNetworkDir then
        edtImagingNetworkDirectory.Text := aDirectory
      else if Sender = btnSelTermCachePath then
        edtCachePath.Text := aDirectory;
    end;
end;

procedure TfraEditCPParameters.chkbxCachedQueriesClick(Sender: TObject);
begin
  edtCachePath.Enabled := chkbxCachedQueries.Checked;
end;

procedure TfraEditCPParameters.chkbxOnlineClick(Sender: TObject);
begin
  edtOfflineMessage.Enabled := (not chkbxOnline.Checked);
  lblOfflineMessage.Enabled := (not chkbxOnline.Checked);
end;

procedure TfraEditCPParameters.edtCPHomePageChange(Sender: TObject);
begin

end;

(*
procedure TfraEditCPParameters.getLatestVersion(var aFramework: string; var aVersion: integer; var aUpdateFile: string; var aDescription: string);
var
  aStream: TMemoryStream;
begin
  aStream := TMemoryStream.Create;
  with NewXMLDocument do
    try
      GetUpdateFile(fCPParameters.getUpdateMaster, aStream);
      LoadFromStream(aStream);
      ShowMessage(XML.Text);
      aFramework := DocumentElement.ChildValues['FRAMEWORK'];
      aVersion := DocumentElement.ChildValues['VERSION'];
      aUpdateFile := DocumentElement.ChildValues['SOURCE'];
      aDescription := DocumentElement.ChildValues['DESCRIPTION'];
    except
      on E: Exception do
        raise Exception.CreateFmt('Error Accessing CP Terminology Server' + #13 + 'Error: %s', [e.Message]);
    end;
  FreeAndNil(aStream);
end;
*)

procedure TfraEditCPParameters.Initialize(aItem: IInterface);
begin
  if aItem.QueryInterface(ICPParameters, fCPParameters) = 0 then
    begin
      edtCPHomePage.Text := fCPParameters.getCPHomePage;
      chkbxOnline.Checked := (not fCPParameters.getOffLine);
      edtOfflineMessage.Text := fCPParameters.getOfflineMessage;
      chkbxBypassCRCCheck.Checked := fCPParameters.getByPassCRC;
      chkbxExternalAttachments.Checked := fCPParameters.getAllowExternalAttachments;
      chkbxCachedQueries.Checked := fCPParameters.getTerminologyCached;
      edtCachePath.Text := fCPParameters.getTerminologyPath;
      edtCachePath.Enabled := fCPParameters.getTerminologyCached;
      udLoadSize.Position := fCPParameters.getBatchLoadSize;
      udUnverifedPurge.Position := fCPParameters.getUnverifiedRetention;
      udInstrumentDataRetention.Position := fCPParameters.getInstrumentDataRetention;
      edtImagingNetworkDirectory.Text := fCPParameters.getImagingXferDirectory;
    end;
  Visible := True;
end;

procedure TfraEditCPParameters.Save;
var
  aCacheSetting: string;
begin
  if not chkbxCachedQueries.Checked then {Just file it :)}
    fCPParameters.setTerminologyCacheConnectionString('0;' + edtCachePath.Text)
  else
    begin
      aCacheSetting := '1;' + edtCachePath.Text;
      {Compare this cache setting to the one on file - non case sensitive}
      if AnsiCompareText(aCacheSetting, fCPParameters.getTerminologyCacheConnectionString) <> 0 then
        if MessageDlg508('Caching options have changed. Rebuild Cache Now?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          try
            if MyTerminology.CacheAllQueries(edtCachePath.text) then
              fCPParameters.setTerminologyCacheConnectionString(aCacheSetting)
            else
              MessageDlg508('Error caching queries', mtError, [mbOk], 0);
          except
            fCPParameters.setTerminologyCacheConnectionString('0;' + fCPParameters.getTerminologyPath);
            MessageDlg508('Error caching queries.', mtError, [mbOk], 0);
          end
        else
          MessageDlg508('Original Cache settings have been restored.', mtConfirmation, [mbOk], 0);

      chkbxCachedQueries.Checked := fCPParameters.getTerminologyCached;
      edtCachePath.Text := fCPParameters.getTerminologyPath;
    end;

  fCPParameters.setCPHomePage(edtCPHomePage.Text);
  fCPParameters.setOffLine(not chkbxOnline.Checked);
  fCPParameters.setOfflineMessage(edtOfflineMessage.Text);
  fCPParameters.setAllowExternalAttachments(chkbxExternalAttachments.Checked);
  fCPParameters.setByPassCRC(chkbxBypassCRCCheck.Checked);
  fCPParameters.setBatchLoadSize(udLoadSize.Position);
  fCPParameters.setUnverifedRetention(udUnverifedPurge.Position);
  fCPParameters.setInstrumentDataRetention(udInstrumentDataRetention.Position);
  fCPParameters.setImagingXferDirectory(edtImagingNetworkDirectory.Text);
  fCPParameters.Save;
end;

procedure TfraEditCPParameters.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

(*
procedure TfraEditCPParameters.UpdateTerminology(aSourceFile: string);
var
  aURL: string;
  aStream: TMemoryStream;
  aXMLDocument: IXMLDocument;
  aDBImport: IDBImport;
begin
  if MyTerminology.DBConnection.QueryInterface(IDBImport, aDBImport) <> 0 then
    MessageDlg508('Your terminology server does not support importing', mtError, [mbOk], 0)
  else
    if MessageDlg508(
      'Press OK to download and install the new terminology' + #13 +
      'file ''' + Format(fCPParameters.getUpdateSource, [aSourceFile]) + '''',
      mtConfirmation, [mbOK, mbCancel], 0) = mrOK then
      begin
        aStream := TMemoryStream.Create;
        try
          aURL := Format(fCPParameters.getUpdateSource, [aSourceFile]);
          //ShowMessage('Attempting to update with: ' + #13 + aURL);
          GetUpdateFile(aURL, aStream);
          aXMLDocument := NewXMLDocument;
          aXMLDocument.LoadFromStream(aStream);
          aDBImport.ExecuteImport(aXMLDocument);
          fCPParameters.setTerminologyVersion(399);
          fCPParameters.setTerminologyDescription('woo hoo');
          fCPParameters.Save;
          MessageDlg508('New terminology has been successfully installed!', mtInformation, [mbOk], 0);
        except
          on E: Exception do
            MessageDlg508('Error with update.' + #13 + 'Error: ' + E.Message, mtError, [mbok], 0);
        end;
        FreeAndNil(aStream);
      end;

    (*
    with TOpenDialog.Create(Application) do
      try
        if Execute then
          try
            aXMLDocument := NewXMLDocument;
            aXMLDocument.LoadFromFile(FileName);
            aDBImport.ExecuteImport(aXMLDocument);
            fCPParameters.setTerminologyVersion(399);
            fCPParameters.setTerminologyDescription('woo hoo');
            fCPParameters.Save;
            ShowMessage('Boom!');
          except
            on E: Exception do
              MessageDlg508('Error with update.' + #13 + 'Error: ' + E.Message, mtError, [mbok], 0);
          end;
      finally
        Free;
      end;

end;
*)

procedure TfraEditCPParameters.Validate(var aErrorLog: TStringList);
begin
  //
end;

end.

