unit mEditFlowsheetTotal;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  oInterfaces,
  oCPTypes,
  oCliOTypes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  Buttons,
  ComCtrls, VA508AccessibilityManager;

type
  TfraEditFlowsheetTotal = class(TForm, ICPEditor)
    gbDisplayProperties: TGroupBox;
    chkbxTotal: TCheckBox;
    chkbxCount: TCheckBox;
    chkbxAverage: TCheckBox;
    chkbxRatio: TCheckBox;
    chkbxDifference: TCheckBox;
    chkbxDisplayMinimum: TCheckBox;
    chkbxDisplayMaximum: TCheckBox;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    pnlGeneral: TPanel;
    chkbxActive: TCheckBox;
    edtComment: TEdit;
    Label1: TLabel;
    edtDisplayName: TEdit;
    lblDisplayName: TLabel;
    cbxDefaultUnit: TComboBox;
    lblDefaultUnit: TLabel;
    edtDecPrec: TEdit;
    udDecPrec: TUpDown;
    Label2: TLabel;
    lblGeneral: TLabel;
    Shape1: TShape;
    pnlTerminology: TPanel;
    Panel2: TPanel;
    lblAvailableTerms: TLabel;
    lblSelectedTerms: TLabel;
    lbxTermAll: TListBox;
    bbtnAdd: TBitBtn;
    lbxTermSelected: TListBox;
    Shape2: TShape;
    lblTerminology: TLabel;
    bbnRemove: TBitBtn;
    bbtnDown: TBitBtn;
    bbtnUp: TBitBtn;
    Bevel1: TBevel;
    procedure sbtnAddClick(Sender: TObject);
    procedure cbxDefaultUnitKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure bbnRemoveClick(Sender: TObject);
    procedure bbtnDownClick(Sender: TObject);
    procedure bbtnUpClick(Sender: TObject);
    procedure cbxDefaultUnitDropDown(Sender: TObject);
  private
    fFlowsheetTotal: ICPFlowsheetTotal;
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Save; stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditFlowsheetTotalFrame(aParent: TWinControl): TfraEditFlowsheetTotal;

implementation

uses
  u508Dialogs,
  uCP_GUI;

{$R *.dfm}

function NewEditFlowsheetTotalFrame(aParent: TWinControl): TfraEditFlowsheetTotal;
begin
  Result := TfraEditFlowsheetTotal.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblGeneral.Font.Style := [fsBold];
      lblTerminology.Font.Style := [fsBold];
    end;
end;

{ TfraEditFlowsheetTotal }

procedure TfraEditFlowsheetTotal.bbnRemoveClick(Sender: TObject);
begin
  if lbxTermSelected.ItemIndex > -1 then
    lbxTermSelected.DeleteSelected;
end;

procedure TfraEditFlowsheetTotal.bbtnDownClick(Sender: TObject);
begin
  if (lbxTermSelected.ItemIndex > -1) and (lbxTermSelected.ItemIndex < lbxTermSelected.Items.Count - 1) then
    lbxTermSelected.Items.Exchange(lbxTermSelected.ItemIndex, lbxTermSelected.ItemIndex + 1)
  else
    MessageDlg508('This term is already at the bottom of the list.', mtError, [mbOk], 0);
end;

procedure TfraEditFlowsheetTotal.bbtnUpClick(Sender: TObject);
begin
  if lbxTermSelected.ItemIndex > 0 then
    lbxTermSelected.Items.Exchange(lbxTermSelected.ItemIndex, lbxTermSelected.ItemIndex - 1)
  else
    MessageDlg508('This term is already at the top of the list.', mtError, [mbOk], 0);
end;

procedure TfraEditFlowsheetTotal.cbxDefaultUnitDropDown(Sender: TObject);
begin
  SetDropDownWidth(Sender as TComboBox);
end;

procedure TfraEditFlowsheetTotal.cbxDefaultUnitKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_ESCAPE) and (not cbxDefaultUnit.DroppedDown) then
    cbxDefaultUnit.ItemIndex := -1;
end;

procedure TfraEditFlowsheetTotal.Initialize(aItem: IInterface);
var
  i: integer;
begin
  if aItem.QueryInterface(ICPFlowsheetTotal, fFlowsheetTotal) = 0 then
    begin
      edtDisplayName.Text := fFlowsheetTotal.getDisplayName;
      edtComment.Text := fFlowsheetTotal.getComment;
      chkbxTotal.Checked := fFlowsheetTotal.getDisplayTotal;
      chkbxCount.Checked := fFlowsheetTotal.getDisplayCount;
      chkbxAverage.Checked := fFlowsheetTotal.getDisplayAverage;
      chkbxDisplayMinimum.Checked := fFlowsheetTotal.getDisplayMinimum;
      chkbxDisplayMaximum.Checked := fFlowsheetTotal.getDisplayMaximum;
      chkbxRatio.Checked := fFlowsheetTotal.getDisplayRatio;
      chkbxDifference.Checked := fFlowsheetTotal.getDisplayDifference;
      chkbxActive.Checked := fFlowsheetTotal.getActive;
      udDecPrec.Position := fFlowsheetTotal.getDecimalPrec;
      if Assigned(fFlowsheetTotal.getDefaultUnit) then
        cbxDefaultUnit.ItemIndex := fFlowsheetTotal.getDefaultUnit.getIndex(cbxDefaultUnit.Items)
      else
        cbxDefaultUnit.ItemIndex := -1;
      lbxTermSelected.Items.Clear;
      for i := 0 to fFlowsheetTotal.getTermCount - 1 do
        fFlowSheetTotal.getTerm(i).AddToList(lbxTermSelected.Items, tdTerm);
    end;
  Visible := true;
end;

procedure TfraEditFlowsheetTotal.Save;
var
  aTerm: ITerm;
  i: integer;
begin
  try
    with fFlowsheetTotal do
      begin
        setDisplayName(edtDisplayName.Text);
        setComment(edtComment.Text);
        setActive(chkbxActive.Checked);
        if cbxDefaultUnit.ItemIndex > -1 then
          if cbxDefaultUnit.Items.Objects[cbxDefaultUnit.ItemIndex].getInterface(ITerm, aTerm) then
            setDefaultUnit(aTerm)
          else
            setDefaultUnit(nil)
        else
          setDefaultUnit(nil);
        setDecimalPrec(udDecPrec.Position);
        setDisplayTotal(chkbxTotal.Checked);
        setDisplayCount(chkbxCount.Checked);
        setDisplayAverage(chkbxAverage.Checked);
        setDisplayMinimum(chkbxDisplayMinimum.Checked);
        setDisplayMaximum(chkbxDisplayMaximum.Checked);
        setDisplayDifference(chkbxDifference.Checked);
        setDisplayRatio(chkbxRatio.Checked);
        ClearAllTerms;
        for i := 0 to lbxTermSelected.Items.Count - 1 do
          if lbxTermSelected.Items.Objects[i].GetInterface(ITerm, aTerm) then
            AddTerm(aTerm);
        Save;
      end;
  except {Raise this back to the main form}
    raise;
  end;
end;

procedure TfraEditFlowsheetTotal.sbtnAddClick(Sender: TObject);
var
  aTerm: ITerm;
begin
  if lbxTermAll.ItemIndex > -1 then
    if lbxTermSelected.Items.IndexOfObject(lbxTermAll.Items.Objects[lbxTermAll.ItemIndex]) < 0 then
      begin
        lbxTermAll.Items.Objects[lbxTermAll.ItemIndex].GetInterface(ITerm, aTerm);
        aTerm.AddToList(lbxTermSelected.Items, tdTerm);
      end
    else
      MessageDlg508('This term is already in your selected list.' + #13 +
        'Terms can only be placed in a totals definition once.',
        mtError, [mbOk], 0);
end;

procedure TfraEditFlowsheetTotal.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditFlowsheetTotal.Validate(var aErrorLog: TStringList);
begin
  //
end;

end.

