unit mEditFlowsheetView;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 8 $  $Modtime: 8/04/09 4:44p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Edits a single Observation View Definition
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditFlowsheetView.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Buttons,
  StdCtrls,
  ExtCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  ActnList,
  XMLDoc,
  XMLIntf,
  ImgList,
  ComCtrls,
  ToolWin,
  Menus,
  VA508AccessibilityManager;

type
  TfraEditFlowsheetView = class(TForm, ICPEditor)
    lblComment: TLabel;
    lblDisplayName: TLabel;
    edtComment: TEdit;
    edtDisplayName: TEdit;
    lblTerminology: TLabel;
    lblTimeInterval: TLabel;
    cbxInterval: TComboBox;
    lblXAxis: TLabel;
    cbxXAxis: TComboBox;
    lblGeneral: TLabel;
    pnlGeneral: TPanel;
    Shape1: TShape;
    Shape2: TShape;
    ActionList1: TActionList;
    acAddTerm: TAction;
    acRemoveTerm: TAction;
    acMoveUp: TAction;
    acMoveDown: TAction;
    ImageList1: TImageList;
    ckbxAllowPivot: TCheckBox;
    pnlControls: TPanel;
    acRefreshListView: TAction;
    acEditTerm: TAction;
    chkbxActive: TCheckBox;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    bbtnMoveUp: TBitBtn;
    bbtnAdd: TBitBtn;
    bbtnEdit: TBitBtn;
    bbtnRemove: TBitBtn;
    bbtnMoveDown: TBitBtn;
    lvTerminology: TListView;
    procedure acRefreshListViewExecute(Sender: TObject);
    procedure acMoveUpExecute(Sender: TObject);
    procedure acEditTermExecute(Sender: TObject);
    procedure lvTermsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure acMoveDownExecute(Sender: TObject);
    procedure acRemoveTermExecute(Sender: TObject);
    procedure acAddTermExecute(Sender: TObject);
    procedure ActionList1Update(Action: TBasicAction; var Handled: Boolean);
  private
    fView: ICPFlowsheetView;
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
  end;

function NewEditFlowsheetViewFrame(aParent: TWinControl): TfraEditFlowsheetView;

implementation

uses
  iCliOGrid,
  fMain,
  uCPXML,
  oCP_Exceptions,
  fEditViewTerm,
  u508Dialogs;

{$R *.dfm}

function NewEditFlowsheetViewFrame(aParent: TWinControl): TfraEditFlowsheetView;
var
  aGridInterval: TGridInterval;
  aGridStyle: TGridStyle;
begin
  Result := TfraEditFlowsheetView.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblGeneral.Font.Style := [fsBold];
      lblTerminology.Font.Style := [fsBold];

      for aGridInterval := Low(TGridInterval) to High(TGridInterval) do
        cbxInterval.Items.Add(GRID_INTERVAL_NAME[aGridInterval]);

      for aGridStyle := Low(TGridStyle) to High(TGridStyle) do
        cbxXAxis.Items.Add(GRID_STYLE_NAME[aGridStyle]);
    end;
end;

{ TfraEditObservationView }

procedure TfraEditFlowsheetView.acAddTermExecute(Sender: TObject);
var
  aViewTerm: ICPFlowsheetViewTerm;
begin
  aViewTerm := MyCPService.CreateNewCPFlowsheetViewTerm(fView);
  with NewEditViewTermForm(aViewTerm) do
    try
      if ShowModal = mrOk then
        begin
          aViewTerm.AddToListView(lvTerminology);
          fView.InsertViewTerm(aViewTerm);
          acRefreshListView.Execute;
        end;
    finally
      Free;
    end;
end;

procedure TfraEditFlowsheetView.acEditTermExecute(Sender: TObject);
var
  aViewTerm: ICPFlowsheetViewTerm;
begin
  if lvTerminology.ItemIndex > -1 then
    if TObject(lvTerminology.Items[lvTerminology.ItemIndex].Data).GetInterface(ICPFlowsheetViewTerm, aViewTerm) then
      with NewEditViewTermForm(aViewTerm) do
        begin
          ShowModal;
        end;
  acRefreshListView.Execute;
end;

procedure TfraEditFlowsheetView.acMoveDownExecute(Sender: TObject);
var
  Item: TListItem;
  SelectedIndex: Integer;
begin
  {
  Unlike the TStrings class, the TListItems class doesn't have an Exchange()
  method.  So we have to do this the hard way.
  }
  SelectedIndex := lvTerminology.ItemIndex;
  if SelectedIndex < (lvTerminology.Items.Count - 1) then
    begin
      Item := lvTerminology.Items.Insert(SelectedIndex + 2);
      Item.Assign(lvTerminology.Selected);
      lvTerminology.Items.Delete(SelectedIndex);
      lvTerminology.Selected := Item;
      lvTerminology.SetFocus;
    end;
end;

procedure TfraEditFlowsheetView.acMoveUpExecute(Sender: TObject);
var
  Item: TListItem;
  SelectedIndex: Integer;
begin
  {
  Unlike the TStrings class, the TListItems class doesn't have an Exchange()
  method.  So we have to do this the hard way.
  }
  SelectedIndex := lvTerminology.Selected.Index;
  if SelectedIndex > 0 then
    begin
      Item := lvTerminology.Items.Insert(SelectedIndex - 1);
      Item.Assign(lvTerminology.Selected);
      lvTerminology.Items.Delete(lvTerminology.Selected.Index);
      lvTerminology.Selected := Item;
      lvTerminology.SetFocus;
    end;
end;

procedure TfraEditFlowsheetView.acRefreshListViewExecute(Sender: TObject);
var
  aTerm: ITerm;
  aViewTerm: ICPFlowsheetViewTerm;
  Enum: TListItemsEnumerator;
begin
  Enum := lvTerminology.Items.GetEnumerator;
  try
    while Enum.MoveNext do
      begin
        if Assigned(Enum.GetCurrent.Data) then
          if TObject(Enum.GetCurrent.Data).GetInterface(ICPFlowsheetViewTerm, aViewTerm) then
            begin
              aTerm := MyTerminology.GetTerm(aViewTerm.getAttributeID(ttObservation));
              Enum.GetCurrent.Caption := aViewTerm.getDisplayName;
              Enum.GetCurrent.SubItems.Add(aTerm.getTerm);
            end;
      end;
  finally
    FreeAndNil(Enum);
  end;
end;

procedure TfraEditFlowsheetView.acRemoveTermExecute(Sender: TObject);
var
  aViewTerm: ICPFlowsheetViewTerm;
begin
  if lvTerminology.ItemIndex > -1 then
    if TObject(lvTerminology.Items[lvTerminology.ItemIndex].Data).GetInterface(ICPFlowsheetViewTerm, aViewTerm) then
      if MessageDlg508(Format('Remove the term ''%s'' from this view defintion?',
        [aViewTerm.getDisplayName]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        begin
          fView.DeleteViewTerm(aViewTerm);
          lvTerminology.DeleteSelected;
        end;
end;

procedure TfraEditFlowsheetView.Initialize(aItem: IInterface);
var
  aInterval: TGridInterval;
  i: integer;
begin
  if aItem.QueryInterface(ICPFlowsheetView, fView) = 0 then
    begin
      fView.ForcedLoad;
      edtDisplayName.Text := fView.getDisplayName;
      edtComment.Text := fView.getComment;

      for aInterval := Low(TGridInterval) to High(TGridInterval) do
        if GRID_INTERVAL_VALUE[aInterval] = fView.getDisplayInterval then
          cbxInterval.ItemIndex := Ord(aInterval);
      cbxXAxis.ItemIndex := Ord(fView.getViewStyle);
      ckbxAllowPivot.Checked := fView.getAllowPivot;
      chkbxActive.Checked := fView.getActive;
      lvTerminology.Items.Clear;
      for i := 0 to fView.getViewTermCount - 1 do
        fView.getViewTermByIndex(i).AddToListView(lvTerminology);

      acRefreshListView.Execute;
      Enabled := (not fView.getIsNationalTitle);
    end;
  Visible := true;
end;

procedure TfraEditFlowsheetView.lvTermsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  Compare := StrToIntDef(Item1.Caption, 0) - StrToIntDef(Item2.Caption, 0);
end;

procedure TfraEditFlowsheetView.Save;
var
  aViewTerm: ICPFlowsheetViewTerm;
  Enum: TListItemsEnumerator;
begin
  if fView.getIsNationalTitle then
    MessageDlg508('Sorry, this is a national title and cannot be saved locally.', mtInformation, [mbOk], 0)
  else
    try
      fView.setAllowPivot(ckbxAllowPivot.Checked);
      fView.setDisplayInterval(GRID_INTERVAL_VALUE[TGridInterval(cbxInterval.ItemIndex)]);
      fView.setDisplayName(edtDisplayName.Text);
      fView.setComment(edtComment.Text);
      fView.setViewStyle(TObservationViewStyle(cbxXAxis.ItemIndex));
      fView.setActive(chkbxActive.Checked);
      {Reset the display order}
      Enum := lvTerminology.Items.GetEnumerator;
      try
        while Enum.MoveNext do
          begin
            if TObject(Enum.GetCurrent.Data).GetInterface(ICPFlowsheetViewTerm, aViewTerm) then
              aViewTerm.setDisplayOrder(Enum.GetCurrent.Index + 1);
          end;
      finally
        FreeAndNil(Enum);
      end;
      fView.Save;
    except
      on E: ECPSaveFailure do
        MessageDlg508('Error saving view ''' + fView.getID + '''.' + #13 +
          'Error: ' + e.Message, mtError, [mbOk], 0);
      on E: Exception do
        MessageDlg508('Unknown error saving view.' + #13 +
          'Error: ' + e.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfraEditFlowsheetView.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditFlowsheetView.Validate(var aErrorLog: TStringList);
begin
  if edtDisplayName.Text = '' then
    aErrorLog.Add('Display Name required.');
  if cbxInterval.ItemIndex = -1 then
    aErrorLog.Add('Time Interval required.');
  if cbxXAxis.ItemIndex = -1 then
    aErrorLog.Add('X-Axis required.');
end;

procedure TfraEditFlowsheetView.ActionList1Update(Action: TBasicAction; var Handled: Boolean);
begin
  acMoveUp.Enabled := lvTerminology.ItemIndex > 0;
  acMoveDown.Enabled := (lvTerminology.ItemIndex < lvTerminology.Items.Count - 1) and (lvTerminology.ItemIndex > -1); //and (lvTerminology.Items.Count > 1);
  acRemoveTerm.Enabled := (lvTerminology.ItemIndex > -1);
  acEditTerm.Enabled := (lvTerminology.ItemIndex > -1);
  Handled := True;
end;

end.

