unit mEditGatewayConfig;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 13 $  $Modtime: 5/20/09 1:45p $
*       Developer:    brian.juergensmeyer@va.gov
*       Site:         Hines OIFO
*
*       Description:  UI for CP Gateway Administrator.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditGatewayConfig.pas $
================================================================================
}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  StdCtrls,
  oInterfaces,
  ComCtrls,
  Spin,
  VA508AccessibilityManager;

type
  TfraEditGatewayConfig = class(TForm, ICPEditor)
    pnlVistAServer: TPanel;
    lblAccessCode: TLabel;
    lblVerifyCode: TLabel;
    pnlGatewayServer: TPanel;
    Label1: TLabel;
    edtGatewayIPAddress: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    edtLogDirectory: TEdit;
    edtNotifyPort: TEdit;
    udNotifyPort: TUpDown;
    edtAccessCode: TEdit;
    edtVerifyCode: TEdit;
    edtDaysToKeepData: TEdit;
    Label8: TLabel;
    udDaysToKeepData: TUpDown;
    edtSharedDirectory: TEdit;
    Label9: TLabel;
    cbDataRetrievalType: TComboBox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    edtBrokerPort: TEdit;
    lblBrokerPort: TLabel;
    udBrokerPort: TUpDown;
    edtVistAIPAddress: TEdit;
    lblIPAddress: TLabel;
    udPollingInterval: TUpDown;
    edtPollingInterval: TEdit;
    cbxLogLevel: TComboBox;
    Shape2: TShape;
    lblVistAServerAddress: TLabel;
    lblAccessVerifyCodes: TLabel;
    Shape1: TShape;
    Label2: TLabel;
    lblServiceAccountDescription: TLabel;
    edtServiceAccountDescription: TEdit;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure cbDataRetrievalTypeChange(Sender: TObject);
  private
    { Private declarations }
    fConfig: ICPGatewayConfig;
  public
    { Public declarations }
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Save; stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  end;

function NewGatewayAdminFrame(aParent: TWinControl): TfraEditGatewayConfig;

implementation

uses
  oGateway,
  oCliOTypes,
  oCPTypes;

{$R *.dfm}

function NewGatewayAdminFrame(aParent: TWinControl): TfraEditGatewayConfig;
var
  aLogLevel: TLogLevel;
begin
  Result := TfraEditGatewayConfig.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblAccessVerifyCodes.Font.Style := [fsBold];
      lblVistAServerAddress.Font.Style := [fsBold];
      for aLogLevel := llCritical to llDetail do
        cbxLogLevel.Items.Add(GATEWAY_LOG_LEVEL[aLogLevel]);
    end;
end;

{ TfraGatewayAdmin }

procedure TfraEditGatewayConfig.cbDataRetrievalTypeChange(Sender: TObject);
begin
  edtPollingInterval.Enabled := cbDataRetrievalType.ItemIndex = 1;
end;

procedure TfraEditGatewayConfig.Initialize(aItem: IInterface);
begin
  if aItem.QueryInterface(ICPGatewayConfig, fConfig) = 0 then
    begin
      {Only do these if the gateway is installed on this workstation}
      if fConfig.getIsGatewayInstalled then
        begin
          edtAccessCode.Text := fConfig.getAccessCode;
          edtVerifyCode.Text := fConfig.getVerifyCode;
          edtVistAIPAddress.Text := fConfig.getBrokerServerIP;
          udBrokerPort.Position := fConfig.getBrokerServerPort;
          edtServiceAccountDescription.Text := fConfig.getGatewayServerHint;
        end
      else
        begin
          edtAccessCode.Visible := False;
          edtVerifyCode.Visible := False;
          edtVistAIPAddress.Visible := False;
          edtBrokerPort.Visible := False;
          edtServiceAccountDescription.Visible := False;
          udBrokerPort.Visible := False;
          lblAccessCode.Visible := False;
          lblVerifyCode.Visible := False;
          lblBrokerPort.Visible := False;
          lblIPAddress.Visible := False;
          lblServiceAccountDescription.Visible := False;
          pnlVistAServer.Caption := 'CPGateway Service not installed on this computer.';
          pnlVistAServer.Enabled := False;
        end;

      edtGatewayIPAddress.Text := fConfig.getGatewayServerIP;
      udNotifyPort.Position := fConfig.getGatewayServerPort;

      //udLogLevel.Position := fConfig.getLogLevel;
      cbxLogLevel.ItemIndex := fConfig.getLogLevel;
      udDaysToKeepData.Position := fConfig.getDaysToKeepData;
      edtLogDirectory.Text := fConfig.getLogFileDirectory;
      edtSharedDirectory.Text := fConfig.getSharedDirectory;
      cbDataRetrievalType.ItemIndex := ord(fConfig.getDataRetrievalType);
      edtPollingInterval.Enabled := fConfig.getDataRetrievalType = drtPoll;
      udPollingInterval.Position := fConfig.getPollingInterval div 10;
    end;
  Visible := true;
end;

procedure TfraEditGatewayConfig.Save;
begin
  fConfig.setAccessCode(edtAccessCode.Text);
  fConfig.setVerifyCode(edtVerifyCode.Text);
  fConfig.setBrokerServerIP(edtVistAIPAddress.Text);
  fConfig.setBrokerServerPort(udBrokerPort.Position);
  fConfig.setGatewayServerIP(edtGatewayIPAddress.Text);
  fConfig.setGatewayServerPort(udNotifyPort.Position);
  fConfig.setGatewayServerHint(edtServiceAccountDescription.Text);
  //fConfig.setLogLevel(udLogLevel.Position);
  fCOnfig.setLogLevel(cbxLogLevel.ItemIndex);
  fConfig.setLogFileDirectory(edtLogDirectory.Text);
  fConfig.setSharedDirectory(edtSharedDirectory.Text);
  fCOnfig.setDaysToKeepData(udDaysToKeepData.Position);
  fConfig.setDataRetrievalType(TDataRetrievalType(cbDataRetrievalType.ItemIndex));
  fConfig.setPollingInterval(udPollingInterval.Position * 10);
  fConfig.Save;
end;

procedure TfraEditGatewayConfig.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditGatewayConfig.Validate(var aErrorLog: TStringList);
begin

end;

end.

