unit mEditInstrument;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 16 $  $Modtime: 5/27/09 2:01p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Edits a single CP Instrument
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditInstrument.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  oInterfaces,
  oCliOTypes,
  StdCtrls,
  XMLDoc,
  XMLIntf,
  mLookup,
  ExtCtrls,
  VA508AccessibilityManager, Menus;

type
  TfraEditInstrument = class(TForm, ICPEditor)
    edtComment: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    edtPrintableName: TEdit;
    Label3: TLabel;
    edtDefaultFileExtension: TEdit;
    Label4: TLabel;
    edtSerialNumber: TEdit;
    Label5: TLabel;
    lblGeneral: TLabel;
    Shape1: TShape;
    cbxDeleteWhenSubmitted: TCheckBox;
    edtMRoutine: TEdit;
    cbxPackageCode: TComboBox;
    Label6: TLabel;
    Label7: TLabel;
    Shape3: TShape;
    lblBiDirectional: TLabel;
    cbxBiDirectional: TCheckBox;
    edtIPAddress: TEdit;
    Label9: TLabel;
    edtPort: TEdit;
    Label10: TLabel;
    edtHL7InstID: TEdit;
    Label11: TLabel;
    edtHL7SvcID: TEdit;
    Label12: TLabel;
    Label13: TLabel;
    frafra: TPanel;
    pnlAttachments: TPanel;
    pnlBiDirectional: TPanel;
    Shape2: TShape;
    lblAttachment: TLabel;
    cbxActive: TCheckBox;
    GroupBox1: TGroupBox;
    cbxUNC: TCheckBox;
    cbxText: TCheckBox;
    cbxURL: TCheckBox;
    cbxDLL: TCheckBox;
    cbxUUEncode: TCheckBox;
    cbxXML: TCheckBox;
    cbxXMS: TCheckBox;
    fraLookupMailGroup: TfraLookup;
    fraLookupHL7Link: TfraLookup;
    MainMenu1: TMainMenu;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
  private
    fInstrument: ICPInstrument;
    procedure LookupMailGroup(aPartial: string; aResult: TStrings);
    procedure LookupHL7Link(aPartial: string; aResult: TStrings);
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditInstrumentFrame(aParent: TWinControl): TfraEditInstrument;

implementation

uses
  fMain;

{$R *.dfm}

function NewEditInstrumentFrame(aParent: TWinControl): TfraEditInstrument;
begin
  Result := TfraEditInstrument.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblGeneral.Font.Style := [fsBold];
      lblAttachment.Font.Style := [fsBold];
      lblBiDirectional.Font.Style := [fsBold];
      Visible := true;
    end;
end;

{ TfraEditInstrument }

procedure TfraEditInstrument.Initialize(aItem: IInterface);
//var
//  aNode: IXMLNode;
//  aXMLDoc: IXMLDocument;
begin
  if aItem.QueryInterface(ICPInstrument, fInstrument) = 0 then
    begin
      edtComment.Text := fInstrument.getComment;
      edtPrintableName.Text := fInstrument.getPrintableName;
      cbxActive.Checked := fInstrument.getActive;
      edtSerialNumber.Text := fInstrument.getSerialNumber;
      {Mailgroup}
      fraLookupMailGroup.Initialize(
        fInstrument.getNotificationMailgroupID,
        fInstrument.getNotificationMailgroup,
        LookupMailGroup, nil, Label2);
      cbxDeleteWhenSubmitted.Checked := fInstrument.getDeleteWhenSubmitted;
      edtDefaultFileExtension.Text := fInstrument.getDefaultFileExtension;
      edtMRoutine.Text := fInstrument.getProcessingRoutine;
      cbxPackageCode.ItemIndex := Pos(fInstrument.getProcessingCode, 'MCO') - 1;
      cbxUNC.Checked := fInstrument.getProcessUNC;
      cbxText.Checked := fInstrument.getProcessText;
      cbxURL.Checked := fInstrument.getProcessURL;
      cbxDLL.Checked := fInstrument.getProcessDLL;
      cbxUUEncode.Checked := fInstrument.getProcessUUEncode;
      cbxXML.Checked := fInstrument.getProcessXML;
      cbxXMS.Checked := fInstrument.getProcessXMS;
      cbxBiDirectional.Checked := fInstrument.getBiDirectional;
      edtIPAddress.Text := fInstrument.getIPAddress;
      edtPort.Text := fInstrument.getPort;
      edtHL7InstID.Text := fInstrument.getHL7InstrumentID;
      edtHL7SvcID.Text := fInstrument.getHL7UniversalServiceID;
      {HL7 Link}
      fraLookupHL7Link.Initialize(
        fInstrument.getHL7LogicalLinkID,
        fInstrument.getHL7LogicalLink, LookupHL7Link, nil, Label13);
    end;
  Visible := true;
end;

procedure TfraEditInstrument.LookupMailGroup(aPartial: string; aResult: TStrings);
{Change these types of calls to utilize ExecQuery}
begin
  DoLookup('GetMailGroupList', aPartial, 60, aResult);
end;

procedure TfraEditInstrument.LookupHL7Link(aPartial: string; aResult: TStrings);
{Change these types of calls to utilize ExecQuery}
begin
  DoLookup('GetHL7LogicalLinkList', aPartial, 60, aResult);
end;

procedure TfraEditInstrument.Save;
begin
  with fInstrument do
    begin
      setComment(edtComment.Text);
      setActive(cbxActive.Checked);
      setPrintableName(edtPrintableName.Text);
      setSerialNumber(edtSerialNumber.Text);
      setNotificationMailgroup(fraLookupMailGroup.ExternalValue);
      setNotificationMailgroupID(fraLookupMailGroup.InternalValue);
      setDeleteWhenSubmitted(cbxDeleteWhenSubmitted.Checked);
      setDefaultFileExtension(edtDefaultFileExtension.Text);
      setProcessingRoutine(edtMRoutine.Text);
      setProcessingCode(Copy('MCO', cbxPackageCode.ItemIndex + 1, 1));
      setProcessUNC(cbxUNC.Checked);
      setProcessText(cbxText.Checked);
      setProcessURL(cbxURL.Checked);
      setProcessDLL(cbxDLL.Checked);
      setProcessUUEncode(cbxUUEncode.Checked);
      setProcessXML(cbxXML.Checked);
      setProcessXMS(cbxXMS.Checked);
      setBiDirectional(cbxBiDirectional.Checked);
      setIPAddress(edtIPAddress.Text);
      setPort(edtPort.Text);
      setHL7InstrumentID(edtHL7InstID.Text);
      setHL7UniversalServiceID(edtHL7SvcID.Text);
      setHL7LogicalLink(fraLookupHL7Link.ExternalValue);
      setHL7LogicalLinkID(fraLookupHL7Link.InternalValue);

      Save;
    end;
  {
  SetLength(aParams, 27);
  aParams[0] := fInstrument.getName;
  aParams[1] := edtDescription.Text;
  aParams[2] := edtPrintableName.Text;
  aParams[3] := BOOLEAN_AS_INTEGER[cbxActive.Checked];
  aParams[4] := edtSerialNumber.Text;
  aParams[5] := fraLookupMailGroup.InternalValue;
  aParams[6] := BOOLEAN_AS_INTEGER[cbxDeleteWhenSubmitted.Checked];
  aParams[7] := edtDefaultFileExtension.Text;
  aParams[8] := edtMRoutine.Text;
  aParams[9] := Copy('MCO', cbxPackageCode.ItemIndex + 1, 1);
  aParams[10] := BOOLEAN_AS_INTEGER[cbxUNC.Checked];
  aParams[11] := BOOLEAN_AS_INTEGER[cbxText.Checked];
  aParams[12] := BOOLEAN_AS_INTEGER[cbxURL.Checked];
  aParams[13] := BOOLEAN_AS_INTEGER[cbxDLL.Checked];
  aParams[14] := BOOLEAN_AS_INTEGER[cbxUUEncode.Checked];
  aParams[15] := BOOLEAN_AS_INTEGER[cbxXML.Checked];
  aParams[16] := BOOLEAN_AS_INTEGER[cbxXMS.Checked];
  aParams[17] := BOOLEAN_AS_INTEGER[cbxBiDirectional.Checked];
  aParams[18] := edtIPAddress.Text;
  aParams[19] := edtPort.Text;
  aParams[20] := edtHL7InstID.Text;
  aParams[21] := edtHL7SvcID.Text;
  aParams[22] := fraLookupHL7Link.InternalValue;

  if not MyDBConnection.ExecCommand('UpdateInstrument', aParams) then
    raise Exception.Create(MyDBConnection.ExecResults.Text)
  else
    fInstrument.setActive(cbxActive.Checked);
  }
end;

procedure TfraEditInstrument.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditInstrument.Validate(var aErrorLog: TStringList);
begin
  //aErrorLog.Add('Ha Ha, you got bipped');
end;

end.

