unit mEditMappingTable;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 17 $  $Modtime: 5/20/09 1:55p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Manages a CliO Mapping Table
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditMappingTable.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ActnList,
  StdCtrls,
  ComCtrls,
  oCliOTypes,
  oInterfaces,
  XMLDoc,
  XMLIntf,
  ToolWin,
  ExtCtrls,
  ImgList,
  VA508AccessibilityManager,
  Buttons;

type
  TfraEditMappingTable = class(TForm, ICPEditor)
    pnlGeneral: TPanel;
    lblTerminology: TLabel;
    lblComment: TLabel;
    lblDisplayName: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    lblGeneral: TLabel;
    Shape1: TShape;
    Shape2: TShape;
    edtComment: TEdit;
    edtSourceID: TEdit;
    lvTerms: TListView;
    ToolBar1: TToolBar;
    Label1: TLabel;
    ToolButton5: TToolButton;
    Panel1: TPanel;
    edtVendorKey: TEdit;
    cbxType: TComboBox;
    cbxTerm: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Button1: TButton;
    ActionList1: TActionList;
    acUpdate: TAction;
    edtSourceApplication: TEdit;
    edtSourceVersion: TEdit;
    acAddItem: TAction;
    acRemoveItem: TAction;
    ckbxActive: TCheckBox;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    acUpdateTermList: TAction;
    ImageList1: TImageList;
    ckbxTrusted: TCheckBox;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    procedure lvTermsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure acUpdateTermListExecute(Sender: TObject);
    procedure acUpdateExecute(Sender: TObject);
    procedure acAddItemExecute(Sender: TObject);
    procedure acRemoveItemExecute(Sender: TObject);
  private
    fTable: ICPConsoleItem;
    fTerms: array[TTermType] of TStringList;
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    function TermList(aTermType: TTermType): TStringList;
    { Public declarations }
  end;

function NewEditMappingTableFrame(aParent: TWinControl): TfraEditMappingTable;

implementation

uses
  uCPXML,
  fMain,
  u508Dialogs;

{$R *.dfm}

function NewEditMappingTableFrame(aParent: TWinControl): TfraEditMappingTable;
var
  tt: TTermType;
begin
  Result := TfraEditMappingTable.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblGeneral.Font.Style := [fsBold];
      lblTerminology.Font.Style := [fsBold];
      edtVendorKey.Text := '';
      for tt := Low(TTermType) to High(TTermType) do
        begin
          cbxType.Items.Add(QUALNAME[tt]);
          fTerms[tt] := TStringList.Create;
        end;
      cbxType.ItemIndex := -1;
      cbxTerm.Items.Clear;
      {Need to add item type to the combobox}
    end;
end;

{ TfraEditMappingTable }

procedure TfraEditMappingTable.Initialize(aItem: IInterface);
var
  aTableDoc, aEntriesDoc: IXMLDocument;
  aNode: IXMLNode;
begin
  if aItem.QueryInterface(ICPConsoleItem, fTable) = 0 then
    begin
      aTableDoc := NewXMLDocument;
      aEntriesDoc := NewXMLDocument;

      MyDBConnection.ExecQuery('GetMappingTableByID', [fTable.getID], aTableDoc);
      aTableDoc.Active := True;
      aNode := aTableDoc.DocumentElement.ChildNodes.FindNode('RECORD');

      edtSourceID.Text := XMLValueAsStr(aNode, 'SOURCE_ID');
      edtSourceApplication.Text := XMLValueAsStr(aNode, 'SOURCE_APPLICATION');
      edtSourceVersion.Text := XMLValueAsStr(aNode, 'SOURCE_VERSION');
      edtComment.Text := XMLValueAsStr(aNode, 'COMMENT');
      ckbxActive.Checked := (XMLValueAsInt(aNode, 'ACTIVE') = 1);
      ckbxTrusted.Checked := (XMLValueAsInt(aNode, 'SOURCE_TRUSTED') = 1);

      MyDBConnection.ExecQuery('GetMappingTableTermsByID', [fTable.getID], aEntriesDoc);
      aEntriesDoc.Active := True;
      aNode := aEntriesDoc.DocumentElement.ChildNodes.FindNode('RECORD');
      while aNode <> nil do
        with lvTerms.Items.Add do
          begin
            Caption := XMLValueAsStr(aNode, 'VENDOR_KEY');
            try
              SubItems.Add(QUALNAME[TTermType(XMLValueAsInt(aNode, 'TERM_TYPE'))]);
            except
              SubItems.Add('Missing term type');
            end;
            SubItems.Add(XMLValueAsStr(aNode, 'TERM_ID'));
            SubItems.Add(XMLValueAsStr(aNode, 'TERM_NAME'));
            //SubItems.Add(XMLValueAsStr(aNode, 'PARSER_KEY'));
            aNode := aNode.NextSibling;
          end;
      lvTerms.AlphaSort;
    end;
  Visible := true;
end;

procedure TfraEditMappingTable.Save;
var
  i: integer;
begin
  if not MyDBConnection.ExecCommand(
    'UpdateMappingTable',
    [fTable.getID, fTable.getName, BOOLEAN_AS_INTEGER[ckbxActive.Checked], edtComment.Text, edtSourceID.Text, edtSourceApplication.Text, edtSourceVersion.Text, BOOLEAN_AS_INTEGER[ckbxTrusted.Checked]]) then
    raise Exception.Create('Unable to update the existing mapping table.');

  if not MyDBConnection.ExecCommand(
    'DeleteMappingTableTerms',
    [fTable.getID]) then
    raise Exception.Create('Unable to clear the existing mapping table entries.');

  for i := 0 to lvTerms.Items.Count - 1 do
    with lvTerms.Items[i] do
      try
        if not MyDBConnection.ExecCommand(
          'AddMappingTableTerm',
          [fTable.getID, Caption, cbxType.Items.IndexOf(SubItems[0]), SubItems[1]]) then
        MessageDlg508('Error Saving Item ' + IntToStr(i+1)+ '. ' + Caption + ' ' + SubItems[0], mtError, [mbOk], 0);
      except
        MessageDlg508('Error Saving Item ' + IntToStr(i+1)+ '. ' + Caption + ' ' + SubItems[0], mtError, [mbOk], 0);
        //raise Exception.Create('Error updating mapping table entries.');
      end;
  fTable.setActive(ckbxActive.Checked);
end;

procedure TfraEditMappingTable.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditMappingTable.Validate(var aErrorLog: TStringList);
var
  i: integer;
begin
  for i := 0 to lvTerms.Items.Count - 1 do
    with lvTerms.Items[i] do
      begin
        if Caption = '' then
          aErrorLog.Add(Format('Term %d is missing the Vendor Key.', [i]))
        else
          begin
            if SubItems[0] = '' then
              aErrorLog.Add(Format('Vendor Key ''%s'' is missing the Item Type.', [Caption]));
            if SubItems[1] = '' then
              aErrorLog.Add(Format('Vendor Key ''%s'' is missing the ID.', [Caption]));
            if SubItems[2] = '' then
              aErrorLog.Add(Format('Vendor Key ''%s'' is missing the Term.', [Caption]));
          end
      end;
end;

procedure TfraEditMappingTable.lvTermsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  i: integer;
  aTerm: ITerm;
begin
  edtVendorKey.Text := '';
  cbxType.ItemIndex := -1;
  cbxTerm.ItemIndex := -1;

  edtVendorKey.Enabled := Selected;
  cbxType.Enabled := Selected;
  cbxTerm.Enabled := Selected;
  acUpdate.Enabled := Selected;

  if Selected then
    with Item do
      begin
        edtVendorKey.Text := Caption;
        //edtParserKey.Text := SubItems[3];
        cbxType.ItemIndex := cbxType.Items.IndexOf(SubItems[0]);
        acUpdateTermList.Execute;
        for i := 0 to cbxTerm.Items.Count - 1 do
          if cbxTerm.Items.Objects[i].GetInterface(ITerm, aTerm) then
            if aTerm.getID = SubItems[1] then
              begin
                cbxTerm.ItemIndex := i;
                Exit;
              end;
      end;
end;

function TfraEditMappingTable.TermList(aTermType: TTermType): TStringList;
begin
  Result := fTerms[aTermType];
end;

procedure TfraEditMappingTable.acUpdateTermListExecute(Sender: TObject);
begin
  cbxTerm.Items.Clear;
  if cbxType.ItemIndex > -1 then
    cbxTerm.Items.Assign(fTerms[TTermType(cbxType.ItemIndex)]);
end;

procedure TfraEditMappingTable.acUpdateExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if lvTerms.Selected <> nil then
    with lvTerms.Selected do
      begin
        Caption := edtVendorKey.Text;
        SubItems[0] := cbxType.Text;
        //SubItems[3] := edtParserKey.Text;
        if cbxTerm.ItemIndex > -1 then
          if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, aTerm) then
            begin
              SubItems[1] := aTerm.getID;
              SubItems[2] := aTerm.getTerm;
            end
          else
            begin
              SubItems[1] := '';
              SubItems[2] := '';
            end;
      end;
end;

procedure TfraEditMappingTable.acAddItemExecute(Sender: TObject);
var
  i: integer;
begin
  i := 1;
  while lvTerms.FindCaption(0, Format('V Key %d', [i]), False, False, False) <> nil do
    inc(i);
  with lvTerms.Items.Add do
    begin
      Caption := Format('V Key %d', [i]);
      SubItems.Add(QUALNAME[ttObservation]);
      SubItems.Add('');
      SubItems.Add('');
      SubItems.Add('');
      Selected := True;
    end;
end;

procedure TfraEditMappingTable.acRemoveItemExecute(Sender: TObject);
begin
  if lvTerms.Selected <> nil then
    lvTerms.Items.Delete(lvTerms.Selected.Index);
end;

end.

