unit mEditObservationView;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 20 $  $Modtime: 5/26/09 5:30p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Edits a single Observation View Definition
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditObservationView.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Buttons,
  StdCtrls,
  ExtCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  ActnList,
  XMLDoc,
  XMLIntf,
  ImgList,
  ComCtrls,
  ToolWin,
  Menus;

type
  TfraEditObservationView = class(TFrame, ICPEditor)
    lblComment: TLabel;
    lblDisplayName: TLabel;
    edtComment: TEdit;
    edtDisplayName: TEdit;
    lblTerminology: TLabel;
    lblTimeInterval: TLabel;
    cbxInterval: TComboBox;
    lblXAxis: TLabel;
    cbxXAxis: TComboBox;
    lblGeneral: TLabel;
    scbxTerms: TScrollBox;
    pnlGeneral: TPanel;
    Shape1: TShape;
    Shape2: TShape;
    ActionList1: TActionList;
    acAddTerm: TAction;
    acRemoveTerm: TAction;
    acMoveUp: TAction;
    acMoveDown: TAction;
    ImageList1: TImageList;
    ckbxAllowPivot: TCheckBox;
    lvTerms: TListView;
    Bevel1: TBevel;
    pnlControls: TPanel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Button1: TButton;
    acRefreshListView: TAction;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    acEditTerm: TAction;
    chkbxActive: TCheckBox;
    acReorderList: TAction;
    procedure Button1Click(Sender: TObject);
    procedure acRefreshListViewExecute(Sender: TObject);
    procedure acMoveUpExecute(Sender: TObject);
    procedure acEditTermExecute(Sender: TObject);
    procedure lvTermsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure acMoveDownExecute(Sender: TObject);
    procedure acRemoveTermExecute(Sender: TObject);
    procedure acAddTermExecute(Sender: TObject);
    procedure acReorderListExecute(Sender: TObject);
    procedure lvTermsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
  private
    fView: IObservationView;
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
  end;

function NewEditObservationViewFrame(aParent: TWinControl): TfraEditObservationView;

implementation

uses
  iCliOGrid,
  fMain,
  uCPXML,
  oCP_Exceptions,
  fEditViewTerm,
  u508Dialogs;

{$R *.dfm}

function NewEditObservationViewFrame(aParent: TWinControl): TfraEditObservationView;
var
  aGridInterval: TGridInterval;
  aGridStyle: TGridStyle;
begin
  Result := TfraEditObservationView.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblGeneral.Font.Style := [fsBold];
      lblTerminology.Font.Style := [fsBold];

      for aGridInterval := Low(TGridInterval) to High(TGridInterval) do
        cbxInterval.Items.Add(GRID_INTERVAL_NAME[aGridInterval]);

      for aGridStyle := Low(TGridStyle) to High(TGridStyle) do
        cbxXAxis.Items.Add(GRID_STYLE_NAME[aGridStyle]);
    end;
end;

{ TfraEditObservationView }

procedure TfraEditObservationView.acAddTermExecute(Sender: TObject);
begin
  //with TfrmEditViewTerm.Create(Self) do
  with NewEditViewTermForm do
    try
      MyTerminology.LoadList(cbxTerm.Items, ttObservation);
      ShowModal;
      if getViewTerm <> nil then
        begin
          getViewTerm.setDisplayOrder(lvTerms.Items.Count + 1);
          fView.InsertViewTerm(getViewTerm);
          getViewTerm.AddToListView(lvTerms);
          acRefreshListView.Execute;
        end;
    finally
      Free;
    end;
end;

procedure TfraEditObservationView.acEditTermExecute(Sender: TObject);
var
  aViewTerm: IObservationViewTerm;
begin
  if Assigned(lvTerms.Selected) then
    if TObject(lvTerms.Selected.Data).GetInterface(IObservationViewTerm, aViewTerm) then
      //with TfrmEditViewTerm.Create(Self) do
      with NewEditViewTermForm do
        begin
          MyTerminology.LoadList(cbxTerm.Items, ttObservation);
          setViewTerm(aViewTerm);
          ShowModal;
          acRefreshListView.Execute;
        end;
end;

procedure TfraEditObservationView.acMoveDownExecute(Sender: TObject);
var
  aViewTerm: IObservationViewTerm;
begin
  if lvTerms.Selected.Index < (lvTerms.Items.Count - 1) then
    begin
      if TObject(lvTerms.Selected.Data).GetInterface(IObservationViewTerm, aViewTerm) then
        aViewTerm.setDisplayOrder(aViewTerm.getDisplayOrder + 1);
      if TObject(lvTerms.Items[lvTerms.Selected.Index + 1].Data).GetInterface(IObservationViewTerm, aViewTerm) then
        aViewTerm.setDisplayOrder(aViewTerm.getDisplayOrder - 1);
    end;
  acRefreshListView.Execute;
end;

procedure TfraEditObservationView.acMoveUpExecute(Sender: TObject);
var
  aViewTerm: IObservationViewTerm;
begin
  if lvTerms.Selected.Index > 0 then
    begin
      if TObject(lvTerms.Selected.Data).GetInterface(IObservationViewTerm, aViewTerm) then
        aViewTerm.setDisplayOrder(aViewTerm.getDisplayOrder - 1);
      if TObject(lvTerms.Items[lvTerms.Selected.Index - 1].Data).GetInterface(IObservationViewTerm, aViewTerm) then
        aViewTerm.setDisplayOrder(aViewTerm.getDisplayOrder + 1);
    end;
  acRefreshListView.Execute;
end;

procedure TfraEditObservationView.acRefreshListViewExecute(Sender: TObject);
var
  aTerm: ITerm;
  aViewTerm: IObservationViewTerm;
  i: integer;
begin
  for i := 0 to lvTerms.Items.Count - 1 do
    if TObject(lvTerms.Items[i].Data).GetInterface(IObservationViewTerm, aViewTerm) then
      with lvTerms.Items[i] do
        begin
          SubItems.Clear;
          aTerm := MyTerminology.GetTerm(aViewTerm.getAttributeID(ttObservation));
          Caption := IntToStr(aViewTerm.getDisplayOrder);
          SubItems.Add(aViewTerm.getDisplayName);
          SubItems.Add(aTerm.getTerm);
          SubItems.Add(CP_YN_TEXT[aViewTerm.getDisplayOnly]);
          SubItems.Add(IntToStr(aViewTerm.getDisplayWidth));
        end;
  lvTerms.AlphaSort;
end;

procedure TfraEditObservationView.acRemoveTermExecute(Sender: TObject);
var
  aViewTerm: IObservationViewTerm;
begin
  if Assigned(lvTerms.Selected) then
    if TObject(lvTerms.Selected.Data).GetInterface(IObservationViewTerm, aViewTerm) then
      if MessageDlg508(Format('Remove the term ''%s'' from this view defintion?',
        [aViewTerm.getDisplayName]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        begin
          fView.DeleteViewTerm(aViewTerm);
          lvTerms.DeleteSelected;
          acReorderList.Execute;
          acRefreshListView.Execute;
        end;
end;

procedure TfraEditObservationView.acReorderListExecute(Sender: TObject);
var
  i: integer;
  aViewTerm: IObservationViewTerm;
begin
  for i := 0 to lvTerms.Items.Count - 1 do
    if TObject(lvTerms.Items[i].Data).GetInterface(IObservationViewTerm, aViewTerm) then
      aViewTerm.setDisplayOrder(i + 1);
  aViewTerm := nil;
end;

procedure TfraEditObservationView.Button1Click(Sender: TObject);
begin
  fView.DumpToFile(Format('.\View_%s.txt', [fView.getName]));
  MessageDlg508('Current definition saved to ' + ExpandFileName(Format('.\View_%s.txt', [fView.getName])), mtInformation, [mbOk], 0);
end;

procedure TfraEditObservationView.Initialize(aItem: IInterface);
var
  aInterval: TGridInterval;
  i: integer;
begin
  if aItem.QueryInterface(IObservationView, fView) = 0 then
    begin
      edtDisplayName.Text := fView.getDisplayName;
      edtComment.Text := fView.getComment;

      for aInterval := Low(TGridInterval) to High(TGridInterval) do
        if GRID_INTERVAL_VALUE[aInterval] = fView.getDisplayInterval then
          cbxInterval.ItemIndex := Ord(aInterval);
      cbxXAxis.ItemIndex := Ord(fView.getViewStyle);
      ckbxAllowPivot.Checked := fView.getAllowPivot;
      chkbxActive.Checked := fView.getActive;

      for i := 0 to fView.getViewTermCount - 1 do
        fView.getViewTermByIndex(i).AddToListView(lvTerms);

      acRefreshListView.Execute;
      Enabled := (not fView.getIsNationalTitle);
    end;
end;

procedure TfraEditObservationView.lvTermsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  Compare := StrToIntDef(Item1.Caption, 0) - StrToIntDef(Item2.Caption, 0);
end;

procedure TfraEditObservationView.lvTermsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  if Selected then
    begin
      acMoveUp.Enabled := (Item.Index > 0);
      acMoveDown.Enabled := (Item.Index < lvTerms.Items.Count - 1);
      acRemoveTerm.Enabled := True;
    end
  else
    begin
      acMoveUp.Enabled := False;
      acMoveDown.Enabled := False;
      acRemoveTerm.Enabled := False;
    end;
end;

procedure TfraEditObservationView.Save;
var
  i: integer;
  aViewTerm: IObservationViewTerm;
begin
  if fView.getIsNationalTitle then
    MessageDlg508('Sorry, this is a national title and cannot be saved locally.', mtInformation, [mbOk], 0)
  else
    try
      fView.setAllowPivot(ckbxAllowPivot.Checked);
      fView.setDisplayInterval(GRID_INTERVAL_VALUE[TGridInterval(cbxInterval.ItemIndex)]);
      fView.setDisplayName(edtDisplayName.Text);
      fView.setComment(edtComment.Text);
      fView.setViewStyle(TObservationViewStyle(cbxXAxis.ItemIndex));
      fView.setActive(chkbxActive.Checked);
      {Reset the display order}
      for i := 0 to lvTerms.Items.Count - 1 do
        if TObject(lvTerms.Items[i].Data).GetInterface(IObservationViewTerm, aViewTerm) then
          aViewTerm.setDisplayOrder(i + 1);
      fView.Save;
    except
      on E: ECPSaveFailure do
        MessageDlg508('Error saving view ''' + fView.getID + '''.' + #13 +
          'Error: ' + e.Message, mtError, [mbOk], 0);
      on E: Exception do
        MessageDlg508('Unknown error saving view.' + #13 +
          'Error: ' + e.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfraEditObservationView.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditObservationView.Validate(var aErrorLog: TStringList);
begin
  if edtDisplayName.Text = '' then
    aErrorLog.Add('Display Name required.');
  if cbxInterval.ItemIndex = -1 then
    aErrorLog.Add('Time Interval required.');
  if cbxXAxis.ItemIndex = -1 then
    aErrorLog.Add('X-Axis required.');
end;

end.

