unit mEditObservationViewTerm;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 5 $  $Modtime: 5/27/07 10:25a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Repeating frame for each term in an observation view
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditObservationViewTerm.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  oCliOTypes,
  oInterfaces,
  XMLDoc,
  XMLIntf,
  mEditObservationViewFilter;

type
  TfraEditObservationViewTerm = class(TFrame)
    cbxTerm: TComboBox;
    edtDisplayName: TEdit;
    edtDisplayWidth: TEdit;
    Panel1: TPanel;
    lblTerm: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    udWidth: TUpDown;
    Label1: TLabel;
    Shape1: TShape;
    Shape2: TShape;
    cbxUnit: TComboBox;
    Label4: TLabel;
    ckbxMandatory: TCheckBox;
    ckbxTotal: TCheckBox;
    ckbxCount: TCheckBox;
    ckbxAverage: TCheckBox;
    gbxCalculations: TGroupBox;
    cbxDisplayOnly: TComboBox;
    Label5: TLabel;
    procedure cbxTermChange(Sender: TObject);
    procedure cbxUnitClick(Sender: TObject);
  private
    fFrameList: array[TTermType] of TfraEditObservationViewFilter;
  public
    procedure SetFilter(aFilterType: TTermType; aFilterVUID: string; aFilterMandatory: Boolean);
    procedure Save(aViewID: string; aTermOrder: integer; aDBConnection: IDBConnection);
  end;

function NewEditObservationViewTermFrame(aParent: TWinControl): TfraEditObservationViewTerm;

implementation

{$R *.dfm}

function NewEditObservationViewTermFrame(aParent: TWinControl): TfraEditObservationViewTerm;
var
  tt: TTermType;
begin
  Result := TfraEditObservationViewTerm.Create(aParent);
  with Result do
    begin
      Name := Format('%s_%d', [Result.ClassName, aParent.ControlCount]);
      Top := aParent.ComponentCount * Height;
      Parent := aParent;
      Align := alTop;
      lblTerm.Font.Style := [fsBold];
      for tt := Low(TTermType) to High(TTermType) do
        if tt in TERMFILTERS then
          begin
            fFrameList[tt] := NewEditObservationViewFilterFrame(Result);
            fFrameList[tt].ckbxFilterType.Caption := Format('%s:', [TERMTYPE[tt]]);
          end;
    end;
end;

{ TfraEditObservationViewTerm }

procedure TfraEditObservationViewTerm.cbxTermChange(Sender: TObject);
var
  aTerm: ITerm;
  tt: TTermType;
begin
  if cbxTerm.ItemIndex > -1 then
    if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, aTerm) then
      begin
        edtDisplayName.Text := aTerm.getDisplayName;
        aTerm.LoadListOfPairs(ttUnit, cbxUnit.Items);
        cbxUnit.ItemIndex := -1;
        if cbxUnit.Items.Count > 0 then
          begin
            cbxUnit.Enabled := True;
            cbxUnit.Color := clWindow;
          end
        else
          begin
            cbxUnit.Enabled := False;
            cbxUnit.Color := clBtnFace;
          end;

        ckbxMandatory.Checked := False;
        ckbxTotal.Checked := False;
        ckbxCount.Checked := False;
        ckbxAverage.Checked := False;
        gbxCalculations.Enabled := False;
        ckbxMandatory.Enabled := False;
        ckbxTotal.Enabled := False;
        ckbxCount.Enabled := False;
        ckbxAverage.Enabled := False;

        for tt := Low(TTermType) to High(TTermType) do
          if fFrameList[tt] <> nil then
            begin
              aTerm.LoadListOfPairs(tt, fFrameList[tt].cbxFilter.Items, tdTerm);
              fFrameList[tt].ckbxFilterType.Checked := False;
              fFrameList[tt].ckbxFilterType.Enabled := (fFrameList[tt].cbxFilter.Items.Count > 0);
              fFrameList[tt].rbtnDefault.Checked := False;
              fFrameList[tt].rbtnDefault.Enabled := False;
              fFrameList[tt].rbtnMandatory.Checked := False;
              fFrameList[tt].rbtnMandatory.Enabled := False;
              fFrameList[tt].cbxFilter.ItemIndex := -1;
              fFrameList[tt].cbxFilter.Enabled := False;

              if fFrameList[tt].cbxFilter.Items.Count > 0 then
                fFrameList[tt].cbxFilter.Color := clWindow
              else
                fFrameList[tt].cbxFilter.Color := clBtnFace;

              //fFrameList[tt].SetEnabled(fFrameList[tt].cbxFilter.Items.Count > 0);
            end;
      end;
end;

procedure TfraEditObservationViewTerm.Save(aViewID: string; aTermOrder: integer; aDBConnection: IDBConnection);
var
  aTerm: ITerm;
  aUnit: ITerm;
  tt: TTermType;
begin
  if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, aTerm) then
    begin
      if not aDBConnection.ExecCommand(
        'AddViewTerminology',
        [aViewID, aTermOrder, aTerm.getVUID, edtDisplayName.Text, udWidth.Position,
        BOOLEAN_AS_INTEGER[ckbxTotal.Checked], BOOLEAN_AS_INTEGER[ckbxCount.Checked],
          BOOLEAN_AS_INTEGER[ckbxAverage.Checked], BOOLEAN_AS_INTEGER[(cbxDisplayOnly.ItemIndex = 1)]]) then
        raise Exception.Create('Error saving terminology');

      if cbxUnit.Items.Count > 0 then
        if cbxUnit.ItemIndex > -1 then
          begin
            if cbxUnit.Items.Objects[cbxUnit.ItemIndex].GetInterface(Iterm, aUnit) then
              if not aDBConnection.ExecCommand(
                'AddViewFilter',
                [aViewID, aTermOrder, ttUnit, aUnit.getVUID, BOOLEAN_AS_INTEGER[ckbxMandatory.Checked]]) then
                raise Exception.Create('Error saving terminology unit specifications');
          end
        else
          if not aDBConnection.ExecCommand(
            'AddViewFilter',
            [aViewID, aTermOrder, ttUnit, '', '']) then
            raise Exception.Create('Error saving terminology unit specifications');

      for tt := Low(TTermType) to High(TTermType) do
        if fFrameList[tt] <> nil then
          TfraEditObservationViewFilter(fFrameList[tt]).Save(aViewID, aTermOrder, tt, aDBConnection);
    end;
end;

procedure TfraEditObservationViewTerm.SetFilter(aFilterType: TTermType; aFilterVUID: string; aFilterMandatory: Boolean);
var
  i: integer;
  aUnit: ITerm;
begin
  if aFilterType = ttUnit then
    begin
      cbxUnit.ItemIndex := -1;
      for i := 0 to cbxUnit.Items.Count - 1 do
        if cbxUnit.Items.Objects[i].GetInterface(ITerm, aUnit) then
          if aUnit.getVUID = aFilterVUID then
            begin
              cbxUnit.ItemIndex := i;
              ckbxMandatory.Checked := aFilterMandatory;
            end;

      gbxCalculations.Enabled := (cbxUnit.ItemIndex > -1);
      ckbxMandatory.Enabled := (cbxUnit.ItemIndex > -1);
      ckbxTotal.Enabled := (cbxUnit.ItemIndex > -1);
      ckbxCount.Enabled := (cbxUnit.ItemIndex > -1);
      ckbxAverage.Enabled := (cbxUnit.ItemIndex > -1);
    end
  else
    if fFrameList[aFilterType] <> nil then
      TfraEditObservationViewFilter(fFrameList[aFilterType]).SetFilter(aFilterVUID, aFilterMandatory);
end;

procedure TfraEditObservationViewTerm.cbxUnitClick(Sender: TObject);
begin
  gbxCalculations.Enabled := (cbxUnit.ItemIndex > -1);
  ckbxTotal.Enabled := (cbxUnit.ItemIndex > -1);
  ckbxCount.Enabled := (cbxUnit.ItemIndex > -1);
  ckbxAverage.Enabled := (cbxUnit.ItemIndex > -1);
  ckbxMandatory.Enabled := (cbxUnit.ItemIndex > -1);
end;

end.

