unit mEditProcedure;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 15 $  $Modtime: 5/22/09 1:49p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Edit a single procedure definition.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Console/mEditProcedure.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  XMLDoc,
  XMLIntf,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  StdCtrls,
  mLookup,
  ExtCtrls,
  CheckLst,
  ComCtrls, VA508AccessibilityManager;

type
  TfraEditProcedure = class(TForm, ICPEditor)
    pnlGeneral: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    lblGeneral: TLabel;
    Shape1: TShape;
    edtDescription: TEdit;
    ckbxActive: TCheckBox;
    Label4: TLabel;
    ckbxReqExtData: TCheckBox;
    ckbxAutoSubmit: TCheckBox;
    ckbxHighVolume: TCheckBox;
    Label3: TLabel;
    edtExternalAttachmentDirectory: TEdit;
    Label5: TLabel;
    rgProcessingApplication: TRadioGroup;
    rgProcessedResults: TRadioGroup;
    pnlInstruments: TPanel;
    Shape2: TShape;
    lblInstruments: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Bevel4: TBevel;
    cklstbxInstruments: TCheckListBox;
    fraLookupTreatingSpecialty: TfraLookup;
    fraLookupHospitalLocation: TfraLookup;
    fraLookupTIUNoteTitle: TfraLookup;
    A508Mgr: TVA508AccessibilityManager;
    procedure cklstbxInstrumentsDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
  private
    fProcedure: ICPProcedure;
    procedure LookupTreatingSpecialty(aPartial: string; aResult: TStrings);
    procedure LookupHospitalLocation(aPartial: string; aResult: TStrings);
    procedure LookupTIUNote(aPartial: string; aResult: TStrings);
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditProcedureFrame(aParent: TWinControl): TfraEditProcedure;

implementation

uses fMain;

{$R *.dfm}

function NewEditProcedureFrame(aParent: TWinControl): TfraEditProcedure;
begin
  Result := TfraEditProcedure.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblGeneral.Font.Style := [fsBold];
      lblInstruments.Font.Style := [fsBold];
    end;
end;

{ TfraEditProcedure }

procedure TfraEditProcedure.Initialize(aItem: IInterface);
var
//  aNode: IXMLNode;
//  aXMLDoc: IXMLDocument;
  aInstrument: ICPInstrument;
  i: integer;
//  x: string;
begin
  if aItem.QueryInterface(ICPProcedure, fProcedure) = 0 then
    begin
      fraLookupHospitalLocation.Initialize(
        fProcedure.getHospitalLocationID,
        fProcedure.getHospitalLocation,
        LookupHospitalLocation, nil, Label4);

      fraLookupTreatingSpecialty.Initialize(
        fProcedure.getTreatingSpecialtyID,
        fProcedure.getTreatingSpecialty,
        LookupTreatingSpecialty, nil, Label2);

      fraLookupTIUNoteTitle.Initialize(
        fProcedure.getDefaultTIUNoteID,
        fProcedure.getDefaultTIUNote,
        LookupTIUNote, nil, Label3);

      //fraLookupTIUNoteTitle.Enabled := False;

      edtDescription.Text := fProcedure.getDescription;
      edtExternalAttachmentDirectory.Text := fProcedure.getExternalDataDirectory;
      ckbxReqExtData.Checked := fProcedure.getRequireExternalData;
      ckbxAutoSubmit.Checked := fProcedure.getAutoSubmit;
      ckbxHighVolume.Checked := fProcedure.getHighVolume;
      ckbxActive.Checked := fProcedure.getActive;
      rgProcessingApplication.ItemIndex := Ord(fProcedure.getProcessingApplication) - 1;
      rgProcessedResults.ItemIndex := Ord(fProcedure.getProcessedResult);

      for i := 0 to cklstbxInstruments.Count - 1 do
        if cklstbxInstruments.Items.Objects[i].GetInterface(ICPInstrument, aInstrument) then
          cklstbxInstruments.Checked[i] := fProcedure.getInstrumentAssigned(aInstrument.getID);
    end;
  Visible := true;
end;

procedure TfraEditProcedure.LookupHospitalLocation(aPartial: string; aResult: TStrings);
begin
  DoLookup('GetHospitalLocationList', aPartial, 60, aResult);
end;

procedure TfraEditProcedure.LookupTIUNote(aPartial: string; aResult: TStrings);
begin
  DoLookup('GetNoteTitlesCP', aPartial, 60, aResult);
end;

procedure TfraEditProcedure.LookupTreatingSpecialty(aPartial: string; aResult: TStrings);
begin
  if aPartial = '' then
    aPartial := '*'; {Small enough file, we can yank it all if we want}
  DoLookup('GetTreatingSpecialtyList', aPartial, 60, aResult);
end;

procedure TfraEditProcedure.Save;
var
  i: integer;
  aInstrument: ICPInstrument;
begin
  with fProcedure do
    begin
      setTreatingSpecialty(fraLookupTreatingSpecialty.ExternalValue);
      setTreatingSpecialtyID(fraLookupTreatingSpecialty.InternalValue);
      setHospitalLocation(fraLookupHospitalLocation.ExternalValue);
      setHospitalLocationID(fraLookupHospitalLocation.InternalValue);
      setDefaultTIUNote(fraLookupTIUNoteTitle.ExternalValue);
      setDefaultTIUNoteID(fraLookupTIUNoteTitle.InternalValue);
      setDescription(edtDescription.Text);
      setRequireExternalData(ckbxReqExtData.Checked);
      setAutoSubmit(ckbxAutoSubmit.Checked);
      setHighVolume(ckbxHighVolume.Checked);

      if rgProcessingApplication.ItemIndex > -1 then
        setProcessingApplication(TCPProcessingApplication(rgProcessingApplication.ItemIndex + 1))
      else
        setProcessingApplication(paDefault);

      setProcessedResult(TCPProcessedResult(rgProcessedResults.ItemIndex));
      setExternalDataDirectory(edtExternalAttachmentDirectory.Text);

      setActive(ckbxActive.Checked);
      ClearInstruments;
      for i := 0 to cklstbxInstruments.Count - 1 do
        if cklstbxInstruments.Checked[i] then
          if cklstbxInstruments.Items.Objects[i].GetInterface(ICPInstrument, aInstrument) then
            AddInstrument(aInstrument.getID);

      Save;
    end;
end;

procedure TfraEditProcedure.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditProcedure.Validate(var aErrorLog: TStringList);
begin
  if rgProcessingApplication.ItemIndex < 0 then
    aErrorLog.Add('Processing Application is a required field.');
  if rgProcessedResults.ItemIndex < 0 then
    aErrorLog.Add('Processed Results is a required field.');
end;

procedure TfraEditProcedure.cklstbxInstrumentsDrawItem(
  Control: TWinControl; Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  aItem: ICPConsoleItem;
begin
  if not cklstbxInstruments.Items.Objects[Index].GetInterface(ICPConsoleItem, aItem) then
    begin
      cklstbxInstruments.Canvas.TextOut(0, 0, cklstbxInstruments.Items[Index]);
      Exit;
    end;

  if odSelected in State then
    begin
      cklstbxInstruments.Canvas.Brush.Color := clHighlight;
      cklstbxInstruments.Canvas.Font.Color := clHighlightText;
      cklstbxInstruments.Canvas.Font.Style := [fsbold];
    end
  else
    begin
      cklstbxInstruments.Canvas.Brush.Color := clWindow;
      cklstbxInstruments.Canvas.Font.Color := clWindowText;
      cklstbxInstruments.Canvas.Font.Style := [];
    end;

  cklstbxInstruments.Canvas.FillRect(Rect);

  with cklstbxInstruments do
    begin
      if aItem.getActive then
        frmMain.ImageList1.Draw(Canvas, Rect.Left + 4, Rect.Top + 4, 8)
      else
        frmMain.ImageList1.Draw(Canvas, Rect.Left + 4, Rect.Top + 4, 7);

      (*
      Canvas.TextOut(Rect.Left + ItemHeight, Rect.Top, aItem.getName);
      Canvas.TextOut(Rect.Left + (ItemHeight * 2), Rect.Bottom - Canvas.TextHeight('|'), aItem.getComment);
      *)

      Canvas.TextOut(
        Rect.Left + ItemHeight,
        Rect.Top + ((ItemHeight - Canvas.TextHeight('|')) div 2),
        Format('  %s - %s', [aItem.getName, aItem.getComment]))
    end;
end;

end.

