unit mEditSchedule;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  oInterfaces,
  StdCtrls,
  CheckLst,
  ExtCtrls, VA508AccessibilityManager;

type
  TfraEditSchedule = class(TForm, ICPEditor)
    edtDisplayName: TEdit;
    chklstbxCustom: TCheckListBox;
    cbxInterval: TComboBox;
    memComment: TMemo;
    Label1: TLabel;
    lblCustomTimes: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Shape1: TShape;
    lblScheduleInformation: TLabel;
    chkbxActive: TCheckBox;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure cbxIntervalChange(Sender: TObject);
  private
    fSchedule: ICPSchedule;

    function getCustomSchedule: string;
    procedure setCustomSchedule(const aValue: string);

    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditScheduleFrame(aParent: TWinControl): TfraEditSchedule;

implementation

{$R *.dfm}

uses
  DateUtils,
  fMain,
  u508Dialogs;

type
  TCPInterval = (
    ivCustom,
    ivMinute,
    iv5Minutes,
    iv10Minutes,
    iv15Minutes,
    iv20Minutes,
    iv30Minutes,
    iv1Hour,
    iv2Hours,
    iv3Hours,
    iv4Hours,
    iv6Hours,
    iv8Hours,
    iv12Hours,
    iv24Hours,
    iv36Hours,
    iv2Days,
    iv3Days,
    iv4Days,
    iv5Days,
    iv6Days,
    iv7Days,
    iv2Weeks,
    iv3Weeks,
    ivMonthly,
    iv2Months,
    iv3Months,
    iv4Months,
    iv5Months,
    iv6Months,
    ivAnnual);

const
  CP_INTERVAL_NAME: array[TCPInterval] of string = (
    'Custom',
    '1 Minute',
    '5 Minutes',
    '10 Minutes',
    '15 Minutes',
    '20 Minutes',
    '30 Minutes',
    '1 Hour',
    '2 Hours',
    '3 Hours',
    '4 Hours',
    '6 Hours',
    '8 Hours',
    '12 Hours',
    '24 Hours',
    '36 Hours',
    '2 Days',
    '3 Days',
    '4 Days',
    '5 Days',
    '6 Days',
    '7 Days',
    '2 Weeks',
    '3 Weeks',
    'Monthly',
    '2 Months',
    '3 Months',
    '4 Months',
    '5 Months',
    '6 Months',
    'Annual');

  CP_INTERVAL_MINUTES: array[TCPInterval] of integer = (
    0,
    1,
    5,
    10,
    15,
    20,
    30,
    60,
    120,
    180,
    240,
    360,
    480,
    720,
    1440,
    2160,
    2880,
    4320,
    5760,
    7200,
    8640,
    10080,
    20160,
    30240,
    43200,
    86400,
    129600,
    172800,
    216000,
    259200,
    525600);

function NewEditScheduleFrame(aParent: TWinControl): TfraEditSchedule;
var
  aInt: TCPInterval;
  aTime: TDateTime;
  aMin: integer;
begin
  Result := TfraEditSchedule.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblScheduleInformation.Font.Style := [fsBold];
      for aInt := Low(TCPInterval) to High(TCPInterval) do
        cbxInterval.Items.Add(CP_INTERVAL_NAME[aInt]);
      cbxInterval.ItemIndex := 0;
      lblCustomTimes.Enabled := False;
      chklstbxCustom.Enabled := False;
      aMin := 0;
      while aMin < 1440 do
        begin
          aTime := IncMinute(0, aMin);
          chklstbxCustom.Items.Add(FormatDateTime('hhmm', aTime));
          inc(aMin, 15);
        end;
      Visible := true;
    end;
end;

{ TfraEditSchedule }

procedure TfraEditSchedule.Initialize(aItem: IInterface);
var
  aInt: TCPInterval;
begin
  if aItem.QueryInterface(ICPSchedule, fSchedule) = 0 then
    begin
      edtDisplayName.Text := fSchedule.getDisplayName;
      chkbxActive.Checked := fSchedule.getActive;
      cbxInterval.ItemIndex := -1;
      for aInt := Low(TCPInterval) to High(TCPInterval) do
        if fSchedule.getScheduleInterval = CP_INTERVAL_MINUTES[aInt] then
          cbxInterval.ItemIndex := Ord(aInt);
      if cbxInterval.ItemIndex = 0 then
        if fSchedule.getSchedule <> '' then
          setCustomSchedule(fSchedule.getSchedule);
      lblCustomTimes.Enabled := (cbxInterval.ItemIndex = 0);
      chklstbxCustom.Enabled := (cbxInterval.ItemIndex = 0);
      memComment.Text := fSchedule.getComment;
      Visible := true;
    end
  else
    MessageDlg508('Error, item is not a schedule', mtError, [mbOk], 0);
end;

procedure TfraEditSchedule.Save;
begin
  fSchedule.setDisplayName(edtDisplayName.Text);
  fSchedule.setActive(chkbxActive.Checked);
  fSchedule.setComment(memComment.Text);
  if cbxInterval.ItemIndex = 0 then
    begin
      fSchedule.setScheduleInterval(CP_INTERVAL_MINUTES[TCPInterval(cbxInterval.ItemIndex)]);
      fSchedule.setCustomSchedule(getCustomSchedule);
    end
  else
    begin
      fSchedule.setScheduleInterval(CP_INTERVAL_MINUTES[TCPInterval(cbxInterval.ItemIndex)]);
      fSchedule.setCustomSchedule('');
    end;
  MyCPService.SaveConsoleItem(fSchedule);
end;

procedure TfraEditSchedule.setCustomSchedule(const aValue: string);
var
  i: integer;
begin
  with TStringList.Create do
    try
      CommaText := aValue;
      for i := 0 to chklstbxCustom.Items.Count - 1 do
        chklstbxCustom.Checked[i] := (IndexOf(chklstbxCustom.Items[i]) > -1);
    finally
      Free;
    end;
end;

function TfraEditSchedule.getCustomSchedule: string;
var
  i: integer;
begin
  with TStringList.Create do
    try
      for i := 0 to chklstbxCustom.Items.Count - 1 do
        if chklstbxCustom.Checked[i] then
          Add(chklstbxCustom.Items[i]);
      Result := CommaText;
    finally
      Free;
    end;
end;

procedure TfraEditSchedule.cbxIntervalChange(Sender: TObject);
begin
  if cbxInterval.ItemIndex = 0 then
    begin
      lblCustomTimes.Enabled := True;
      chklstbxCustom.Enabled := True;
      setCustomSchedule(fSchedule.getCustomSchedule);
    end
  else
    begin
      chklstbxCustom.ItemIndex := -1;
      lblCustomTimes.Enabled := False;
      chklstbxCustom.Enabled := False;
      setCustomSchedule('');
    end;
end;

procedure TfraEditSchedule.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditSchedule.Validate(var aErrorLog: TStringList);
begin

end;

end.

