unit mEditShift;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  XMLDoc,
  XMLIntf,
  oCP_Exceptions,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ComCtrls, VA508AccessibilityManager;

type
  TfraEditShift = class(TForm, ICPEditor)
    lblShiftInformation: TLabel;
    Shape1: TShape;
    edtDisplayName: TEdit;
    cxbActive: TCheckBox;
    edtComment: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    cbxStartTime: TComboBox;
    cbxStopTime: TComboBox;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
  private
    fShift: ICPShift;
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure Save; stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  public
    { Public declarations }
  end;

function NewEditShiftFrame(aParent: TWinControl): TfraEditShift;

implementation

uses
  DateUtils,
  u508Dialogs;

const
  SHIFT_INC = 15;

{$R *.dfm}

function NewEditShiftFrame(aParent: TWinControl): TfraEditShift;
var
  dt: TDateTime;
begin
  Result := TfraEditShift.Create(aParent);
  with Result do
    begin
      Name := Format('f%s', [NewGUID(True)]);
      Top := aParent.ControlCount * 20;
      Parent := aParent;
      Align := alClient;
      lblShiftInformation.Font.Style := [fsBold];
      dt := trunc(Now);
      while dt < (trunc(now) + 1) do
        begin
          cbxStartTime.Items.Add(FormatDateTime('hh:mm ampm', dt));
          cbxStopTime.Items.Add(FormatDateTime('hh:mm ampm', dt));
          dt := IncMinute(dt, SHIFT_INC);
        end;
    end;
end;

{ TfraEditShift }

procedure TfraEditShift.Initialize(aItem: IInterface);
begin
  if aItem.QueryInterface(ICPShift, fShift) = 0 then
    begin
      edtDisplayName.Text := fShift.getDisplayName;
      cxbActive.Checked := fShift.getActive;
      edtComment.Text := fShift.getComment;
      cbxStartTime.ItemIndex := cbxStartTime.Items.IndexOf(FormatDateTime('hh:mm ampm', fShift.getStartTime));
      cbxStopTime.ItemIndex := cbxStopTime.Items.IndexOf(FormatDateTime('hh:mm ampm', fShift.getStopTime));
      Visible := true;
    end
  else
    MessageDlg508('Error, trying to edit non-shift item with the shift editor',
      mtError, [mbOk], 0);
end;

procedure TfraEditShift.Save;
begin
  if (edtDisplayName.Text = '') or (edtComment.Text = '') then
    MessageDlg508('Please enter both a display name and a comment', mtInformation, [mbOk], 0)
  else
    try
      fShift.setStartTime(IncMinute(Trunc(now), cbxStartTime.ItemIndex * SHIFT_INC));
      fShift.setStopTime(IncMinute(Trunc(now), cbxStopTime.ItemIndex * SHIFT_INC));
      fShift.setActive(cxbActive.Checked);
      fShift.setDisplayName(edtDisplayName.Text);
      fShift.setComment(edtComment.Text);
      fShift.Save;
    except
      on E: Exception do
        MessageDlg508('Error Saving Shift:' + #13 + e.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfraEditShift.SetReadOnly(const aValue: boolean);
begin
  Enabled := (not aValue);
end;

procedure TfraEditShift.Validate(var aErrorLog: TStringList);
begin
  //
end;

end.

