unit fFileManLookup;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 1 $  $Modtime: 10/05/06 9:39a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Standard Fileman Lookup component
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/forms/fFileManLookup.pas $
*
* $History: fFileManLookup.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:01p
 * Created in $/CP Modernization/CliO/source/Common/forms
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  oInterfaces,
  oFilemanField;

type
  TLkupVal = class(TObject)
    IEN: string;
    Value: string;
  end;

  TLkupDir = (ldBackwards, ldForwards);

  TfrmFileManLookup = class(TForm)
    lbx: TListBox;
    procedure lbxKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure lbxDblClick(Sender: TObject);
  private
    fDBConnection: IDBConnection;
    fFileManField: TFilemanField;
    function getResults(aStartingValue: string; aDirection: TLkupDir): boolean;
    procedure setResults(aDirection: TLkupDir);
  public
    { Public declarations }
  end;

procedure LookupPointer(aDBConnection: IDBConnection; aFileManField: TFileManField; aStartValue: string);

implementation

const
  LKUP_DIR: array[TLkupDir] of string = ('-', '+');

{$R *.dfm}

procedure LookupPointer(aDBConnection: IDBConnection; aFileManField: TFileManField; aStartValue: string);
var
  pt: TPoint;
  Ctrl: TControl;
begin
  with TfrmFileManLookup.Create(Application) do
    try
      Ctrl := aFileManField.getControl;
      pt := Ctrl.Parent.ClientToScreen(Point(Ctrl.Left, Ctrl.Top));
      Left := pt.X;
      Top := pt.Y + Ctrl.Height;
      Width := Ctrl.Width;
      Height := Width;
      fDBConnection := aDBConnection;
      fFileManField := aFileManField;
      getResults(aStartValue, ldForwards);
      if lbx.Items.Count = 0 then
        MessageDlg('Unable to match anything like ''' + aStartValue + '''', mtInformation, [mbOk], 0)
      else
        begin
          if ShowModal = mrOK then {We have a valid result, set it into the I_FilemanField}
            with TLkupVal(lbx.Items.Objects[lbx.ItemIndex]) do
              begin
                aFileManField.setInternalValue(IEN);
                aFileManField.setExternalValue(Value);
              end;
          while lbx.Items.Count > 0 do {Cleanup all item objects}
            begin
              TLkupVal(lbx.Items.Objects[0]).Free;
              lbx.Items.Delete(0);
            end;
        end;
    finally
      Free;
    end;
end;

function TfrmFileManLookup.getResults(aStartingValue: string; aDirection: TLkupDir): boolean;
begin
  (*
  with fDBConnection do
    try
      if ExecStoredProc(
        'ZZDRP FMAN',
        ['PTRLKUP', fFileManField.getDDNumber, '', fFileManField.getFieldNumber],
        [aStartingValue, LKUP_DIR[aDirection]]) then
        setResults(aDirection);
      Result := True;
    except
      //
    end;
  *)
end;

procedure TfrmFileManLookup.setResults(aDirection: TLkupDir);
var
  i: integer;
  LkupVal: TLkupVal;
  CurrentValue: TLkupVal;
begin
  {Find the current object we are on if any}
  if lbx.ItemIndex > -1 then
    CurrentValue := TLkupVal(lbx.Items.Objects[lbx.ItemIndex])
  else
    CurrentValue := nil;

  {Release the list box from visual constraints}
  lbx.Items.BeginUpdate;

  with fDBConnection do
    for i := 1 to ExecResults.Count - 1 do
      begin
        LkupVal := TLkupVal.Create;
        LkupVal.IEN := Copy(ExecResults[i], 1, Pos('^', ExecResults[i]) - 1);
        LkupVal.Value := Copy(ExecResults[i], Pos('^', ExecResults[i]) + 1, Length(ExecResults[i]));
        lbx.Items.AddObject(LkupVal.Value, LkupVal);
      end;

  {Tie back into the UI}
  lbx.Items.EndUpdate;

  if Assigned(CurrentValue) then
    begin
      i := lbx.Items.IndexOfObject(CurrentValue);
      lbx.ItemIndex := i;
      if aDirection = ldBackwards then
        lbx.TopIndex := i;
      lbx.Selected[i] := True;
    end
  else
    if lbx.Items.Count > 0 then
      lbx.ItemIndex := 0;
end;

procedure TfrmFileManLookup.lbxKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_DOWN: {get more if on the last item}
      begin
        if lbx.ItemIndex = lbx.Items.Count - 1 then
          getResults(lbx.Items[lbx.ItemIndex], ldForwards);
      end;
    VK_UP: {get more if on the first item}
      begin
        if lbx.ItemIndex = 0 then
          getResults(lbx.Items[lbx.ItemIndex], ldBackwards);
      end;
    VK_RETURN: {Select the current item if any and close the form}
      begin
        if lbx.ItemIndex > -1 then
          ModalResult := mrOk;
      end;
    VK_ESCAPE: {Close the form, the user wants out}
      begin
        ModalResult := mrCancel;
      end;
  end;
end;

procedure TfrmFileManLookup.lbxDblClick(Sender: TObject);
begin
  if lbx.ItemIndex > -1 then
    ModalResult := mrOk;
end;

end.

