unit fKardexViewer;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  oInterfaces,
  oCPTypes,
  uGlobalVars,
  StdCtrls,
  ComCtrls,
  ActnList,
  ExtCtrls,
  ToolWin, VA508AccessibilityManager;

type
  TfrmKardexViewer = class(TForm)
    btnAction: TButton;
    btnCancel: TButton;
    ActionList1: TActionList;
    acComplete: TAction;
    acClose: TAction;
    acUndo: TAction;
    CoolBar1: TCoolBar;
    pnlTop: TPanel;
    lblPatientName: TLabel;
    lblPtDOBSSN: TLabel;
    lblPtWardRoomBed: TLabel;
    pnlDisplay: TPanel;
    Label1: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    Label2: TLabel;
    Label9: TLabel;
    Label11: TLabel;
    Label13: TLabel;
    pnlComplete: TPanel;
    lblDateCompleted: TLabel;
    lblCompletedComment: TLabel;
    memComment: TMemo;
    dtpDate: TDateTimePicker;
    dtpTime: TDateTimePicker;
    btnAltAction01: TButton;
    chkbxOutOfRange: TCheckBox;
    acOutOfRange: TAction;
    acRefused: TAction;
    btnAltAction02: TButton;
    acHold: TAction;
    acUnHold: TAction;
    btnAltAction03: TButton;
    acAudits: TAction;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    lblCareAction: TVA508StaticText;
    lblStatus: TVA508StaticText;
    lblSchedule: TVA508StaticText;
    lblDueDateTime: TVA508StaticText;
    lblSpecialInstructions: TVA508StaticText;
    lblCompletedDateTime: TVA508StaticText;
    lblCompletedBy: TVA508StaticText;
    lblComment: TVA508StaticText;
    procedure FormCreate(Sender: TObject);
    procedure acUndoExecute(Sender: TObject);
    procedure acCompleteExecute(Sender: TObject);
    procedure acCloseExecute(Sender: TObject);
    procedure acOutOfRangeExecute(Sender: TObject);
    procedure acRefusedExecute(Sender: TObject);
    procedure acHoldExecute(Sender: TObject);
    procedure acUnHoldExecute(Sender: TObject);
    procedure acAuditsExecute(Sender: TObject);
    procedure pnlTopEnter(Sender: TObject);
  private
    fAction: ICPKardexAction;
    fEvent: ICPKardexEvent;
    { Private declarations }
  public
    { Public declarations }
    procedure ReadPnlTop;
  end;

function ViewKardexEvent(aKardex: ICPKardexService; aKardexEvent: ICPKardexEvent): boolean;

implementation

{$R *.dfm}

uses
  fComment,
  DateUtils,
  fKardexAudits,
  u508Func,
  VA508AccessibilityRouter,
  u508Dialogs;

const
  WINDOW_COMPLETION = 90; {I'm sure this will need to be moved to a parameter setting :(}
  WINDOW_UNDO = 15; {I'm sure this will need to be moved to a parameter setting :(}

function getTimeDisplay(aMinutes: integer): string;
begin
  if aMinutes < 120 then
    Result := Format('%d hour', [trunc(aMinutes / 60)])
  else
    Result := Format('%d hours', [trunc(aMinutes / 60)]);

  aMinutes := aMinutes - (trunc(aMinutes / 60) * 60);

  if aMinutes = 1 then
    Result := Format('%s and %d minute', [Result, aMinutes])
  else
    Result := Format('%s and %d minutes', [Result, aMinutes]);
end;

function ViewKardexEvent(aKardex: ICPKardexService; aKardexEvent: ICPKardexEvent): boolean;
//var
//  aMinutes: integer;

begin
  with TfrmKardexViewer.Create(application) do
    try
      lblPatientName.Caption := MyPatient.getDisplayLine(1);
      lblPtDOBSSN.Caption := MyPatient.getDisplayLine(2);
      lblPtWardRoomBed.Caption := MyPatient.getDisplayLine(3);
      fAction := aKardex.getAction(aKardexEvent.getActionID);
      fEvent := aKardexEvent;
      lblCareAction.Caption := fAction.getCareAction.getTerm;
      if fAction.getStatus <> asActive then
        lblCareAction.Caption := Format(
          '%s (%s)',
          [lblCareAction.Caption, CP_KARDEX_ACTION_STATUS[fAction.getStatus]]);
      lblSchedule.Caption := fAction.getScheduleDisplay;
      lblDueDateTime.Caption := FormatDateTime('YYYY-MM-DD hh:mm', fEvent.getScheduledDateTime);
      lblSpecialInstructions.Caption := fAction.getSpecialInstructions;
      lblStatus.Caption := CP_KARDEX_EVENT_STATUS[fEvent.getStatus];
      case fEvent.getStatus of
        esPending:
          begin
            pnlDisplay.Height := lblCompletedDateTime.Top - 1;
            pnlComplete.Top := pnlDisplay.Top + pnlDisplay.Height + 8;
            pnlComplete.Visible := True;
            dtpDate.DateTime := Trunc(now);
            dtpTime.Time := Frac(Now);
            memComment.Text := '';
            btnAction.Action := acComplete;
            btnAltAction01.Action := acRefused;
            btnAltAction02.Action := acHold;
            acOutOfRange.Execute;
          end;
        esCompleted:
          begin
            lblCompletedDateTime.Caption := FormatDateTime('YYYY-MM-DD hh:mm', fEvent.getScheduledDateTime);
            lblCompletedBy.Caption := MyPersonService.getPerson(fEvent.getCompletedByID).getName; //'Me!';
            lblComment.Caption := fEvent.getComment;
            btnAction.Action := acUndo;
            btnAltAction01.Visible := False;
            btnAltAction02.Visible := False;
            pnlComplete.Visible := False;
            if fAction.getStatus = asActive then
              if (fEvent.getCompletedByID = MyUser.getDUZ) or MyUser.HasRole('MD MANAGER') then
                acUndo.Enabled :=
                  (MinutesBetween(Now, fEvent.getCompletedDateTime) < WINDOW_UNDO) or MyUser.HasRole('MD MANAGER')
              else
                acUndo.Enabled := False
            else
              acUndo.Enabled := False;
          end;
        esHeld:
          begin
            lblCompletedDateTime.Caption := '';
            lblCompletedBy.Caption := '';
            lblComment.Caption := fEvent.getComment;
            btnAction.Action := acUndo;
            btnAltAction01.Action := acUnHold;
            btnAltAction02.Visible := False;
            pnlComplete.Visible := False;
          end;
        esRefused:
          begin
            btnAction.Action := acUndo;
            btnAltAction01.Visible := False;
            btnAltAction02.Visible := False;
          end;
      end;
      acAudits.Enabled := fEvent.getHasAudits;
      pnlTop.SetFocus;
      ShowModal;
    finally
      Free;
    end;
  result := true;
end;

procedure TfrmKardexViewer.acAuditsExecute(Sender: TObject);
var
  aList: IInterfaceList;
//  i: integer;
//  aAudit: ICPKardexAudit;
begin
  TInterfaceList.Create.GetInterface(IInterfaceList, aList);
  MyCPKardex.LoadAudits(fEvent, aList);
  if aList.Count = 0 then
    MessageDlg508('No audits found for this event.', mtInformation, [mbOk], 0)
  else
    DisplayAuditLog(fEvent, aList, 'Event Audit Log');
end;

procedure TfrmKardexViewer.acCloseExecute(Sender: TObject);
begin
  ModalResult := mrIgnore;
end;

procedure TfrmKardexViewer.acCompleteExecute(Sender: TObject);
begin
  try
    MyCPKardex.UpdateEvent(fEvent, MyUser, trunc(dtpDate.Date) + frac(dtpTime.Time), esCompleted, memComment.Text);
  except
    on E: Exception do
      MessageDlg508(e.Message, mtError, [mbOk], 0);
  end;
  ModalResult := mrOk;
end;

procedure TfrmKardexViewer.acHoldExecute(Sender: TObject);
begin
  if (Length(memComment.Text) > 4) and (Length(memComment.Text) < 251) then
    begin
      MyCPKardex.HoldEvent(fEvent, MyUser, Now, memComment.Text);
      ModalResult := mrOk;
    end
  else
    MessageDlg508('Comment of 5-250 characters is required to palce a Hold on this event.', mtInformation, [mbOk], 0);
end;

procedure TfrmKardexViewer.acUnHoldExecute(Sender: TObject);
begin
  if (Length(memComment.Text) > 4) and (Length(memComment.Text) < 251) then
    begin
      MyCPKardex.UnHoldEvent(fEvent, MyUser, Now, memComment.Text);
      ModalResult := mrOk;
    end
  else
    MessageDlg508('Comment of 5-250 characters is required to palce a Hold on this event.', mtInformation, [mbOk], 0);
end;

procedure TfrmKardexViewer.acOutOfRangeExecute(Sender: TObject);
var
  aMinutes: integer;
begin
  aMinutes := MinutesBetween(Now, fEvent.getScheduledDateTime);
  if aMinutes > WINDOW_COMPLETION then
    if fEvent.getScheduledDateTime < Now then
      chkbxOutOfRange.Caption :=
        Format('This event is overdue by %s, continue?', [getTimeDisplay(aMinutes)])
    else if fEvent.getScheduledDateTime > Now then
      chkbxOutOfRange.Caption :=
        Format('This event is not due for %s, continue?', [getTimeDisplay(aMinutes)]);
  chkbxOutOfRange.Visible := (aMinutes > WINDOW_COMPLETION);
  if chkbxOutOfRange.Visible then
    begin
      lblDateCompleted.Enabled := chkbxOutOfRange.Checked;
      lblCompletedComment.Enabled := chkbxOutOfRange.Checked;
      memComment.Enabled := chkbxOutOfRange.Checked;
      dtpDate.Enabled := chkbxOutOfRange.Checked;
      dtpTime.Enabled := chkbxOutOfRange.Checked;
      acComplete.Enabled := chkbxOutOfRange.Checked;
      acRefused.Enabled := chkbxOutOfRange.Checked;
      acHold.Enabled := chkbxOutOfRange.Checked;
    end;
end;

procedure TfrmKardexViewer.acRefusedExecute(Sender: TObject);
begin
  if memComment.Text = '' then
    MessageDlg508('When marking an event ''Refused'' you must enter in a comment.', mtInformation, [mbOk], 0)
  else
    try
      MyCPKardex.UpdateEvent(fEvent, MyUser, Now, esRefused, memComment.Text);
      ModalResult := mrOK;
    except
      on E: Exception do
        MessageDlg508('Unable to mark event refused.' + #13 + E.Message,
          mtInformation, [mbOk], 0);
    end;
end;

procedure TfrmKardexViewer.acUndoExecute(Sender: TObject);
var
  aComment: string;
begin
  if MessageDlg508('This will undo the completion of this event.' + #13 + #13 +
    'Are you sure you want to do this?', mtConfirmation, mbYesNo, 0) = mrYes then
    if GetComment(aComment, 'Undo Event', 'Undo Reason:') then
      try
        MyCPKardex.UpdateAudit(fEvent, MyUser, 'Undo', aComment);
        MyCPKardex.UpdateEvent(fEvent, nil, 0, esPending, '');
      except
        on E: Exception do
          MessageDlg508(e.Message, mtError, [mbOk], 0);
      end;
  ModalResult := mrOk;
end;

procedure TfrmKardexViewer.FormCreate(Sender: TObject);
begin
  lblCareAction.Font.Style := [fsBold];
  lblSchedule.Font.Style := [fsBold];
  lblDueDateTime.Font.Style := [fsBold];
  lblStatus.Font.Style := [fsBold];
  lblCompletedDateTime.Font.Style := [fsBold];
  lblCompletedBy.Font.Style := [fsBold];
  lblSpecialInstructions.Font.Style := [fsBold];
  lblPatientName.Font.Style := [fsBold];
  lblComment.Font.Style := [fsBold];
end;

procedure TfrmKardexViewer.pnlTopEnter(Sender: TObject);
begin
  ReadPnlTop;
end;

procedure TfrmKardexViewer.ReadPnlTop;
var
  JAWSReadback: String;
begin
  JAWSReadback :=
    Caption + ' for ' +
    GetVerbalPID(MyPatient);
  GetScreenReader.Speak(JAWSReadback);
end;

end.

