unit fNoteWriter;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  oInterfaces,
  uCPXML,
  XMLDoc,
  XMLIntf,
  ExtCtrls,
  ComCtrls;

type
  TfrmNoteWriter = class(TForm)
    Panel1: TPanel;
    memReadOnly: TMemo;
    lblEndGeneratedText: TLabel;
    memUserText: TMemo;
    lblBeginGeneratedText: TLabel;
    Panel2: TPanel;
    lblSigBlockName: TLabel;
    edtESig: TEdit;
    lblES: TLabel;
    btnSign: TButton;
    btnCancel: TButton;
    Shape1: TShape;
    Bevel1: TBevel;
    lblSigBlockTitle: TLabel;
    Panel3: TPanel;
    lblNoteTitle: TLabel;
    Label1: TLabel;
    edtCoSigner: TEdit;
    dtpDateOfNote: TDateTimePicker;
    dtpTimeOfNote: TDateTimePicker;
    Label3: TLabel;
    lblCosigner: TLabel;
    btnCosigner: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
    procedure btnSignClick(Sender: TObject);
    procedure btnCosignerClick(Sender: TObject);
  private
    fCPTIUDocument: ICPTIUDocument;
    fCosigRequired: boolean;
    fCosigID: string;
    fCosigName: string;
  public
    { Public declarations }
  end;

procedure WriteTIUDocument(aCPTIUDocument: ICPTIUDocument; aReadOnlyText: TStrings = nil);

implementation

uses
  fPersonLookup,
  uGlobalVars,
  u508Dialogs;

{$R *.dfm}

procedure WriteTIUDocument(aCPTIUDocument: ICPTIUDocument; aReadOnlyText: TStrings = nil);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  with TfrmNoteWriter.Create(application) do
    try
      aXMLDoc := NewXMLDocument;
      fCPTIUDocument := aCPTIUDocument;

      if Assigned(aReadOnlyText) then
        memReadOnly.Lines.Assign(aReadOnlyText);

      MyVistALogin.ExecQuery('GetNotePrivs', [aCPTIUDocument.getTitleID], aXMLDoc);
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);

      fCosigRequired := (XMLValueAsInt(aXMLRec, 'REQUIRE_COSIGN') = 1);
      fCosigID := XMLValueAsStr(aXMLRec, 'DEF_COSIGN_ID');
      fCosigName := XMLValueAsStr(aXMLRec, 'DEF_COSIGN_NAME');

      lblNoteTitle.Caption := fCPTIUDocument.getTitle;

      lblSigBlockName.Caption := fCPTIUDocument.getUser.getESIGName;
      lblSigBlockTitle.Caption := fCPTIUDocument.getUser.getESIGTitle;

      dtpDateOfNote.DateTime := Now;
      dtpTimeOfNote.DateTime := Now;

      lblCosigner.Visible := fCosigRequired;
      edtCoSigner.Visible := fCosigRequired;
      btnCosigner.Visible := fCosigRequired;

      edtCosigner.Text := fCosigName;
      btnCosigner.Enabled := fCosigRequired;

      {Set to patient location???}
      (*
      if MyUserOptions.getUsePatientLocation then
        cbxLocation.ItemIndex := MyPatientSource.GetLocationIndexByName(cbxLocation.Items, fCPTIUDocument.getPatient.getWardLocation);
      *)

      {If still no location and MyUser has a default, try that}
      (*
      if (MyUserOptions.getDefaultLocationID <> '') and (cbxLocation.ItemIndex < 0) then
        cbxLocation.ItemIndex := MyPatientSource.GetLocationIndexByID(cbxLocation.Items, MyUserOptions.getDefaultLocationID);
      *)
      
      {CQ579 resolved below in btnSignClick() to require a location before filing because it can still be blank at this point}

      ShowModal;
    finally
      aXMLDoc := nil;
      aXMLRec := nil;
      Free;
    end;
end;

procedure TfrmNoteWriter.btnCosignerClick(Sender: TObject);
var
  aNewPerson: IPerson;
begin
  aNewPerson := LookupPerson(MyPersonService, 'Select Co-Signer');
  if Assigned(aNewPerson) then
    begin
      fCosigID := aNewPerson.getID;
      fCosigName := aNewPerson.getName;
      edtCoSigner.Text := fCosigName;
    end;
end;

procedure TfrmNoteWriter.btnSignClick(Sender: TObject);
//var
//  aCPLocation: ICPLocation;
begin
  try
    with fCPTIUDocument do
      begin
        ClearText;
        AppendText(memReadOnly.Lines);
        AppendText('');
        AppendText(memUserText.Lines);

        {
        setLocationID('');

        CQ579 - Location is now a required field drp/2011-07-21 1014
        Will now raise exception to abort the filing if blank

        CQ 579 pt. 2 - We have to actually build a new encounter if we want
        to switch locations.  This is beyond the scope of what we're doing
        right now, so we're going to completely remove the location selector.
        }

        (*
        if cbxLocation.ItemIndex > -1 then
          begin
            if cbxLocation.Items.Objects[cbxLocation.ItemIndex].GetInterface(ICPLocation, aCPLocation) then
              setLocationID(aCPLocation.getID);
          end
        else
          raise Exception.Create('No location provided.');
        *)

        setDateTime(Trunc(dtpDateOfNote.DateTime) + Frac(dtpTimeOfNote.DateTime));

        if edtESig.Text = '' then
          raise Exception.Create('No electronic signature entered.');

        (*
        if getLocationID = '' then
        begin
          setLocationID(getPatient.getWardLocation);
          raise Exception.Create('Note is missing a location.');
        end;
        *)
        
        setCosignerID('');
        if fCosigRequired then
          if fCosigID = '' then
            raise Exception.Create('A Co-Signer is required but none is selected.')
          else
            setCosignerID(fCosigID);

        if not Sign(edtESig.Text) then
          raise Exception.Create('Invalid Electronic Signature Code.');

        MessageDlg508('TIU Document has been signed and filed', mtInformation, [mbOk], 0);

        ModalResult := mrOk;
      end
  except
    on E: Exception do
      begin
        MessageDlg508('Error filing note.' + #13 + e.Message, mtError, [mbOk], 0);
        edtESig.SetFocus;
        edtESig.SelectAll;
      end;
  end;
end;

procedure TfrmNoteWriter.FormCreate(Sender: TObject);
begin
  Font := Screen.IconFont;
  memReadOnly.Color := clInfoBk;
  memUserText.Lines.Clear;
  lblEndGeneratedText.Color := clInfoBk;
  lblSigBlockName.Font.Style := [fsBold];
  lblSigBlockTitle.Font.Style := [fsBold];
  lblES.Font.Style := [fsBold];
  lblNoteTitle.Font.Style := [fsBold];
  edtESig.Text := '';
  edtESig.Font.Style := [fsBold];

  (*
  MyPatientSource.LoadLocations(cbxLocation.Items);
  *)
end;

procedure TfrmNoteWriter.Panel1Resize(Sender: TObject);
begin
  memReadOnly.Height := Trunc(Panel1.Height * 0.4);
end;

end.

