unit fPatientLookup;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 17 $  $Modtime: 5/22/09 2:00p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Temporary Patient Lookup Component
*
*       Notes:        This component will need to be vastly imporved if there
*                     is no movement on a common component for Delhi Apps.
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/forms/fPatientLookup.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ActnList,
  ImgList,
  ComCtrls,
  StdCtrls,
  oDBController_Patient,
  oInterfaces,
  ExtCtrls, VA508AccessibilityManager, VA508ImageListLabeler, System.ImageList,
  System.Actions;

type
  TfrmPatientLookup = class(TForm)
    ActionList1: TActionList;
    acDisplayPatientInfo: TAction;
    acOpen: TAction;
    acCancelLookup: TAction;
    acPerformLookup: TAction;
    acInitializeForm: TAction;
    acCloseForm: TAction;
    acClearList: TAction;
    Timer1: TTimer;
    Panel1: TPanel;
    lblDOB: TLabel;
    Label5: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    lblSelectedPatient: TLabel;
    Label4: TLabel;
    lblGender: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    lblRoomBed: TLabel;
    lblLocation: TLabel;
    btnOpen: TButton;
    btnCancel: TButton;
    lvPatients: TListView;
    imgList: TImageList;
    cbxLookupValue: TComboBox;
    rgTypeOfLookup: TRadioGroup;
    vamOpenPtMgr: TVA508AccessibilityManager;
    vcaTypeOfLookup: TVA508ComponentAccessibility;
    vcaLookupValue: TVA508ComponentAccessibility;
    VA508ImageListLabeler1: TVA508ImageListLabeler;
    lblName: TStaticText;
    lblSSN: TStaticText;
    lblLookup: TLabel;
    procedure acDisplayPatientInfoExecute(Sender: TObject);
    procedure acPerformLookupExecute(Sender: TObject);
    procedure acOpenExecute(Sender: TObject);
    procedure acCancelLookupExecute(Sender: TObject);
    procedure acCloseFormExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure zzedtLookupValueKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure zzedtLookupValueEnter(Sender: TObject);
    procedure lvPatientsGetImageIndex(Sender: TObject; Item: TListItem);
    procedure lvPatientsGetSubItemImage(Sender: TObject; Item: TListItem; SubItem: Integer; var ImageIndex: Integer);
    procedure lvPatientsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure zzedtLookupValueKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure rgTypeOfLookupClick(Sender: TObject);
    procedure cbxLookupValueChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure lvPatientsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  protected
    fPatientController: TDBController_Patient;
    fListOfPatients: IInterfaceList;
    fCurrentPatient: IPatient;
    fKeyDownValue: string;
    fCounter: Integer;
    procedure ResetCounter;
    function getCurrentPatientInfo:String;
  public
    function getCurrentPatient: IPatient;
    function Execute: boolean;
  end;

function NewPatientLookupForm(aPatientController: TDBController_Patient; aDefaultLocationID: string = ''): TfrmPatientLookup;

implementation

uses
  uGlobalVars,
  oCPTypes,
  fRestrictedRecord,
  u508Func,
  VA508AccessibilityRouter,
  u508Dialogs;

{$R *.dfm}
const
  COUNTER_LIMIT = 5;
  SENSITIVE = '*SENSITIVE*';


function NewPatientLookupForm(aPatientController: TDBController_Patient; aDefaultLocationID: string = ''): TfrmPatientLookup;
begin
  Result := TfrmPatientLookup.Create(Application.MainForm);
  with Result do
    begin
      fPatientController := aPatientController;
      if aDefaultLocationID <> '' then
        begin
          rgTypeOfLookup.ItemIndex := 1;
          cbxLookupValue.ItemIndex := MyPatientSource.GetLocationIndexByID(cbxLookupValue.Items, aDefaultLocationID);
          if cbxLookupValue.ItemIndex > -1 then
            acPerformLookup.Execute;
        end;
    end;
end;

procedure TfrmPatientLookup.FormCreate(Sender: TObject);
begin
  Font := Screen.IconFont;
  TInterfaceList.Create.GetInterface(IInterfaceList, fListOfPatients);
  lblSelectedPatient.Font.Style := [fsBold];
  lblName.Font.Style := [fsBold];
  lblName.Caption := '';
  lblDOB.Font.Style := [fsBold];
  lblDOB.Caption := '';
  lblSSN.Font.Style := [fsBold];
  lblSSN.Caption := '';
  lblGender.Font.Style := [fsBold];
  lblGender.Caption := '';
  lblLocation.Font.Style := [fsBold];
  lblLocation.Caption := '';
  lblRoomBed.Font.Style := [fsBold];
  lblRoomBed.Caption := '';
  rgTypeOfLookup.ItemIndex := 0;
end;

function TfrmPatientLookup.getCurrentPatientInfo:String;
begin
  Result := '';
  if Assigned(fCurrentPatient) then
    begin
      Text := FCurrentPatient.getName;
      if fCurrentPatient.getSensitive then
        Text := Text + ' (Sensitive)'
      else
        Text := Text +
          Format('%s %s',[FCurrentPatient.getSSN, FCurrentPatient.getDOB]);
    end
  else
    Result := 'No patient selected';
end;

procedure TfrmPatientLookup.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = char(VK_ESCAPE) then
    ModalResult := mrCancel;
end;

function TfrmPatientLookup.getCurrentPatient: IPatient;
begin
  Result := fCurrentPatient;
end;

procedure TfrmPatientLookup.lvPatientsGetImageIndex(Sender: TObject; Item: TListItem);
var
  aPatient: IPatient;
begin
  if Assigned(Item.Data) then
    if TObject(Item.Data).GetInterface(IPatient, aPatient) then
      if aPatient.getSensitive then
        Item.ImageIndex := 0
      else
        Item.ImageIndex := -1
    else
      Item.ImageIndex := -1
  else
    Item.ImageIndex := -1;
end;

procedure TfrmPatientLookup.lvPatientsGetSubItemImage(Sender: TObject; Item: TListItem; SubItem: Integer; var ImageIndex: Integer);
var
  aPatient: IPatient;
begin
  if SubItem = 0 then
    if Assigned(Item.Data) then
      if TObject(Item.Data).GetInterface(IPatient, aPatient) then
        if aPatient.getSensitive then
          ImageIndex := 0
        else
          ImageIndex := -1
      else
        ImageIndex := -1
    else
      ImageIndex := -1
  else
    ImageIndex := -1;
end;

procedure TfrmPatientLookup.lvPatientsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
    acOpen.Execute;
end;

procedure TfrmPatientLookup.lvPatientsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  fCurrentPatient := nil;
  if Selected then
    if Assigned(Item.Data) then
      TObject(Item.Data).GetInterface(IPatient, fCurrentPatient)
    else
      fCurrentPatient := nil;
  acDisplayPatientInfo.Execute;
end;

procedure TfrmPatientLookup.rgTypeOfLookupClick(Sender: TObject);
begin
  //lblCount.Caption := ''; // zzzzzzandria 2008-07-07

  cbxLookupValue.OnKeyDown := nil;
  cbxLookupValue.OnKeyUp := nil;
  cbxLookupValue.OnEnter := nil;
  cbxLookupValue.OnChange := nil;
  lvPatients.Items.Clear;

  case rgTypeOfLookup.ItemIndex of
    0:
      begin
        Timer1.Enabled := False;
        cbxLookupValue.Style := csSimple;
        cbxLookupValue.Items.Clear;
        cbxLookupValue.Text := '';
        cbxLookupValue.OnKeyDown := zzedtLookupValueKeyDown;
        cbxLookupValue.OnKeyUp := zzedtLookupValueKeyUp;
        cbxLookupValue.OnEnter := zzedtLookupValueEnter;
        cbxLookupValue.Hint :=
          'Enter in a patient by LastName,FirstName or you' + #13 +
          'may enter the last initial and last four of the SSN.';
        lblLookup.Caption := 'Patient Name or Last Initial and Last 4:';
      end;
    1:
      begin
        Timer1.Enabled := False;
        cbxLookupValue.Style := csDropDownList;
        cbxLookupValue.Text := '';
        cbxLookupValue.OnChange := acPerformLookupExecute; //cbxLookupValueChange;
        MyPatientSource.LoadLocations(cbxLookupValue.Items, ltWard);
        cbxLookupValue.Hint :=
          'Select a ward location from this list to display' + #13 +
          'all patients on that ward.';
        lblLookup.Caption := 'Select Ward Location:';
      end;
  end;
end;

procedure TfrmPatientLookup.acDisplayPatientInfoExecute(Sender: TObject);
var
  IDString: string;
begin
  lblName.Caption := '';
  lblSSN.Caption := '';
  lblDOB.Caption := '';
  lblGender.Caption := '';
  lblLocation.Caption := '';
  lblRoomBed.Caption := '';

  if Assigned(fCurrentPatient) then
    begin
      lblName.Caption := fCurrentPatient.getName;
      lblSSN.caption := fCurrentPatient.getSSN;
      lblDOB.Caption := fCurrentPatient.getDateOfBirth;
      lblGender.Caption := fCurrentPatient.getSex;
      lblLocation.Caption := fCurrentPatient.getWardLocation;
      lblRoomBed.Caption := fCurrentPatient.getRoomBed;
      {
      blj 6 November 2008: 508 stuff
      }
      IDString :=
        'Name ' + fCurrentPatient.GetName + 
        ' S S N ' + GetVerbalSSN(fCurrentPatient.getSSN) +
        ' Date of Birth ' + GetVerbalDate(fCurrentPatient.getDateOfBirth);
      if fCurrentPatient.getSex = 'M' then
        IDString := IDString + ' Male '
      else
        IDString := IDString + ' Female ';
      if lblLocation.Caption <> '' then
        IDString := IDString + 'Ward Location ' + lblLocation.Caption + ' '
      else
        IDString := IDString + 'no ward location ';
      if lblRoomBed.Caption <> '' then
        IDString := IDString + 'Room bed' + lblRoomBed.Caption + ' '
      else
        IDString := IDString + 'no room or bed ';

      GetScreenReader.Speak(IDString);
      {
      end 508 stuff.
      }
    end;

  acOpen.Enabled := Assigned(fCurrentPatient);
end;

procedure TfrmPatientLookup.acPerformLookupExecute(Sender: TObject);
var
  aPatient: IPatient;
  aPartial: string;
  i: integer;
begin
  if rgTypeOfLookup.ItemIndex = 0 then
    begin
      inc(fCounter);
      (*
      if fCounter < COUNTER_LIMIT then
        begin
          lblCount.Caption := Format('Start Lookup of <%s> in %d sec',
            [cbxLookupValue.Text,COUNTER_LIMIT - fCounter]); // zzzzzzandria 2008-07-07
          GetScreenReader.Speak(Format('Searching in %d sec', [COUNTER_LIMIT - fCounter]));
          Exit;
        end;
      lblCount.Caption := Format(' Looking for <%s>',[cbxLookupValue.Text]); // zzzzzzandria 2008-07-07
      *)
      ResetCounter;
    end;

  Timer1.Enabled := False;
  lvPatients.Items.Clear;

  case rgTypeOfLookup.ItemIndex of
    0: {Patient List}
      begin
        aPartial := cbxLookupValue.Text;
        if cbxLookupValue.Text <> '' then
          try
            lvPatients.Items.BeginUpdate;
            fPatientController.ExecuteLookupByName(aPartial, fListOfPatients);
            for i := 0 to fListOfPatients.Count - 1 do
              if fListOfPatients[i].QueryInterface(IPatient, aPatient) = 0 then
                aPatient.AddToListView(lvPatients);

            for i := 0 to lvPatients.Items.Count - 1 do
              if AnsiCompareText(aPartial, Copy(lvPatients.Items[i].Caption, 1, Length(aPartial))) = 0 then
                begin
                  lvPatients.Selected := lvPatients.Items[i];
                  lvPatients.Items[i].Selected := True;
                  cbxLookupValue.Text := lvPatients.Items[i].Caption;
                  cbxLookupValue.SelStart := Length(aPartial);
                  cbxLookupValue.SelLength := Length(cbxLookupValue.Text);
                  acDisplayPatientInfo.Execute;
                  Break; {Pops me out of the for loop :)}
                end;
          finally
            lvPatients.Items.EndUpdate;
            cbxLookupValue.SetFocus;
          end;
      end;
    1: {Ward Location}
      begin
        try
          lvPatients.Items.BeginUpdate;
          fPatientController.ExecuteLookupByWard(cbxLookupValue.Text, fListOfPatients);
          for i := 0 to fListOfPatients.Count - 1 do
            if fListOfPatients[i].QueryInterface(IPatient, aPatient) = 0 then
              aPatient.AddToListView(lvPatients);
        finally
          lvPatients.Items.EndUpdate;
          { Removed so that we can perform a ward lookup on execute
          if lvPatients.Items.Count = 0 then
            cbxLookupValue.SetFocus
          else
            lvPatients.SetFocus;
          }
        end;
      end;
  end;
  (*
  if lvPatients.Items.Count = 0 then
    lblCount.Caption := Format('No patients found matching <%s>',[cbxLookupValue.Text])
  else
    lblCount.Caption := Format('%d patients found matching <%s>',
      [lvPatients.Items.Count,cbxLookupValue.Text]); // zzzzzzandria 2008-07-07
  *)
end;

procedure TfrmPatientLookup.cbxLookupValueChange(Sender: TObject);
begin
  ShowMessageFmt(
    'Lookup all patients on ward %s',
    [cbxLookupValue.Items[cbxLookupValue.ItemIndex]]);
end;

procedure TfrmPatientLookup.acOpenExecute(Sender: TObject);
begin
  if fCurrentPatient <> nil then
    try
      if fCurrentPatient.getSensitive then
        if not IsOkayToAccess then
          Exit
        else
          fPatientController.UnLockSensitivePatient(fCurrentPatient, MyUser);
      acCloseForm.Execute;
      ModalResult := mrOK;
    except
      on E: Exception do
        MessageDlg508('Error opening patient record.' + #13 + #13 + 'Error: ' + E.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfrmPatientLookup.acCancelLookupExecute(Sender: TObject);
begin
  acCloseForm.Execute;
  ModalResult := mrCancel;
end;

procedure TfrmPatientLookup.acCloseFormExecute(Sender: TObject);
begin
  fListOfPatients.Clear;
end;

procedure TfrmPatientLookup.FormShow(Sender: TObject);
begin
  BringToFront;
  rgTypeOfLookup.SetFocus;
end;

procedure TfrmPatientLookup.FormActivate(Sender: TObject);
begin
  BringToFront;
  FormShow(Sender);
  rgTypeOfLookup.SetFocus;
end;

procedure TfrmPatientLookup.zzedtLookupValueEnter(Sender: TObject);
begin
  Timer1.Enabled := False;
  cbxLookupValue.SelectAll;
end;

procedure TfrmPatientLookup.zzedtLookupValueKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  fKeyDownValue := cbxLookupValue.Text;
end;

procedure TfrmPatientLookup.zzedtLookupValueKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  {Reset the timer so the delay restarts after each key stroke}
  Timer1.Enabled := False;

  if Key = VK_Down then
    if rgTypeOfLookup.ItemIndex = 0 then
      if lvPatients.Items.Count>0 then
          begin
            lvPatients.SetFocus;
            if lvPatients.ItemIndex = -1  then
              lvPatients.ItemIndex := 0;
            exit;
          end;
  // zzzzzzandria 2008-07-07 ============================================ begin
  if (Key = VK_RETURN) and (Length(cbxLookupValue.Text) > 2) then
    begin
      fCounter := COUNTER_LIMIT;
      acPerformLookup.Execute;
      exit;
    end;
  // zzzzzzandria 2008-07-07 ============================================ end
  (*
  {Enable only if we have >3 chars and the key was [A-Z] or [,] or [-] or [Del] or [Backspace]}
  Timer1.Enabled :=
    (Length(edtLookupValue.Text) > 2) and
    (((Key > 64) and (Key < 91)) or ((Key > 47) and (Key < 58)) or (Key = 188) or (Key = 189) or (Key = 8) or (Key = 48));

  *)
  {Enable only if the length of the value has changed due to this keystroke}
  Timer1.Enabled :=
    (Length(cbxLookupValue.Text) > 2) and
    (cbxLookupValue.Text <> fKeyDownValue);
  // zzzzzzandria 2008-07-07 ============================================ begin
  if Timer1.Enabled then
    begin
      ResetCounter;
      (*
      lblCount.Caption := Format('Start Lookup of <%s> in %d sec',
        [cbxLookupValue.Text,COUNTER_LIMIT - fCounter]); // zzzzzzandria 2008-07-07
        *)
    end
  else
    (*
    lblCount.Caption := '';
    *)
  // zzzzzzandria 2008-07-07 ============================================ end
end;

function TfrmPatientLookup.Execute: boolean;
begin
  Result := (ShowModal = mrOk);
end;

procedure TfrmPatientLookup.ResetCounter;
begin
  fCounter := 0;
end;

end.

