unit oCPFlowsheetPage;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 5 $  $Modtime: 7/29/09 1:50p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Flowsheet Page
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPFlowsheetPage.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  SysUtils,
  XMLIntf;

type
  TCPFlowsheetPage = class(TInterfacedObject, ICPFlowsheetPage)
  private
    [weak] fCPService: ICPService;
    fFlowsheet: ICPFlowsheet;
    fFlowsheetView: ICPFlowsheetView;

    fDisplayName: string;
    fPageType: TCPFlowsheetPageType;
    fPageOrder: integer;
    fSpecialInstructions: string;

    {ICPFlowsheetPage Interface}
    function getDisplayName: string; stdcall;
    function getSpecialInstructions: string; stdcall;
    function getPageType: TCPFlowsheetPageType; stdcall;
    function getPageOrder: integer; stdcall;

    function getFlowsheet: ICPFLowsheet; stdcall;
    function getFlowsheetID: string; stdcall;
    function getFlowsheetName: string; stdcall;

    function getFlowsheetView: ICPFlowsheetView; stdcall;
    function getFlowsheetViewID: string; stdcall;
    function getFlowsheetViewName: string; stdcall;

    procedure setDisplayName(aValue: string); stdcall;
    procedure setSpecialInstructions(aValue: string); stdcall;
    procedure setPageType(aValue: TCPFlowsheetPageType); stdcall;
    procedure setPageOrder(aValue: integer); stdcall;
    procedure setFlowsheetView(aValue: ICPFlowsheetView); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    {Private to this object}
    function LoadFromXML(aXMLNode: IXMLNode): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPFlowsheetPage(aXMLNode: IXMLNode; aCPFlowsheet: ICPFlowsheet; aCPService: ICPService): TCPFlowsheetPage;

implementation

uses
  uCPXML;

function NewCPFlowsheetPage(aXMLNode: IXMLNode; aCPFlowsheet: ICPFlowsheet; aCPService: ICPService): TCPFlowsheetPage;
begin
  Result := TCPFlowsheetPage.Create;
  with Result do
    try
      fCPService := aCPService;
      fFlowsheet := aCPFlowsheet;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode);
    except
      FreeAndNil(Result);
    end;
end;

{------------------------------------------------------------------------------}
{-------------------------- TFlowsheet_Page -----------------------------------}
{------------------------------------------------------------------------------}

constructor TCPFlowsheetPage.Create;
begin
  inherited Create;
  fFlowsheet := nil;
  fFlowsheetView := nil;
  fPageType := ptMandatory;
  fPageOrder := 0;
  fDisplayName := 'New Flowsheet Page';
end;

destructor TCPFlowsheetPage.Destroy;
begin
  fCPService := nil;
  fFlowsheet := nil;
  fFlowsheetView := nil;

  inherited;
end;

{------------------------------------------------------------------------------}
{-------------------------- private methods -----------------------------------}
{------------------------------------------------------------------------------}

procedure TCPFlowsheetPage.AddToList(aList: TStrings);
begin
  aList.AddObject(fDisplayName, Self);
end;

procedure TCPFlowsheetPage.AddToListView(aListView: TListView);
begin
  with aListView.Items.Add do
    begin
      Caption := fDisplayName;
      Data := Self;
    end;
end;

function TCPFlowsheetPage.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TCPFlowsheetPage.getFlowsheet: ICPFLowsheet;
begin
  Result := fFlowsheet;
end;

function TCPFlowsheetPage.getFlowsheetID: string;
begin
  if Assigned(fFlowsheet) then
    Result := fFlowsheet.getID
  else
    Result := '';
end;

function TCPFlowsheetPage.getFlowsheetName: string;
begin
  if Assigned(fFlowsheet) then
    Result := fFlowsheet.getName
  else
    Result := '';
end;

function TCPFlowsheetPage.getPageOrder: integer;
begin
  Result := fPageOrder;
end;

function TCPFlowsheetPage.getPageType: TCPFlowsheetPageType;
begin
  Result := fPageType;
end;

function TCPFlowsheetPage.getSpecialInstructions: string;
begin
  Result := fSpecialInstructions;
end;

function TCPFlowsheetPage.getFlowsheetView: ICPFlowsheetView;
begin
  Result := fFlowsheetView;
end;

function TCPFlowsheetPage.getFlowsheetViewID: string;
begin
  if fFlowsheetView <> nil then
    Result := fFlowsheetView.getID
  else
    Result := '';
end;

function TCPFlowsheetPage.getFlowsheetViewName: string;
begin
  if fFlowsheetView <> nil then
    Result := fFlowsheetView.getName
  else
    Result := '';
end;

function TCPFlowsheetPage.LoadFromXML(aXMLNode: IXMLNode): Boolean;
begin
  with aXMLNode do
    try
      fPageType := TCPFlowsheetPageType(XMLValueAsInt(aXMLNode, 'PAGE_TYPE', 0));
      fPageOrder := XMLValueAsInt(aXMLNode, 'PAGE_ORDER');
      fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
      fSpecialInstructions := XMLValueAsStr(aXMLNode, 'SPECIAL_INSTRUCTIONS');
      fFlowsheetView := fCPService.CPFlowsheetView(XMLValueAsStr(aXMLNode, 'VIEW_ID'));
      Result := True;
    except
      Result := False;
    end;
end;

procedure TCPFlowsheetPage.setDisplayName(aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TCPFlowsheetPage.setPageOrder(aValue: integer);
begin
  fPageOrder := aValue;
end;

procedure TCPFlowsheetPage.setPageType(aValue: TCPFlowsheetPageType);
begin
  fPageType := aValue;
end;

procedure TCPFlowsheetPage.setSpecialInstructions(aValue: string);
begin
  fSpecialInstructions := aValue;
end;

procedure TCPFlowsheetPage.setFlowsheetView(aValue: ICPFlowsheetView);
begin
  fFlowsheetView := aValue;
end;

end.

