unit oCPFlowsheetTotal;

interface

uses
  Classes,
  ComCtrls,
  oCliO,
  oInterfaces,
  oCPTypes,
  uCPXML,
  SysUtils,
  XMLDoc,
  XMLIntf,
  Grids;

type
  TCPFlowsheetTotal = class(TInterfacedObject, ICPConsoleItem, ICPFlowsheetTotal, ICliOReportItem)
  protected
    fTerminology: ITerminology;
    [weak] fCPService: ICPService;
    fIsLoaded: boolean;
    fID: string;
    fDisplayName: string;
    fTerms: IInterfaceList;
    fTotal: Double;
    fCount: integer;
    fMin: Double;
    fMax: Double;
    fDefaultUnit: ITerm;
    fDecimalPrec: integer;
    fUnitAbbv: string;
    fDisplayTotal: boolean;
    fDisplayCount: boolean;
    fDisplayAverage: boolean;
    fDisplayMinimum: boolean;
    fDisplayMaximum: boolean;
    fDisplayRatio: boolean;
    fDisplayDifference: boolean;
    fValues: array of array of double;
    fName: string;
    fActive: boolean;
    fComment: string;

    {ICPConsoleItem}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;

    {ICliOReportItem}
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;

    procedure AppendToGrid(aStringGrid: TStringGrid); stdcall;

    {ICPFlowsheetTotal}
    function getFlowsheetID: string; stdcall;
    function getDefaultUnit: ITerm; stdcall;
    function getDefaultUnitAbbv: string; stdcall;
    function getDefaultUnitID: string; stdcall;
    function getDecimalPrec: integer; stdcall;
    function getDisplayTotal: boolean; stdcall;
    function getDisplayCount: boolean; stdcall;
    function getDisplayAverage: boolean; stdcall;
    function getDisplayMinimum: boolean; stdcall;
    function getDisplayMaximum: boolean; stdcall;
    function getDisplayRatio: boolean; stdcall;
    function getDisplayDifference: boolean; stdcall;
    function getTermCount: integer; stdcall;
    function getTerm(aIndex: integer): ITerm; stdcall;

    procedure setFlowsheetID(const aValue: string); stdcall;
    procedure setDefaultUnit(const aValue: ITerm); stdcall;
    procedure setDecimalPrec(const aValue: integer); stdcall;
    procedure setDisplayTotal(const aValue: boolean); stdcall;
    procedure setDisplayCount(const aValue: boolean); stdcall;
    procedure setDisplayAverage(const aValue: boolean); stdcall;
    procedure setDisplayMinimum(const aValue: boolean); stdcall;
    procedure setDisplayMaximum(const aValue: boolean); stdcall;
    procedure setDisplayRatio(const aValue: boolean); stdcall;
    procedure setDisplayDifference(const aValue: boolean); stdcall;
    procedure ClearAllTerms; stdcall;
    procedure AddTerm(aTerm: ITerm); stdcall;

    function Execute(aList: IInterface): boolean; stdcall;
    function getTotal: Double; stdcall;
    function getCount: integer; stdcall;
    function getAverage: Double; stdcall;
    function getMinimum: Double; stdcall;
    function getMaximum: Double; stdcall;
    function getRatio: string; stdcall;

    function getTotalByTerm(aIndex: integer): Double; stdcall;
    function getCountByTerm(aIndex: integer): integer; stdcall;
    function getAverageByTerm(aIndex: integer): Double; stdcall;
    function getMinimumByTerm(aIndex: integer): Double; stdcall;
    function getMaximumByTerm(aIndex: integer): Double; stdcall;

    {Private internal methods}
    procedure LoadTerms;
    procedure LoadFromXML(aXMLNode: IXMLNode);
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPFlowsheetTotal(aXMLNode: IXMLNOde; aCPService: ICPService; aTerminology: ITerminology): TCPFlowsheetTotal;

implementation

uses
  oCliOTypes;

function NewCPFlowsheetTotal(aXMLNode: IXMLNOde; aCPService: ICPService; aTerminology: ITerminology): TCPFlowsheetTotal;
begin
  Result := TCPFlowsheetTotal.Create;
  with Result do
    try
      fTerminology := aTerminology;
      fCPService := aCPService;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand(
          'AddTotal',
          [fID, fName, fDisplayName, 2, fActive, fComment,
          True, True, False, False, False, False, False]);
    except
      FreeAndNil(Result);
    end;
end;

{ TCPFlowsheetTotal }

constructor TCPFlowsheetTotal.Create;
begin
  inherited Create;
  fID := NewGUID;
  fName := 'NEW FLOWSHEET TOTAL';
  fDisplayName := 'New Flowsheet Total';
  fComment := 'New Flowsheet Total Definition';
  fActive := False;
  TInterfaceList.Create.GetInterface(IInterfaceList, fTerms);
  fDecimalPrec := 1;
end;

procedure TCPFlowsheetTotal.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
//  aTerm: ITerm;
begin
  aXMLDoc := NewXMLDocument;
  fCPService.ExecuteDBQuery('GetTotal', [fID], aXMLDoc);
  aXMLDoc.Active := True;
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  LoadFromXML(aXMLRec);
  LoadTerms;
end;

procedure TCPFlowsheetTotal.LoadTerms;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aTerm: ITerm;
begin
  fTerms.Clear;
  aXMLDoc := NewXMLDocument;
  fCPService.ExecuteDBQuery('GetTotalTerms', [fID], aXMLDoc);
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aTerm := fTerminology.GetTerm(XMLValueAsStr(aXMLRec, 'TERM_ID'));
      fTerms.Add(aTerm);
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;
end;

procedure TCPFlowsheetTotal.Save;
var
  i: integer;
  aTerm: ITerm;
begin
  with fCPService do
    try
      DBTransactionBegin;

      ExecuteDBCommand('UpdateTotal',
        [fID, fDisplayName, getDefaultUnitID, fDecimalPrec, fActive, fComment,
        fDisplayTotal, fDisplayCount, fDisplayAverage, fDisplayDifference,
          fDisplayRatio, fDisplayMinimum, fDisplayMaximum]);

      ExecuteDBCommand('DeleteTotalTerms', [fID]);

      for i := 0 to fTerms.Count - 1 do
        if fTerms[i].QueryInterface(ITerm, aTerm) = 0 then
          ExecuteDBCommand('AddTotalTerm', [fID, aTerm.getID, i + 1]);

      DBTransactionCommit;
    except
      on E: Exception do
        begin
          DBTransactionRollback;
          raise Exception.Create('Unable to save Flowsheet Totals Definition.');
        end;
    end;
end;

procedure TCPFlowsheetTotal.AddTerm(aTerm: ITerm);
begin
  fTerms.Add(aTerm);
end;

procedure TCPFlowsheetTotal.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPFlowsheetTotal.AddToListView(aListView: TListView);
begin
  with aListView.Items.Add do
    begin
      Caption := fDisplayName;
      Data := Self;
    end;
end;

procedure TCPFlowsheetTotal.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  i: integer;
  aDisplayName: string;
  aLine: string;
  aDecFmt: string;
  aDblFmt: string;
  aStrFmt: string;
begin
  Execute(aList);

  aDblFmt := Format('%%9.%df %3s', [fDecimalPrec, fUnitAbbv]);
  aDecFmt := '%12d';
  aStrFmt := '%12s';

  aReportText.Add('');

  aLine := Format('%-' + IntToStr(aMargin - 60) + 's', [getDisplayName]);

  if not getActive then
    begin
      aReportText.Add('   Total: ' + aLine);
      aReportText.Add('   +-----------------------------------------------+');
      aReportText.Add('   | This total as heen marked inactive and cannot |');
      aReportText.Add('   | be displayed on reports at this time.         |');
      aReportText.Add('   +-----------------------------------------------+');
      Exit;
    end;

  if fDisplayTotal then
    aLine := aLine + Format('%12s', ['Total'])
  else
    aLine := aLine + Format('%12s', ['']);

  if fDisplayCount then
    aLine := aLine + Format('%12s', ['Count'])
  else
    aLine := aLine + Format('%12s', ['']);

  if fDisplayAverage then
    aLine := aLine + Format('%12s', ['Average'])
  else
    aLine := aLine + Format('%12s', ['']);

  if fDisplayMinimum then
    aLine := aLine + Format('%12s', ['Minimum'])
  else
    aLine := aLine + Format('%12s', ['']);

  if fDisplayMaximum then
    aLine := aLine + Format('%12s', ['Maximum'])
  else
    aLine := aLine + Format('%12s', ['']);

  aReportText.Add(aLine);

  if True then {Display individual terms?}
    begin
      aReportText.Add(StringOfChar('-', aMargin));

      for i := 0 to getTermCount - 1 do
        begin
          aDisplayName := getTerm(i).getDisplayName;

          if Length(aDisplayName) > (aMargin - 60) then
            aDisplayName := Copy(aDisplayName, 1, aMargin - 63) + '...';

          aLine := Format('%-' + IntToStr(aMargin - 60) + 's', [aDisplayName]);

          if fDisplayTotal then
            aLine := aLine + Format(aDblFmt, [getTotalByTerm(i)])
          else
            aLine := aLine + Format(aStrFmt, ['']);

          if fDisplayCount then
            aLine := aLine + Format(aDecFmt, [getCountByTerm(i)])
          else
            aLine := aLine + Format(aStrFmt, ['']);

          if fDisplayAverage then
            if getCountByTerm(i) > 0 then
              aLine := aLine + Format(aDblFmt, [getAverageByTerm(i)])
            else
              aLine := aLine + Format(aStrFmt, ['***'])
          else
            aLine := aLine + Format(aStrFmt, ['']);

          if fDisplayMinimum then
            if getCountByTerm(i) > 0 then
              aLine := aLine + Format(aDblFmt, [getMinimumByTerm(i)])
            else
              aLine := aLine + Format(aStrFmt, ['***'])
          else
            aLine := aLine + Format(aStrFmt, ['']);

          if fDisplayMaximum then
            if getCountByTerm(i) > 0 then
              aLine := aLine + Format(aDblFmt, [getMaximumByTerm(i)])
            else
              aLine := aLine + Format(aStrFmt, ['***'])
          else
            aLine := aLine + Format(aStrFmt, ['']);

          aReportText.Add(aLine);
        end;

      aLine := Format('%-' + IntToStr(aMargin - 60) + 's', ['']);

      if fDisplayTotal then
        aLine := aLine + Format(aStrFmt, ['----------']);

      if fDisplayCount then
        aLine := aLine + Format(aStrFmt, ['----------']);

      if fDisplayAverage then
        aLine := aLine + Format(aStrFmt, ['----------']);

      aReportText.Add(aLine);
    end;

  aLine := Format('%-' + IntToStr(aMargin - 60) + 's', ['']);

  if fDisplayTotal then
    aLine := aLine + Format(aDblFmt, [getTotal])
  else
    aLine := aLine + Format(aStrFmt, ['']);

  if fDisplayCount then
    aLine := aLine + Format(aDecFmt, [getCount])
  else
    aLine := aLine + Format(aStrFmt, ['']);

  if fDisplayAverage then
    if getCount > 0 then
      aLine := aLine + Format(aDblFmt, [getAverage])
    else
      aLine := aLine + Format(aStrFmt, ['***']);

  aReportText.Add(aLine);

  aReportText.Add('');
end;

function TCPFlowsheetTotal.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

procedure TCPFlowsheetTotal.AppendToGrid(aStringGrid: TStringGrid);
var
  aRow: integer;
  aCol: integer;
  i: integer;
  aTerm: ITerm;
begin
  with aStringGrid do
    begin
      ColWidths[0] := 180;
      aCol := 1;
      if fDisplayTotal then
        begin
          Cells[aCol, 0] := 'Total';
          inc(aCol);
        end;
      if fDisplayCount then
        begin
          Cells[aCol, 0] := 'Count';
          ColWidths[aCol] := 60;
          inc(aCol);
        end;
      if fDisplayAverage then
        begin
          Cells[aCol, 0] := 'Average';
          inc(aCol);
        end;
      if fDisplayMinimum then
        begin
          Cells[aCol, 0] := 'Minimum';
          inc(aCol);
        end;
      if fDisplayMaximum then
        begin
          Cells[aCol, 0] := 'Maximum';
          inc(aCol);
        end;
      //ColWidths[0] := 180;
      //ColWidths[2] := 60;
      ColCount := aCol;

      aRow := RowCount - 1;
      RowCount := RowCount + fTerms.Count;
      for i := 0 to fTerms.Count - 1 do
        begin
          fTerms[i].QueryInterface(ITerm, aTerm);
          Cells[0, aRow] := aTerm.getDisplayName;
          aCol := 1;

          if fDisplayTotal then
            begin
              Cells[aCol, aRow] := Format('%0.2f %s', [getTotalByTerm(i), fUnitAbbv]);
              inc(aCol);
            end;

          if fDisplayCount then
            begin
              Cells[aCol, aRow] := Format('%d', [getCountByTerm(i)]);
              inc(aCol);
            end;

          if fDisplayAverage then
            if getCountByTerm(i) > 0 then
              begin
                Cells[aCol, aRow] := Format('%0.2f %s', [getAverageByTerm(i), fUnitAbbv]);
                inc(aCol);
              end
            else
              begin
                Cells[aCol, aRow] := Format('%s', ['***']);
                inc(aCol);
              end;

          if fDisplayMinimum then
            if getCountByTerm(i) > 0 then
              begin
                Cells[aCol, aRow] := Format('%0.2f %s', [getMinimumByTerm(i), fUnitAbbv]);
                inc(aCol);
              end
            else
              begin
                Cells[aCol, aRow] := Format('%s', ['***']);
                inc(aCol);
              end;

          if fDisplayMaximum then
            if getCountByTerm(i) > 0 then
              begin
                Cells[aCol, aRow] := Format('%0.2f %s', [getMaximumByTerm(i), fUnitAbbv]);
//                inc(aCol);
              end
            else
              begin
                Cells[aCol, aRow] := Format('%s', ['***']);
//                inc(aCol);
              end;

          {
          if getCountByTerm(i) > 0 then
            begin
              Cells[3, aRow] := Format('%0.2f %s', [getAverageByTerm(i), fUnitAbbv]);
              Cells[4, aRow] := Format('%0.2f %s', [getMinimumByTerm(i), fUnitAbbv]);
              Cells[5, aRow] := Format('%0.2f %s', [getMaximumByTerm(i), fUnitAbbv]);
            end
          else
            begin
              Cells[3, aRow] := Format('%s', ['***']);
              Cells[4, aRow] := Format('%s', ['***']);
              Cells[5, aRow] := Format('%s', ['***']);
            end;
          }
          inc(aRow);
        end;
      Cells[0, aRow] := '   - Total';
      Cells[1, aRow] := Format('%0.2f %s', [getTotal, fUnitAbbv]);
      Cells[2, aRow] := Format('%d', [getCount]);
      if getCount > 0 then
        Cells[3, aRow] := Format('%0.2f %s', [getAverage, fUnitAbbv])
      else
        Cells[3, aRow] := Format('%s', ['***']);
      Cells[4, aRow] := ''; //Format('%0.2f %s', [getMinimum, fUnitAbbv]);
      Cells[5, aRow] := ''; //Format('%0.2f %s', [getMaximum, fUnitAbbv]);
    end;
end;

procedure TCPFlowsheetTotal.ClearAllTerms;
begin
  fTerms.Clear;
end;

destructor TCPFlowsheetTotal.Destroy;
begin
  fTerms.Clear;
  fTerms := nil;
  fTerminology := nil;
  fCPService := nil;
  inherited;
end;

function TCPFlowsheetTotal.Execute(aList: IInterface): boolean;
var
  aCliOList: ICliOList;
  aObs: IObservation;
  aTerm: ITerm;
//  aTermID: string;
  i: integer;
  aMsg: string;
  aVal: Double;
begin
  result := false;
  {Initialize}
  SetLength(fValues, fTerms.Count, 4);
  fTotal := 0;
  fCount := 0;
  fMin := 0;
  fMax := 0;
  for i := Low(fValues) to High(fValues) do
    begin
      fValues[i, 0] := 0;
      fValues[i, 1] := 0;
      fValues[i, 2] := 0;
      fValues[i, 3] := 0;
    end;

  {Get the list and get to counting}
  if aList.QueryInterface(ICliOList, aCliOList) <> 0 then
  begin
    result := false;
    Exit;
  end
  else
    begin
      aObs := aCliOList.FirstObservation;
      while Assigned(aObs) do
        try
          if aObs.getStatus = osVerified then
            begin
              aTerm := aObs.getTerm(ttObservation);
              i := fTerms.IndexOf(aTerm);
              if i > -1 then
                begin
                  aVal := fTerminology.GetConvertedValue(aObs.getNumericValue, aObs.getObsUnit, fDefaultUnit);
                  fValues[i, 0] := fValues[i, 0] + aVal;
                  fValues[i, 1] := fValues[i, 1] + 1;
                  if fValues[i, 1] = 1 then {First one, set max min Min/Max}
                    begin
                      fValues[i, 2] := aVal;
                      fValues[i, 3] := aVal;
                    end
                  else {Compare Min/Max}
                    begin
                      if aVal < fValues[i, 2] then
                        fValues[i, 2] := aVal; {New Min}
                      if aVal > fValues[i, 3] then
                        fValues[i, 3] := aVal; {New Max}
                    end;
                  fTotal := fTotal + aVal;
                  fCount := fCount + 1;
                  if fCount = 1 then
                    begin
                      fMin := aVal;
                      fMax := aVal;
                    end
                  else
                    begin
                      if aVal < fMin then
                        fMin := aVal;
                      if aVal > fMax then
                        fMax := aVal;
                    end;
                end;
            end;
          aObs := aCliOList.NextObservation(aObs);
        except
          if Assigned(aObs.getObsUnit) and Assigned(fDefaultUnit) then
            aMsg := Format('Error executing total ''%s''. No conversion exists for %s to %s',
              [fDisplayName, aObs.getObsUnit.getTerm, fDefaultUnit.getTerm])
          else
            aMsg := 'Error executing total ' + fDisplayName + '. Check for missing unit conversions.';
          raise Exception.Create(aMsg);
        end;
    end;
end;

function TCPFlowsheetTotal.getActive: boolean;
begin
  Result := fActive;
end;

function TCPFlowsheetTotal.getAverage: Double;
begin
  if fCount > 0 then
    Result := fTotal / fCount
  else
    Result := 0;
end;

function TCPFlowsheetTotal.getCanDelete: boolean;
begin
  Result := True;
end;

function TCPFlowsheetTotal.getCanRename: boolean;
begin
  Result := True;
end;

function TCPFlowsheetTotal.getComment: string;
begin
  Result := fComment;
end;

function TCPFlowsheetTotal.getCount: integer;
begin
  Result := fCount;
end;

function TCPFlowsheetTotal.getTotal: Double;
begin
  Result := fTotal;
end;

function TCPFlowsheetTotal.getAverageByTerm(aIndex: integer): Double;
begin
  if fValues[aIndex, 1] > 0 then
    Result := fValues[aIndex, 0] / fValues[aIndex, 1]
  else
    Result := 0;
end;

function TCPFlowsheetTotal.getCountByTerm(aIndex: integer): integer;
begin
  Result := trunc(fValues[aIndex, 1]);
end;

function TCPFlowsheetTotal.getTotalByTerm(aIndex: integer): Double;
begin
  Result := fValues[aIndex, 0];
end;

function TCPFlowsheetTotal.getMaximum: Double;
begin
  Result := fMax;
end;

function TCPFlowsheetTotal.getMaximumByTerm(aIndex: integer): Double;
begin
  Result := fValues[aIndex, 3]
end;

function TCPFlowsheetTotal.getMinimum: Double;
begin
  Result := fMin;
end;

function TCPFlowsheetTotal.getMinimumByTerm(aIndex: integer): Double;
begin
  Result := fValues[aIndex, 2]
end;

function TCPFlowsheetTotal.getDecimalPrec: integer;
begin
  Result := fDecimalPrec;
end;

function TCPFlowsheetTotal.getDefaultUnit: ITerm;
begin
  Result := fDefaultUnit;
end;

function TCPFlowsheetTotal.getDefaultUnitAbbv: string;
begin
  if Assigned(fDefaultUnit) then
    Result := fDefaultUnit.getAbbreviation
  else
    Result := '';
end;

function TCPFlowsheetTotal.getDefaultUnitID: string;
begin
  if Assigned(fDefaultUnit) then
    Result := fDefaultUnit.getID
  else
    Result := '';
end;

function TCPFlowsheetTotal.getDisplayAverage: boolean;
begin
  Result := fDisplayAverage;
end;

function TCPFlowsheetTotal.getDisplayCount: boolean;
begin
  Result := fDisplayCount;
end;

function TCPFlowsheetTotal.getDisplayDifference: boolean;
begin
  Result := fDisplayDifference;
end;

function TCPFlowsheetTotal.getDisplayMaximum: boolean;
begin
  Result := fDisplayMaximum;
end;

function TCPFlowsheetTotal.getDisplayMinimum: boolean;
begin
  Result := fDisplayMinimum;
end;

function TCPFlowsheetTotal.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TCPFlowsheetTotal.getDisplayRatio: boolean;
begin
  Result := fDisplayRatio;
end;

function TCPFlowsheetTotal.getDisplayTotal: boolean;
begin
  Result := fDisplayTotal;
end;

function TCPFlowsheetTotal.getFlowsheetID: string;
begin
  Result := fID;
end;

function TCPFlowsheetTotal.getID: string;
begin
  Result := fID;
end;

function TCPFlowsheetTotal.getIsLoaded: boolean;
begin
  Result := fIsLoaded;
end;

function TCPFlowsheetTotal.getItemType: TCPItemType;
begin
  Result := ciFlowsheetTotal;
end;

function TCPFlowsheetTotal.getName: string;
begin
  Result := fName;
end;

function TCPFlowsheetTotal.getRatio: string;
begin
  Result := '?/?';
end;

function TCPFlowsheetTotal.getTerm(aIndex: integer): ITerm;
begin
  try
    fTerms[aIndex].QueryInterface(ITerm, Result);
  except
    Result := nil;
  end;
end;

function TCPFlowsheetTotal.getTermCount: integer;
begin
  Result := fTerms.Count;
end;

procedure TCPFlowsheetTotal.LoadFromXML(aXMLNode: IXMLNode);
begin
  fID := XMLValueAsStr(aXMLNode, 'ID');
  fName := XMLValueAsStr(aXMLNode, 'NAME');
  fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
  fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
  fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
  fDisplayTotal := (XMLValueAsInt(aXMLNode, 'DISPLAY_TOTAL', 0) = 1);
  fDisplayCount := (XMLValueAsInt(aXMLNode, 'DISPLAY_COUNT', 0) = 1);
  fDisplayAverage := (XMLValueAsInt(aXMLNode, 'DISPLAY_AVERAGE', 0) = 1);
  fDisplayRatio := (XMLValueAsInt(aXMLNode, 'DISPLAY_RATIO', 0) = 1);
  fDisplayDifference := (XMLValueAsInt(aXMLNode, 'DISPLAY_DIFFERENCE', 0) = 1);
  fDefaultUnit := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_UNIT'));
  fDecimalPrec := XMLValueAsInt(aXMLNode, 'DECIMAL_PRECISION', 0);
  fDisplayMinimum := (XMLValueAsInt(aXMLNode, 'DISPLAY_MINIMUM', 0) = 1);
  fDisplayMaximum := (XMLValueAsInt(aXMLNode, 'DISPLAY_MAXIMUM', 0) = 1);

  if Assigned(fDefaultUnit) then
    fUnitAbbv := Format('(%s)', [fDefaultUnit.getAbbreviation])
  else
    fUnitAbbv := '';
end;

procedure TCPFlowsheetTotal.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TCPFlowsheetTotal.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TCPFlowsheetTotal.setDecimalPrec(const aValue: integer);
begin
  fDecimalPrec := aValue;
end;

procedure TCPFlowsheetTotal.setDefaultUnit(const aValue: ITerm);
begin
  fDefaultUnit := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayAverage(const aValue: boolean);
begin
  fDisplayAverage := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayCount(const aValue: boolean);
begin
  fDisplayCount := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayDifference(const aValue: boolean);
begin
  fDisplayDifference := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayMaximum(const aValue: boolean);
begin
  fDisplayMaximum := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayMinimum(const aValue: boolean);
begin
  fDisplayMinimum := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayRatio(const aValue: boolean);
begin
  fDisplayRatio := aValue;
end;

procedure TCPFlowsheetTotal.setDisplayTotal(const aValue: boolean);
begin
  fDisplayTotal := aValue;
end;

procedure TCPFlowsheetTotal.setFlowsheetID(const aValue: string);
begin
  fID := aValue;
end;

procedure TCPFlowsheetTotal.setName(const aValue: string);
begin
  fName := aValue;
end;

end.

