unit oCPFlowsheetViewTerm;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 9 $  $Modtime: 1/07/11 11:58a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Defines a terms methods within a view.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPFlowsheetViewTerm.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  SysUtils,
  oCliOTypes,
  oInterfaces,
  Variants,
  XMLIntf,
  XMLDoc,
  iCliOGrid; {Used in the display frame}

type
  TCPFlowsheetViewTerm = class(TInterfacedObject, ICPFlowsheetViewTerm, IGridTerm)
  protected
    fAttributesAllowed: array[TTermType] of boolean;
    fAttributesMandatory: array[TTermType] of boolean;
    fAttributesID: array[TTermType] of string;

    fDisplayOrder: integer;
    fDisplayName: string;
    fDisplayWidth: Integer;
    fDisplayTotal: boolean;
    fDisplayCount: boolean;
    fDisplayAverage: boolean;
    fDisplayMinimum: boolean;
    fDisplayMaximum: boolean;
    fDisplayInDropdown: boolean;
    fRequiredTerm: boolean;
    fDisplayOnly: boolean;
    fDisplayColumns: integer;
    fDefaultSetID: string;
    fDisplayInCellTotal: boolean;

    function getAttributeAllowed(Index: TTermType): boolean; stdcall;
    function getAttributeMandatory(Index: TTermType): boolean; stdcall;
    function getAttributeID(Index: TTermType): string; stdcall;

    function getDisplayOrder: integer; stdcall;
    function getDisplayName: string; stdcall;
    function getDisplayWidth: integer; stdcall;
    function getDisplayColumns: integer; stdcall;
    function getDisplayTotal: boolean; stdcall;
    function getDisplayCount: boolean; stdcall;
    function getDisplayAverage: boolean; stdcall;
    function getDisplayOnly: boolean; stdcall;
    function getDefaultSetID: string; stdcall;
    function getDisplayInCellTotal: boolean; stdcall;
    function getDisplayMinValue: boolean; stdcall;
    function getDisplayMaxValue: boolean; stdcall;
    function getRequiredTerm: boolean; stdcall;
    function getDisplayInDropdown: boolean; stdcall;

    procedure setAttribute(aTermType: TTermType; aAllowed: boolean; aMandatory: boolean; aID: string); stdcall;
    procedure setDisplayOrder(const aValue: integer); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setDisplayWidth(const aValue: integer); stdcall;
    procedure setDisplayColumns(const aValue: integer); stdcall;
    procedure setDisplayTotal(const aValue: boolean); stdcall;
    procedure setDisplayCount(const aValue: boolean); stdcall;
    procedure setDisplayAverage(const aValue: boolean); stdcall;
    procedure setDisplayOnly(const aValue: boolean); stdcall;
    procedure setDisplayInCellTotal(const aValue: boolean); stdcall;
    procedure setDisplayMinValue(const aValue: boolean); stdcall;
    procedure setDisplayMaxValue(const aValue: boolean); stdcall;
    procedure setRequiredTerm(const aValue: boolean); stdcall;
    procedure setDisplayInDropdown(const aValue: boolean); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    function LoadFromXML(aXMLNode: IXMLNode): boolean;
    function ContainsObservation(aObservation: IObservation): boolean; stdcall;
    function GetAllAttributes: string; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPFlowsheetViewTerm(aXMLNode: IXMLNode; aDefaultSetID: string): TCPFlowsheetViewTerm;

implementation

uses
  uGlobalVars, {Need access to terminology to set a viewterm to read-only if it's inactive drp - 12/29/2010}
  uCPXML;

function NewCPFlowsheetViewTerm(aXMLNode: IXMLNode; aDefaultSetID: string): TCPFlowsheetViewTerm;
begin
  Result := TCPFlowsheetViewTerm.Create;
  with Result do
    try
      fDefaultSetID := aDefaultSetID;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode);
    except
      FreeAndNil(Result);
    end;
end;

{ TObservation_View_Term }

constructor TCPFlowsheetViewTerm.Create;
var
  aTermType: TTermType;
begin
  inherited Create;
  for aTermType := Low(TTermType) to High(TTermType) do
    begin
      fAttributesAllowed[aTermType] := False;
      fAttributesMandatory[aTermType] := False;
      fAttributesID[aTermType] := '';
    end;
  fDisplayInCellTotal := False;
end;

destructor TCPFlowsheetViewTerm.Destroy;
begin
  inherited;
end;

function TCPFlowsheetViewTerm.LoadFromXML(aXMLNode: IXMLNode): boolean;
var
  aTerm: ITerm;
begin
  try
    fDisplayOrder := XMLValueAsInt(aXMLNode, 'TERM_ORDER');
    fDisplayWidth := XMLValueAsInt(aXMLNode, 'DISPLAY_WIDTH');
    fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
    fDisplayTotal := (XMLValueAsInt(aXMLNode, 'DISPLAY_TOTAL', 0) = 1);
    fDisplayCount := (XMLValueAsInt(aXMLNode, 'DISPLAY_COUNT', 0) = 1);
    fDisplayAverage := (XMLValueAsInt(aXMLNode, 'DISPLAY_AVERAGE', 0) = 1);
    fDisplayOnly := (XMLValueAsInt(aXMLNode, 'DISPLAY_ONLY', 0) = 1);
    fDisplayColumns := XMLValueAsInt(aXMLNode, 'DISPLAY_COLUMNS', 0);
    fDisplayInCellTotal := (XMLValueAsInt(aXMLNode, 'DISPLAY_IN_CELL_TOTAL', 0) = 1);
    fDisplayMinimum := (XMLValueAsInt(aXMLNode, 'DISPLAY_MINIMUM', 0) = 1);
    fDisplayMaximum := (XMLValueAsInt(aXMLNode, 'DISPLAY_MAXIMUM', 0) = 1);
    fRequiredTerm := (XMLValueAsInt(aXMLNode, 'REQUIRED_TERM', 0) = 1);
    fDisplayInDropDown := (XMLValueAsInt(aXMLNode, 'USE_DROPDOWN', 0) = 1);
    setAttribute(ttObservation, True, True, XMLValueAsStr(aXMLNode, 'TERM_ID'));
    {Check to see if term is inactive and set Display Only accordingly drp/12/29/2010}
    if Assigned(MyTerminology) then // this thing is in uGLobalvars so this guy sees it from there when compiled, flowsheets is fine, console has it's own local copy.
      if MyTerminology.GetTerm(getAttributeID(ttObservation)).QueryInterface(ITerm, aTerm) = 0 then
        if not aTerm.getActive then
          fDisplayOnly := True;
    Result := True;
  except
    Result := False;
  end;
end;

procedure TCPFlowsheetViewTerm.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

function TCPFlowsheetViewTerm.ContainsObservation(aObservation: IObservation): boolean;
begin
  Result := False;
  if getAttributeID(ttObservation) = aObservation.getTermID(ttObservation) then
    begin {You are on the right track, you at least have a matching observation}
      if (not getAttributeMandatory(ttMethod)) or (getAttributeID(ttMethod) = aObservation.getTermID(ttMethod)) then
        if (not getAttributeMandatory(ttPosition)) or (getAttributeID(ttPosition) = aObservation.getTermID(ttPosition)) then
          if (not getAttributeMandatory(ttLocation)) or (getAttributeID(ttLocation) = aObservation.getTermID(ttLocation)) then
            if (not getAttributeMandatory(ttQuality)) or (getAttributeID(ttQuality) = aObservation.getTermID(ttQuality)) then
              if (not getAttributeMandatory(ttProduct)) or (getAttributeID(ttProduct) = aObservation.getTermID(ttProduct)) then
                if (not getAttributeMandatory(ttNullFlavor)) or (getAttributeID(ttNullFlavor) = aObservation.getTermID(ttNullFlavor)) then
                  Result := True; {if you are here, everything matches up -- Woo Hoo!}
    end;
end;

{ Getters and Setters }

function TCPFlowsheetViewTerm.getAttributeAllowed(Index: TTermType): boolean;
begin
  Result := fAttributesAllowed[Index];
end;

function TCPFlowsheetViewTerm.getAttributeMandatory(Index: TTermType): boolean;
begin
  Result := fAttributesMandatory[Index];
end;

function TCPFlowsheetViewTerm.getAttributeID(Index: TTermType): string;
begin
  Result := fAttributesID[Index];
end;

function TCPFlowsheetViewTerm.getDisplayAverage: boolean;
begin
  Result := fDisplayAverage;
end;

function TCPFlowsheetViewTerm.getDisplayColumns: integer;
begin
  Result := fDisplayColumns;
end;

function TCPFlowsheetViewTerm.getDisplayCount: boolean;
begin
  Result := fDisplayCount;
end;

function TCPFlowsheetViewTerm.getDisplayInCellTotal: boolean;
begin
  Result := fDisplayInCellTotal;
end;

function TCPFlowsheetViewTerm.getDisplayInDropdown: boolean;
begin
  Result := fDisplayInDropdown;
end;

function TCPFlowsheetViewTerm.getDisplayMaxValue: boolean;
begin
  Result := fDisplayMaximum;
end;

function TCPFlowsheetViewTerm.getDisplayMinValue: boolean;
begin
  Result := fDisplayMinimum;
end;

function TCPFlowsheetViewTerm.getDisplayTotal: boolean;
begin
  Result := fDisplayTotal;
end;

function TCPFlowsheetViewTerm.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TCPFlowsheetViewTerm.getDisplayOnly: boolean;
begin
  Result := fDisplayOnly;
end;

function TCPFlowsheetViewTerm.getDisplayOrder: integer;
begin
  Result := fDisplayOrder;
end;

function TCPFlowsheetViewTerm.getDefaultSetID: string;
begin
  Result := fDefaultSetID;
end;

function TCPFlowsheetViewTerm.getDisplayWidth: integer;
begin
  Result := fDisplayWidth;
end;

function TCPFlowsheetViewTerm.getRequiredTerm: boolean;
begin
  Result := fRequiredTerm;
end;

procedure TCPFlowsheetViewTerm.setAttribute(aTermType: TTermType; aAllowed, aMandatory: boolean; aID: string);
begin
  fAttributesAllowed[aTermType] := aAllowed;
  fAttributesMandatory[aTermType] := aMandatory;
  fAttributesID[aTermType] := aID;
end;

procedure TCPFlowsheetViewTerm.setDisplayAverage(const aValue: boolean);
begin
  fDisplayAverage := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayColumns(const aValue: integer);
begin
  fDisplayColumns := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayCount(const aValue: boolean);
begin
  fDisplayCount := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayInCellTotal(const aValue: boolean);
begin
  fDisplayInCellTotal := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayInDropdown(const aValue: boolean);
begin
  fDisplayInDropdown := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayMaxValue(const aValue: boolean);
begin
  fDisplayMaximum := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayMinValue(const aValue: boolean);
begin
  fDisplayMinimum := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayOnly(const aValue: boolean);
begin
  fDisplayOnly := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayOrder(const aValue: integer);
begin
  fDisplayOrder := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayTotal(const aValue: boolean);
begin
  fDisplayTotal := aValue;
end;

procedure TCPFlowsheetViewTerm.setDisplayWidth(const aValue: integer);
begin
  fDisplayWidth := aValue;
end;

procedure TCPFlowsheetViewTerm.setRequiredTerm(const aValue: boolean);
begin
  fRequiredTerm := aValue;
end;

{------------------------------- DEBUG METHOD ---------------------------------}

function TCPFlowsheetViewTerm.GetAllAttributes: string;
var
  tt: TTermType;
const
  B2STR: array[boolean] of string = ('False', 'True');
begin
  with TStringList.Create do
    try
      for tt := Low(TTermType) to High(TTermType) do
        begin
          Add(Format('Attribute: %s', [TERMTYPE[tt]]));
          Add(Format('    Allowed: %s', [B2STR[fAttributesAllowed[tt]]]));
          Add(Format('    Term: %s', [fAttributesID[tt]]));
          Add(Format('    Mandatory: %s', [B2STR[fAttributesMandatory[tt]]]));
          Add('-----------------------------------------------');
          Add('');
        end;
      Result := Text;
    finally
      Free;
    end;
end;

procedure TCPFlowsheetViewTerm.AddToList(aList: TStrings);
begin
  aList.AddObject(fDisplayName, Self);
end;

end.

