unit oCPInstrument;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 7 $  $Modtime: 10/19/11 2:59p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  CP Instrument object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oCPInstrument.pas $
================================================================================
}
interface

uses
  Dialogs,
  Classes,
  ComCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  XMLDoc,
  XMLIntf;

type
  TCPInstrument = class(TInterfacedObject, ICPConsoleItem, ICPConsoleCopy, ICPInstrument)
  protected
    //fDBConnection: IDBConnection;
    [weak] fCPService: ICPService;
    fIsLoaded: boolean;
    fActive: boolean;
    fComment: string;
    fName: string;
    fID: string;

    fPrintableName: string;
    fSerialNumber: string;
    fNotificationMailgroup: string;
    fNotificationMailgroupID: string;
    fDeleteWhenSubmitted: boolean;
    fDefaultFileExtension: string;
    fProcessingRoutine: string;
    fProcessingCode: string;
    fProcessUNC: boolean;
    fProcessText: boolean;
    fProcessURL: boolean;
    fProcessDLL: boolean;
    fProcessUUEncode: boolean;
    fProcessXML: boolean;
    fProcessXMS: boolean;
    fBiDirectional: boolean;
    fIPAddress: string;
    fPort: string;
    fHL7InstrumentID: string;
    fHL7UniversalServiceID: string;
    fHL7LogicalLink: string;
    fHL7LogicalLinkID: string;

    {ICPConsole Interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: Boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function Delete: boolean; stdcall;

    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
    procedure CopyFrom(aID: string); stdcall;

    {ICPInstrument Interface}
    function getPrintableName: string; stdcall;
    function getSerialNumber: string; stdcall;
    function getNotificationMailgroup: string; stdcall;
    function getNotificationMailgroupID: string; stdcall;
    function getDeleteWhenSubmitted: boolean; stdcall;
    function getDefaultFileExtension: string; stdcall;
    function getProcessingRoutine: string; stdcall;
    function getProcessingCode: string; stdcall;
    function getProcessUNC: boolean; stdcall;
    function getProcessText: boolean; stdcall;
    function getProcessURL: boolean; stdcall;
    function getProcessDLL: boolean; stdcall;
    function getProcessUUEncode: boolean; stdcall;
    function getProcessXML: boolean; stdcall;
    function getProcessXMS: boolean; stdcall;
    function getBiDirectional: boolean; stdcall;
    function getIPAddress: string; stdcall;
    function getPort: string; stdcall;
    function getHL7InstrumentID: string; stdcall;
    function getHL7UniversalServiceID: string; stdcall;
    function getHL7LogicalLink: string; stdcall;
    function getHL7LogicalLinkID: string; stdcall;

    procedure setPrintableName(const aValue: string); stdcall;
    procedure setSerialNumber(const aValue: string); stdcall;
    procedure setNotificationMailgroup(const aValue: string); stdcall;
    procedure setNotificationMailgroupID(const aValue: string); stdcall;
    procedure setDeleteWhenSubmitted(const aValue: boolean); stdcall;
    procedure setDefaultFileExtension(const aValue: string); stdcall;
    procedure setProcessingRoutine(const aValue: string); stdcall;
    procedure setProcessingCode(const aValue: string); stdcall;
    procedure setProcessUNC(const aValue: boolean); stdcall;
    procedure setProcessText(const aValue: boolean); stdcall;
    procedure setProcessURL(const aValue: boolean); stdcall;
    procedure setProcessDLL(const aValue: boolean); stdcall;
    procedure setProcessUUEncode(const aValue: boolean); stdcall;
    procedure setProcessXML(const aValue: boolean); stdcall;
    procedure setProcessXMS(const aValue: boolean); stdcall;
    procedure setBiDirectional(const aValue: boolean); stdcall;
    procedure setIPAddress(const aValue: string); stdcall;
    procedure setPort(const aValue: string); stdcall;
    procedure setHL7InstrumentID(const aValue: string); stdcall;
    procedure setHL7UniversalServiceID(const aValue: string); stdcall;
    procedure setHL7LogicalLink(const aValue: string); stdcall;
    procedure setHL7LogicalLinkID(const aValue: string); stdcall;

    {Private stuff}
    procedure LoadFromXML(aXMLNode: IXMLNode);
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPInstrument(aXMLNode: IXMLNode; aCPService: ICPService): TCPInstrument;

implementation

uses
  uCPXML,
  oCP_Exceptions,
  Variants,
  SysUtils;

function NewCPInstrument(aXMLNode: IXMLNode; aCPService: ICPService): TCPInstrument;
begin
  Result := TCPInstrument.Create;
  with Result do
    try
      fCPService := aCPService;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand(
          'AddInstrument',
          [fID, fName, fComment, fActive]);
    except
      FreeAndNil(Result);
    end;
end;

{ TCPInstrument }

procedure TCPInstrument.CopyFrom(aID: string);
var
  aOriginalID: string;
  aOriginalName: string;
begin
  aOriginalID := fID;
  aOriginalName := fName;

  fID := aID;
  try
    Load;
    fID := aOriginalID;
    fName := aOriginalName;
    Save;
  except
    raise ECPCommandFail.Create('Error creating copy');
  end;
end;

constructor TCPInstrument.Create;
begin
  inherited Create;
  fIsLoaded := False;
  fID := NewGUID;
  fName := 'NEW INSTRUMENT';
  fComment := 'New Instrument';
  fActive := False;
  fPrintableName := '';
  fSerialNumber := '';
  fNotificationMailgroup := '';
  fNotificationMailgroupID := '';
  fDeleteWhenSubmitted := False;
  fDefaultFileExtension := '';
  fProcessingRoutine := '';
  fProcessingCode := '';
  fProcessUNC := False;
  fProcessText := False;
  fProcessURL := False;
  fProcessDLL := False;
  fProcessUUEncode := False;
  fProcessXML := False;
  fProcessXMS := False;
  fBiDirectional := False;
  fIPAddress := '';
  fPort := '';
  fHL7InstrumentID := '';
  fHL7UniversalServiceID := '';
  fHL7LogicalLink := '';
  fHL7LogicalLinkID := '';
end;

function TCPInstrument.Delete: boolean;
begin
  Result := False;
end;

destructor TCPInstrument.Destroy;
begin
  fCPService := nil;
  inherited;
end;

procedure TCPInstrument.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPInstrument.AddToListView(aListView: TListView);
begin
  with aListView.Items.Add do
    begin
      Caption := fName;
      Data := Self;
    end;
end;

function TCPInstrument.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TCPInstrument.getActive: Boolean;
begin
  Result := fActive;
end;

function TCPInstrument.getBiDirectional: boolean;
begin
  Result := fBiDirectional
end;

function TCPInstrument.getCanDelete: boolean;
begin
  Result := False;
end;

function TCPInstrument.getCanRename: boolean;
begin
  Result := True;
end;

function TCPInstrument.getComment: string;
begin
  Result := fComment;
end;

function TCPInstrument.getName: string;
begin
  Result := fName;
end;

function TCPInstrument.getNotificationMailgroup: string;
begin
  Result := fNotificationMailgroup;
end;

function TCPInstrument.getNotificationMailgroupID: string;
begin
  Result := fNotificationMailgroupID;
end;

function TCPInstrument.getPort: string;
begin
  Result := fPort;
end;

function TCPInstrument.getPrintableName: string;
begin
  Result := fPrintableName;
end;

function TCPInstrument.getProcessDLL: boolean;
begin
  Result := fProcessDLL;
end;

function TCPInstrument.getProcessingCode: string;
begin
  Result := fProcessingCode;
end;

function TCPInstrument.getProcessingRoutine: string;
begin
  Result := fProcessingRoutine;
end;

function TCPInstrument.getProcessText: boolean;
begin
  Result := fProcessText;
end;

function TCPInstrument.getProcessUNC: boolean;
begin
  Result := fProcessUNC;
end;

function TCPInstrument.getProcessURL: boolean;
begin
  Result := fProcessURL
end;

function TCPInstrument.getProcessUUEncode: boolean;
begin
  Result := fProcessUUEncode;
end;

function TCPInstrument.getProcessXML: boolean;
begin
  Result := fProcessXML;
end;

function TCPInstrument.getProcessXMS: boolean;
begin
  Result := fProcessXMS;
end;

function TCPInstrument.getSerialNumber: string;
begin
  Result := fSerialNumber;
end;

function TCPInstrument.getDefaultFileExtension: string;
begin
  Result := fDefaultFileExtension;
end;

function TCPInstrument.getDeleteWhenSubmitted: boolean;
begin
  Result := fDeleteWhenSubmitted;
end;

procedure TCPInstrument.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  try
    fIsLoaded := False;
    aXMLDoc := NewXMLDocument;

    //if not fDBConnection.ExecQuery('GetInstrument', [fID], aXMLDoc) then
    if not fCPService.ExecuteDBQuery('GetInstrument', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Error loading instrument ID: ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    if Assigned(aXMLRec) then
      LoadFromXML(aXMLRec)
    else
      raise ECPLoadFailure.CreateFmt('No record returned for instrument ''%s''', [fID]);

    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Exception ''%s''', [e.Message]);
  end;
end;

procedure TCPInstrument.LoadFromXML(aXMLNode: IXMLNode);
begin
  try
    fID := XMLValueAsStr(aXMLNode, 'ID');
    fName := XMLValueAsStr(aXMLNode, 'NAME');
    fComment := XMLValueAsStr(aXMLNode, 'DESCRIPTION');
    fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
    fPrintableName := XMLValueAsStr(aXMLNode, 'PRINTABLE_NAME');
    fSerialNumber := XMLValueAsStr(aXMLNode, 'SERIAL_NUMBER');
    fNotificationMailgroup := XMLValueAsStr(aXMLNode, 'NOTIFICATION_MAILGROUP');
    fNotificationMailgroupID := XMLValueAsStr(aXMLNode, 'NOTIFICATION_MAILGROUP_ID');
    fDeleteWhenSubmitted := (XMLValueAsInt(aXMLNode, 'DELETE_WHEN_SUBMITTED') = 1);
    fDefaultFileExtension := XMLValueAsStr(aXMLNode, 'DEFAULT_FILE_EXT');
    fProcessingRoutine := XMLValueAsStr(aXMLNode, 'PROCESSING_ROUTINE');
    fProcessingCode := XMLValueAsStr(aXMLNode, 'PROCESSING_CODE');
    fProcessUNC := (XMLValueAsInt(aXMLNode, 'PROCESS_UNC') = 1);
    fProcessText := (XMLValueAsInt(aXMLNode, 'PROCESS_TEXT') = 1);
    fProcessURL := (XMLValueAsInt(aXMLNode, 'PROCESS_URL') = 1);
    fProcessDLL := (XMLValueAsInt(aXMLNode, 'PROCESS_DLL') = 1);
    fProcessUUEncode := (XMLValueAsInt(aXMLNode, 'PROCESS_UUENCODE') = 1);
    fProcessXML := (XMLValueAsInt(aXMLNode, 'PROCESS_XML') = 1);
    fProcessXMS := (XMLValueAsInt(aXMLNode, 'PROCESS_XMS') = 1);
    fBiDirectional := (XMLValueAsInt(aXMLNode, 'BI_DIRECTIONAL') = 1);
    fIPAddress := XMLValueAsStr(aXMLNode, 'IP_ADDRESS');
    fPort := XMLValueAsStr(aXMLNode, 'PORT');
    fHL7InstrumentID := XMLValueAsStr(aXMLNode, 'HL7_INSTRUMENT_ID');
    fHL7UniversalServiceID := XMLValueAsStr(aXMLNode, 'HL7_UNIVERSAL_SERVICE_ID');
    fHL7LogicalLink := XMLValueAsStr(aXMLNode, 'HL7_LOGICAL_LINK');
    fHL7LogicalLinkID := XMLValueAsStr(aXMLNode, 'HL7_LOGICAL_LINK_ID');
  except
    on E: Exception do
      raise ECPLoadFailure.Create(
        'Error loading instrument definition:' + #13 +
        aXMLNode.XML + #13 + #13 +
        'Message: ' + e.Message);
  end;
end;

procedure TCPInstrument.Save;
begin
  try
    fIsLoaded := False;
    if not fCPService.ExecuteDBCommand('UpdateInstrument',
      [fID,
      fComment,
        fPrintableName,
        CP_YN[fActive],
        fSerialNumber,
        fNotificationMailgroupID,
        CP_YN[fDeleteWhenSubmitted],
        fDefaultFileExtension,
        fProcessingRoutine,
        fProcessingCode,
        CP_YN[fProcessUNC],
        CP_YN[fProcessText],
        CP_YN[fProcessURL],
        CP_YN[fProcessDLL],
        CP_YN[fProcessUUEncode],
        CP_YN[fProcessXML],
        CP_YN[fProcessXMS],
        CP_YN[fBiDirectional],
        fIPAddress,
        fPort,
        fHL7InstrumentID,
        fHL7UniversalServiceID,
        fHL7LogicalLinkID]) then
      raise ECPSaveFailure.CreateFmt('Command UpdateInstrument failed for ID ''%s''', [fID]);
    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Expection ''%s''', [e.Message]);
  end;
end;

function TCPInstrument.getDisplayName: string;
begin
  Result := fName;
end;

function TCPInstrument.getHL7InstrumentID: string;
begin
  Result := fHL7InstrumentID
end;

function TCPInstrument.getHL7LogicalLink: string;
begin
  Result := fHL7LogicalLink;
end;

function TCPInstrument.getHL7LogicalLinkID: string;
begin
  Result := fHL7LogicalLinkID;
end;

function TCPInstrument.getHL7UniversalServiceID: string;
begin
  Result := fHL7UniversalServiceID;
end;

function TCPInstrument.getID: string;
begin
  Result := fID;
end;

function TCPInstrument.getIPAddress: string;
begin
  Result := fIPAddress;
end;

function TCPInstrument.getIsLoaded: boolean;
begin
  Result := False;
end;

function TCPInstrument.getItemType: TCPItemType;
begin
  Result := ciInstrument;
end;

procedure TCPInstrument.setActive(const aValue: Boolean);
begin
  fActive := aValue;
end;

procedure TCPInstrument.setBiDirectional(const aValue: boolean);
begin
  fBiDirectional := aValue;
end;

procedure TCPInstrument.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TCPInstrument.setDefaultFileExtension(const aValue: string);
begin
  fDefaultFileExtension := aValue;
end;

procedure TCPInstrument.setDeleteWhenSubmitted(const aValue: boolean);
begin
  fDeleteWhenSubmitted := aValue;
end;

procedure TCPInstrument.setDisplayName(const aValue: string);
begin
  //fName := aValue;
end;

procedure TCPInstrument.setHL7InstrumentID(const aValue: string);
begin
  fHL7InstrumentID := aValue;
end;

procedure TCPInstrument.setHL7LogicalLink(const aValue: string);
begin
  fHL7LogicalLink := aValue;
end;

procedure TCPInstrument.setHL7LogicalLinkID(const aValue: string);
begin
  fHL7LogicalLinkID := aValue;
end;

procedure TCPInstrument.setHL7UniversalServiceID(const aValue: string);
begin
  fHL7UniversalServiceID := aValue;
end;

procedure TCPInstrument.setIPAddress(const aValue: string);
begin
  fIPAddress := aValue;
end;

procedure TCPInstrument.setName(const aValue: string);
begin
  fName := aValue;
end;

procedure TCPInstrument.setNotificationMailgroup(const aValue: string);
begin
  fNotificationMailgroup := aValue;
end;

procedure TCPInstrument.setNotificationMailgroupID(const aValue: string);
begin
  fNotificationMailgroupID := aValue;
end;

procedure TCPInstrument.setPort(const aValue: string);
begin
  fPort := aValue;
end;

procedure TCPInstrument.setPrintableName(const aValue: string);
begin
  fPrintableName := aValue;
end;

procedure TCPInstrument.setProcessDLL(const aValue: boolean);
begin
  fProcessDLL := aValue;
end;

procedure TCPInstrument.setProcessingCode(const aValue: string);
begin
  fProcessingCode := aValue;
end;

procedure TCPInstrument.setProcessingRoutine(const aValue: string);
begin
  fProcessingRoutine := aValue;
end;

procedure TCPInstrument.setProcessText(const aValue: boolean);
begin
  fProcessText := aValue;
end;

procedure TCPInstrument.setProcessUNC(const aValue: boolean);
begin
  fProcessUNC := aValue;
end;

procedure TCPInstrument.setProcessURL(const aValue: boolean);
begin
  fProcessURL := aValue;
end;

procedure TCPInstrument.setProcessUUEncode(const aValue: boolean);
begin
  fProcessUUEncode := aValue;
end;

procedure TCPInstrument.setProcessXML(const aValue: boolean);
begin
  fProcessXML := aValue;
end;

procedure TCPInstrument.setProcessXMS(const aValue: boolean);
begin
  fProcessXMS := aValue;
end;

procedure TCPInstrument.setSerialNumber(const aValue: string);
begin
  fSerialNumber := aValue;
end;

end.

