unit oCPLocation;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 2 $  $Modtime: 4/08/08 9:57a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  CP Location object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPLocation.pas $
================================================================================
}
interface

uses
  Dialogs,
  Classes,
  ComCtrls,
  oInterfaces,
  oCPTypes,
  XMLDoc,
  XMLIntf;

type
  TCPLocation = class(TInterfacedObject, ICPLocation)
  private
    fID: string;
    fName: string;
    fType: TCPLocationType;
    fDivisionID: string;
    fDivisionNumber: string;
    fDivisionName: string;

    function getID: string; stdcall;
    function getType: TCPLocationType; stdcall;
    function getName: string; stdcall;
    function getDivisionID: string; stdcall;
    function getDivisionNumber: string; stdcall;
    function getDivisionName: string; stdcall;
    procedure AddToList(aList: TStrings); stdcall;

    procedure LoadFromXML(aXMLNode: IXMLNode);
  public
    constructor Create(aXMLNode: IXMLNode);
    destructor Destroy; override;
  end;

implementation

uses
  uCPXML,
  oCP_Exceptions,
  Variants,
  SysUtils;

{ TCPLocation }

constructor TCPLocation.Create(aXMLNode: IXMLNode);
begin
  inherited Create;
  LoadFromXML(aXMLNode);
end;

destructor TCPLocation.Destroy;
begin
  inherited;
end;

procedure TCPLocation.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

function TCPLocation.getDivisionID: string;
begin
  Result := fDivisionID;
end;

function TCPLocation.getDivisionName: string;
begin
  Result := fDivisionName;
end;

function TCPLocation.getDivisionNumber: string;
begin
  Result := fDivisionNumber;
end;

function TCPLocation.getID: string;
begin
  Result := fID;
end;

function TCPLocation.getName: string;
begin
  Result := fName;
end;

function TCPLocation.getType: TCPLocationType;
begin
  Result := fType;
end;

procedure TCPLocation.LoadFromXML(aXMLNode: IXMLNode);
var
  aType: TCPLocationType;
begin
  try
    fID := XMLValueAsStr(aXMLNode, 'LOCATION_ID');
    fName := XMLValueAsStr(aXMLNode, 'LOCATION_NAME');
    fDivisionID := XMLValueAsStr(aXMLNode, 'DIVISION_ID');
    fDivisionNumber := XMLValueAsStr(aXMLNode, 'DIVISION_NUMBER');
    fDivisionName := XMLValueAsStr(aXMLNode, 'DIVISION_NAME');

    for aType := High(TCPLocationType) downto Low(TCPLocationType) do
      if CP_LOCATION_TYPE_ABBV[aType] = XMLValueAsStr(aXMLNode, 'LOCATION_TYPE') then
        begin
          fType := aType;
          Break;
        end;
  except
    raise ECPLoadFailure.Create('Error loading location');
  end;
end;

end.

