unit oCPPatient;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 8 $  $Modtime: 11/01/11 9:24p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default CP Patient Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oCPPatient.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  XMLIntf,
  oInterfaces,
  SysUtils;

type
  TCPPatient = class(TInterfacedObject, IPatient, ISensitivePatient)
  private
    fAge: Integer;
    fDFN: string;
    fDOB: TDateTime;
    fICN: string;
    fName: string;
    fRace: string;
    fRoomBed: string;
    fSex: string;
    fSSN: string;
    fWardLocation: string;
    fSensitive: boolean;
    fLastMessageID: integer;
    function getAge: Integer; stdcall;
    function getDFN: string; stdcall;
    function getDOB: TDateTime; stdcall;
    function getDateOfBirth: string; stdcall;
    function getICN: string; stdcall;
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getRace: string; stdcall;
    function getRoomBed: string; stdcall;
    function getSex: string; stdcall;
    function getSSN: string; stdcall;
    function getWardLocation: string; stdcall;
    function getSensitive: boolean; stdcall;
    function getDisplayLine(aLine: integer): string; stdcall;
    function getLastMessageID: integer; stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;

    //I know this breaks encapsulation a LITTLE bit, but it's either this or go commit
    // major surgery on a half a dozen script runner calls.
    procedure setLastMessageID(const AID: integer); stdcall;
    procedure UnLock; stdcall;

  public
    procedure LoadFromXML(aXMLNode: IXMLNode);
  end;

function NewCPTestPatient: TCPPatient;

implementation

uses
  uCPXML,
  Variants;

function NewCPTestPatient: TCPPatient;
begin
  Result := TCPPatient.Create;
  with Result do
    try
      fDFN := '-1';
      fICN := '-1';
      fName := 'PATIENT,TEST';
      fSex := 'M';
      fSSN := '666000000';
      fDOB := Now - 29;
      fAge := trunc((Now - fDOB) / 365.25);
      fWardLocation := '1-4C MICU';
      fRoomBed := '18';
    except
      Result := nil;
    end;
end;

{ TCPPatient }

{
****************************************************** TCPPatient ******************************************************
}

procedure TCPPatient.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

function TCPPatient.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  Result.Caption := getName;
  Result.Data := Self;
end;

function TCPPatient.getAge: Integer;
begin
  Result := fAge;
end;

function TCPPatient.getDFN: string;
begin
  Result := fDFN;
end;

function TCPPatient.getDisplayLine(aLine: integer): string;
begin
  case aLine of
    1: Result := Format('%s - %s (Age: %d)', [fName, fSex, fAge]);
    2: Result := Format('Date Of Birth: %s  SSN: (%s)', [FormatDateTime('mmm d, yyyy', fDOB), Copy(fSSN, 6, 4)]);
    3:
      begin
        if fWardLocation <> '' then
          Result := Format('Ward: %s', [fWardLocation])
        else
          Result := 'Ward: No Ward Location';

        if fRoomBed <> '' then
          Result := Format('%s  Room-Bed: %s', [Result, fRoomBed])
        else
          Result := Format('%s  Room-Bed: %s', [Result, 'No Room Bed']);
      end;
  else
    Result := '';
  end;
end;

function TCPPatient.getDateOfBirth: string;
begin
  if fSensitive then
    Result := '*SENSITIVE*'
  else
    Result := FormatDateTime('mm/dd/yyyy', fDOB) + Format(' (%d)', [getAge]);
end;

function TCPPatient.getDOB: TDateTime;
begin
  Result := fDOB;
end;

function TCPPatient.getICN: string;
begin
  Result := fICN;
end;

function TCPPatient.getID: string;
begin
  Result := Format('`%s', [fDFN]);
end;

function TCPPatient.getLastMessageID: integer;
begin
  result := fLastMessageID;
end;

function TCPPatient.getName: string;
begin
  Result := fName;
end;

function TCPPatient.getRace: string;
begin
  Result := fRace;
end;

function TCPPatient.getRoomBed: string;
begin
  Result := fRoomBed;
end;

function TCPPatient.getSensitive: boolean;
begin
  Result := fSensitive;
end;

function TCPPatient.getSex: string;
begin
  Result := fSex;
end;

function TCPPatient.getSSN: string;
begin
  if fSensitive then
    Result := '*SENSITIVE*'
  else
    Result := fSSN;
end;

function TCPPatient.getWardLocation: string;
begin
  Result := fWardLocation;
end;

procedure TCPPatient.LoadFromXML(aXMLNode: IXMLNode);
{Private method to load a single instance}
begin
  fDFN := XMLValueAsStr(aXMLNode, 'DFN');
  fICN := XMLValueAsStr(aXMLNode, 'ICN');
  fName := XMLValueAsStr(aXMLNode, 'NAME');
  fRace := XMLValueAsStr(aXMLNode, 'RACE');
  fSex := XMLValueAsStr(aXMLNode, 'SEX');
  fDOB := XMLValueAsDateTime(aXMLNode, 'DOB');
  fAge := XMLValueAsInt(aXMLNode, 'AGE');
  fWardLocation := XMLValueAsStr(aXMLNode, 'WARD_LOCATION');
  fRoomBed := XMLValueAsStr(aXMLNode, 'ROOM_BED');
  fSSN := XMLValueAsStr(aXMLNode, 'SSN');
  fSensitive := (XMLValueAsInt(aXMLNode, 'SENSITIVE', 0) = 1);
end;

procedure TCPPatient.setLastMessageID(const AID: integer);
begin
  fLastMessageID := AID;
end;

procedure TCPPatient.UnLock;
begin
  fSensitive := False;
end;

end.

