unit oCPProcedure;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 6 $  $Modtime: 10/19/11 3:26p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default Procedure Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oCPProcedure.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  SysUtils,
  oCPTypes,
  oInterfaces,
  XMLDoc,
  XMLIntf;

type
  TCPProcedure = class(TInterfacedObject, ICPConsoleCopy, ICPConsoleItem, ICPProcedure)
  protected
    fIsLoaded: Boolean;
    fInstruments: TStringList;
    [weak] fCPService: ICPService;
    fID: string;
    fActive: Boolean;
    fName: string;
    fTreatingSpecialty: string;
    fTreatingSpecialtyID: string;
    fHospitalLocation: string;
    fHospitalLocationID: string;
    fDefaultTIUNote: string;
    fDefaultTIUNoteID: string;
    fDescription: string;
    fRequireExternalData: boolean;
    fAutoSubmit: boolean;
    fHighVolume: boolean;
    fProcessingApplication: TCPProcessingApplication;
    fProcessedResult: TCPProcessedResult;
    fExternalDataDirectory: string;

    {ICPConsole Interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: Boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function Delete: boolean; stdcall;

    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
    procedure CopyFrom(aID: string); stdcall;

    {ICPProcedure Interface}
    function getTreatingSpecialty: string; stdcall;
    function getTreatingSpecialtyID: string; stdcall;
    function getHospitalLocation: string; stdcall;
    function getHospitalLocationID: string; stdcall;
    function getDefaultTIUNote: string; stdcall;
    function getDefaultTIUNoteID: string; stdcall;
    function getDescription: string; stdcall;
    function getRequireExternalData: boolean; stdcall;
    function getAutoSubmit: boolean; stdcall;
    function getHighVolume: boolean; stdcall;
    function getProcessingApplication: TCPProcessingApplication; stdcall;
    function getProcessedResult: TCPProcessedResult; stdcall;
    function getExternalDataDirectory: string; stdcall;

    procedure setTreatingSpecialty(const aValue: string); stdcall;
    procedure setTreatingSpecialtyID(const aValue: string); stdcall;
    procedure setHospitalLocation(const aValue: string); stdcall;
    procedure setHospitalLocationID(const aValue: string); stdcall;
    procedure setDefaultTIUNote(const aValue: string); stdcall;
    procedure setDefaultTIUNoteID(const aValue: string); stdcall;
    procedure setDescription(const aValue: string); stdcall;
    procedure setRequireExternalData(const aValue: boolean); stdcall;
    procedure setAutoSubmit(const aValue: boolean); stdcall;
    procedure setHighVolume(const aValue: boolean); stdcall;
    procedure setProcessingApplication(const aValue: TCPProcessingApplication); stdcall;
    procedure setProcessedResult(const aValue: TCPProcessedResult); stdcall;
    procedure setExternalDataDirectory(const aValue: string); stdcall;

    function getInstrumentID(aIndex: integer): string; stdcall;
    function getInstrumentIDCount: integer; stdcall;
    function getInstrumentAssigned(aID: string): boolean; stdcall;
    procedure AddInstrument(const aInstrumentID: string); stdcall;
    procedure ClearInstruments; stdcall;

    {Private}
    procedure LoadFromXML(aXMLNode: IXMLNode);
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPProcedure(aXMLNode: IXMLNode; aCPService: ICPService): TCPProcedure;

implementation

uses
  uCPXML,
  oCP_Exceptions,
  Variants;

function NewCPProcedure(aXMLNode: IXMLNode; aCPService: ICPService): TCPProcedure;
begin
  Result := TCPProcedure.Create;
  with Result do
    try
      fCPService := aCPService;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand('AddProcedure', [fID, fName, CP_YN[fActive], fDescription]);
    except
      FreeAndNil(Result);
    end;
end;

{ TCPProcedure }

procedure TCPProcedure.ClearInstruments;
begin
  fInstruments.Clear;
end;

procedure TCPProcedure.CopyFrom(aID: string);
var
  aOriginalID: string;
  aOriginalName: string;
begin
  aOriginalID := fID;
  aOriginalName := fName;

  fID := aID;
  try
    Load;
    fID := aOriginalID;
    fName := aOriginalName;
    Save;
  except
    raise ECPCommandFail.Create('Error creating copy');
  end;
end;

constructor TCPProcedure.Create;
begin
  inherited Create;
  fID := NewGUID;
  fName := 'NEW PROCEDURE';
  fDescription := 'New Procedure';
  fActive := False;
  fInstruments := TStringList.Create;
  fIsLoaded := False;
  fTreatingSpecialty := '';
  fTreatingSpecialtyID := '';
  fHospitalLocation := '';
  fHospitalLocationID := '';
  fDefaultTIUNote := '';
  fDefaultTIUNoteID := '';
  fRequireExternalData := False;
  fAutoSubmit := False;
  fHighVolume := False;
  fProcessingApplication := paDefault;
  fProcessedResult := prFinal;
  fExternalDataDirectory := '';
end;

function TCPProcedure.Delete: boolean;
begin
  Result := False;
end;

destructor TCPProcedure.Destroy;
begin
  FreeAndNil(fInstruments);
  fCPService := nil;
  inherited;
end;

procedure TCPProcedure.AddInstrument(const aInstrumentID: string);
begin
  if fInstruments.IndexOf(aInstrumentID) < 0 then
    fInstruments.Add(aInstrumentID);
end;

procedure TCPProcedure.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPProcedure.AddToListView(aListView: TListView);
begin
  {
  with aListView.Items.Add do
    begin
    end;
  }
end;

function TCPProcedure.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TCPProcedure.getActive: Boolean;
begin
  Result := fActive;
end;

function TCPProcedure.getAutoSubmit: boolean;
begin
  Result := fAutoSubmit;
end;

function TCPProcedure.getCanDelete: boolean;
begin
  Result := False;
end;

function TCPProcedure.getCanRename: boolean;
begin
  Result := True;
end;

function TCPProcedure.getComment: string;
begin
  Result := fDescription;
end;

function TCPProcedure.getName: string;
begin
  Result := fName;
end;

function TCPProcedure.getProcessedResult: TCPProcessedResult;
begin
  Result := fProcessedResult;
end;

function TCPProcedure.getProcessingApplication: TCPProcessingApplication;
begin
  Result := fProcessingApplication;
end;

function TCPProcedure.getRequireExternalData: boolean;
begin
  Result := fRequireExternalData;
end;

function TCPProcedure.getTreatingSpecialty: string;
begin
  Result := fTreatingSpecialty;
end;

function TCPProcedure.getTreatingSpecialtyID: string;
begin
  Result := fTreatingSpecialtyID;
end;

function TCPProcedure.getDefaultTIUNote: string;
begin
  Result := fDefaultTIUNote;
end;

function TCPProcedure.getDefaultTIUNoteID: string;
begin
  Result := fDefaultTIUNoteID;
end;

function TCPProcedure.getDescription: string;
begin
  Result := fDescription;
end;

function TCPProcedure.getDisplayName: string;
begin
  Result := fName;
end;

function TCPProcedure.getExternalDataDirectory: string;
begin
  Result := fExternalDataDirectory;
end;

function TCPProcedure.getHighVolume: boolean;
begin
  Result := fHighVolume;
end;

function TCPProcedure.getHospitalLocation: string;
begin
  Result := fHospitalLocation;
end;

function TCPProcedure.getHospitalLocationID: string;
begin
  Result := fHospitalLocationID;
end;

function TCPProcedure.getID: string;
begin
  Result := fID;
end;

function TCPProcedure.getInstrumentAssigned(aID: string): boolean;
begin
  Result := (fInstruments.IndexOf(aID) > -1);
end;

function TCPProcedure.getInstrumentID(aIndex: integer): string;
begin
  try
    Result := fInstruments[aIndex]
  except
    Result := '';
  end;
end;

function TCPProcedure.getInstrumentIDCount: integer;
begin
  Result := fInstruments.Count;
end;

function TCPProcedure.getIsLoaded: boolean;
begin
  Result := False;
end;

function TCPProcedure.getItemType: TCPItemType;
begin
  Result := ciProcedure;
end;

procedure TCPProcedure.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  try
    fIsLoaded := False;
    fInstruments.Clear;
    aXMLDoc := NewXMLDocument;

    if not fCPService.ExecuteDBQuery('GetProcedure', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Query ''GetProcedure'' failed for ID ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    if Assigned(aXMLRec) then
      LoadFromXML(aXMLRec)
    else
      raise ECPLoadFailure.CreateFmt('No record returned for procedure ''%s''', [fID]);

    if not fCPService.ExecuteDBQuery('GetProcedureInstrumentList', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Query ''GetProcedureInstrumentList'' failed for ID ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    while Assigned(aXMLRec) do
      try
        fInstruments.Add(XMLValueAsStr(aXMLRec, 'ID'));
      finally
        aXMLRec := XMLDocumentNextRecord(aXMLRec);
      end;

    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Expection ''%s''', [e.Message]);
  end;
end;

procedure TCPProcedure.LoadFromXML(aXMLNode: IXMLNode);
begin
  try
    fID := XMLValueAsStr(aXMLNode, 'ID');
    fName := XMLValueAsStr(aXMLNode, 'NAME');
    fDescription := XMLValueAsStr(aXMLNode, 'DESCRIPTION');
    fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
    fTreatingSpecialty := XMLValueAsStr(aXMLNode, 'TREATING_SPECIALTY');
    fTreatingSpecialtyID := XMLValueAsStr(aXMLNode, 'TREATING_SPECIALTY_ID');
    fHospitalLocation := XMLValueAsStr(aXMLNode, 'HOSPITAL_LOCATION');
    fHospitalLocationID := XMLValueAsStr(aXMLNode, 'HOSPITAL_LOCATION_ID');
    fDefaultTIUNote := XMLValueAsStr(aXMLNode, 'DEFAULT_TIU_NOTE');
    // DRM - I9914178FY16/584942 - Correct tag name
    //fDefaultTIUNoteID := XMLValueAsStr(aXMLNode, 'DEFAULT_TIU_NOTE');
    fDefaultTIUNoteID := XMLValueAsStr(aXMLNode, 'DEFAULT_TIU_NOTE_ID');
    // DRM - I9914178FY16/584942 ---
    fDescription := XMLValueAsStr(aXMLNode, 'DESCRIPTION');
    fRequireExternalData := (XMLValueAsInt(aXMLNode, 'REQUIRE_EXTERNAL_DATA') = 1);
    fAutoSubmit := (XMLValueAsInt(aXMLNode, 'AUTO_SUBMIT') = 1);
    fHighVolume := (XMLValueAsInt(aXMLNode, 'HIGH_VOLUME') = 1);
    fProcessingApplication := TCPProcessingApplication(XMLValueAsInt(aXMLNode, 'PROCESSING_APPLICATION', 0));
    fProcessedResult := TCPProcessedResult(XMLValueAsInt(aXMLNode, 'PROCESSED_RESULT', 0));
    fExternalDataDirectory := XMLValueAsStr(aXMLNode, 'EXTERNAL_DATA_DIRECTORY');
  except
    on E: Exception do
      raise ECPLoadFailure.Create(
        'Error loading procedure definition:' + #13 +
        aXMLNode.XML + #13 + #13 +
        'Message: ' + e.Message);
  end;
end;

procedure TCPProcedure.Save;
var
  i: integer;
begin
  try
    fIsLoaded := False;
    if not fCPService.ExecuteDBCommand(
      'UpdateProcedure',
      [fID,
      fName,
        fTreatingSpecialtyID,
        CP_YN[fRequireExternalData],
        fDefaultTIUNoteID,
        fHospitalLocationID,
        fProcessingApplication,
        CP_YN[fAutoSubmit],
        fExternalDataDirectory,
        CP_YN[fActive],
        CP_YN[fHighVolume],
        fProcessedResult,
        fDescription]) then
      raise ECPSaveFailure.CreateFmt('Command ''UpdateProcedure'' failed for ID ''%s''', [fID]);

    if not fCPService.ExecuteDBCommand('DeleteProcedureInstrumentList', [fID]) then
      raise ECPSaveFailure.CreateFmt('Command ''DeleteProcedureInstrumentList'' failed for ID ''%s''', [fID]);

    for i := 0 to fInstruments.Count - 1 do
      if not fCPService.ExecuteDBCommand('AddProcedureInstrument', [fID, fInstruments[i]]) then
        raise ECPSaveFailure.CreateFmt('Command ''AddProcedureInstrument'' failed for ID ''%s''', [fID]);

    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Expection ''%s''', [e.Message]);
  end;
end;

procedure TCPProcedure.setActive(const aValue: Boolean);
begin
  fActive := aValue;
end;

procedure TCPProcedure.setAutoSubmit(const aValue: boolean);
begin
  fAutoSubmit := aValue;
end;

procedure TCPProcedure.setComment(const aValue: string);
begin
  //
end;

procedure TCPProcedure.setDefaultTIUNote(const aValue: string);
begin
  fDefaultTIUNote := aValue;
end;

procedure TCPProcedure.setDefaultTIUNoteID(const aValue: string);
begin
  fDefaultTIUNoteID := aValue;
end;

procedure TCPProcedure.setDescription(const aValue: string);
begin
  fDescription := aValue;
end;

procedure TCPProcedure.setDisplayName(const aValue: string);
begin
  //
end;

procedure TCPProcedure.setExternalDataDirectory(const aValue: string);
begin
  fExternalDataDirectory := aValue;
end;

procedure TCPProcedure.setHighVolume(const aValue: boolean);
begin
  fHighVolume := aValue;
end;

procedure TCPProcedure.setHospitalLocation(const aValue: string);
begin
  fHospitalLocation := aValue;
end;

procedure TCPProcedure.setHospitalLocationID(const aValue: string);
begin
  fHospitalLocationID := aValue;
end;

procedure TCPProcedure.setName(const aValue: string);
begin
  fName := aValue;
end;

procedure TCPProcedure.setProcessedResult(const aValue: TCPProcessedResult);
begin
  fProcessedResult := aValue;
end;

procedure TCPProcedure.setProcessingApplication(const aValue: TCPProcessingApplication);
begin
  fProcessingApplication := aValue;
end;

procedure TCPProcedure.setRequireExternalData(const aValue: boolean);
begin
  fRequireExternalData := aValue;
end;

procedure TCPProcedure.setTreatingSpecialty(const aValue: string);
begin
  fTreatingSpecialty := aValue;
end;

procedure TCPProcedure.setTreatingSpecialtyID(const aValue: string);
begin
  fTreatingSpecialtyID := aValue;
end;

end.

