{-----------------------------------------------------------------------------
 Unit Name: oCPProtocolLocation
 Author:    Brian Juergensmeyer
 Date:      15-Aug-2007
 Purpose:
 History:
-----------------------------------------------------------------------------}

unit oCPProtocolLocation;

interface

uses
  oInterfaces,
  Classes,
  ComCtrls,
  oCPTypes;

function NewCPProtocolLocation(DBConnection: IDBConnection): ICPProtocolLocation;

implementation

uses
  XMLIntf,
  XMLDoc,
  SysUtils,
  Dialogs,
  oCP_Exceptions,
  Windows,
  ActiveX;

type

  { TODO 1 -oBrian : Modify IEnumerator interface to make GetCurrent and MoveNext stdcall; }

   {----------------------------------------------------------------------------
   | Containers
   ----------------------------------------------------------------------------}
  TCPProtocolLocation = class(TInterfacedObject, ICPConsoleItem, ICPProtocolLocation)
  private
    FName: string;
    FID: string;
    FDBConnection: IDBConnection;
    FProtocolList: ICPProtocolList;
    FADTTargetList: ICPADTTargetList;
    FDivisionList: ICPDivisionList;
  protected
  public
    constructor Create(DBConnection: IDBConnection);
    destructor Destroy; override;
    function getProtocolList: IInterfaceList;
    function getADTTargetList: IInterfaceList;
    function getDivisionList: IInterfaceList;

    {ICPConsole Interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
  published
  end;

  TCPProtocolList = class(TInterfaceList, IInterfaceList, ICPProtocolList)
  private
    FDBConnection: IDBConnection;
  protected
  public
    constructor Create(DBConnection: IDBConnection);
    procedure Load; stdcall;
  published
  end;

  TADTTargetList = class(TInterfaceList, IInterfaceList, ICPADTTargetList)
  private
    FDBConnection: IDBConnection;
    FDivisionList: ICPDivisionList;
    FProtocolList: ICPProtocolList;
  protected
  public
    constructor Create(DBConnection: IDBConnection; DivisionList: ICPDivisionList;
      ProtocolList: ICPProtocolList);
    procedure DeleteItem(AItem: ICPADTTarget); stdcall;
    procedure Load; stdcall;
    procedure Save; stdcall;
    function NewTarget(ASubscriber: ICPProtocol; ADivision: ICPDivision; AWard: ICPWard;
      AName: string; AMessageType: string; AEventType: string): ICPADTTarget; stdcall;
  published
  end;

  TCPDivisionList = class(TInterfaceList, IInterfaceList, ICPDivisionList)
  private
    FDBConnection: IDBConnection;
  protected
  public
    constructor Create(DBConnection: IDBConnection);
    procedure Load; stdcall;
  published
  end;

  {----------------------------------------------------------------------------
  | Contained.
  ----------------------------------------------------------------------------}

  TCPWard = class(TInterfacedObject, ICPWard)
  private
    FIFN: Integer;
    FDivision: Integer;
    FWard: string;
  protected
  public
    constructor Create;
    function getIFN: integer; stdcall;
    function getWard: string; stdcall;
  published
  end;

  TCPDivision = class(TInterfacedObject, ICPDivision)
  private
    FIFN: Integer;
    FName: string;
    FWardList: IInterfaceList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    function getIFN: integer; stdcall;
    function getName: string; stdcall;
    function getWardList: IInterfaceList; stdcall;
  published
  end;

  TCPProtocol = class(TInterfacedObject, ICPProtocol)
  private
    FIFN: Integer;
    FName: string;
  protected
  public
    function getIFN: integer; stdcall;
    function getName: string; stdcall;
  published
  end;

  TCPADTTarget = class(TInterfacedObject, ICPADTTarget)
  private
    FDivision: ICPDivision;
    FSubscriber: ICPProtocol;
    FWard: ICPWard;
    FMessageType: string;
    FEventType: string;
    FID: string;
    FName: string;
  protected
  public
    destructor Destroy; override;
    function getDivision: ICPDivision; stdcall;
    function getSubscriber: ICPProtocol; stdcall;
    function getWard: ICPWard; stdcall;
    function getMessageType: string; stdcall;
    function getEventType: string; stdcall;
    function getID: string; stdcall;
    function getName: string; stdcall;
  published
  end;

  {----------------------------------------------------------------------------
  | Factories
  ----------------------------------------------------------------------------}

function NewCPProtocolLocation(DBConnection: IDBConnection): ICPProtocolLocation;
begin
  result := TCPProtocolLocation.Create(DBConnection);
end;

function NewADTTargetList(DBConnection: IDBConnection; DivisionList: ICPDivisionList;
  ProtocolList: ICPProtocolList): ICPADTTargetList;
begin
  result := TADTTargetList.Create(DBConnection, DivisionList, ProtocolList);
end;

function NewCPDivisionList(DBConnection: IDBConnection): ICPDivisionList;
begin
  result := TCPDivisionList.Create(DBConnection);

end;

function NewCPProtocolList(DBConnection: IDBConnection): ICPProtocolList;
begin
  result := TCPProtocolList.Create(DBConnection);
end;

{ TCPProtocolLocation }

procedure TCPProtocolLocation.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPProtocolLocation.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

function TCPProtocolLocation.AddToTreeView(aTreeView: TTreeView;
  aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

constructor TCPProtocolLocation.Create(DBConnection: IDBConnection);
begin
  inherited Create;
  FDBConnection := DBConnection;
  fID := 'CP ADT Feed Configuration';
  fName := 'CP ADT Feed Configuration';
end;

destructor TCPProtocolLocation.Destroy;
begin
  FDBConnection := nil;
  inherited;
end;

function TCPProtocolLocation.getActive: boolean;
begin
  result := true;
end;

function TCPProtocolLocation.getADTTargetList: IInterfaceList;
begin
  result := FADTTargetList;
end;

function TCPProtocolLocation.getCanDelete: boolean;
begin
  result := true;
end;

function TCPProtocolLocation.getCanRename: boolean;
begin
  result := false;
end;

function TCPProtocolLocation.getComment: string;
begin
  Result := Format('''%s'' ', [getDisplayName]) +
    'holds the configuration settings for the outbound patient information feed. ' +
    'Extreme care should be taken when modifiying these settings.';
end;

function TCPProtocolLocation.getDisplayName: string;
begin
  Result := 'CP Patient information feed configuration';
end;

function TCPProtocolLocation.getDivisionList: IInterfaceList;
begin
  result := FDivisionList;
end;

function TCPProtocolLocation.getID: string;
begin
  result := FID;
end;

function TCPProtocolLocation.getIsLoaded: boolean;
begin
  Result := True;
end;

function TCPProtocolLocation.getItemType: TCPItemType;
begin
  result := ciParameters;
end;

function TCPProtocolLocation.getName: string;
begin
  result := FName;
end;

function TCPProtocolLocation.getProtocolList: IInterfaceList;
begin
  result := FProtocolList;
end;

procedure TCPProtocolLocation.Load;
begin
  FDivisionList := NewCPDivisionList(FDBConnection);
  FProtocolList := NewCPProtocolList(FDBConnection);
  FADTTargetList := NewADTTargetList(FDBConnection, FDivisionList, FProtocolList);
end;

procedure TCPProtocolLocation.setActive(const aValue: boolean);
begin

end;

procedure TCPProtocolLocation.setComment(const aValue: string);
begin

end;

procedure TCPProtocolLocation.setDisplayName(const aValue: string);
begin

end;

procedure TCPProtocolLocation.setName(const aValue: string);
begin

end;

{ TCPProtocolList }

constructor TCPProtocolList.Create(DBConnection: IDBConnection);
begin
  inherited Create;
  FDBConnection := DBConnection;
  Load;
end;

procedure TCPProtocolList.Load;
var
  Protocols: IXMLDocument;
  Items: IXMLNode;
  I: Integer;
  Protocol: TCPProtocol;
begin
  Clear;

  Protocols := NewXMLDocument;

  FDBConnection.ExecQuery('GetSubscriberProtocols', [], Protocols);
  Protocols.Active := True;

  Items := Protocols.ChildNodes.FindNode('RESULTS');
  for I := 0 to Items.ChildNodes.Count - 1 do
    begin
      if Items.ChildNodes[I].NodeType = ntComment then
        Continue;

      Protocol := TCPProtocol.Create;
      Protocol.FName := Items.ChildNodes[I].ChildNodes.FindNode('NAME').NodeValue;
      Protocol.FIFN := Items.ChildNodes[I].ChildNodes.FindNode('VALUE').NodeValue;
      Add(Protocol as ICPProtocol);
    end;
end;

{ TADTTargetList }

constructor TADTTargetList.Create(DBConnection: IDBConnection; DivisionList: ICPDivisionList;
  ProtocolList: ICPProtocolList);
begin
  inherited Create;
  FDBConnection := DBConnection;
  FDivisionList := DivisionList;
  FProtocolList := ProtocolList;
  Load;
end;

procedure TADTTargetList.DeleteItem(AItem: ICPADTTarget);
var
  I: Integer;
begin
  I := IndexOf(AItem);
  if I > -1 then
    begin
      if not FDBConnection.ExecCommand('DeleteADTFeedByID', [AItem.getID]) then
        raise ECPDeleteFailure.CreateFmt('Unable to delete ADT Feed for ''%s''', [AItem.getName]);
    end;
  Load;
end;

procedure TADTTargetList.Load;
var
  Targets: IXMLDocument;
  Items: IXMLNode;
  I, J, K: Integer;
  Target: TCPADTTarget;
  Division: string;
  Subscriber: string;
  WardIFN: Integer;
  Ward: ICPWard;
  CPDiv: ICPDivision;
  Prot: ICPProtocol;
begin
  Clear;

  Targets := NewXMLDocument;

  FDBConnection.ExecQuery('GetADTTargets', [], Targets);
  Targets.Active := true;
  Items := Targets.ChildNodes.FindNode('RESULTS');

  for I := 0 to Items.ChildNodes.Count - 1 do
    begin
      if Items.ChildNodes[I].NodeType = ntComment then
        Continue;

      Target := TCPADTTarget.Create;
      { DONE 1 -oBrian : Need to add the NAME field to the script that populates this. }
      Target.FName := Items.ChildNodes[I].ChildNodes.FindNode('NAME').NodeValue;
      Division := Items.ChildNodes[I].ChildNodes.FindNode('DIVISION').NodeValue;
      Subscriber := Items.ChildNodes[I].ChildNodes.FindNode('SUBSCRIBER').NodeValue;
      if Assigned(Items.ChildNodes[I].ChildNodes.FindNode('WARD')) then
        WardIFN := StrToIntDef(Items.ChildNodes[I].ChildNodes.FindNode('WARD').NodeValue, -1)
      else
        WardIFN := -1;
      Target.FMessageType := Items.ChildNodes[I].ChildNodes.FindNode('MESSAGE_TYPE').NodeValue;
      Target.FEventType := Items.ChildNodes[I].ChildNodes.FindNode('EVENT_TYPE').NodeValue;
      Target.FID := Items.ChildNodes[I].ChildNodes.FindNode('ID').NodeValue;

      // Now, we need to do lookups in the list for Division, Subscriber, and wardBed.

      for J := 0 to FDivisionList.Count - 1 do
        begin
          if FDivisionList[J].QueryInterface(ICPDivision, CPDiv) = 0 then
            begin
              if IntToStr(CPDiv.getIFN) = Division then
                begin
                  Target.FDivision := CPDiv;
                  for K := 0 to CPDiv.getWardList.Count - 1 do
                    begin
                      if CPDiv.getWardList[K].QueryInterface(ICPWard, Ward) = 0 then
                        if Ward.getIFN = WardIFN then
                          Target.FWard := Ward;
                    end;
                end;
            end;
        end;

      for J := 0 to FProtocolList.Count - 1 do
        begin
          if FProtocolList[J].QueryInterface(ICPProtocol, Prot) = 0 then
            begin
              if IntToStr(Prot.getIFN) = Subscriber then
                Target.FSubscriber := Prot;
            end;
        end;
      Add(Target as ICPADTTarget);
    end;
end;

function TADTTargetList.NewTarget(ASubscriber: ICPProtocol; ADivision: ICPDivision;
  AWard: ICPWard; AName: string; AMessageType: string; AEventType: string): ICPADTTarget;
var
  Target: TCPADTTarget;
  Ward: string;
begin
  Target := TCPADTTarget.Create;
  Target.FDivision := ADivision;
  Target.FSubscriber := ASubscriber;
  Target.FWard := AWard;
  Target.FName := AName;
  Target.FMessageType := AMessageType;
  Target.FEventType := AEventType;

  if Target.getWard = nil then
    Ward := ''
  else
    Ward := IntToStr(Target.getWard.getIFN);

  FDBConnection.ExecCommand('AddADTTarget', [Target.getID, Target.getName,
    Target.getSubscriber.getIFN, Target.getDivision.getIFN, Ward,
      Target.FMessageType, Target.FEventType]);
  result := Target;
end;

procedure TADTTargetList.Save;
begin

end;

{ TCPDivisionList }

constructor TCPDivisionList.Create(DBConnection: IDBConnection);
begin
  inherited Create;
  FDBConnection := DBConnection;
  Load;
end;

procedure TCPDivisionList.Load;
var
  I, J: Integer;
  Divisions: IXMLDocument;
  Items: IXMLNode;
  Division: TCPDivision;
  WardList: TList;
  Ward: TCPWard;
//  IWard: ICPWard;
  DivisionNode: IXMLNode;
begin
  //CliO Script: GetHospitalDivisions
  // Also, run GetWardList to get the wards for various divisions.  Then, link them
  // up.
  Clear;

  Divisions := NewXMLDocument;

  WardList := TList.Create;
  try
    FDBConnection.ExecQuery('GetWardList', [], Divisions);
    Divisions.Active := True;

    Items := Divisions.ChildNodes.FindNode('RESULTS');
    for I := 0 to Items.ChildNodes.Count - 1 do
      begin
        if Items.ChildNodes[I].NodeType = ntComment then
          Continue;
        Ward := TCPWard.Create;
        Ward.FIFN := Items.ChildNodes[I].ChildNodes.FindNode('ROWID').NodeValue;
        Ward.FWard := Items.ChildNodes[I].ChildNodes.FindNode('NAME').NodeValue;
        DivisionNode := Items.ChildNodes[I].ChildNodes.FindNode('DIVISION');
        if Assigned(DivisionNode) then
          Ward.FDivision := DivisionNode.NodeValue
        else
          Ward.FDivision := -1;
        WardList.Add(Ward);
      end;

    FDBConnection.ExecQuery('GetHospitalDivisions', [], Divisions);
    Divisions.Active := true;
    Items := Divisions.ChildNodes.FindNode('RESULTS');

    for I := 0 to Items.ChildNodes.Count - 1 do
      begin
        if Items.ChildNodes[I].NodeType = ntComment then
          Continue;
        Division := TCPDivision.Create;
        Division.FIFN := Items.ChildNodes[I].ChildNodes.FindNode('ROWID').NodeValue;
        Division.FName := Items.ChildNodes[I].ChildNodes.FindNode('NAME').NodeValue;
        Division.FWardList := TInterfaceList.Create;
        for J := 0 to WardList.Count - 1 do
          begin
            Ward := TCPWard(WardList[J]);
            if (Ward.FDivision = Division.FIFN) or (Ward.FDivision = -1) then
              Division.FWardList.Add(Ward as ICPWard);
          end;
        Add(Division as ICPDivision);
      end;
  finally
    FreeAndNil(WardList);
  end;
end;

{ TCPWard }

constructor TCPWard.Create;
begin
  inherited Create;
end;

function TCPWard.getIFN: integer;
begin
  result := FIFN;
end;

function TCPWard.getWard: string;
begin
  result := FWard;
end;

{ TCPDivision }

constructor TCPDivision.Create;
begin
  inherited Create;
end;

destructor TCPDivision.Destroy;
begin
  FWardList.Clear;
  FWardList := nil;
  inherited;
end;

function TCPDivision.getIFN: integer;
begin
  result := FIFN;
end;

function TCPDivision.getName: string;
begin
  result := FName;
end;

function TCPDivision.getWardList: IInterfaceList;
begin
  result := FWardList;
end;

{ TCPProtocol }

function TCPProtocol.getIFN: integer;
begin
  result := FIFN;
end;

function TCPProtocol.getName: string;
begin
  result := FName;
end;

{ TCPADTTarget }

destructor TCPADTTarget.Destroy;
begin

  inherited;
end;

function TCPADTTarget.getDivision: ICPDivision;
begin
  result := FDivision;
end;

function TCPADTTarget.getWard: ICPWard;
begin
  result := FWard;
end;

function TCPADTTarget.getEventType: string;
begin
  result := FEventType;
end;

function TCPADTTarget.getID: string;
var
  GUID: TGUID;
begin
  if FID = '' then
    begin
      CoInitialize(nil);
      CreateGUID(GUID);
      FID := GUIDToString(GUID);
      FID := StringReplace(FID, '{', '', [rfReplaceAll]);
      FID := StringReplace(FID, '}', '', [rfReplaceAll]);
      FID := StringReplace(FID, '-', '', [rfReplaceAll]);
      CoUninitialize;
    end;
  result := FID;
end;

function TCPADTTarget.getMessageType: string;
begin
  result := FMessageType;
end;

function TCPADTTarget.getName: string;
begin
  result := FName;
end;

function TCPADTTarget.getSubscriber: ICPProtocol;
begin
  result := FSubscriber;
end;

end.

