unit oCPUser;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 11 $  $Modtime: 5/22/09 1:12p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default CP User object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPUser.pas $
================================================================================
}
interface

uses
  Classes,
  XMLDoc,
  XMLIntf,
  oCliO,
  oInterfaces;

type
  TCPUser = class(TInterfacedObject, IUser, ICliOSource, IPerson)
  protected
    fDUZ: string;
    fName: string;
    fInitials: string;
    fSex: string;
    fSSN: string;
    fDOB: TDateTime;
    fStationID: string;
    fStationName: string;
    fCCOWDFNItemName: string;
    fRoles: TStringList;
    fSourceID: string;
    fSourceApplication: string;
    fSourceVersion: string;
    fInternalESig: string;
    fExternalESig: string;
    fESIGName: string;
    fESIGTitle: string;
    fProductionSignOn: boolean;
    fDBConnection: IDBConnection;
    fTimeOut: integer;
    function getID: string; stdcall;
    function getDUZ: string; stdcall;
    function getInternalID: string;
    function getName: string; stdcall;
    function getInitials: string; stdcall;
    function getFormalName: string; stdcall;
    function getESIGName: string; stdcall;
    function getESIGTitle: string; stdcall;
    function getStation_ID: string; stdcall;
    function getStation_Name: string; stdcall;
    function getCCOWDFNItemName: string; stdcall;
    function HasRole(aRoleName: string): boolean; stdcall;
    function IsThisExecutableValid: boolean; stdcall;
    function IsValidElectronicSignature(aSignature: string): boolean; stdcall;
    function IsProductionSignOn: boolean; stdcall;
    function getTimeOut: integer; stdcall;
    function GetLastError: string; stdcall;

    function getSourceID: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;

    function getSex: string; stdcall;
    function getSSN: string; stdcall;
    function getDOB: TDateTime; stdcall;
    function getTitle: string; stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    procedure RefreshAccess; stdcall;

    procedure LoadFromXML(aXMLDoc: IXMLDocument);
    procedure LoadRoles(aXMLDoc: IXMLDocument);
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPUser(aID: string; aDBConnection: IDBConnection): TCPUser;

implementation

uses
  Forms,
  oVersionInformation,
  uCPXML,
  SysUtils;

function NewCPUser(aID: string; aDBConnection: IDBConnection): TCPUser;
var
  aXMLDoc: IXMLDocument;
  aVistASystem: IVistASystem;
begin
  Result := TCPUser.Create;
  aXMLDoc := NewXMLDocument;
  try
    Result.fDBConnection := aDBConnection;
    aDBConnection.ExecQuery('GetUserInformation', [aID], aXMLDoc);
    Result.LoadFromXML(aXMLDoc);
    aDBConnection.ExecQuery('GetUserRoles', [aID], aXMLDoc);
    Result.LoadRoles(aXMLDoc);
    if aDBConnection.QueryInterface(IVistASystem, aVistASystem) = 0 then
      begin
        Result.fDUZ := aVistASystem.DUZ;
        Result.fStationID := aVistASystem.Station_ID;
        Result.fStationName := aVistASystem.Station_Name;
      end;
    aVistASystem := nil;

    with TVersionInformation.Create(Application.ExeName, 'MD') do
      try
        Result.fSourceID := Format('%s:%s', [getFileName, getCRC32]);
        Result.fSourceApplication := getProductName; //Format('%s (%s)', [getFileNameAndVersion, getCRC32]);
        Result.fSourceVersion := getFileVersion;
        //Result.fSourceTrusted := (not Result.HasRole('MD TRAINEE'));
      finally
        Free;
      end;

  except
    FreeAndNil(Result);
  end
end;

{ TCPUser }

{
******************************************************* TCPUser ********************************************************
}

procedure TCPUser.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

constructor TCPUser.Create;
begin
  inherited;
  fRoles := TStringList.Create;
end;

destructor TCPUser.Destroy;
begin
  FreeAndNil(fRoles);
  fDBConnection := nil;
  inherited;
end;

function TCPUser.getCCOWDFNItemName: string;
begin
  Result := fCCOWDFNItemName;
end;

function TCPUser.getDOB: TDateTime;
begin
  Result := fDOB;
end;

function TCPUser.getDUZ: string;
begin
  Result := fDUZ;
end;

function TCPUser.getESIGName: string;
begin
  Result := fESIGName;
end;

function TCPUser.getESIGTitle: string;
begin
  Result := fESIGTitle;
end;

function TCPUser.getFormalName: string;
begin
  Result := Format('%s (DUZ: %s)', [fName, fDUZ]);
end;

function TCPUser.getID: string;
begin
  Result := fDUZ;
end;

function TCPUser.getInitials: string;
begin
  Result := fInitials;
end;

function TCPUser.getInternalID: string;
begin
  Result := fDUZ;
end;

function TCPUser.GetLastError: string;
begin
  Result := '';
end;

function TCPUser.HasRole(aRoleName: string): boolean;
begin
  if (aRoleName <> 'MD TRAINEE') and (aRoleName <> 'MD READ-ONLY') then
    Result := (fRoles.IndexOf(aRoleName) > -1) or (fRoles.IndexOf('MD ADMINISTRATOR') > -1)
  else
    Result := (fRoles.IndexOf(aRoleName) > -1)
end;

function TCPUser.IsProductionSignOn: boolean;
begin
  Result := fProductionSignOn;
end;

function TCPUser.IsThisExecutableValid: boolean;
begin
  Result := True;
end;

function TCPUser.IsValidElectronicSignature(aSignature: string): boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
begin
  aXMLDoc := NewXMLDocument;
  try
    fDBConnection.ExecQuery('GetValidESignature', [aSignature], aXMLDoc);
    aXMLDoc.Active := True;
    aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
    Result := (XMLValueAsInt(aXMLNode, 'VALID') = 1);
    fInternalESig := XMLValueAsStr(aXMLNode, 'INTERNAL');
    fExternalESig := XMLValueAsStr(aXMLNode, 'EXTERNAL');
  except
    Result := False;
  end;
end;

procedure TCPUser.LoadFromXML(aXMLDoc: IXMLDocument);
var
  aXMLNode: IXMLNode;
begin
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  fDUZ := XMLValueAsStr(aXMLNode, 'ID');
  fName := XMLValueAsStr(aXMLNode, 'NAME');
  fInitials := XMLValueAsStr(aXMLNode, 'INITIAL');
  fSSN := XMLValueAsStr(aXMLNode, 'SSN');
  fSex := XMLValueAsStr(aXMLNode, 'SEX');
  fStationID := XMLValueAsStr(aXMLNode, 'STATION_ID');
  fStationName := XMLValueAsStr(aXMLNode, 'STATION');
  fESIGName := XMLValueAsStr(aXMLNode, 'SIGNATURE_BLOCK_PRINTED_NAME');
  fESIGTitle := XMLValueAsStr(aXMLNode, 'SIGNATURE_BLOCK_TITLE');
  fProductionSignOn := (XMLValueAsInt(aXMLNode, 'PRODUCTION_ACCOUNT') = 1);
  fCCOWDFNItemName := XMLValueAsStr(aXMLNode, 'CCOW');
  fTimeOut := XMLValueAsInt(aXMLNode, 'TIMED_READ', 300);
end;

procedure TCPUser.LoadRoles(aXMLDoc: IXMLDocument);
var
  aXMLNode: IXMLNode;
begin
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  fRoles.Clear;
  while Assigned(aXMLNode) do
    try
      fRoles.Add(XMLValueAsStr(aXMLNode, 'ROLE_ID'));
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;
end;

procedure TCPUser.RefreshAccess;
var
  aXMLDoc: IXMLDocument;
begin
  aXMLDoc := NewXMLDocument;
  try
    fDBConnection.ExecQuery('GetUserInformation', [fDUZ], aXMLDoc);
    LoadFromXML(aXMLDoc);
    fDBConnection.ExecQuery('GetUserRoles', [fDUZ], aXMLDoc);
    LoadRoles(aXMLDoc);
  except
    raise;
  end;
end;

function TCPUser.getName: string;
begin
  Result := fName;
end;

function TCPUser.getSex: string;
begin
  Result := fSex;
end;

function TCPUser.getSourceApplication: string;
begin
  Result := fSourceApplication;
end;

function TCPUser.getSourceID: string;
begin
  Result := fSourceID;
end;

function TCPUser.getSourceTrusted: boolean;
begin
  {If not assigned role of MD TRAINEE then we trust you!}
  Result := (fRoles.IndexOf('MD TRAINEE') < 0);
end;

function TCPUser.getSourceVersion: string;
begin
  Result := fSourceVersion;
end;

function TCPUser.getSSN: string;
begin
  Result := fSSN;
end;

function TCPUser.getStation_ID: string;
begin
  Result := fStationID;
end;

function TCPUser.getStation_Name: string;
begin
  Result := fStationName;
end;

function TCPUser.getTimeOut: integer;
begin
  Result := fTimeOut;
end;

function TCPUser.getTitle: string;
begin
  Result := fESIGTitle;
end;

end.

