unit oCPUserOptions;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 7 $  $Modtime: 1/20/10 1:38p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  CP Parameters
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPUserOptions.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  Graphics,
  oCliOTypes,
  oCPTypes,
  oCPParameterFile,
  oInterfaces;

{
  TODO: 20 January 2009 blj - We need to modify this to persist out a setting for
    time formats (specifically allowing display of time in a 24 vs. 12 hour format.
}

type
  TCPUserOptions = class(TInterfacedObject, ICPUserOptions)
  protected
    fConfirmOnClose: boolean;
    fAutoRefreshEnabled: boolean;
    fAutoRefreshInterval: integer;
    fDefaultFlowsheetID: string;
    fUsePatientLocation: boolean;
    fDefaultLocationID: string;
    fLookAndFeel: TCPLookAndFeel;
    fUseFlowsheetExplorer: boolean;

    fInternalTestValue: string;
    fDBConnection: IDBConnection;
    fDisplayFonts: array[TObservationRange] of TFont;
    fDisplayBackground: array[TObservationRange] of TColor;

    function getConfirmOnClose: boolean; stdcall;
    function getFlowsheetItemFont(aRange: TObservationRange): TFont; stdcall;
    function getFlowsheetItemColor(aRange: TObservationRange): TColor; stdcall;
    function getAutoRefreshEnabled: boolean; stdcall;
    function getAutoRefreshInterval: integer; stdcall;
    function getDefaultFlowsheetID: string; stdcall;
    function getUsePatientLocation: boolean; stdcall;
    function getDefaultLocationID: string; stdcall;
    function getLookAndFeel: TCPLookAndFeel; stdcall;
    function getUseFlowsheetExplorer: boolean; stdcall;

    procedure setConfirmOnClose(aValue: boolean); stdcall;
    procedure setFlowsheetItemFont(aRange: TObservationRange; aFont: TFont); stdcall;
    procedure setFLowsheetItemColor(aRange: TObservationRange; aColor: TColor); stdcall;
    procedure setAutoRefreshEnabled(aValue: boolean); stdcall;
    procedure setAutoRefreshInterval(aValue: integer); stdcall;
    procedure setDefaultFlowsheetID(aValue: string); stdcall;
    procedure setUsePatientLocation(aValue: boolean); stdcall;
    procedure setDefaultLocationID(aValue: string); stdcall;
    procedure setLookAndFeel(aValue: TCPLookAndFeel); stdcall;
    procedure setUseFlowsheetExplorer(aValue: boolean); stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

uses
  MFunStr,
  Math,
  SysUtils
  , System.UITypes // 2FA
  ;

const
  BOOL_STR: array[boolean] of string = ('0', '1');
  FONT_PARAM: array[TObservationRange] of string = ('FONT_UNKNOWN', 'FONT_NORMAL', 'FONT_OUT_OF_BOUNDS_LOW', 'FONT_OUT_OF_BOUNDS_HIGH', 'FONT_LOW', 'FONT_HIGH');

  AUTO_REFRESH_ENABLED = 'AUTO_REFRESH_ENABLED';
  AUTO_REFRESH_INTERVAL = 'AUTO_REFRESH_INTERVAL';
  CONFIRM_ON_CLOSE = 'CONFIRM_ON_CLOSE';
  DEFAULT_FLOWSHEET_ID = 'DEFAULT_FLOWSHEET_ID';
  DEFAULT_LOCATION_ID = 'DEFAULT_LOCATION_ID';
  DEFAULT_LOCATION_NAME = 'DEFAULT_LOCATION_NAME';
  USE_PATIENT_LOCATION = 'USE_PATIENT_LOCATION';
  LOOK_AND_FEEL = 'LOOK_AND_FEEL';
  USE_FLOWSHEET_EXPLORER = 'USE_FLOWSHEET_EXPLORER';

  { TCPParameters }

constructor TCPUserOptions.Create(aDBConnection: IDBConnection);
var
  aObsRange: TObservationRange;
begin
  inherited Create;
  fDBConnection := aDBConnection;
  for aObsRange := Low(TObservationRange) to High(TObservationRange) do
    begin
      fDisplayFonts[aObsRange] := TFont.Create;
      fDisplayFonts[aObsRange].Color := clWindowText;
      fDisplayFonts[aObsRange].Style := [];
      fDisplayBackground[aObsRange] := clWindow;
    end;
  fDisplayFonts[orHigh].Color := clRed;
  fDisplayFonts[orHigh].Style := [fsBold, fsUnderline];
end;

destructor TCPUserOptions.Destroy;
var
  aObsRange: TObservationRange;
begin
  for aObsRange := Low(TObservationRange) to High(TObservationRange) do
    FreeAndNil(fDisplayFonts[aObsRange]);
  inherited;
end;

function TCPUserOptions.getAutoRefreshEnabled: boolean;
begin
  Result := fAutoRefreshEnabled
end;

function TCPUserOptions.getAutoRefreshInterval: integer;
begin
  Result := fAutoRefreshInterval;
end;

function TCPUserOptions.getConfirmOnClose: boolean;
begin
  Result := fConfirmOnClose;
end;

function TCPUserOptions.getDefaultFlowsheetID: string;
begin
  Result := fDefaultFlowsheetID;
end;

function TCPUserOptions.getDefaultLocationID: string;
begin
  Result := fDefaultLocationID;
end;

function TCPUserOptions.getFlowsheetItemColor(aRange: TObservationRange): TColor;
begin
  Result := fDisplayBackground[aRange];
end;

function TCPUserOptions.getFlowsheetItemFont(aRange: TObservationRange): TFont;
begin
  Result := fDisplayFonts[aRange];
end;

function TCPUserOptions.getLookAndFeel: TCPLookAndFeel;
begin
  Result := fLookAndFeel;
end;

function TCPUserOptions.getUsePatientLocation: boolean;
begin
  Result := fUsePatientLocation;
end;

function TCPUserOptions.getUseFlowsheetExplorer: boolean;
begin
  Result := False; //fUseFlowsheetExplorer;
end;

procedure TCPUserOptions.Load;
var
  aObsRange: TObservationRange;
  i: integer;
  aFontString: string;
  aFontStyle: TFontStyle;
begin
  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
    try
      fConfirmOnClose := ReadBoolean(pkUser, CONFIRM_ON_CLOSE, False);
      fAutoRefreshEnabled := ReadBoolean(pkUser, AUTO_REFRESH_ENABLED, False);
      fAutoRefreshInterval := ReadInteger(pkUser, AUTO_REFRESH_INTERVAL, 300);
      fDefaultFlowsheetID := ReadString(pkUser, DEFAULT_FLOWSHEET_ID, '');
      fUsePatientLocation := ReadBoolean(pkUser, USE_PATIENT_LOCATION, True);
      fDefaultLocationID := ReadString(pkUser, DEFAULT_LOCATION_ID, '');
      fLookAndFeel := TCPLookAndFeel(ReadInteger(pkUser, LOOK_AND_FEEL, 0));
      fUseFlowsheetExplorer := ReadBoolean(pkUser, USE_FLOWSHEET_EXPLORER, False);

      for aObsRange := Low(TObservationRange) to High(TObservationRange) do
        begin
          {Build the default fonts}
          case aObsRange of
            orLow:
              begin
                fDisplayFonts[aObsRange].Style := [];
                fDisplayFonts[aObsRange].Color := clBlue;
                fDisplayBackground[aObsRange] := clWindow;
                aFontString := '0;clBlue;clWindow';
              end;
            orHigh:
              begin
                fDisplayFonts[aObsRange].Style := [];
                fDisplayFonts[aObsRange].Color := clRed;
                fDisplayBackground[aObsRange] := clWindow;
                aFontString := '0;clRed;clWindow';
              end;
          else
            begin
              fDisplayFonts[aObsRange].Style := [];
              fDisplayFonts[aObsRange].Color := clWindowText;
              fDisplayBackground[aObsRange] := clWindow;
              aFontString := '0;clWindowText;clWindow';
            end;
          end;

          aFontString := ReadString(pkUser, FONT_PARAM[aObsRange], aFontString);

          i := StrToIntDef(Piece(aFontString, ';', 1), 0);
          for aFontStyle := Low(TFontStyle) to High(TFontStyle) do
            if (i and Trunc(Power(2, Ord(aFontStyle)))) = Trunc(Power(2, Ord(aFontStyle))) then
              fDisplayFonts[aObsRange].Style := fDisplayFonts[aObsRange].Style + [aFontStyle];
          fDisplayFonts[aObsRange].Color := StringToColor(Piece(aFontString, ';', 2));
          fDisplayBackground[aObsRange] := StringToColor(Piece(aFontString, ';', 3));
        end;
    except
      raise Exception.Create('Error loading user options.');
    end;
end;

procedure TCPUserOptions.Save;
var
  aObsRange: TObservationRange;
  i: integer;
  aFontStyle: TFontStyle;
begin
  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
    try
      ClearValues(pkUser);
      WriteBoolean(pkUser, CONFIRM_ON_CLOSE, fConfirmOnClose);
      WriteBoolean(pkUser, AUTO_REFRESH_ENABLED, fAutoRefreshEnabled);
      WriteInteger(pkUser, AUTO_REFRESH_INTERVAL, fAutoRefreshInterval);
      WriteString(pkUser, DEFAULT_FLOWSHEET_ID, fDefaultFlowsheetID);
      WriteBoolean(pkUser, USE_PATIENT_LOCATION, fUsePatientLocation);
      WriteString(pkUser, DEFAULT_LOCATION_ID, fDefaultLocationID);
      WriteInteger(pkUser, LOOK_AND_FEEL, Ord(fLookAndFeel));
      WriteBoolean(pkUser, USE_FLOWSHEET_EXPLORER, fUseFlowsheetExplorer);

      for aObsRange := Low(TObservationRange) to High(TObservationRange) do
        begin
          i := 0;
          for aFontStyle := Low(TFontStyle) to High(TFontStyle) do
            if aFontStyle in fDisplayFonts[aObsRange].Style then
              i := i + trunc(Power(2, ord(aFontStyle)));
          WriteString(pkUser, FONT_PARAM[aObsRange],
            format('%d;%s;%s', [i, ColorToString(fDisplayFonts[aObsRange].Color), ColorToString(fDisplayBackground[aObsRange])]));
        end;
    except
      raise;
    end;
end;

procedure TCPUserOptions.setAutoRefreshEnabled(aValue: boolean);
begin
  fAutoRefreshEnabled := aValue;
end;

procedure TCPUserOptions.setAutoRefreshInterval(aValue: integer);
begin
  fAutoRefreshInterval := aValue;
end;

procedure TCPUserOptions.setConfirmOnClose(aValue: boolean);
begin
  fConfirmOnClose := aValue;
end;

procedure TCPUserOptions.setDefaultFlowsheetID(aValue: string);
begin
  fDefaultFlowsheetID := aValue;
end;

procedure TCPUserOptions.setDefaultLocationID(aValue: string);
begin
  fDefaultLocationID := aValue;
end;

procedure TCPUserOptions.setFLowsheetItemColor(aRange: TObservationRange; aColor: TColor);
begin
  fDisplayBackground[aRange] := aColor;
end;

procedure TCPUserOptions.setFlowsheetItemFont(aRange: TObservationRange; aFont: TFont);
begin
  fDisplayFonts[aRange].Assign(aFont);
end;

procedure TCPUserOptions.setLookAndFeel(aValue: TCPLookAndFeel);
begin
  fLookAndFeel := aValue;
end;

procedure TCPUserOptions.setUsePatientLocation(aValue: boolean);
begin
  fUsePatientLocation := aValue;
end;

procedure TCPUserOptions.setUseFlowsheetExplorer(aValue: boolean);
begin
  fUseFlowsheetExplorer := aValue;
end;

end.

