unit oCliO;

interface

uses
  SysUtils,
  Classes,
  ComCtrls,
  Graphics,
  Types,
  XMLDoc,
  XMLIntf,
  oInterfaces,
  oCliOTypes;

type
  ICliOService = interface;
  ICliOFiler = interface;
  ICliOFilerDoc = interface;
  ICliOFilerObs = interface;
  ICliOFilerSet = interface;
  ICliOFilerXML = interface;
  ICliOLoader = interface;
  ICliOLogEntry = interface;
  ICliOList = interface;
  ICliOHL7Log = interface;
  ICliOHL7LogEntry = interface;
  ICliOReport = interface;
  ICliOReportItem = interface;
  ICliOSource = interface;
  ICliOAlarm = interface;
  ICliOAlarmList = interface;

  ICliOService = interface(IInterface)
    ['{5C209ED3-C253-4886-B111-0AA152DE31EE}']
    function Initialize(aName: string; aDBConnection: IDBConnection): Boolean; overload; stdcall;
    function Initialize(aName: string; aDBType: integer; aParams: array of Variant): Boolean; overload; stdcall;
    function Initialize(aName: string; aXMLDocument: IXMLDocument): Boolean; overload; stdcall;

    function getNewFiler: ICliOFiler; stdcall;
    function getNewLoader: ICliOLoader; stdcall;
    function getNewReport(aLoader: ICliOLoader): ICliOReport; stdcall;

    function getTotal(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function getAverage(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function getCount(aList: IInterfaceList; aTerm: IInterface): integer; stdcall;
    function getMaximumValue(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function getMinimumValue(aList: IInterfaceList; aTerm: IInterface): string; stdcall;

    procedure LoadLogEntries(aList: IInterfaceList; aPatientID: string; aFromDateTime, aToDateTime: TDateTime); stdcall;
  end;

  ICliOFiler = interface(IInterface)
    ['{8CDB7EC0-681D-4A07-B1B0-3851F3B6BE27}']
    function getNewDocument(aSource: IInterface): ICliOFilerDoc; stdcall;
    function getError(aIndex: integer): string; stdcall;
    function getErrorCount: Integer; stdcall;
    function getErrorLog: WideString; stdcall;

    procedure SetAlarmList(aCliOAlarmList: ICliOAlarmList); stdcall;
    procedure Execute(aDoc: ICliOFilerDoc); stdcall;
    procedure Correct(aError, aCorrection: IObservation; aUser: IUser; aComment: string); stdcall;
    procedure UpdateObservationAsVerified(aObs: IObservation; aUser: IUser; aComment: string = 'Observation Verified'); stdcall;
    procedure UpdateObservationAsPurged(aObs: IObservation; aUser: IUser; aComment: string); stdcall;
    procedure UpdateObservationAsRescinded(aObs: IObservation; aUser: IUser; aComment: string); stdcall;
    procedure UpdateObservationAsUnverified(aObs: IObservation; aUser: IUser; aComment: string = 'Observation Recovered'); stdcall;
  end;

  ICliOFilerXML = interface(IInterface)
    ['{35B53BED-4248-4BA2-827F-0C6B7E6C479C}']
    function getXML: WideString; stdcall;
  end;

  ICliOFilerObs = interface(ICliOFilerXML)
    ['{4CABDBAB-D5C2-4528-8987-BBE368C8BB88}']
    function getID: string; stdcall;
    function getFacilityID: string; stdcall;
    function getFacilityLocationID: string; stdcall;
    function getPatientID: string; stdcall;
    function getEnteredDateTime: TDateTime; stdcall;
    function getEnteredByID: string; stdcall;
    function getComment: string; stdcall;
    function getCorrectionForID: string; stdcall;
    function getCorrectionReason: string; stdcall;
    function getObservationStatus: TObservationRecordStatus; stdcall;
    function getObservationRange: TObservationRange; stdcall;
    function getObservedByID: string; stdcall;
    function getObservedDateTime: TDateTime; stdcall;
    function getParentID: string; stdcall;
    function getSource: string; stdcall;
    function getSourceComment: string; stdcall;
    function getSourceDataItemID: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;
    function getTermID: string; stdcall;
    function getUnitID: string; stdcall;
    function getValue: string; stdcall;
    function getQualifier(aTermType: TTermType): string; stdcall;

    procedure setFacilityID(const aValue: string); stdcall;
    procedure setFacilityLocationID(const aValue: string); stdcall;
    procedure setPatientID(const aValue: string); stdcall;
    procedure setEnteredDateTime(const aValue: TDateTime); stdcall;
    procedure setEnteredByID(const aValue: string); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setCorrectionForID(const aValue: string); stdcall;
    procedure setCorrectionReason(const aValue: string); stdcall;
    procedure setObservationStatus(const aValue: TObservationRecordStatus); stdcall;
    procedure setObservationRange(const aValue: TObservationRange); stdcall;
    procedure setObservedByID(const aValue: string); stdcall;
    procedure setObservedDateTime(const aValue: TDateTime); stdcall;
    procedure setParentID(const aValue: string); stdcall;

    procedure setSource(const aValue: string); stdcall;
    procedure setSourceComment(const aValue: string); stdcall;
    procedure setSourceDataItemID(const aValue: string); stdcall;
    procedure setSourceVersion(const aValue: string); stdcall;
    
    procedure setTermID(const aValue: string); stdcall;
    procedure setUnitID(const aValue: string); stdcall;
    procedure setValue(const aValue: string); stdcall;
    procedure setQualifier(aTermType: TTermType; aTermID: string); stdcall;
    {Manage the sets as a many to many releation}
    procedure AddToSet(aSetID: string); stdcall;
    procedure GetSetIDs(aList: TStrings); stdcall;
  end;

  ICliOFilerDoc = interface(ICliOFilerXML)
    ['{83F2BDF0-9476-4948-90F5-2E1F229160DC}']
    function getFirstObs: ICliOFilerObs; stdcall;
    function getNextObs(aCliOFilerObs: ICliOFilerObs): ICliOFilerObs; stdcall;

    function getFirstSet: ICliOFilerSet; stdcall;
    function getNextSet(aCliOSet: ICliOFilerSet): ICliOFilerSet; stdcall;

    function CreateNewObs(aSetID: string): ICliOFilerObs; stdcall;
    function CreateNewSet: ICliOFilerSet; stdcall;

    procedure SaveToFile(aFileName: string); stdcall;
  end;

  ICliOFilerSet = interface(ICliOFilerXML)
    ['{8817F39A-3093-46A5-AAD8-4E4A61D4CD03}']
    function getID: string; stdcall;
    function getEnteredByID: string; stdcall;
    function getEnteredDateTime: TDateTime; stdcall;
    function getIsPublic: boolean; stdcall;
    function getComment: string; stdcall;

    procedure setEnteredByID(const aValue: string); stdcall;
    procedure setEnteredDateTime(const aValue: TDateTime); stdcall;
    procedure setIsPublic(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
  end;

  ICliOLoader = interface(IInterface)
    ['{DC28CA76-0D71-4F4F-A64A-B7847D13535F}']
    function CreateNewList: ICliOList;
    function CreateNewHL7Log: ICliOHL7Log;

    function GetObservationByID(aID: string): IObservation; stdcall;
    procedure LoadObservations(aList: ICliOList; aPatientID: string; aStartDateTime, aStopDateTime: TDateTime); stdcall;
    procedure LoadUnverifedObservations(aList: ICliOList; aPatientID: string); stdcall;
    procedure LoadObservationsByStatus(aList: ICliOList; aPatientID: string; aStartDateTime, aStopDateTime: TDateTime; aStatus: TObservationRecordStatusSet); stdcall;
    procedure LoadAuditLog(aList: TStrings; aObservationID: string); stdcall;
    procedure SetLoadAtATime(aValue: integer); stdcall;
    procedure SetOnNotify(aOnNotifyOnProgress: TNotifyOnProgress); stdcall;
  end;

  ICliOList = interface(IInterface)
    ['{28344785-8C3D-4C1B-ACBD-DBC442C26321}']
    function FirstObservation: IObservation; stdcall;
    function NextObservation(aObservation: IObservation): IObservation; stdcall;
    function Observation(aObservationID: string): IObservation; overload; stdcall;
    function Observation(aIndex: integer): IObservation; overload; stdcall;
    function ObservationCount: Integer; stdcall;
    procedure Add(aObservation: IObservation); stdcall;
    procedure Clear; stdcall;
    procedure CopyTo(aList: ICliOList); stdcall;
    procedure SortByDateTime; stdcall;
  end;

  ICliOReport = interface(IInterface)
    ['{05038AC0-F7A0-46A7-9CD3-602D576C3D5B}']
    function getPage(aPageNumber: integer; aLines: TStrings): boolean; stdcall;
    function getPageCount: integer; stdcall;
    function getSectionBreak(aTitle: string; aRepeatingCharacter: Char; aAlign: TAlignment): ICliOReportItem; stdcall;
    function getText(aLines: TStrings): integer; stdcall;

    procedure setTitle(aTitle: string); stdcall;
    procedure setMargins(aWidth, aHeight: integer); stdcall;

    procedure Refresh(
      aStartDateTime, aStopDateTime: TDateTime;
      aPatient: IPatient;
      aReportItemList: IInterfaceList;
      aIncludeComments: boolean = False); stdcall;
    procedure PrintPage(aPageNumber: Integer; aCanvas: TCanvas; aRect: TRect); stdcall;
  end;

  ICliOReportItem = interface(IInterface)
    ['{1A75F7DA-4363-4D9C-832E-58F6A4E3B100}']
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;
  end;

  (*
  ICliOReportGenerator = interface(IInterface)
    ['{FF09AA03-D531-4E3B-ACC7-A1690CA5A5C7}']
    procedure PrintTestPage; stdcall;
    procedure PrintListView(aListView: TListView); stdcall;
    procedure SetTitle(aTitle: string); stdcall;
    procedure SetPatient(aPatient: IPatient); stdcall;
    procedure AddSubTitle(aSubTitle: string); stdcall;
  end;
  *)

  ICliOLogEntry = interface(IInterface)
    ['{3B1DF6F1-D691-4D9C-AC09-480E4B8C21C6}']
    function getDateTime: TDateTime; stdcall;
    function getStatus: TObservationRecordStatus; stdcall;
    function getPatientID: string; stdcall;
    procedure AddToTreeView(aTTreeNode: TTreeNode); stdcall;
  end;

  ICliOHL7LogEntry = interface(IInterface)
    ['{9DB67DD4-CBAE-41E4-84FF-6AFBE645B722}']
    function getLogID: string; stdcall;
    function getMessageDateTime: TDateTIme; stdcall;
    function getStatus: TCliOHL7LogStatus; stdcall;
    function getMappingTableID: string; stdcall;
    function getPatientID: string; stdcall;
    function getPatientName: string; stdcall;
    function getPatientSSN: string; stdcall;
    function getPatientDOB: TDateTime; stdcall;
    function getPatientSex: string; stdcall;
    function getHL7PatientID: string; stdcall;
    function getHL7DeviceID: string; stdcall;
    function getHL7DeviceLocation: string; stdcall;
    function getHL7Error: TCliOHL7LogErrorType; stdcall;
    function getHL7ErrorText: string; stdcall;

    procedure setPatient(aPatient: IPatient); stdcall;
    procedure setMappingTableID(aMappingTableID: string); stdcall;
  end;

  ICliOHL7Log = interface(IInterface)
    ['{6DDB7C24-958E-4F69-86E7-D7584FCD4EDB}']
    function FirstLogEntry: ICliOHL7LogEntry; stdcall;
    function NextLogEntry(aCliOHL7LogEntry: ICliOHL7LogEntry): ICliOHL7LogEntry; stdcall;
    function LogEntry(aLogID: string): ICliOHL7LogEntry; overload; stdcall;
    function LogEntry(aIndex: integer): ICliOHL7LogEntry; overload; stdcall;
    function LogEntryCount: Integer; stdcall;

    procedure LoadMessageText(aCliOHL7LogEntry: ICliOHL7LogEntry; aList: TStrings); stdcall;
    procedure LoadMessageErrors(aCliOHL7LogEntry: ICliOHL7LogEntry; aList: TStrings); stdcall;
    procedure Refresh; stdcall;
    procedure Clear; stdcall;
  end;

  ICliOSource = interface(IInterface)
    ['{43B26DBE-513B-45F8-9AD3-AC7DF332214D}']
    function getSourceID: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;
  end;

  ICliOAlarm = interface(IInterface)
    ['{0942C5F1-1323-4778-B54C-BB1CE7D76913}']
    function getID: string; stdcall;
    function getPatientID: string; stdcall;
    function getTerm: ITerm; stdcall;
    function getUnit: ITerm; stdcall;
    function getAlarmType: TCliOAlarmType; stdcall;
    function getMinValue: double; stdcall;
    function getMaxValue: double; stdcall;
    function getPickListValues: string; stdcall;
    function getActive: boolean; stdcall;
    function getActivated: boolean; stdcall;
    function getActivationDateTime: TDateTime; stdcall;
    function getActivationByID: string; stdcall;
    function getActivationComment: string; stdcall;
    function getDeactivationDateTime: TDateTime; stdcall;
    function getDeactivationByID: string; stdcall;
    function getDeactivationComment: string; stdcall;

    procedure setPatientID(const aValue: string); stdcall;
    procedure setTerm(const aValue: ITerm); stdcall;
    procedure setUnit(const aValue: ITerm); stdcall;
    procedure setAlarmType(const aValue: TCliOAlarmType); stdcall;
    procedure setMinValue(const aValue: double); stdcall;
    procedure setMaxValue(const aValue: double); stdcall;
    procedure setPickListValues(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setActivationDateTime(const aValue: TDateTime); stdcall;
    procedure setActivationByID(const aValue: string); stdcall;
    procedure setActivationComment(const aValue: string); stdcall;
    procedure setDeactivationDateTime(const aValue: TDateTime); stdcall;
    procedure setDeactivationByID(const aValue: string); stdcall;
    procedure setDeactivationComment(const aValue: string); stdcall;

    function GetAlarm: string; stdcall;
    function GetAlarmSettings: string; stdcall;
    function GetAlarmValue: variant; stdcall;

    function Execute(aValue: Variant; aUnit: ITerm = nil): boolean; stdcall;
    procedure Clear; stdcall;
    procedure Deactivate(aUser: IUser; aComment: string); stdcall;
    procedure Update(aUser: IUser); stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;
  end;

  ICliOAlarmList = interface(IInterface)
    ['{39DC16A1-F5BB-40C6-9762-3ECF750A1F34}']

    function getOnCliOAlarmSet: TOnCliOAlarmSet; stdcall;
    procedure setOnCliOAlarmSet(const aOnCliOAlarmSet: TOnCliOAlarmSet); stdcall;

    function getOnCliOAlarmClear: TOnCliOAlarmClear; stdcall;
    procedure setOnCliOAlarmClear(const aOnCliOAlarm: TOnCliOAlarmClear); stdcall;

    function getAlarmCount: integer; stdcall;
    function getAlarm(aIndex: integer): ICliOAlarm; overload; stdcall;
    function getAlarm(aID: string): ICliOAlarm; overload; stdcall;
    function getNewAlarm: ICliOAlarm; overload; stdcall;
    function getActivatedAlarmCount: integer; stdcall;

    procedure Load(aPatientID: string); stdcall;
    procedure Refresh; stdcall;
    procedure Execute(aValue: variant; aTerm: ITerm; aUnit: ITerm = nil); stdcall;

    property OnCliOAlarmSet: TOnCliOAlarmSet read getOnCliOAlarmSet write setOnCliOAlarmSet;
    property OnCliOAlarmClear: TOnCliOAlarmClear read getOnCliOAlarmClear write setOnCliOAlarmClear;
  end;

function NewCliOService(aTerminology: ITerminology): ICliOService;
function NewAlarmList(aDBConnection: IDBConnection; aTerminology: ITerminology): ICliOAlarmList;
{For use by ICliOReportItem}
procedure WrapThisValue(aText: string; aCol: integer; aOut: TStrings);

implementation

uses
  oDBController_Observations,
  oCliOAlarmList,
  oCliOFiler,
  oCliOLoader,
  oCliOReport;

function NewCliOService(aTerminology: ITerminology): ICliOService;
begin
  TDBController_Observations.Create(aTerminology).GetInterface(ICliOService, Result);
end;

function NewAlarmList(aDBConnection: IDBConnection; aTerminology: ITerminology): ICliOAlarmList;
begin
  TCliOAlarmList.Create(aDBConnection, aTerminology).GetInterface(ICliOAlarmList, Result);
end;

procedure WrapThisValue(aText: string; aCol: integer; aOut: TStrings);
var
  tmp: string;
  i: integer;
begin
  tmp := aText;
  aOut.Clear;
  while tmp <> '' do
    begin
      if Length(tmp) <= aCol then
        begin
          aOut.Add(tmp);
          tmp := '';
        end
      else if Pos(' ', Copy(tmp, 1, aCol)) = 0 then
        begin
          aOut.Add(Copy(tmp, 1, aCol));
          tmp := Copy(tmp, aCol + 1, Length(tmp));
        end
      else
        begin
          i := aCol;
          while Copy(tmp, i, 1) <> ' ' do
            dec(i);
          aOut.Add(Copy(tmp, 1, i - 1));
          tmp := Copy(tmp, i + 1, Length(tmp));
        end;
    end;
end;

end.

