unit oCliOFiler;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 6 $  $Modtime: 6/11/09 8:36a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Observation Filer - aka XML Carnivore!!!!
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCliOFiler.pas $
================================================================================
}
interface

uses
  //Dialogs,
  SysUtils,
  Classes,
  oCliO,
  oCliOTypes,
  oCP_Exceptions,
  oInterfaces,
  XMLDoc,
  XMLIntf;

type
  TCliOFiler = class(TInterfacedObject, ICliOFiler)
  protected
    fDBConnection: IDBConnection;
    fErrorList: TStringList;
    fTerminology: ITerminology;
    //fDocumentType: string;
    //fDocumentVersion: string;
    fParams: array of variant;
    fCommandName: string;
    fCliOAlarmList: ICliOAlarmList;

    {ICliOFiler interface}
    function getNewDocument(aSource: IInterface): ICliOFilerDoc; stdcall;

    function getError(aIndex: integer): string; stdcall;
    function getErrorCount: Integer; stdcall;
    function getErrorLog: WideString; stdcall;

    procedure SetAlarmList(aCliOAlarmList: ICliOAlarmList); stdcall;

    procedure Execute(aDoc: ICliOFilerDoc); stdcall;
    procedure Correct(aError, aCorrection: IObservation; aUser: IUser; aComment: string); stdcall;
    procedure UpdateObservationStatus(aObsID: string; aNewStatus: TObservationRecordStatus); stdcall;
    procedure UpdateObservationAsVerified(aObs: IObservation; aUser: IUser; aComment: string = 'Observation Verified'); stdcall;
    procedure UpdateObservationAsPurged(aObs: IObservation; aUser: IUser; aComment: string); stdcall;
    procedure UpdateObservationAsRescinded(aObs: IObservation; aUser: IUser; aComment: string); stdcall;
    procedure UpdateObservationAsUnverified(aObs: IObservation; aUser: IUser; aComment: string = 'Observation Recovered'); stdcall;

    {Physical db inserts - access internally only}
    procedure InsertObservation(aObs: ICliOFilerObs);
    procedure InsertSet(aSet: ICliOFilerSet);
    procedure InsertAudit(aObsID, aRepID: string; aUser: IUser;
      aType: TObservationAuditType;
      aStatusOriginal, aStatusNew: TObservationRecordStatus;
      aComment: string);

    {utility type stuff}
    procedure ValidateObservation(aObs: ICliOFilerObs); stdcall;
    procedure ValidateSet(aSet: ICliOFilerSet); stdcall;

    procedure LoadPageParams(aNode: IXMLNode);
    procedure LoadPageDeactivationParams(aNode: IXMLNode);
    function FileSupplementalPage(aXMLDocument: IXMLDocument): boolean;
    function FileSupplementalPageDeactivation(aXMLDocument: IXMLDocument): boolean;
    procedure NotifyOfProgress(const aMsg: string; aProgress,aProgressMax: integer);
  public
    OnNotifyOfProgress: TNotifyOnProgress;
    constructor Create(aTerminology: ITerminology; aDBConnection: IDBConnection);
    destructor Destroy; override;
    procedure ActivateSupplementPage(aViewID: string; aActivationDateTime: TDateTime; aActivationByID, aPrimaryFilterID, aSecondaryFilterID, aPatientID, aComment, aDisplayName: string);
    procedure DeactivateSupplementalPage(aSupplementalID: string; aDeactivatedDateTime: TDateTime; aDeactivatedByID: string; aComment: string);
  end;

implementation

uses
  Variants,
  uCPXML,
  oCliOFilerDoc;

{************************* TObservation_Filer *********************************}

procedure TCliOFiler.Correct(aError, aCorrection: IObservation; aUser: IUser; aComment: string);
var
  i: integer;
begin
  InsertAudit(aError.getID, aCorrection.getID, aUser, atCorrection, aError.getStatus, osCorrected, aComment);
  UpdateObservationStatus(aError.getID, osCorrected);

  for i := 0 to aError.getChildCount - 1 do
    begin
      InsertAudit(aError.getChildID(i), aCorrection.getChildID(i), aUser, atCorrection, aError.getStatus, osCorrected, aComment);
      UpdateObservationStatus(aError.getChildID(i), osCorrected);
    end;
end;

constructor TCliOFiler.Create(aTerminology: ITerminology; aDBConnection: IDBConnection);
begin
  inherited Create;
  fErrorList := TStringList.Create;
  fTerminology := aTerminology;
  fDBConnection := aDBConnection;
  OnNotifyOfProgress := NotifyOfProgress; {prevents null pointers}
end;

destructor TCliOFiler.Destroy;
begin
  FreeAndNil(fErrorList);
  inherited;
end;

function TCliOFiler.getNewDocument(aSource: IInterface): ICliOFilerDoc;
var
  aCliOSource: ICliOSource;
begin
  if aSource.QueryInterface(ICliOSource, aCliOSource) = 0 then
    with TCliOFilerDoc.Create(aCliOSource) do
      try
        GetInterface(ICliOFilerDoc, Result);
      except
        Free;
      end
  else
    Result := nil;
end;

procedure TCliOFiler.NotifyOfProgress(const aMsg: string; aProgress,aProgressMax: integer);
begin
  //
end;

procedure TCliOFiler.SetAlarmList(aCliOAlarmList: ICliOAlarmList);
begin
  fCliOAlarmList := aCliOAlarmList;
end;

function TCliOFiler.getError(aIndex: integer): string;
begin
  try
    Result := fErrorList[aIndex];
  except
    Result := Format('Error %d does not exist.', [aIndex]);
  end;
end;

function TCliOFiler.getErrorCount: Integer;
begin
  Result := fErrorList.Count;
end;

function TCliOFiler.getErrorLog: widestring;
var
  i: integer;
begin
  Result := 'CliO Filer Error Log' + #13 + #13 + Format('%d error(s) found in log.', [fErrorList.Count]) + #13;

  for i := 0 to fErrorList.Count - 1 do
    Result := Result + #13 + Format('%3d. %s', [i + 1, fErrorList[i]]);
end;

procedure TCliOFiler.UpdateObservationStatus(aObsID: string; aNewStatus: TObservationRecordStatus);
begin
  fDBConnection.ExecCommand('UpdateObservationStatus', [aObsID, aNewStatus]);
end;

procedure TCliOFiler.UpdateObservationAsVerified(aObs: IObservation; aUser: IUser; aComment: string = 'Observation Verified');
var
  i: integer;
begin
  fErrorList.Clear;
  try
    fDBConnection.BeginTransaction;
    InsertAudit(aObs.getID, '', aUser, atVerified, aObs.getStatus, osVerified, aComment);
    UpdateObservationStatus(aObs.getID, osVerified);
    for i := 0 to aObs.getChildCount - 1 do
      begin
        InsertAudit(aObs.getChildID(i), '', aUser, atVerified, aObs.getStatus, osVerified, aComment);
        UpdateObservationStatus(aObs.getChildID(i), osVerified);
      end;
    fDBConnection.CommitTransaction;
  except
    fDBConnection.RollbackTransaction;
    fErrorList.AddStrings(fDBConnection.ExecResults);
    raise ECPFilerError.CreateFmt('Error Updating Observation, ID = {%s}', [aObs.getID]);
  end;
end;

procedure TCliOFiler.UpdateObservationAsPurged(aObs: IObservation; aUser: IUser; aComment: string);
var
  i: integer;
begin
  fErrorList.Clear;
  try
    fDBConnection.BeginTransaction;
    InsertAudit(aObs.getID, '', aUser, atPurged, aObs.getStatus, osPurged, aComment);
    UpdateObservationStatus(aObs.getID, osPurged);
    for i := 0 to aObs.getChildCount - 1 do
      begin
        InsertAudit(aObs.getChildID(i), '', aUser, atPurged, aObs.getStatus, osPurged, aComment);
        UpdateObservationStatus(aObs.getChildID(i), osPurged);
      end;
    fDBConnection.CommitTransaction;
  except
    fDBConnection.RollbackTransaction;
    raise ECPFilerError.CreateFmt('Error Updating Observation, ID = {%s}', [aObs.getID]);
  end;
end;

procedure TCliOFiler.UpdateObservationAsRescinded(aObs: IObservation; aUser: IUser; aComment: string);
var
  i: integer;
begin
  fErrorList.Clear;
  try
    fDBConnection.BeginTransaction;
    InsertAudit(aObs.getID, '', aUser, atRescission, aObs.getStatus, osRescinded, aComment);
    UpdateObservationStatus(aObs.getID, osRescinded);
    for i := 0 to aObs.getChildCount - 1 do
      begin
        InsertAudit(aObs.getChildID(i), '', aUser, atRescission, aObs.getStatus, osRescinded, aComment);
        UpdateObservationStatus(aObs.getChildID(i), osRescinded);
      end;
    fDBConnection.CommitTransaction;
  except
    fDBConnection.RollbackTransaction;
    raise ECPFilerError.CreateFmt('Error Updating Observation, ID = {%s}', [aObs.getID]);
  end;
end;

procedure TCliOFiler.UpdateObservationAsUnverified(aObs: IObservation;  aUser: IUser; aComment: string);
var
  i: integer;
begin
  fErrorList.Clear;
  try
    fDBConnection.BeginTransaction;
    InsertAudit(aObs.getID, '', aUser, atStatusChange, aObs.getStatus, osUnverified, aComment);
    UpdateObservationStatus(aObs.getID, osUnverified);
    for i := 0 to aObs.getChildCount - 1 do
      begin
        InsertAudit(aObs.getChildID(i), '', aUser, atStatusChange, aObs.getStatus, osUnverified, aComment);
        UpdateObservationStatus(aObs.getChildID(i), osUnverified);
      end;
    fDBConnection.CommitTransaction;
  except
    fDBConnection.RollbackTransaction;
    raise ECPFilerError.CreateFmt('Error Updating Observation, ID = {%s}', [aObs.getID]);
  end;
end;

procedure TCliOFiler.InsertAudit(aObsID, aRepID: string; aUser: IUser; aType: TObservationAuditType; aStatusOriginal, aStatusNew: TObservationRecordStatus; aComment: string);
begin
  fErrorList.Clear;
  try
    fDBConnection.BeginTransaction;
    fDBConnection.ExecCommand('AddObservationAudit', [NewGUID, aType, aObsID, Now, aUser.getDUZ, aStatusOriginal, aStatusNew, aComment]);
    fDBConnection.CommitTransaction;
  except
    fDBConnection.RollbackTransaction;
    raise ECPFilerError.Create('Error filing audit record');
  end;
end;

procedure TCliOFiler.InsertObservation(aObs: ICliOFilerObs);
var
  aTerm: ITerm;
  i: integer;

  procedure LoadParams;
  begin
    try
      SetLength(fParams, 19);
      fParams[0] := aObs.getID;
      fParams[1] := ''; {Parent ID}
      fParams[2] := aObs.getFacilityID;
      fParams[3] := aObs.getFacilityLocationID;
      fParams[4] := aObs.getObservedDateTime;
      fParams[5] := aObs.getObservedByID;
      fParams[6] := aObs.getTermID;
      fParams[7] := aObs.getPatientID;
      fParams[8] := aObs.getObservationStatus;
      fParams[9] := aObs.getValue;
      fParams[10] := aObs.getSource;
      fParams[11] := aObs.getSourceComment;
      fParams[12] := aObs.getSourceDataItemID;
      fParams[13] := aObs.getSourceVersion;
      fParams[14] := aObs.getEnteredDateTime;
      fParams[15] := aObs.getEnteredByID;
      fParams[16] := ''; {Child Order}
      fParams[17] := aObs.getComment;
      fParams[18] := aObs.getObservationRange;
    except
      on E: Exception do
        raise ECPFilerError.Create('Error extracting Observation Values.');
    end;
  end;

  procedure InsertQualifiers(aObsID: string);
  var
    aQualType: TTermType;
    aQualID: string;
  begin
    for aQualType := Low(TTermType) to High(TTermType) do
      if aQualType in TERMQUALIFIERS then
        begin
          aQualID := aObs.getQualifier(aQualType);
          if aQualID <> '' then
            fDBConnection.ExecCommand('AddQualifier', [aObsID, aQualID]);
        end;
  end;

  procedure AddToSets(aObsID: string);
  var
    ts: TStringList;
    i: integer;
  begin
    ts := TStringList.Create;
    try
      aObs.GetSetIDs(ts);
      for i := 0 to ts.Count - 1 do
        fDBConnection.ExecCommand('AddObservationToSet', [ts[i], aObsID]);
    finally
      FreeAndNil(ts);
    end;
  end;

begin
  {An assumption is made that the filer has already ran a validation on this ICliOFilerObs}
  fErrorList.Clear;
  try
    {Primary observation}
    LoadParams;
    fDBConnection.ExecCommand('AddObservation', fParams);
    InsertQualifiers(aObs.getID);
    AddToSets(aObs.getID);

    if Assigned(fCliOAlarmList) then
      fCliOAlarmList.Execute(
        aObs.getValue,
        fTerminology.GetTerm(aObs.getTermID),
        fTerminology.GetTerm(aObs.getUnitID));

    {Any Kids?}
    aTerm := fTerminology.GetTerm(aObs.getTermID);
    if aTerm.getChildCount > 0 then
      for i := 0 to aTerm.getChildCount - 1 do
        begin
          LoadParams; {Get fresh copy of original ICliOFilerObs}
          {Assign appropriate values to the observation as a child}
          fParams[0] := NewGUID;
          fParams[1] := aObs.getID;
          fParams[6] := aTerm.getChildTerm(i).getID;
          fParams[9] := aTerm.getChildValue(i, aObs.getValue);
          fParams[16] := i + 1;
          fDBConnection.ExecCommand('AddObservation', fParams);

          {Setup aObs with the appropriate unit for this child if any}
          if aTerm.getChildUnit(i) <> nil then
            aObs.setQualifier(ttUnit, aTerm.getChildUnit(i).getID)
          else
            aObs.setQualifier(ttUnit, '');
          InsertQualifiers(fParams[0]);

          if Assigned(fCliOAlarmList) then
            fCliOAlarmList.Execute(
              aTerm.getChildValue(i, aObs.getValue),
              aTerm.getChildTerm(i),
              aTerm.getChildUnit(i));

        end;
    {Woo Hoo!}
  except
    on E: Exception do
      begin
        fErrorList.Add(E.Message);
        raise ECPFilerError.Create('Error filing Observation');
      end;
  end;
end;

procedure TCliOFiler.InsertSet(aSet: ICliOFilerSet);
begin
  fErrorList.Clear;

  try
    SetLength(fParams, 5);
    fParams[0] := aSet.getID;
    fParams[1] := aSet.getEnteredDateTime;
    fParams[2] := aSet.getEnteredByID;
    if aSet.getIsPublic then
      fParams[3] := 1
    else
      fParams[3] := 0;
    fParams[4] := aSet.getComment;
    fDBConnection.ExecCommand('AddSet', fParams);
  except
    on E: Exception do
      begin
        fErrorList.Add(E.Message);
        raise ECPFilerError.Create('Error filing Set');
      end;
  end;
end;

procedure TCliOFiler.Execute(aDoc: ICliOFilerDoc);
var
  aSet: ICliOFilerSet;
  aObs: ICliOFilerObs;
begin
  fErrorList.Clear;

  try
    OnNotifyOfProgress('Verifying data ...',0,0);

    aObs := aDoc.getFirstObs;
    while Assigned(aObs) do
      try
        ValidateObservation(aObs);
      finally
        aObs := aDoc.getNextObs(aObs);
      end;

    aSet := aDoc.getFirstSet;
    while Assigned(aSet) do
      try
        ValidateSet(aSet);
      finally
        aSet := aDoc.getNextSet(aSet);
      end;

  except
    on E: Exception do
      fErrorList.Add(e.Message);
  end;

  if fErrorList.Count > 0 then
    raise ECPFilerError.Create('Error validating Filer Document.');

  OnNotifyOfProgress('Filing data ...',0,0);

  try
    fDBConnection.BeginTransaction;

    {Do the sets first}
    aSet := aDoc.getFirstSet;
    while Assigned(aSet) do
      try
        InsertSet(aSet);
        aSet := aDoc.getNextSet(aSet);
      except
        raise ECPFilerError.Create('Error filing set.');
      end;

    {Now do the observations}
    aObs := aDoc.getFirstObs;
    while Assigned(aObs) do
      try
        InsertObservation(aObs);
        aObs := aDoc.getNextObs(aObs);
      except
        raise ECPFilerError.Create('Error filing observation.');
      end;

    fDBConnection.CommitTransaction;
  except
    on E: Exception do
      begin
        fDBConnection.RollbackTransaction;
        fErrorList.Add(e.Message);
        raise ECPFilerError.CreateFmt('Error writing observations. Error: %s', [e.Message]);
      end;
  end;
end;

procedure TCliOFiler.ValidateObservation(aObs: ICliOFilerObs);
var
  aTerm: ITerm;
  aChildTerm: ITerm;
  aUnit: ITerm;
  aChildUnit: ITerm;
  aValue: string;
  aChildValue: string;
  aChildRange: TObservationRange;
  aID: string;
  i: integer;
begin
  try
    aObs.setObservationRange(orNormal);
    aValue := aObs.getValue;

    aID := aObs.getTermID;
    if aID <> '' then
      aTerm := fTerminology.GetTerm(aID)
    else
      aTerm := nil;

    aID := aObs.getUnitID;
    if aID <> '' then
      aUnit := fTerminology.GetTerm(aID)
    else
      aUnit := nil;

    {Do we have a term?}
    if aTerm = nil then
      begin
        fErrorList.Add('Missing term in observation');
        Exit;
      end;

    {Do we have a value?}
    if aValue = '' then
      begin
        fErrorList.Add('Missing value in observation');
        Exit;
      end;

    {Do we require a unit?}
    if (aUnit = nil) and (aTerm.getUnitCount > 0) then
      begin
        fErrorList.Add(Format('Term %s requires a unit designation for filing.', [aTerm.getTerm]));
        Exit;
      end;

    {Is it a valid unit?}
    if Assigned(aUnit) and (not aTerm.getIsValidUnit(aUnit)) then
      begin
        fErrorList.Add(Format('Unit %s is not valid with term %s', [aUnit.getTerm, aTerm.getTerm]));
        Exit;
      end;

    case aTerm.getChildCount of
      0:
        try
          aObs.setObservationRange(aTerm.Validate(aUnit, aValue));
        except
          on E: Exception do
            fErrorList.Add(E.Message);
        end;
    else
      for i := 0 to aTerm.getChildCount - 1 do
        begin
          aChildTerm := aTerm.getChildTerm(i);
          aChildUnit := aTerm.getChildUnit(i);
          aChildValue := aTerm.getChildValue(i, aValue);
          try
            aChildRange := aChildTerm.Validate(aChildUnit, aChildValue);
            {Only set the Observation Range on the first out of range value found}
            if (aChildRange in [orLow, orHigh]) and (aObs.getObservationRange = orNormal) then
              aObs.setObservationRange(aChildRange);
          except
            on E: Exception do
              fErrorList.Add(e.Message);
          end;
        end;
    end;

  except
    on E: Exception do
      fErrorList.Add('Unknown error validating term/unit pair');
  end;
end;

procedure TCliOFiler.ValidateSet(aSet: ICliOFilerSet);
begin
  try
    if Length(aSet.getComment) > 250 then
      fErrorList.Add('Comment must be between 0 and 250 characters.');
  except
    on E: Exception do
      fErrorList.Add('Unknown error validating set');
  end;
end;

{******************************************************************************}
{******************************************************************************}
{******************************************************************************}
{                not sure why this stuff is still here                         }
{******************************************************************************}
{******************************************************************************}
{******************************************************************************}

procedure TCliOFiler.ActivateSupplementPage(aViewID: string; aActivationDateTime: TDateTime; aActivationByID, aPrimaryFilterID, aSecondaryFilterID, aPatientID, aComment, aDisplayName: string);
begin
  try
    {
    SetLength(fParams, 10);
    fParams[0] := NewGUID;
    fParams[1] := aViewID;
    fParams[2] := aPatientID;
    fParams[3] := aPrimaryFilterID;
    fParams[4] := aSecondaryFilterID;
    fParams[5] := 1;
    fParams[6] := aActivationDateTime;
    fParams[7] := aActivationByID;
    fParams[8] := aComment;
    fParams[9] := aDisplayName;
    }
    if not fDBConnection.ExecCommand(
      'AddSupplementalPage',
      [NewGUID, aViewID, aPatientID, aPrimaryFilterID, aSecondaryFilterID, 1,
      aActivationDateTime, aActivationByID, aComment, aDisplayName]) then
      raise Exception.Create('');
  except
    raise Exception.Create('Error Filing New Supplement Page.');
  end;
end;

procedure TCliOFiler.DeactivateSupplementalPage(
  aSupplementalID: string; aDeactivatedDateTime: TDateTime;
  aDeactivatedByID, aComment: string);
begin
  try
    if not fDBConnection.ExecCommand(
      'UpdateSupplementalPage',
      [aSupplementalID, aDeactivatedDateTime, aDeactivatedByID, 0, aComment]) then
      raise exception.Create('');
  except
    raise Exception.Create('Unable to properly deactivate supplemental page');
  end;
end;

procedure TCliOFiler.LoadPageParams(aNode: IXMLNode);
begin
  try
    fCommandName := 'AddSupplementalPage';
    SetLength(fParams, 10);
    fParams[0] := XMLValueAsStr(aNode, 'SUPPLEMENTAL_VIEW_ID');
    fParams[1] := XMLValueAsStr(aNode, 'VIEW_ID');
    fParams[2] := XMLValueAsStr(aNode, 'PATIENT_ID');
    fParams[3] := XMLValueAsStr(aNode, 'PRIMARY_FILTER_ID');
    fParams[4] := XMLValueAsStr(aNode, 'SECONDARY_FILTER_ID');
    fParams[5] := XMLValueAsInt(aNode, 'STATUS');
    fParams[6] := XMLValueAsStr(aNode, 'ACTIVATED_DATE_TIME');
    fParams[7] := XMLValueAsStr(aNode, 'ACTIVATED_BY_ID');
    fParams[8] := XMLValueAsStr(aNode, 'ACTIVATED_COMMENT');
    fParams[9] := XMLValueAsStr(aNode, 'DISPLAY_NAME');
  except
    raise Exception.Create('Error parsing SUPPLEMENTAL PAGE XML');
  end;
end;

procedure TCliOFiler.LoadPageDeactivationParams(aNode: IXMLNode);
begin
  try
    fCommandName := 'UpdateSupplementalPage';
    SetLength(fParams, 5);
    fParams[0] := XMLValueAsStr(aNode, 'SUPPLEMENTAL_VIEW_ID');
    fParams[1] := XMLValueAsStr(aNode, 'DEACTIVATED_DATE_TIME');
    fParams[2] := XMLValueAsStr(aNode, 'DEACTIVATED_BY_ID');
    fParams[3] := XMLValueAsInt(aNode, 'STATUS');
    fParams[4] := XMLValueAsStr(aNode, 'DEACTIVATED_COMMENT');
  except
    raise Exception.Create('Error parsing SUPPLEMENTAL PAGE XML');
  end;
end;

function TCliOFiler.FileSupplementalPage(aXMLDocument: IXMLDocument): boolean;
var
  aPageNode: IXMLNode;
begin
  with aXMLDocument do
    try
      Active := True;
      fDBConnection.BeginTransaction;

      aPageNode := DocumentElement.ChildNodes.FindNode('PAGE');
      if aPageNode = nil then
        raise Exception.Create('No page definition found in the document');

      LoadPageParams(aPageNode);

      if not fDBConnection.ExecCommand(fCommandName, fParams) then
        raise Exception.Create('Unable to insert supplemental page');

      fDBConnection.CommitTransaction;
      Result := True;
    except
      on E: Exception do
        begin
          fDBConnection.RollbackTransaction;
          fErrorList.Add(e.Message);
          Result := False;
        end;
    end;
end;

function TCliOFiler.FileSupplementalPageDeactivation(aXMLDocument: IXMLDocument): boolean;
var
  aPageNode: IXMLNode;
begin
  with aXMLDocument do
    try
      Active := True;
      fDBConnection.BeginTransaction;

      aPageNode := DocumentElement.ChildNodes.FindNode('PAGE');
      if aPageNode = nil then
        raise Exception.Create('No page definition found in the document');

      LoadPageDeactivationParams(aPageNode);

      if not fDBConnection.ExecCommand(fCommandName, fParams) then
        raise Exception.Create('Unable to insert supplemental page');

      fDBConnection.CommitTransaction;
      Result := True;
    except
      on E: Exception do
        begin
          fDBConnection.RollbackTransaction;
          fErrorList.Add(e.Message);
          Result := False;
        end;
    end;
end;

end.

