unit oCliOFilerDoc;

interface

uses
  Classes,
  oCliO,
  oCliOTypes,
  oCP_Exceptions,
  oInterfaces,
  XMLDoc,
  XMLIntf;

type
  TCliOFilerDoc = class(TInterfacedObject, ICliOFilerDoc, ICliOFilerXML)
  private
    fDocRoot: IXMLDocument;
    fObsRoot: IXMLNode;
    fSetRoot: IXMLNode;
    fSetList: IInterfaceList;
    fObsList: IInterfaceList;
    fCliOSource: ICliOSource;

    {ICliOFilerXML interface}
    function getXML: WideString; stdcall;

    {ICliOFilerDoc interface}
    function getFirstObs: ICliOFilerObs; stdcall;
    function getNextObs(aObs: ICliOFilerObs): ICliOFilerObs; stdcall;
    function getFirstSet: ICliOFilerSet; stdcall;
    function getNextSet(aSet: ICliOFilerSet): ICliOFilerSet; stdcall;
    function CreateNewObs(aSetID: string): ICliOFilerObs; stdcall;
    function CreateNewSet: ICliOFilerSet; stdcall;

    procedure SaveToFile(aFileName: string); stdcall;
  public
    constructor Create(aCliOSource: ICliOSource);
    destructor Destroy; override;
  end;

implementation

uses
  oCliOFilerSet,
  oCliOFilerObs;

{ TObservation_Filer_Document }

constructor TCliOFilerDoc.Create(aCliOSource: ICliOSource);
begin
  inherited Create;
  fCliOSource := aCLiOSource;
  {Create the lists for the objects}
  TInterfaceList.Create.GetInterface(IInterfaceList, fSetList);
  TInterfaceList.Create.GetInterface(IInterfaceList, fObsList);

  {Setup the XML document}
  with NewXMLDocument do
    begin
      DocumentElement := AddChild('CLIO');
      DocumentElement.Attributes['TYPE'] := 'CLIO_FILER_DOCUMENT';
      DocumentElement.Attributes['VERSION'] := 1.0;
      fSetRoot := DocumentElement.AddChild('SETS');
      fObsRoot := DocumentElement.AddChild('OBSERVATIONS');
      QueryInterface(IXMLDocument, fDocRoot);
    end;
end;

destructor TCliOFilerDoc.Destroy;
begin
  fObsList.Clear;
  fSetList.Clear;
  fObsRoot := nil;
  fSetRoot := nil;
  fDocRoot := nil;
  inherited;
end;

function TCliOFilerDoc.CreateNewObs(aSetID: string): ICliOFilerObs;
begin
  //TCliOFilerObs.Create(fObsRoot).GetInterface(ICliOFilerObs, Result);
  //fObsList.Add(Result);
  with TCliOFilerObs.Create(fObsRoot, fCliOSource) do
    try
      GetInterface(ICliOFilerObs, Result);
      fObsList.Add(Result);
    except
      Free;
    end;
end;

function TCliOFilerDoc.CreateNewSet: ICliOFilerSet;
begin
  TCliOFilerSet.Create(fSetRoot).GetInterface(ICliOFilerSet, Result);
  fSetList.Add(Result);
end;

function TCliOFilerDoc.getXML: WideString;
begin
  Result := FormatXMLData(fDocRoot.XML.Text);
end;

procedure TCliOFilerDoc.SaveToFile(aFileName: string);
begin
  with TStringList.Create do
    try
      Text := FormatXMLData(fDocRoot.XML.Text);
      //SaveToFile(aFileName);
    finally
      Free;
    end;
end;

function TCliOFilerDoc.getFirstObs: ICliOFilerObs;
begin
  if fObsList.Count > 0 then
    fObsList[0].QueryInterface(ICliOFilerObs, Result)
  else
    Result := nil;
end;

function TCliOFilerDoc.getFirstSet: ICliOFilerSet;
begin
  if fSetList.Count > 0 then
    fSetList[0].QueryInterface(ICliOFilerSet, Result)
  else
    Result := nil;
end;

function TCliOFilerDoc.getNextObs(aObs: ICliOFilerObs): ICliOFilerObs;
var
  i: integer;
begin
  i := fObsList.IndexOf(aObs) + 1;
  if i < fObsList.Count then
    fObsList[i].QueryInterface(ICliOFilerObs, Result)
  else
    Result := nil;
end;

function TCliOFilerDoc.getNextSet(aSet: ICliOFilerSet): ICliOFilerSet;
var
  i: integer;
begin
  i := fSetList.IndexOf(aSet) + 1;
  if i < fSetList.Count then
    fSetList[i].QueryInterface(ICliOFilerSet, Result)
  else
    Result := nil;
end;

end.

