unit oCliOFilerObs;

interface

uses
  Classes,
  oCliO,
  oCliOTypes,
  oCP_Exceptions,
  oInterfaces,
  SysUtils,
  XMLDoc,
  XMLIntf;

type
  TCliOFilerObsField = (
    ofFacilityID,
    ofFacilityLocationID,
    ofEnteredDateTime,
    ofEnteredByID,
    ofComment,
    ofPatientID,
    ofParentID,
    ofSetID,
    ofObservedDateTime,
    ofObservedByID,
    ofTermID,
    ofStatus,
    ofSValue,
    ofSource,
    ofSourceComments,
    ofSourceDataItemID,
    ofSourceVersion,
    ofCorrectionForID,
    ofUnitID,
    ofCorrectionReason,
    ofObservationRange);

  TCliOFilerObs = class(TInterfacedObject, ICliOFilerObs, ICliOFilerXML)
  private
    fNode: IXMLNode;
    fSetNode: IXMLNode;

    function getStringValue(aField: TCliOFilerObsField): string;
    function getIntegerValue(aField: TCliOFilerObsField): integer;
    procedure setIntegerValue(aField: TCliOFilerObsField; aValue: integer);
    procedure setStringValue(aField: TCliOFilerObsField; aValue: string);

    {ICliOFilerObs interface}
    function getID: string; stdcall;
    function getFacilityID: string; stdcall;
    function getFacilityLocationID: string; stdcall;
    function getPatientID: string; stdcall;
    function getEnteredDateTime: TDateTime; stdcall;
    function getEnteredByID: string; stdcall;
    function getComment: string; stdcall;
    //function getSetID: string; stdcall;
    function getCorrectionForID: string; stdcall;
    function getCorrectionReason: string; stdcall;
    function getObservationStatus: TObservationRecordStatus; stdcall;
    function getObservationRange: TObservationRange; stdcall;
    function getObservedByID: string; stdcall;
    function getObservedDateTime: TDateTime; stdcall;
    function getParentID: string; stdcall;
    function getSource: string; stdcall;
    function getSourceComment: string; stdcall;
    function getSourceDataItemID: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;
    function getTermID: string; stdcall;
    function getUnitID: string; stdcall;
    function getValue: string; stdcall;

    function getXML: WideString; stdcall;

    procedure setFacilityID(const aValue: string); stdcall;
    procedure setFacilityLocationID(const aValue: string); stdcall;
    procedure setPatientID(const aValue: string); stdcall;
    procedure setEnteredDateTime(const aValue: TDateTime); stdcall;
    procedure setEnteredByID(const aValue: string); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setCorrectionForID(const aValue: string); stdcall;
    procedure setCorrectionReason(const aValue: string); stdcall;
    procedure setObservationStatus(const aValue: TObservationRecordStatus); stdcall;
    procedure setObservationRange(const aValue: TObservationRange); stdcall;
    procedure setObservedByID(const aValue: string); stdcall;
    procedure setObservedDateTime(const aValue: TDateTime); stdcall;
    procedure setParentID(const aValue: string); stdcall;
    procedure setTermID(const aValue: string); stdcall;
    procedure setUnitID(const aValue: string); stdcall;
    procedure setValue(const aValue: string); stdcall;

    procedure setQualifier(aTermType: TTermType; aTermID: string); stdcall;
    function getQualifier(aTermType: TTermType): string; stdcall;
    procedure setSource(const aValue: string); stdcall;
    procedure setSourceComment(const aValue: string); stdcall;
    procedure setSourceDataItemID(const aValue: string); stdcall;
    procedure setSourceVersion(const aValue: string); stdcall;
    procedure AddToSet(aSetID: string); stdcall;
    procedure GetSetIDs(aList: TStrings); stdcall;
  public
    constructor Create(aXMLRoot: IXMLNode; aCliOSource: ICliOSource);
  end;

implementation

uses
  uCPXML;

const
  XMLTAGS: array[TCliOFilerObsField] of string = (
    'FACILITY_ID',
    'FACILITY_LOCATION_ID',
    'ENTERED_DATE_TIME',
    'ENTERED_BY_ID',
    'COMMENT',
    'PATIENT_ID',
    'PARENT_ID',
    'BATTERY_ID',
    'OBSERVED_DATE_TIME',
    'OBSERVED_BY_ID',
    'TERM_ID', 'STATUS',
    'SVALUE',
    'SOURCE',
    'SOURCE_COMMENTS',
    'SOURCE_DATA_ITEM_ID',
    'SOURCE_VERSION',
    'CORRECTION_FOR_ID',
    'UNIT_ID',
    'CORRECTION_REASON',
    'OBSERVATION_RANGE');

{ ----------------------TCliOFilerObs ----------------------------------- }

procedure TCliOFilerObs.AddToSet(aSetID: string);
begin
  if aSetID <> '' then
    with fSetNode.AddChild('SET') do
      ChildValues['VALUE'] := aSetID;
end;

constructor TCliOFilerObs.Create(aXMLRoot: IXMLNode; aCliOSource: ICliOSource);
begin
  inherited Create;
  fNode := aXMLRoot.AddChild('OBSERVATION');
  fNode.Attributes['ID'] := NewGUID;
  fSetNode := fNode.AddChild('SETS');

  fNode.ChildValues['SOURCE'] := aCliOSource.getSourceApplication;
  fNode.ChildValues['SOURCE_DATA_ITEM_ID'] := aCliOSource.getSourceID;
  fNode.ChildValues['SOURCE_VERSION'] := aCliOSource.getSourceVersion;

  if aCliOSource.getSourceTrusted then
    fNode.ChildValues['STATUS'] := osVerified
  else
    fNode.ChildValues['STATUS'] := osUnverified;

  //fNode.ChildValues['SOURCE_COMMENTS'] := aCliOSource.Comments;
end;

function TCliOFilerObs.getStringValue(aField: TCliOFilerObsField): string;
begin
  Result := XMLValueAsStr(fNode, XMLTAGS[aField]);
end;

function TCliOFilerObs.getIntegerValue(aField: TCliOFilerObsField): integer;
begin
  Result := XMLValueAsInt(fNode, XMLTAGS[aField]);
end;

procedure TCliOFilerObs.setIntegerValue(aField: TCliOFilerObsField; aValue: integer);
begin
  fNode.ChildValues[XMLTAGS[aField]] := aValue;
end;

procedure TCliOFilerObs.setStringValue(aField: TCliOFilerObsField; aValue: string);
begin
  fNode.ChildValues[XMLTAGS[aField]] := aValue;
end;

function TCliOFilerObs.getID: string;
begin
  Result := fNode.Attributes['ID'];
end;

function TCliOFilerObs.getFacilityID: string;
begin
  Result := getStringValue(ofFacilityID);
end;

function TCliOFilerObs.getFacilityLocationID: string;
begin
  Result := getStringValue(ofFacilityLocationID);
end;

function TCliOFilerObs.getEnteredByID: string;
begin
  Result := getStringValue(ofEnteredByID);
end;

function TCliOFilerObs.getEnteredDateTime: TDateTime;
begin
  try
    Result := XMLStringToDateTime(getStringValue(ofEnteredDateTime));
  except
    Result := 0;
  end;
end;

function TCliOFilerObs.getComment: string;
begin
  Result := getStringValue(ofComment);
end;

function TCliOFilerObs.getPatientID: string;
begin
  Result := getStringValue(ofPatientID);
end;

{
function TCliOFilerObs.getSetID: string;
begin
  Result := getStringValue(ofSetID);
end;
}

function TCliOFilerObs.getCorrectionForID: string;
begin
  Result := getStringValue(ofCorrectionForID);
end;

function TCliOFilerObs.getCorrectionReason: string;
begin
  Result := getStringValue(ofCorrectionReason);
end;

function TCliOFilerObs.getObservationRange: TObservationRange;
begin
  Result := TObservationRange(getIntegerValue(ofObservationRange));
end;

function TCliOFilerObs.getObservationStatus: TObservationRecordStatus;
begin
  Result := TObservationRecordStatus(getIntegerValue(ofStatus));
end;

function TCliOFilerObs.getObservedByID: string;
begin
  Result := getStringValue(ofObservedByID);
end;

function TCliOFilerObs.getObservedDateTime: TDateTime;
begin
  Result := XMLStringToDateTime(getStringValue(ofObservedDateTime));
end;

function TCliOFilerObs.getParentID: string;
begin
  Result := getStringValue(ofParentID);
end;

procedure TCliOFilerObs.getSetIDs(aList: TStrings);
var
  aNode: IXMLNode;
begin
  try
    aList.Clear;
    aNode := fSetNode.ChildNodes.First;
    while Assigned(aNode) do
      try
        aList.Add(aNode.ChildValues['VALUE']);
      finally
        aNode := aNode.NextSibling;
      end;
  except
    aList.Clear;
  end;
end;

function TCliOFilerObs.getSource: string;
begin
  Result := getStringValue(ofSource);
end;

function TCliOFilerObs.getSourceComment: string;
begin
  Result := getStringValue(ofSourceComments);
end;

function TCliOFilerObs.getSourceDataItemID: string;
begin
  Result := getStringValue(ofSourceDataItemID);
end;

function TCliOFilerObs.getSourceTrusted: boolean;
begin
  result := false;
end;

function TCliOFilerObs.getSourceVersion: string;
begin
  Result := getStringValue(ofSourceVersion);
end;

function TCliOFilerObs.getTermID: string;
begin
  Result := getStringValue(ofTermID);
end;

function TCliOFilerObs.getUnitID: string;
begin
  Result := getStringValue(ofUnitID);
end;

function TCliOFilerObs.getValue: string;
begin
  Result := getStringValue(ofSValue);
end;

function TCliOFilerObs.getXML: WideString;
begin
  Result := fNode.XML;
end;

{ ---------------------- Property setters -------------------------------------}

procedure TCliOFilerObs.setEnteredByID(const aValue: string);
begin
  setStringValue(ofEnteredByID, aValue);
end;

procedure TCliOFilerObs.setEnteredDateTime(const aValue: TDateTime);
begin
  setStringValue(ofEnteredDateTime, DateTimeToXMLString(aValue));
end;

procedure TCliOFilerObs.setFacilityLocationID(const aValue: string);
begin
  setStringValue(ofFacilityLocationID, aValue);
end;

procedure TCliOFilerObs.setFacilityID(const aValue: string);
begin
  setStringValue(ofFacilityID, aValue);
end;

procedure TCliOFilerObs.setComment(const aValue: string);
begin
  setStringValue(ofComment, aValue);
end;

procedure TCliOFilerObs.setPatientID(const aValue: string);
begin
  setStringValue(ofPatientID, aValue);
end;

procedure TCliOFilerObs.setCorrectionForID(const aValue: string);
begin
  setStringValue(ofCorrectionForID, aValue);
end;

procedure TCliOFilerObs.setCorrectionReason(const aValue: string);
begin
  setStringValue(ofCorrectionReason, aValue);
end;

procedure TCliOFilerObs.setObservationRange(const aValue: TObservationRange);
begin
  setIntegerValue(ofObservationRange, Ord(aValue));
end;

procedure TCliOFilerObs.setObservationStatus(const aValue: TObservationRecordStatus);
begin
  setIntegerValue(ofStatus, Ord(aValue));
end;

procedure TCliOFilerObs.setObservedByID(const aValue: string);
begin
  setStringValue(ofObservedByID, aValue);
end;

procedure TCliOFilerObs.setObservedDateTime(const aValue: TDateTime);
begin
  setStringValue(ofObservedDateTime, DateTimeToXMLString(aValue));
end;

procedure TCliOFilerObs.setParentID(const aValue: string);
begin
  setStringValue(ofParentID, aValue);
end;

procedure TCliOFilerObs.setSource(const aValue: string);
begin
  setStringValue(ofSource, aValue);
end;

procedure TCliOFilerObs.setSourceComment(const aValue: string);
begin
  setStringValue(ofSourceComments, aValue);
end;

procedure TCliOFilerObs.setSourceDataItemID(const aValue: string);
begin
  setStringValue(ofSourceDataItemID, aValue);
end;

procedure TCliOFilerObs.setSourceVersion(const aValue: string);
begin
  setStringValue(ofSourceVersion, aValue);
end;

procedure TCliOFilerObs.setTermID(const aValue: string);
begin
  setStringValue(ofTermID, aValue);
end;

procedure TCliOFilerObs.setUnitID(const aValue: string);
begin
  setStringValue(ofUnitID, aValue);
end;

procedure TCliOFilerObs.setValue(const aValue: string);
begin
  setStringValue(ofSValue, aValue);
end;

procedure TCliOFilerObs.setQualifier(aTermType: TTermType; aTermID: string);
begin
  if aTermType in TERMQUALIFIERS then
    if aTermID <> '' then
      fNode.ChildValues[TERMTYPE_XMLTAG[aTermType]] := aTermID
    else
      if fNode.ChildNodes.FindNode(TERMTYPE_XMLTAG[aTermType]) <> nil then
        fNode.ChildNodes.Delete(TERMTYPE_XMLTAG[aTermType]);
end;

function TCliOFilerObs.getQualifier(aTermType: TTermType): string;
begin
  Result := XMLValueAsStr(fNode, TERMTYPE_XMLTAG[aTermType]);
end;

end.

