unit oCliOList;

interface

uses
  Classes,
  SysUtils,
  oCliO,
  oInterfaces;

type
  TCliOList = class(TInterfacedObject, ICliOList)
  private
    fList: IInterfaceList;
    function FirstObservation: IObservation; stdcall;
    function NextObservation(aObservation: IObservation): IObservation; stdcall;
    function Observation(aObservationID: string): IObservation; overload; stdcall;
    function Observation(aIndex: integer): IObservation; overload; stdcall;
    function ObservationCount: Integer; stdcall;
    procedure Clear; stdcall;
    procedure CopyTo(aList: ICliOList); stdcall;
    procedure Add(aObservation: IObservation); stdcall;
    procedure SortByDateTime; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCliOList: TCliOList;

implementation

function NewCliOList: TCliOList;
begin
  Result := TCliOList.Create;
end;

{ TObservation_List }

constructor TCliOList.Create;
begin
  inherited Create;
  TInterfaceList.Create.GetInterface(IInterfaceList, fList);
end;

destructor TCliOList.Destroy;
begin
  fList.Clear;
  fList := nil;
  inherited;
end;

procedure TCliOList.Clear;
begin
  fList.Clear;
end;

procedure TCliOList.CopyTo(aList: ICliOList);
{Not sure if this is the most efficient way of doing this.}
var
  i: integer;
  aCopy: IObservation;
begin
  for i := 0 to fList.Count - 1 do
    if fList[i].QueryInterface(IObservation, aCopy) = 0 then
      aList.Add(aCopy);
end;

function TCliOList.FirstObservation: IObservation;
begin
  if fList.Count > 0 then
    fList[0].QueryInterface(IObservation, Result)
  else
    Result := nil;
end;

function TCliOList.NextObservation(aObservation: IObservation): IObservation;
var
  i: Integer;
begin
  i := fList.IndexOf(aObservation) + 1;
  if (i > 0) and (i < fList.Count) then
    fList[i].QueryInterface(IObservation, Result)
  else
    Result := nil;
end;

function TCliOList.Observation(aObservationID: string): IObservation;
var
  i: integer;
begin
  if aObservationID <> '' then
    for i := 0 to fList.Count - 1 do
      begin
        fList[i].QueryInterface(IObservation, Result);
        if Result.getID = aObservationID then
          Exit;
      end;
  Result := nil;
end;

function TCliOList.Observation(aIndex: integer): IObservation;
begin
  if aIndex < fList.Count then
    fList[aIndex].QueryInterface(IObservation, Result)
  else
    Result := nil;
end;

function TCliOList.ObservationCount: Integer;
begin
  Result := fList.Count;
end;

procedure TCliOList.Add(aObservation: IObservation);
begin
  fList.Add(aObservation);
end;

procedure TCliOList.SortByDateTime;
var
  i, j: integer;
  aObsA, aObsB: IObservation;
begin
  for i := 0 to fList.Count - 2 do
    for j := i + 1 to fList.Count - 1 do
      begin
        fList[i].QueryInterface(IObservation, aObsA);
        fList[j].QueryInterface(IObservation, aObsB);
        if aObsB.getObservedDateTime < aObsA.getObservedDateTime then
          fList.Exchange(i, j);
      end;
end;

end.

