unit oCliOReportSection;

interface

uses
  Classes,
  SysUtils,
  XMLDoc,
  XMLIntf,
  oCliOReportWriter;

type
  TCliOReportSection = class(TInterfacedObject, ICliOReportSection)
  private
    fSectionNode: IXMLNode;
    fColumnNode: IXMLNode;
    fItemNode: IXMLNode;

    fColumns: IInterfaceList;
    fItems: IInterfaceList;

    function getTitle: string; stdcall;
    function getPrintHeader: boolean; stdcall;
    function getPrintSubHeader: boolean; stdcall;
    function getTopLevel: boolean; stdcall;

    procedure setTitle(aValue: string); stdcall;
    procedure setPrintHeader(aValue: boolean); stdcall;
    procedure setPrintSubHeader(aValue: boolean); stdcall;
    procedure setTopLevel(aValue: boolean); stdcall;

    function FirstColumn: ICliOReportColumn; stdcall;
    function NextColumn(aCliOReportColumn: ICliOReportColumn): ICliOReportColumn; stdcall;
    function Column(aIndex: Integer): ICliOReportColumn; stdcall;
    function ColumnCount: Integer; stdcall;

    function Item(aIndex: Integer): ICliOReportItem; stdcall;
    function ItemCount: integer; stdcall;

    function NewColumn: ICliOReportColumn; stdcall;
    function NewItem: ICliOReportItem; stdcall;
  public
    constructor Create(aXMLRoot: IXMLNode);
    destructor Destroy; override;
  end;

implementation

uses
  oCliOReportColumn,
  oCliOReportItem;

{ TCliOReportSection }

function TCliOReportSection.Column(aIndex: Integer): ICliOReportColumn;
begin
  try
    fColumns[aIndex].QueryInterface(ICliOReportColumn, Result);
  except
    Result := nil;
  end;
end;

function TCliOReportSection.ColumnCount: Integer;
begin
  Result := fColumns.Count;
end;

constructor TCliOReportSection.Create(aXMLRoot: IXMLNode);
begin
  inherited Create;
  TInterfaceList.Create.GetInterface(IInterfaceList, fColumns);
  TInterfaceList.Create.GetInterface(IInterfaceList, fItems);

  fSectionNode := aXMLRoot.AddChild('SECTION');
  fColumnNode := fSectionNode.AddChild('COLUMNS');
  fItemNode := fSectionNode.AddChild('ITEMS');

  setTitle(Format('Section %d', [aXMLRoot.ChildNodes.Count]));
  setPrintHeader(True);
  setPrintSubHeader(False);
  setTopLevel(False);
end;

destructor TCliOReportSection.Destroy;
begin

  inherited;
end;

function TCliOReportSection.FirstColumn: ICliOReportColumn;
begin
  if fColumns.Count > 0 then
    fColumns[0].QueryInterface(ICliOReportColumn, Result)
  else
    Result := nil;
end;

function TCliOReportSection.getPrintHeader: boolean;
begin
  try
    Result := fSectionNode.ChildValues['PRINT_HEADER'];
  except
    Result := False;
  end;
end;

function TCliOReportSection.getPrintSubHeader: boolean;
begin
  try
    Result := fSectionNode.ChildValues['PRINT_SUB_HEADER'];
  except
    Result := False;
  end;
end;

function TCliOReportSection.getTitle: string;
begin
  Result := fSectionNode.ChildValues['TITLE'];
end;

function TCliOReportSection.getTopLevel: boolean;
begin
  try
    Result := fSectionNode.ChildValues['TOP_LEVEL'];
  except
    Result := False;
  end;
end;

function TCliOReportSection.Item(aIndex: Integer): ICliOReportItem;
begin
  try
    fItems[aIndex].QueryInterface(ICliOReportItem, Result);
  except
    Result := nil;
  end;
end;

function TCliOReportSection.ItemCount: integer;
begin
  Result := fItems.Count;
end;

function TCliOReportSection.NewColumn: ICliOReportColumn;
begin
  TCliOReportColumn.Create(fColumnNode).GetInterface(ICliOReportColumn, Result);
  fColumns.Add(Result);
end;

function TCliOReportSection.NewItem: ICliOReportItem;
begin
  TCliOReportItem.Create(fItemNode).GetInterface(ICliOReportItem, Result);
  fItems.Add(Result);
end;

function TCliOReportSection.NextColumn(aCliOReportColumn: ICliOReportColumn): ICliOReportColumn;
var
  i: integer;
begin
  i := fColumns.IndexOf(aCliOReportColumn) + 1;
  if (i > 0) and (i < fColumns.Count) then
    fColumns[i].QueryInterface(ICliOReportColumn, Result)
  else
    Result := nil;
end;

procedure TCliOReportSection.setPrintHeader(aValue: boolean);
begin
  fSectionNode.ChildValues['PRINT_HEADER'] := aValue;
end;

procedure TCliOReportSection.setPrintSubHeader(aValue: boolean);
begin
  fSectionNode.ChildValues['PRINT_SUB_HEADER'] := aValue;
end;

procedure TCliOReportSection.setTitle(aValue: string);
begin
  fSectionNode.ChildValues['TITLE'] := aValue;
end;

procedure TCliOReportSection.setTopLevel(aValue: boolean);
begin
  fSectionNode.ChildValues['TOP_LEVEL'] := aValue;
end;

end.

