unit oDBConnection_ADO;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 3 $  $Modtime: 5/22/09 12:47p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  ADO Connector for IDBConnection
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oDBConnection_ADO.pas $
*
* $History: oDBConnection_ADO.pas $
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 5/22/09    Time: 2:17p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Build Pre-275:  
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 4/08/08    Time: 8:13p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:15p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 11/07/07   Time: 10:28a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 10/27/06   Time: 2:59p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:06p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  Forms,
  ADODB,
  XMLDoc,
  XMLIntf,
  oInterfaces;

type
  TDBConnection_ADO = class(TInterfacedObject)
  private
    fADOConnection: TADOConnection;
    fCommands: TStringList;
  protected
    function ConnectADO(aConnectionString: string): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    function InitCommands(aCommandFileName: string): Boolean; stdcall;
    procedure BeginTransaction; stdcall;
    procedure CommitTransaction; stdcall;
    function Connect(aParams: array of Variant): Boolean; stdcall;
    procedure Disconnect; stdcall;
    function ExecCommand(aCommandName: string; aParams: array of variant): Boolean; stdcall;
    function ExecQuery(aQueryName: string; aParams: array of variant; aResults: IXMLDocument): boolean; stdcall;
    function ExecResults: TStringList; stdcall;
    function ExecStoredProc(aStoredProcName: string; aParams: array of variant; aParamList: TStringList): Boolean; stdcall;
    function getDescription: string; stdcall;
    function IsConnected: Boolean; stdcall;
    procedure RollbackTransaction; stdcall;
    procedure SaveCommandList(aFileName: string); stdcall;
  end;

implementation

uses ComObj;

{
************************************************** TDBConnection_ADO ***************************************************
}

constructor TDBConnection_ADO.Create;
begin
  inherited Create;
  fADOConnection := TADOConnection.Create(nil);
  fCommands := TStringList.Create;
end;

destructor TDBConnection_ADO.Destroy;
begin
  inherited;
end;

function TDBConnection_ADO.InitCommands(aCommandFileName: string): Boolean;
var
  aXMLNode: IXMLNode;
  aCmdName, aCmdText: string;
begin
  with NewXMLDocument do
    begin
      if FileExists(aCommandFileName) then
        LoadFromFile(aCommandFileName);

      if DocumentElement = nil then
        DocumentElement := AddChild('COMMANDS');

      aXMLNode := DocumentElement.ChildNodes.First;
      while aXMLNode <> nil do
        try
          aCmdName := aXMLNode.ChildValues['NAME'];
          aCmdText := aXMLNode.ChildValues['SQL'];
          fCommands.Values[aCmdName] := aCmdText;
        finally
          aXMLNode := aXMLNode.NextSibling;
        end;
    end;
  Result := True;
end;

procedure TDBConnection_ADO.BeginTransaction;
begin
  fADOConnection.BeginTrans;
end;

procedure TDBConnection_ADO.CommitTransaction;
begin
  fADOConnection.CommitTrans;
end;

function TDBConnection_ADO.Connect(aParams: array of Variant): Boolean;
begin
  {
  Inherited types take parameters and build a single connect string
  and then calls this objects protected ConnectADO method.
  }
  Result := False;
end;

function TDBConnection_ADO.ConnectADO(aConnectionString: string): Boolean;
begin
  try
    fADOConnection.ConnectionString := aConnectionString;
    fADOConnection.Open;
    Result := True;
  except
    Result := False;
  end;
end;

procedure TDBConnection_ADO.Disconnect;
begin
  fADOConnection.Close;
end;

function TDBConnection_ADO.ExecCommand(aCommandName: string; aParams: array of variant): Boolean;
var
  i: Integer;
  p: string;
begin
  with TADOCommand.Create(nil) do
    try
      Connection := fADOConnection;
      CommandType := cmdText;
      CommandText := fCommands.Values[aCommandName];

      if CommandText = '' then
        begin
//          Result := False;
          raise Exception.CreateFmt('Unknown command ''%s'' called', [aCommandName]);
          //MessageDlg('Unknown command called ''' + aCommandName + '''.', mtError, [mbOK], 0);
          //Exit;
        end;

      if Length(aParams) > 0 then
        for i := Low(aParams) to High(aParams) do
          begin
            p := Format('P%d', [i]);
            Parameters.ParamByName(p).Value := aParams[i];
          end;

      Execute;
      Result := True;
    except
      Result := False;
    end;
end;

function TDBConnection_ADO.ExecQuery(aQueryName: string; aParams: array of variant; aResults: IXMLDocument): boolean;
var
  i: Integer;
  f: string;
  p: string;
  XMLDoc: IXMLDocument;
  XMLRec: IXMLNode;
  XMLVal: IXMLNode;
begin
  if fCommands.IndexOfName(aQueryName) < 0 then
    begin
      Result := False;
      Exit;
    end;

  with TADOQuery.Create(nil) do
    try
      Connection := fADOConnection;
      SQL.Text := fCommands.Values[aQueryName];
      if Length(aParams) > 0 then
        for i := Low(aParams) to High(aParams) do
          begin
            p := Format('P%d', [i]);
            Parameters.ParamByName(p).Value := aParams[i];
          end;

      Open;
      XMLDoc := NewXMLDocument;
      XMLDoc.DocumentElement := XMLDoc.AddChild('RESULTS');
      XMLDoc.CreateNode('Query Results Generated By ' + getDescription, ntComment);
      First;
      while not EOF do
        begin
          XMLRec := XMLDoc.DocumentElement.AddChild('RECORD');
          for i := 0 to FieldDefList.Count - 1 do
            begin
              f := FieldDefList[i].Name;
              XMLVal := XMLRec.AddChild(f);
              XMLVal.NodeValue := FieldValues[f];
            end;
          Next;
        end;
      aResults.XML.Text := FormatXMLData(XMLDoc.XML.Text);
      Result := True;
    except
      on E:Exception do
        begin
          aResults.XML.Clear;
          aResults.DocumentElement := aResults.AddChild('ERROR');
          aResults.DocumentElement.ChildValues['MESSAGE'] := e.Message;
          Result := False;
        end
    end;
end;

function TDBConnection_ADO.ExecResults: TStringList;
begin
  Result := nil;
end;

function TDBConnection_ADO.ExecStoredProc(aStoredProcName: string; aParams: array of variant; aParamList: TStringList):
  Boolean;
begin
  // Not allowed under VA Standards - Only here to give the broker type connection a place to breathe
  Result := False;
end;

function TDBConnection_ADO.getDescription: string;
begin
  Result := 'Default ADO Connection';
end;

function TDBConnection_ADO.IsConnected: Boolean;
begin
  Result := fADOConnection.Connected;
end;

procedure TDBConnection_ADO.RollbackTransaction;
begin
  fADOConnection.RollbackTrans;
end;

procedure TDBConnection_ADO.SaveCommandList(aFileName: string);
begin
  //fCommands.SaveToFile(aFileName);
end;

end.

