unit oDBConnection_VistA;
{
================================================================================
*       Application:  Clinical Observations
*       Revision:     $Revision: 16 $  $Modtime: 5/22/09 1:05p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default Fileman Connector for IDBConnection
*
*       Notes:
*
*       $Archive: /CP Modernization/CliO/source/Common/objects/oDBConnection_VistA.pas $
================================================================================
}
interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  Forms,
  XMLIntf,
  inifiles,
  XMLDoc,
  oInterfaces,
  SyncObjs,
  TRPCB,
  uCCOW,
  CCOWRPCBroker,
  VERGENCECONTEXTORLib_TLB;

type
  TDBConnection_VistA = class(TInterfacedObject, IDBConnection, ICCOWContextor, ILegacyBroker, IVistASystem, IDBExport, IDBImport)
  protected
    fCommandNames: TStringList;
    fRPCBroker: TRPCBroker;
    fRPCResults: TStringList;
    fCCOWContextor: TVACCOW;
    fRoles: TStringList;
    fLastError: string;
    fCriticalQuery: TCriticalSection;
    fExportDocument: IXMLDocument;
    fExportHeader: IXMLNode;
    fExportTables: IXMLNode;
    fImportDocument: IXMLDocument;
    fOnNotifyImportStatus: TNotifyOnProgress;
    function CallServer(aRPCName: string; aParams: array of variant; aMultlist: TStringList): Boolean;
    function FMDateTime(aDateTime: TDateTime): string;
    procedure ProcessDateTypes;
    function XMLDateTime(aString: string): string;
    function getExportTableNode(aTableName: string): IXMLNode;
    function getRowID(aTableName: string; aKeyFields: array of variant; aKeyValues: array of variant): string;
    procedure NotifyImportStatus(const aMsg: string; aProgress, aProgressMax: integer);

    {ICCOWContextor}
    procedure SetCCOWNotifyNewPatientID(aProcedure: TNotifyNewPatientID); stdcall;
    procedure SetCCOWNotifyNewLinkStatus(aProcedure: TNotifyNewLinkStatus); stdcall;
    procedure SetCCOWNotifyAbleToChange(aProcedure: TNotifyAbleToChange); stdcall;
    procedure SetCCOWPatientContextItem(aContextItem: string); stdcall;
    procedure SetCCOWPatientContextID(const aPtId: string); stdcall;
    procedure CCOWContextRefresh; stdcall;
    procedure CCOWContextSuspend; stdcall;
    procedure CCOWContextResume(const aRejoinMode: TVACCOW_RejoinMode); stdcall;

    function CCOWLinkStatus: TVACCOW_LinkStatus; stdcall;

    {ILegacyBroker calls}
    function getRPCTimeLimit: integer; stdcall;
    function getConnected: boolean; stdcall;
    function getListenerPort: integer; stdcall;
    function getResults: TStringList; stdcall;
    function getServer: string;

    {IDBExport calls}
    procedure ClearLastExport; stdcall;
    procedure AddExportTable(aTableName: string; aQueryName: string; aParams: array of Variant);
    procedure SaveExport(aFilename: string; aHeaderFilename: string = ''); stdcall;
    procedure DisplayExport(aList: TStrings); stdcall;
    procedure SetExportDescription(aValue: string); stdcall;
    procedure SetExportFramework(aValue: string); stdcall;
    procedure SetExportVersion(aValue: integer); stdcall;

    {IDBImport Calls}
    procedure ExecuteImport(aImportDocument: IXMLDocument); stdcall;
    procedure ImportRecord(aTableName: string; aKeyFields: array of variant; aKeyValues: array of variant; aFields: array of variant; aValues: array of variant);
    procedure setNotifyImportProgress(aNotifyOnProgress: TNotifyOnProgress); stdcall;

    function DoesRecordExist(aTableName: string; aKeyFields: array of variant; aKeyValues: array of variant): boolean;
    procedure InsertRecord(aTableName: string; aFields: array of variant; aValues: array of variant);
    procedure UpdateRecord(aTableName: string; aKeyFields: array of variant; aKeyValues: array of variant; aFields: array of variant; aValues: array of variant);

    procedure setRPCTimeLimit(const Value: integer); stdcall;
    procedure setConnected(const Value: boolean); stdcall;
    procedure setListenerPort(const Value: integer); stdcall;
    procedure setServer(const Value: string);

    function CreateContext(strContext: string): boolean; stdcall;
  public
    constructor Create;
    constructor CreateNoCCOW;
    destructor Destroy; override;
    function InitCommands(aCommandFileName: string): Boolean; stdcall;
    procedure BeginTransaction; stdcall;
    procedure CommitTransaction; stdcall;
    function Connect(aParams: array of Variant): Boolean; stdcall;
    procedure Disconnect; stdcall;
    function ExecCommand(aCommandName: string; aParams: array of variant): Boolean; stdcall;
    function ExecQuery(aQueryName: string; aParams: array of variant; aResults: IXMLDocument): boolean; stdcall;
    function ExecResults: TStringList; stdcall;
    function ExecStoredProc(aStoredProcName: string; aParams: array of variant; aParamList: TStringList): Boolean; stdcall;
    function getDescription: string; stdcall;
    function IsConnected: Boolean; stdcall;
    procedure RollbackTransaction; stdcall;
    procedure SaveCommandList(aFileName: string); stdcall;
    function GetLastError: string; stdcall;

    {IUser}
    function DUZ: string; stdcall;
    function Name: string; stdcall;
    function Station_ID: string; stdcall;
    function Station_Name: string; stdcall;
    function HasRole(aRoleName: string): boolean; stdcall;
    function IsThisExecutableValid: boolean; stdcall;

    {ILegacyBroker Properties}
    property RPCTimeLimit: integer read getRPCTimeLimit write setRPCTimeLimit;
    property Connected: boolean read getConnected write setConnected;
    property Server: string read getServer write setServer;
    property ListenerPort: integer read getListenerPort write setListenerPort;
    property Results: TStringList read getResults;
  end;

implementation

uses
  Dialogs,
  oCP_Exceptions,
  oVersionInformation,
  DateUtils,
  uCPXML,
  Variants,
  MFunStr,
  RPCConf1;

const
  CLIO_RPC = 'MD CLIO';
  CLIO_CONTEXT = 'MD CLIO';

  (*
  function CCOW_Required: Boolean;
  var
    i: Integer;
  begin
    Result := True;
    for i := 0 to ParamCount - 1 do
      begin
        if UpperCase(ParamStr(i)) = '/NOCCOW' then
          Result := False;
      end;
  end;
  *)

  {------------------------------------------------------------------------------}
  {----------------------------- TDBConnection_VistA ----------------------------}
  {------------------------------------------------------------------------------}

constructor TDBConnection_VistA.Create;
begin
  inherited Create;

  if FindCmdLineSwitch('noccow') then
    fRPCBroker := TRPCBroker.Create(application)
  else
    begin
      fRPCBroker := TCCOWRPCBroker.Create(Application);
      fCCOWContextor := NewCCOWContexter(Application, TCCOWRPCBroker(fRPCBroker));
    end;

  fRPCBroker.ShowErrorMsgs := semQuiet;
  fRPCBroker.RPCTimeLimit := 300;
  fRPCResults := TStringList.Create;
  fCriticalQuery := TCriticalSection.Create;
  fRoles := TStringList.Create;
  fRoles.Sorted := True;
  fCommandNames := TStringList.Create;
  fOnNotifyImportStatus := NotifyImportStatus;
end;

constructor TDBConnection_VistA.CreateNoCCOW;
begin
  inherited Create;

  fRPCBroker := TRPCBroker.Create(application);
  fRPCBroker.ShowErrorMsgs := semQuiet;
  fRPCBroker.RPCTimeLimit := 300;
  fRPCResults := TStringList.Create;
  fCriticalQuery := TCriticalSection.Create;
  fRoles := TStringList.Create;
  fRoles.Sorted := True;
  fCommandNames := TStringList.Create;
  fOnNotifyImportStatus := NotifyImportStatus;
end;

destructor TDBConnection_VistA.Destroy;
begin
  if Assigned(fCCOWContextor) then
    FreeAndNil(fCCOWContextor);

  if Assigned(fRPCBroker) then
    begin
      fRPCBroker.Connected := False;
      FreeAndNil(fRPCBroker);
    end;

  FreeAndNil(fRPCResults);
  FreeAndNil(fCommandNames);
  FreeAndNil(fRoles);
  FreeAndNil(fCriticalQuery);
  inherited;
end;

procedure TDBConnection_VistA.ImportRecord(aTableName: string; aKeyFields, aKeyValues, aFields, aValues: array of variant);
var
  aRowID: string;
  i: integer;
  aLst: TStringList;
begin
  aRowID := getRowID(aTableName, aKeyFields, aKeyValues);
  if StrToIntDef(aRowID, 0) < 1 then
    aRowID := '+1,';

  aLst := TStringList.Create;
  try
    for i := Low(aFields) to High(aFields) do
      begin
        aLst.Add(aFields[i]);
        aLst.Add(aValues[i]);
      end;
    CallServer(CLIO_RPC, ['IMPORT', aTableName, aRowID], aLst);
  finally
    FreeAndNil(aLst);
  end;
end;

function TDBConnection_VistA.InitCommands(aCommandFileName: string): Boolean;
var
  aXMLNode: IXMLNode;
  aCmdName, aCmdText: string;
begin
  with NewXMLDocument do
    begin
      if FileExists(aCommandFileName) then
        LoadFromFile(aCommandFileName);

      if DocumentElement = nil then
        DocumentElement := AddChild('COMMANDS');

      aXMLNode := DocumentElement.ChildNodes.First;
      while aXMLNode <> nil do
        try
          aCmdName := aXMLNode.ChildValues['NAME'];
          aCmdText := 'XPAR Handle';
          fCommandNames.Values[aCmdName] := aCmdText;
        finally
          aXMLNode := aXMLNode.NextSibling;
        end;
    end;
  Result := True;
end;

procedure TDBConnection_VistA.AddExportTable(aTableName: string; aQueryName: string; aParams: array of Variant);
var
//  i: integer;
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aTableRoot: IXMLNode;
begin
  if getExportTableNode(aTableName) <> nil then
    raise Exception.Create('Export Table Already Exists');

  {Build the table node}
  aTableRoot := fExportTables.AddChild('TABLE');
  aTableRoot.Attributes['NAME'] := aTableName;

  (*
  {Add the Primary Keys}
  getExportTableNode(aTableName).Attributes['PK_CNT'] := Length(aPrimaryKeys);
  for i := Low(aPrimaryKeys) to High(aPrimaryKeys) do
    getExportTableNode(aTableName).Attributes[Format('PK_%0.2d', [i + 1])] := aPrimaryKeys[i];
  *)

  {Grab the data}
  aXMLDoc := NewXMLDocument;
  ExecQuery(aQueryName, aParams, aXMLDoc);
  aXMLDoc.Active := True;

  {Move query result to the export document}
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aTableRoot.ChildNodes.Add(aXMLRec.CloneNode(True));
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;
end;

procedure TDBConnection_VistA.BeginTransaction;
begin
  { Facade methods, this isn't allowed under VA standards :( }
end;

function TDBConnection_VistA.CallServer(aRPCName: string; aParams: array of variant; aMultlist: TStringList): Boolean;
var
  i: Integer;
  j: Integer;
  aStrParam: string;
//  CallID: string;
//  CallTime: integer;
//  CallStart: TDateTime;
//  CallStop: TDateTime;
begin
  try
//    Result := False;
    if not fRPCBroker.Connected then {let's not bang on the thing endlessly}
      begin
        fRPCResults.Clear;
        raise ECPConnectFail.Create('RPC Broker is not connected');
      end;

    fRPCBroker.RemoteProcedure := aRPCName;

    {Load any params}
    i := 0;
    if SizeOf(aParams) > 0 then
      while i <= High(aParams) do
        begin
          if VarIsType(aParams[i], varDate) then
            aStrParam := DateTimeToXMLString(aParams[i])
          else
            aStrParam := VarToStrDef(aParams[i], '');

          if (Copy(aStrParam, 1, 1) = '@') and (Length(aStrParam) > 1) then // Is it passed as a reference???
            begin
              fRPCBroker.Param[i].PType := reference;
              fRPCBroker.Param[i].Value := Copy(aStrParam, 2, Length(aStrParam));
            end
          else
            begin
              if Length(aStrParam) > 255 then
                raise ECPConnectFail.CreateFmt(
                  'A value greater than 255 characters has been passed to VistA.' + #13 +
                  'Param[%d] = %s', [i, aStrParam]);
              fRPCBroker.Param[i].PType := literal;
              fRPCBroker.Param[i].Value := aStrParam;
            end;
          inc(i);
        end;

    {Load up the multiple if it is assigned}
    if Assigned(aMultList) then
      begin
        fRPCBroker.Param[i].PType := list;
        for j := 0 to aMultList.Count - 1 do
          if Length(aMultList[j]) < 256 then
            fRPCBroker.Param[i].Mult[IntToStr(j)] := aMultList[j]
          else
            raise ECPConnectFail.CreateFmt(
              'A value greater than 255 characters has been passed to VistA.' + #13 +
              'Param[%d] = %s', [j, aMultList[j]]);
      end;

    try
      fRPCResults.Clear;
//      CallStart := Now;
      fRPCBroker.lstCall(fRPCResults);
//      CallStop := Now;

      (* Uncomment for logging into an ini file
      with TIniFile.Create('.\RPCB.log') do
        try
          i := ReadInteger('ROOT', 'CALLID', 0) + 1;
          WriteInteger('ROOT', 'CALLID', i);
          WriteString(Format('Call_%d', [i]), 'RPC', aRPCName);
          for j := Low(aParams) to High(aParams) do
            WriteString(Format('Call_%d', [i]), Format('Param(%d)', [j]), VarToStr(aParams[j]));
          WriteDateTime(Format('Call_%d', [i]), 'START', CallStart);
          WriteDateTime(Format('Call_%d', [i]), 'STOP', CallStop);
          WriteInteger(Format('Call_%d', [i]), 'EXECUTION_TIME', MilliSecondsBetween(CallStart, CallStop));
          UpdateFile;
        finally
          Free;
        end;
      *)
    except
      on E: Exception do
        begin
          fRPCBroker.Connected := False; {Force this thing to stop being STOOOPID!!!}
          raise ECPConnectFail.Create('Broker communication failure.' + #13 + E.Message);
        end;
    end;

    Result := (Length(fRPCResults.Text) > 0) and (Copy(fRPCResults.Text, 1, 2) <> '-1'); {Check for error flag}

    if not Result then {Clear the error flag (-1^) out}
      raise ECPConnectFail.Create(Copy(fRPCResults.Text, 4, Length(fRPCResults.Text)) + '***')
  except
    on E: Exception do
      begin
        fRPCResults.Clear;
        fLastError := E.Message;
        raise ECPConnectFail.Create(E.Message);
      end;
  end;
end;

procedure TDBConnection_VistA.ClearLastExport;
begin
  fExportDocument := NewXMLDocument;
  fExportDocument.DocumentElement := fExportDocument.AddChild('EXPORT');
  fExportHeader := fExportDocument.DocumentElement.AddChild('HEADER');
  fExportHeader.ChildValues['DATE_CREATED'] := Now;
  fExportTables := fExportDocument.DocumentElement.AddChild('TABLES');
end;

procedure TDBConnection_VistA.CommitTransaction;
begin
  { Facade methods, this isn't allowed under VA standards :( }
end;

function TDBConnection_VistA.Connect(aParams: array of Variant): Boolean;
{Raises ECPConnectFail exception}
var
  aSrvr: string;
  aPort: string;
  aCodes: string;
  aContext: string;
  aNode: IXMLNode;
  aXMLDocument: IXMLDocument;

begin
  aSrvr := '';
  aPort := '';
  aCodes := '';
  aContext := CLIO_CONTEXT;

  if Length(aParams) = 4 then
    begin
      aSrvr := aParams[0];
      aPort := aParams[1];
      aCodes := aParams[2];
      aContext := aParams[3];
    end;

  {Fire up the Contextor - Need a flag here to prevent this one}

  if Assigned(fCCOwContextor) then
    try
      fCCOWContextor.JoinContext('CPFlowsheets');
    except
    end;

  if (aSrvr = '') or (aPort = '') then
    if GetServerInfo(aSrvr, aPort) <> mrOK then
      raise ECPConnectFail.Create('Unable to determine Server/Port values');
  try
    fRPCBroker.Server := aSrvr;
    fRPCBroker.ListenerPort := StrToInt(aPort);
    fRPCBroker.AccessVerifyCodes := aCodes;
    fRPCBroker.Connected := True;
  except
    on E: Exception do
      raise ECPConnectFail.CreateFmt('Error connecting: %s', [e.Message]);
  end;

  if not fRPCBroker.Connected then
    raise ECPConnectFail.Create('No connection established to server.');

  if not fRPCBroker.CreateContext(aContext) then
    begin
      fRPCBroker.Connected := False;
      raise ECPContextFail.CreateFmt('Unable to create context ''%s''.', [aContext]);
    end;

  Result := fRPCBroker.Connected;

  if Result then
    begin
      aXMLDocument := NewXMLDocument;
      if ExecQuery('GetUserRoles', [], aXMLDocument) then
        with aXMLDocument do
          begin
            Active := True;
            aNode := DocumentElement.ChildNodes.FindNode('RECORD');
            while aNode <> nil do
              begin
                fRoles.Add(XMLValueAsStr(aNode, 'ROLE_ID'));
                aNode := aNode.NextSibling;
              end;
          end
      else
        begin
//          Result := False;
          raise ECPLoadUserRolesFail.Create('Unable to load user roles');
        end;
    end;
end;

procedure TDBConnection_VistA.Disconnect;
begin
  fRPCBroker.Connected := False;
end;

procedure TDBConnection_VistA.DisplayExport(aList: TStrings);
begin
  aList.Text := FormatXMLData(fExportDocument.XML.Text);
end;

function TDBConnection_VistA.DoesRecordExist(aTableName: string; aKeyFields, aKeyValues: array of variant): boolean;
//var
//  aCMD: string;
//  i: integer;
begin
  Result := StrToIntDef(getRowID(aTableName, aKeyFields, aKeyValues), 0) > 0;

  (* SQL COMMAND
  aCMD := Format('SELECT COUNT(%s) WHERE %s = %s', [aKeyFields[0], aKeyFields[0], aKeyValues[0]]);
  i := 1;
  while i < Length(aKeyFields) do
    try
      aCMD := Format('%s and %s = %s', [aCMD, aKeyFields[i], aKeyValues[i]]);
    finally
      inc(i);
    end;
  ShowMessage(aCMD);
  *)
end;

procedure TDBConnection_VistA.InsertRecord(aTableName: string; aFields, aValues: array of variant);
var
//  aCMD: string;
//  aFld, aVal: string;
  i: integer;
  aLst: TStringList;
//  atxt: string;
begin
  aLst := TStringList.Create;
  try
    for i := Low(aFields) to High(aFields) do
      begin
        aLst.Add(aFields[i]);
        aLst.Add(aValues[i]);
      end;

    CallServer(CLIO_RPC, ['IMPORT', aTableName, '+1,'], aLst);
  finally
    FreeAndNil(aLst);
  end;
end;

procedure TDBConnection_VistA.UpdateRecord(aTableName: string; aKeyFields, aKeyValues, aFields, aValues: array of variant);
var
  aRowID: string;
//  aCMD: string;
//  aFld, aVal: string;
  i: integer;
//  aParams: array of variant;
  aLst: TStringList;
//  atxt: string;
begin
  aRowID := getRowID(aTableName, aKeyFields, aKeyValues);
  aLst := TStringList.Create;
  try
    for i := Low(aFields) to High(aFields) do
      begin
        aLst.Add(aFields[i]);
        aLst.Add(aValues[i]);
      end;
    CallServer(CLIO_RPC, ['IMPORT', aTableName, aRowID], aLst);
  finally
    FreeAndNil(aLst);
  end;
end;

function TDBConnection_VistA.ExecCommand(aCommandName: string; aParams: array of variant): Boolean;
var
  i: Integer;
  aParamList: TStringList;
begin
  aParamList := TStringList.Create;
  try
    for i := Low(aParams) to High(aParams) do
      if VarIsType(aParams[i], varDate) then
        aParamList.Add(DateTimeToXMLString(aParams[i]))
      else if VarIsType(aParams[i], varBoolean) then
        if aParams[i] then
          aParamList.Add('1')
        else
          aParamList.Add('0')
      else
        aParamList.Add(VarToStrDef(aParams[i], ''));

    try
      Result := CallServer(CLIO_RPC, ['EXECUTE', aCommandName], aParamList);
    except
  //    Result := False;
      raise;
    end;
  finally
    aParamList.Free;
  end;
end;

function TDBConnection_VistA.ExecQuery(aQueryName: string; aParams: array of variant; aResults: IXMLDocument): boolean;
begin
  try
//    Result := False;
    try
      fCriticalQuery.Acquire;
      if ExecCommand(aQueryName, aParams) then
        begin
          aResults.XML.Text := fRPCResults.Text;
          Result := True;
        end
      else
        begin
          Result := False;
          raise ECPQueryFail.CreateFmt('Error executing query %s' + #13 + #13 + fRPCResults.Text, [aQueryName]);
        end;
    finally
      fCriticalQuery.Release;
    end;
  except
    on E: ECPQueryFail do
      raise;
    on E: Exception do
      raise ECPQueryFail.CreateFmt('Error executing query %s' + #13 + #13 + E.Message + #13 + fRPCResults.Text, [aQueryName]);
  end;
end;

function TDBConnection_VistA.ExecResults: TStringList;
begin
  Result := fRPCResults;
end;

function TDBConnection_VistA.ExecStoredProc(aStoredProcName: string; aParams: array of variant; aParamList: TStringList): Boolean;
begin
  result := false;
  //Result := CallServer(aStoredProcName, aParams, aParamList);
end;

procedure TDBConnection_VistA.ExecuteImport(aImportDocument: IXMLDocument);
var
  aTableRoot: IXMLNode;
  aTable: IXMLNode;
  aRecord: IXMLNode;
  aValuePair: IXMLNode;
//  aTableName: string;
  aPrimaryKeys: array of variant;
  aPrimaryValues: array of variant;
  aFieldList: array of variant;
  aValueList: array of variant;
  i,
  j: integer;
  aParams: TStringList;
begin
  aImportDocument.Active := True;
  aParams := TStringList.Create;
  aTableRoot := aImportDocument.DocumentElement.ChildNodes.FindNode('TABLES');
  aTable := aTableRoot.ChildNodes.First;
  while Assigned(aTable) do
    try
      fOnNotifyImportStatus(
        Format('Importing %s...', [aTable.Attributes['NAME']]), 0, 0);

      //ShowMessageFmt('TABLE TO IMPORT: %s', [aTable.Attributes['NAME']]);

      j := aTable.Attributes['PK_CNT'];
      SetLength(aPrimaryKeys, j);
      SetLength(aPrimaryValues, j);

      {Build array of field names}
      for i := 1 to j do
        aPrimaryKeys[i - 1] := aTable.Attributes[Format('PK_%0.2d', [i])];

      aRecord := aTable.ChildNodes.First;
      while Assigned(aRecord) do
        try
          {Build list of lookup values}
          for i := Low(aPrimaryKeys) to High(aPrimaryKeys) do
            aPrimaryValues[i] := aRecord.ChildValues[aPrimaryKeys[i]];

          {Build the arrays of field names and values}
          SetLength(aValueList, aRecord.ChildNodes.Count);
          SetLength(aFieldList, aRecord.ChildNodes.Count);
          i := 0;
          aValuePair := aRecord.ChildNodes.First;
          while Assigned(aValuePair) do
            try
              aFieldList[i] := aValuePair.NodeName;
              aValueList[i] := aValuePair.NodeValue;
            finally
              inc(i);
              aValuePair := aValuePair.NextSibling;
            end;

          ImportRecord(aTable.Attributes['NAME'], aPrimaryKeys, aPrimaryValues, aFieldList, aValueList)
            {
if DoesRecordExist(aTable.Attributes['NAME'], aPrimaryKeys, aPrimaryValues) then
UpdateRecord(aTable.Attributes['NAME'], aPrimaryKeys, aPrimaryValues, aFieldList, aValueList)
else
InsertRecord(aTable.Attributes['NAME'], aFieldList, aValueList);
}
        finally
          aRecord := aRecord.NextSibling;
        end;
    finally
      aTable := aTable.NextSibling;
    end;
  FreeAndNil(aParams);
end;

function TDBConnection_VistA.FMDateTime(aDateTime: TDateTime): string;
var
  aYear: Word;
  aMonth: Word;
  aDay: Word;
  aHour: Word;
  aMinute: Word;
  aSecond: Word;
  aMs: Word;
begin
  DecodeDateTime(aDateTime, aYear, aMonth, aDay, aHour, aMinute, aSecond, aMs);
  Result := FloatToStr(((aYear - 1700) * 10000) + (aMonth * 100) + aDay + (aHour * 0.01) + (aMinute * 0.0001) + (aSecond * 0.000001));
end;

function TDBConnection_VistA.getDescription: string;
begin
  Result := Format('VistA RPC Broker - %s@%d', [fRPCBroker.Server, fRPCBroker.ListenerPort]);
end;

function TDBConnection_VistA.getExportTableNode(aTableName: string): IXMLNode;
begin
  Result := fExportTables.ChildNodes.First;
  while Assigned(Result) do
    if Result.Attributes['NAME'] = aTableName then
      Exit
    else
      Result := Result.NextSibling;
end;

function TDBConnection_VistA.IsConnected: Boolean;
begin
  Result := fRPCBroker.Connected;
end;

procedure TDBConnection_VistA.ProcessDateTypes;
var
//  i: Integer;
  aXMLNode: IXMLNode;
  aXMLType: IXMLNode;
//  aXMLValue: IXMLNode;
  aTag: string;
  aType: string;
//  aDumper: TStringList;

  procedure TranslateDateNodes(aRootNode: IXMLNode; aNodeName: string);
  var
    aRecNode: IXMLNode;
    aValNode: IXMLNode;
  begin
    aRecNode := aRootNode.ChildNodes.FindNode('RECORD');
    while aRecNode <> nil do
      begin
        aValNode := aRecNode.ChildNodes.First;
        while aValNode <> nil do
          begin
            if aValNode.NodeName = aNodeName then
              aValNode.NodeValue := XMLDateTime(VarToStr(aValNode.NodeValue));
            aValNode := aValNode.NextSibling;
          end;
        aRecNode := aRecNode.NextSibling;
      end;
  end;

begin
  with NewXMLDocument do
    begin
      XML.Text := fRPCResults.Text;
      Active := True;
      aXMLNode := DocumentElement.ChildNodes.FindNode('DATATYPES');
      if aXMLNode <> nil then
        begin
          aXMLType := aXMLNode.ChildNodes.FindNode('TAG');
          while aXMLType <> nil do
            begin
              aTag := aXMLType.Attributes['NAME'];
              aType := aXMLType.Attributes['TYPE'];
              if aType = 'DATE/TIME' then
                TranslateDateNodes(DocumentElement, aTag);
              aXMLType := aXMLType.NextSibling;
            end;
        end;
      fRPCResults.Text := XML.Text;
    end;
end;

procedure TDBConnection_VistA.RollbackTransaction;
begin
  { Facade methods, this isn't allowed under VA standards :( }
end;

procedure TDBConnection_VistA.SaveExport(aFilename, aHeaderFilename: string);
begin
  fExportDocument.SaveToFile(aFilename);
  if aHeaderFilename <> '' then
    with NewXMLDocument do
      begin
        Active := True;
        DocumentElement := fExportHeader.CloneNode(True);
        DocumentElement.ChildValues['SOURCE'] := ExtractFileName(aFilename);
        SaveToFile(aHeaderFilename);
      end;
end;

procedure TDBConnection_VistA.CCOWContextRefresh;
begin
  fCCOWContextor.RefreshContext;
end;

procedure TDBConnection_VistA.CCOWContextResume(const aRejoinMode: TVACCOW_RejoinMode);
begin
  fCCOWContextor.ResumeContext(aRejoinMode);
end;

procedure TDBConnection_VistA.CCOWContextSuspend;
begin
  fCCOWContextor.SuspendContext;
end;

function TDBConnection_VistA.CCOWLinkStatus: TVACCOW_LinkStatus;
begin
  if Assigned(fCCOWContextor) then
    Result := fCCOWContextor.LinkStatus
  else
    Result := lsNoCCOW;
end;

procedure TDBConnection_VistA.SetCCOWPatientContextID(const aPtId: string);
begin
  fCCOWContextor.SetNewPatientContext(aPtID);
end;

procedure TDBConnection_VistA.SetCCOWPatientContextItem(aContextItem: string);
begin
  fCCOWContextor.PatientContextItem := aContextItem;
end;

procedure TDBConnection_VistA.SetCCOWNotifyAbleToChange(aProcedure: TNotifyAbleToChange);
begin
  if Assigned(fCCOWContextor) then
    fCCOWContextor.NotifyAbleToChange := aProcedure;
end;

procedure TDBConnection_VistA.SetCCOWNotifyNewLinkStatus(aProcedure: TNotifyNewLinkStatus);
begin
  if Assigned(fCCOWContextor) then
    fCCOWContextor.NotifyNewLinkStatus := aProcedure;
end;

procedure TDBConnection_VistA.SetCCOWNotifyNewPatientID(aProcedure: TNotifyNewPatientID);
begin
  if Assigned(fCCOWContextor) then
    fCCOWContextor.NotifyNewPatientID := aProcedure;
end;

procedure TDBConnection_VistA.SaveCommandList(aFileName: string);
begin
  //fCommandNames.SaveToFile(aFileName);
end;

function TDBConnection_VistA.XMLDateTime(aString: string): string;
var
  aYear: Word;
  aMonth: Word;
  aDay: Word;
  aHour: Word;
  aMinute: Word;
  aSecond: Word;
begin
  if aString = '' then
    Result := ''
  else
    begin
      aString := FloatToStr(StrToFloat(aString) + 0.0000001);
      aYear := StrToInt(Copy(aString, 1, 3)) + 1700;
      aMonth := StrToInt(Copy(aString, 4, 2));
      aDay := StrToInt(Copy(aString, 6, 2));
      aHour := StrToIntDef(Copy(aString, 9, 2), 0);
      aMinute := StrToIntDef(Copy(aString, 11, 2), 0);
      aSecond := StrToIntDef(Copy(aString, 13, 2), 0);
      Result := DateTimeToStr(EncodeDateTime(aYear, aMonth, aDay, aHour, aMinute, aSecond, 0));
    end;
end;

{------------------------------- IUser interface ------------------------------}

function TDBConnection_VistA.DUZ: string;
begin
  if fRPCBroker.Connected then
    Result := fRPCBroker.User.DUZ
  else
    Result := '';
end;

function TDBConnection_VistA.Name: string;
begin
  if fRPCBroker.Connected then
    Result := fRPCBroker.User.StandardName
  else
    Result := '';
end;

procedure TDBConnection_VistA.NotifyImportStatus(const aMsg: string; aProgress, aProgressMax: integer);
begin
  // No null pointer
end;

procedure TDBConnection_VistA.SetExportVersion(aValue: integer);
begin
  fExportHeader.ChildValues['VERSION'] := aValue;
end;

function TDBConnection_VistA.Station_ID: string;
begin
  if fRPCBroker.Connected then
    Result := Piece(fRPCBroker.User.Division, '^', 3)
  else
    Result := '';
end;

function TDBConnection_VistA.Station_Name: string;
begin
  if fRPCBroker.Connected then
    Result := Piece(fRPCBroker.User.Division, '^', 2)
  else
    Result := '';
end;

{-------------------------- ILegacyBroker Connection --------------------------}

function TDBConnection_VistA.CreateContext(strContext: string): boolean;
begin
  Result := fRPCBroker.CreateContext(strContext);
end;

function TDBConnection_VistA.getConnected: boolean;
begin
  Result := fRPCBroker.Connected;
end;

function TDBConnection_VistA.getListenerPort: integer;
begin
  Result := fRPCBroker.ListenerPort;
end;

function TDBConnection_VistA.getResults: TStringList;
begin
  Result := fRPCResults;
end;

function TDBConnection_VistA.getRowID(aTableName: string; aKeyFields, aKeyValues: array of variant): string;
var
  aLst: TStringList;
  i: integer;
begin
  aLst := TStringList.Create;
  Result := '';
  try
    for i := Low(aKeyValues) to High(aKeyValues) do
      aLst.Add(aKeyValues[i]);

    CallServer(CLIO_RPC, ['GETIEN', aTableName], aLst);
    Result := fRPCResults[0];
  except
    Result := '';
  end;
  FreeAndNil(aLst);
end;

function TDBConnection_VistA.getRPCTimeLimit: integer;
begin
  Result := fRPCBroker.RPCTimeLimit;
end;

function TDBConnection_VistA.getServer: string;
begin
  Result := fRPCBroker.Server;
end;

procedure TDBConnection_VistA.setConnected(const Value: boolean);
begin
  fRPCBroker.Connected := Value;
end;

procedure TDBConnection_VistA.SetExportDescription(aValue: string);
begin
  fExportHeader.ChildValues['DESCRIPTION'] := aValue;
end;

procedure TDBConnection_VistA.SetExportFramework(aValue: string);
begin
  fExportHeader.ChildValues['FRAMEWORK'] := aValue;
end;

procedure TDBConnection_VistA.setListenerPort(const Value: integer);
begin
  fRPCBroker.ListenerPort := Value;
end;

procedure TDBConnection_VistA.setNotifyImportProgress(aNotifyOnProgress: TNotifyOnProgress);
begin
  fOnNotifyImportStatus := aNotifyOnProgress;
end;

procedure TDBConnection_VistA.setRPCTimeLimit(const Value: integer);
begin
  fRPCBroker.RPCTimeLimit := Value;
end;

procedure TDBConnection_VistA.setServer(const Value: string);
begin
  fRPCBroker.Server := Value;
end;

function TDBConnection_VistA.HasRole(aRoleName: string): boolean;
begin
  Result := fRoles.IndexOf(aRoleName) > -1;
end;

function TDBConnection_VistA.IsThisExecutableValid: boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
begin
  {Prepare for the worst}
  Result := False;
  fLastError := 'Error retrieving version compatiblity.';

  with TVersionInformation.Create(Application.ExeName, 'MD') do
    try
      aXMLDoc := NewXMLDocument;
      ExecQuery('GetVersionInformation', [getFileNameAndVersion], aXMLDoc);
      aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
      if Assigned(aXMLNode) then
        case (XMLValueAsInt(aXMLNode, 'COMPATIBLE') = 1) of
          True:
            begin
              Result := True;
              fLastError := '';
            end;
          False:
            begin
              Result := False;
              fLastError := Format('Version %s is marked as incompatible.', [getFileNameAndVersion]);
            end;
        end
      else
        begin
          Result := False;
          fLastError := Format('Version %s was not found in the compatiblity list.', [getFileNameAndVersion]);
        end
    finally
      aXMLNode := nil;
      aXMLDoc := nil;
      Free;
    end;
end;

function TDBConnection_VistA.GetLastError: string;
begin
  Result := fLastError;
end;

end.

