unit oDBController;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 3 $  $Modtime: 4/08/08 9:52a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Base controller for all other controller types
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oDBController.pas $
*
* $History: oDBController.pas $
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 4/08/08    Time: 8:13p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 1/16/08    Time: 4:22p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * New progress indicator
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:16p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 4/10/07    Time: 3:28p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Revamp of terminology to all interfaces.
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 4/03/07    Time: 9:29a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 2/23/07    Time: 1:54p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Post CMO conference
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:07p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  oInterfaces,
  XMLIntf,
  XMLDoc,
  Classes;

type
  TDBController = class(TInterfacedObject)
  private
    fControllerName: string;
    fDBConnection: IDBConnection;
    procedure NotifyControllerStatus(const aMsg: string; aProgress, aProgressMax: integer);
  public
    OnNotifyControllerStatus: TNotifyOnProgress;
    constructor Create;
    destructor Destroy; override;
    function ControllerName: string; stdcall;
    function DBConnection: IDBConnection; stdcall;
    function Initialize(aName: string; aDBConnection: IDBConnection): Boolean; overload; stdcall;
    function Initialize(aName: string; aDBType: integer; aParams: array of Variant): Boolean; overload; stdcall;
    function Initialize(aName: string; aXMLDocument: IXMLDocument): Boolean; overload; stdcall;
    procedure SaveCommands;
    procedure Shutdown; stdcall;
  end;

implementation

{ TDBController }

{
**************************************************** TDBController *****************************************************
}

constructor TDBController.Create;
begin
  inherited Create;
  OnNotifyControllerStatus := NotifyControllerStatus;
  fControllerName := ClassName;
end;

destructor TDBController.Destroy;
begin
  fDBConnection := nil;
  inherited;
end;

function TDBController.ControllerName: string;
begin
  Result := fControllerName;
end;

function TDBController.DBConnection: IDBConnection;
begin
  Result := fDBConnection;
end;

function TDBController.Initialize(aName: string; aDBConnection: IDBConnection): Boolean;
{This method allows multiple controllers to connect to the same database without a new IDBConnection}
begin
  fControllerName := aName;
  fDBConnection := aDBConnection;
  Result := True;
end;

function TDBController.Initialize(aName: string; aDBType: integer; aParams: array of Variant): Boolean;
begin
  OnNotifyControllerStatus('Initializing DB Controller...',0,0);
  fControllerName := aName;
  case aDBType of
    0: fDBConnection := NewDBConnection_VistA;
    1: fDBConnection := NewDBConnection_MsAccess;
    2: fDBConnection := NewDBConnection_CacheSQL;
    3: fDBConnection := NewDBConnection_J2EE;
  end;

  if fDBConnection.Connect(aParams) then
    begin
      OnNotifyControllerStatus('Controller Ready.',0,0);
      Result := True;
    end
  else
    begin
      OnNotifyControllerStatus('Controller Initialization Failure.',0,0);
      fDBConnection := nil;
      Result := False;
    end;
end;

function TDBController.Initialize(aName: string; aXMLDocument: IXMLDocument): Boolean;
var
  aXMLNode: IXMLNode;
  aDBType: Integer;
  aParams: array of variant;
  i: Integer;
begin
  with aXMLDocument do
    try
      aXMLNode := aXMLDocument.DocumentElement;

      {Make sure we have a connect command file}
      if aXMLNode.ChildValues['TASK_NAME'] <> 'CONNECT' then
        begin
          Result := False;
          Exit;
        end;

      aXMLNode := aXMLNode.ChildNodes[aName];
      aDBType := aXMLNode.ChildValues['DBTYPE'];
      aXMLNode := aXMLNode.ChildNodes['PARAMS'];

      {Load up the params}
      i := aXMLNode.Attributes['COUNT'];
      SetLength(aParams, i);
      i := 0;
      aXMLNode := aXMLNode.ChildNodes.First;
      while aXMLNode <> nil do
        begin
          aParams[i] := aXMLNode.NodeValue;
          aXMLNode := aXMLNode.NextSibling;
          inc(i);
        end;
      Result := Initialize(aName, aDBType, aParams);
    except
      Result := False;
    end;
end;

procedure TDBController.NotifyControllerStatus(const aMsg: string; aProgress, aProgressMax: integer);
begin
  // {Prevents null pointers}
end;

procedure TDBController.SaveCommands;
begin
  DBConnection.SaveCommandList(fControllerName + '.log');
end;

procedure TDBController.Shutdown;
begin
  if Assigned(fDBConnection) then
    if fDBConnection.IsConnected then
      fDBConnection.Disconnect;
end;

end.

