unit oDBController_CPKardex;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 8 $  $Modtime: 5/22/09 1:00p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary controller for interacting with Nursing Care
*                     Actions
*
================================================================================
}
interface

uses
  Dialogs,
  SysUtils,
  Windows,
  Messages,
  Classes,
  oInterfaces,
  oCliO,
  oCliOTypes,
  oDBController,
  oCPTypes,
  oCP_Exceptions,
  ComCtrls,
  XMLDoc,
  XMLIntf;

type
  TDBController_CPKardex = class(TDBController, ICPKardexService, ICliOReportItem)
  protected
    fTerminology: ITerminology;
    fCPService: ICPService;
    fPatient: IPatient;
    fErrors: TStringList;
    fActions: IInterfaceList;
    fEvents: IInterfaceList;

    function getAction(aIndex: integer): ICPKardexAction; overload; stdcall;
    function getAction(aID: string): ICPKardexAction; overload; stdcall;
    function getActionCount: integer; stdcall;
    function getNewAction(aPatient: IPatient): ICPKardexAction; stdcall;

    function getError(aIndex: integer): string; stdcall;
    function getErrorCount: integer; stdcall;

    procedure InsertAction(aCPKardexAction: ICPKardexAction); stdcall;
    procedure UpdateAction(aCPKardexAction: ICPKardexAction); stdcall;
    procedure UpdateActionStatus(aCPKardexAction: ICPKardexAction; aNewStatus: TCPKardexActionStatus);
    procedure InsertEvent(aCPKardexEvent: ICPKardexEvent); stdcall;
    procedure UpdateEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDateTime: TDateTime; aStatus: TCPKardexEventStatus; aComment: string); stdcall;
    procedure UpdateAudit(aCPKardexAction: ICPKardexAction; aUser: IUser; aDescription: string; aComment: string); overload; stdcall;
    procedure UpdateAudit(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDescription: string; aComment: string); overload; stdcall;

    procedure Load(aPatient: IPatient; aStartDateTime, aStopDateTime: TDateTime); stdcall;
    procedure LoadEvents(aCPKardexAction: ICPKardexAction); stdcall;
    procedure LoadActions(aListView: TListView); overload; stdcall;
    procedure LoadActions(aList: TStrings; aActiveOnly: boolean); overload; stdcall;
    procedure LoadAudits(aAction: ICPKardexAction; aList: IInterfaceList); overload; stdcall;
    procedure LoadAudits(aEvent: ICPKardexEvent; aList: IInterfaceList); overload; stdcall;
    procedure DiscontinueAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aComment: string); stdcall;
    procedure HoldAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aHoldStartDate, aHoldStopDate: TDateTime; aComment: string); stdcall;
    procedure HoldEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aHoldDate: TDateTime; aComment: string); stdcall;
    procedure UnHoldAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aUnHoldDate: TDateTime; aComment: string); stdcall;
    procedure UnHoldEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aUnHoldDate: TDateTime; aComment: string); stdcall;
    procedure CompleteEvent(aCPKardexAction: ICPKardexAction; aUser: IUser; aDateTime: TDateTIme; aComment: string); overload; stdcall;
    procedure CompleteEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDateTime: TDateTIme; aComment: string); overload; stdcall;
    procedure RenewAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aNewStopDateTime: TDateTime); stdcall;
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;
  public
    constructor Create(aTerminology: ITerminology; aCPService: ICPService);
    destructor Destroy; override;
  end;

implementation

uses
  uCPXML,
  DateUtils,
  Variants,
  oCPKardexAction,
  oCPKardexEvent,
  oCPKardexAudit;

{------------------------------------------------------------------------------}
{-------------------------- TDBController_CPKardex ----------------------------}
{------------------------------------------------------------------------------}

procedure TDBController_CPKardex.UpdateEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDateTime: TDateTime; aStatus: TCPKardexEventStatus; aComment: string);
begin
  try
    DBConnection.BeginTransaction;
    if Assigned(aUser) and (aDateTime > 0) then
      DBConnection.ExecCommand('UpdateKardexEvent',
        [aCPKardexEvent.getEventID, aDateTime, aUser.getDUZ, aStatus, aComment])
    else
      DBConnection.ExecCommand('UpdateKardexEvent',
        [aCPKardexEvent.getEventID, '', '', aStatus, aComment]);
    DBConnection.CommitTransaction;
  except
    DBConnection.RollbackTransaction;
    raise ECPSaveFailure.CreateFmt('Error completing event %s', [aCPKardexEvent.getActionID]);
  end;
end;

procedure TDBController_CPKardex.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  i: integer;
  aAction: ICliOReportItem;
begin
  aReportText.Add('');
  aReportText.Add(StringOfChar('=', aMargin));
  aReportText.Add(StringOfChar(' ', (aMargin - Length('K A R D E X')) div 2) + 'K A R D E X');
  aReportText.Add(StringOfChar('=', aMargin));

  for i := 0 to getActionCount - 1 do
    if getAction(i).QueryInterface(ICliOReportItem, aAction) = 0 then
      aAction.AddToReport(aReportText, aMargin, aIncludeDetail, aList)
    else
      aReportText.Add('Unable to query interface on CP Kardex Action');
end;

procedure TDBController_CPKardex.CompleteEvent(aCPKardexAction: ICPKardexAction; aUser: IUser; aDateTime: TDateTIme; aComment: string);
var
  aEvent: ICPKardexEvent;
begin
  with TCPKardexEvent.Create do
    try
      setActionID(aCPKardexAction.getID);
      setScheduledDateTime(aDateTime);
      setStatus(esPending);
      GetInterface(ICPKardexEvent, aEvent);
      aCPKardexAction.AddEvent(aEvent);
      DBConnection.BeginTransaction;
      DBConnection.ExecCommand('AddKardexEvent', [getEventID, aCPKardexAction.getID, getScheduledDateTime, esPending]);
      DBConnection.CommitTransaction;
      CompleteEvent(aEvent, aUser, aDateTime, aComment);
    except
      DBConnection.RollbackTransaction;
      raise Exception.Create('Unable to complete event.');
    end;
end;

procedure TDBController_CPKardex.CompleteEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDateTime: TDateTIme; aComment: string);
var
  aAction: ICPKardexAction;
begin
  try
    DBConnection.BeginTransaction;
    DBConnection.ExecCommand('UpdateKardexEvent', [aCPKardexEvent.getEventID, aDateTime, aUser.getDUZ, esCompleted, aComment]);
    aCPKardexEvent.setStatus(esCompleted);
    getAction(aCPKardexEvent.getActionID).QueryInterface(ICPKardexAction, aAction);
    if aAction.getScheduleType in [stOneTime, stNow, stStat] then
      DiscontinueAction(aAction, aUser, 'Auto dc upon event completion.');

    DBConnection.CommitTransaction;
  except
    DBConnection.RollbackTransaction;
    raise Exception.Create('Unable to complete event.');
  end;
end;

constructor TDBController_CPKardex.Create(aTerminology: ITerminology; aCPService: ICPService);
begin
  inherited Create;
  fTerminology := aTerminology;
  fCPService := aCPService;
  fErrors := TStringList.Create;
  TInterfaceList.Create.GetInterface(IInterfaceList, fActions);
  TInterfaceList.Create.GetInterface(IInterfaceList, fEvents);
end;

destructor TDBController_CPKardex.Destroy;
begin
  fTerminology := nil;
  fEvents.Clear;
  fEvents := nil;
  fActions.Clear;
  fActions := nil;
  fErrors.Clear;
  FreeAndNil(fErrors);
  inherited;
end;

procedure TDBController_CPKardex.DiscontinueAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aComment: string);
begin
  try
    DBConnection.BeginTransaction;
    DBConnection.ExecCommand('DeleteKardexEvents', [aCPKardexAction.getID, esPending]);
    DBConnection.ExecCommand('DiscontinueKardexAction', [aCPKardexAction.getID, Now, asDiscontinued]);
    UpdateAudit(aCPKardexAction, aUser, 'Discontinued', aComment);
    aCPKardexAction.setStatus(asDiscontinued);
    DBConnection.CommitTransaction;
  except
    DBConnection.RollbackTransaction;
  end;
end;

function TDBController_CPKardex.getAction(aIndex: integer): ICPKardexAction;
begin
  try
    fActions[aIndex].QueryInterface(ICPKardexAction, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPKardex.getAction(aID: string): ICPKardexAction;
var
  i: integer;
begin
  for i := 0 to fActions.Count - 1 do
    if fActions[i].QueryInterface(ICPKardexAction, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPKardex.getActionCount: integer;
begin
  Result := fActions.Count;
end;

function TDBController_CPKardex.getError(aIndex: integer): string;
begin
  try
    Result := fErrors[aIndex];
  except
    Result := Format('No such error number (%d)', [aIndex]);
  end;
end;

function TDBController_CPKardex.getErrorCount: integer;
begin
  Result := fErrors.Count;
end;

function TDBController_CPKardex.getNewAction(aPatient: IPatient): ICPKardexAction;
begin
  TCPKardexAction.Create(nil, aPatient).GetInterface(ICPKardexAction, Result);
end;

procedure TDBController_CPKardex.HoldAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aHoldStartDate, aHoldStopDate: TDateTime; aComment: string);
var
  i: integer;
  aEvent: ICPKardexEvent;
begin
  try
    LoadEvents(aCPKardexAction);

    for i := 0 to aCPKardexAction.getEventCount - 1 do
      begin
        aCPKardexAction.getEvent(i).QueryInterface(ICPKardexEvent, aEvent);
        if (aEvent.getStatus = esPending) and
          (aEvent.getScheduledDateTime >= aHoldStartDate) and
          (aEvent.getScheduledDateTime <= aHoldStopDate) then
          HoldEvent(aEvent, aUser, Now, aComment);
      end;
    aCPKardexAction.setStatus(asHeld);
    UpdateActionStatus(aCPKardexAction, asHeld);
    UpdateAudit(aCPKardexAction, aUser, 'Action placed in Hold Status', aComment);
  except
    on E: Exception do
      raise ECPSaveFailure.Create('Error placing action in Hold Status');
  end;
end;

procedure TDBController_CPKardex.UnHoldAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aUnHoldDate: TDateTime; aComment: string);
var
  i: integer;
  aEvent: ICPKardexEvent;
begin
  try
    LoadEvents(aCPKardexAction);

    for i := 0 to aCPKardexAction.getEventCount - 1 do
      begin
        aCPKardexAction.getEvent(i).QueryInterface(ICPKardexEvent, aEvent);
        if (aEvent.getStatus = esHeld) and
          (aEvent.getScheduledDateTime >= aUnHoldDate) then
          UnHoldEvent(aEvent, aUser, Now, aComment);
      end;
    aCPKardexAction.setStatus(asActive);
    UpdateActionStatus(aCPKardexAction, asActive);
    UpdateAudit(aCPKardexAction, aUser, 'Action placed in Hold Status', aComment);
  except
    on E: Exception do
      raise ECPSaveFailure.Create('Error removing Hold.');
  end;
end;

procedure TDBController_CPKardex.HoldEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aHoldDate: TDateTime; aComment: string);
begin
  UpdateEvent(aCPKardexEvent, aUser, aHoldDate, esHeld, aComment);
  UpdateAudit(aCPKardexEvent, aUser, 'Event placed in Hold Status', aComment);
end;

procedure TDBController_CPKardex.UnHoldEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aUnHoldDate: TDateTime; aComment: string);
begin
  UpdateEvent(aCPKardexEvent, aUser, aUnHoldDate, esPending, aComment);
  UpdateAudit(aCPKardexEvent, aUser, 'Hold Status cleared for event', aComment);
end;

procedure TDBController_CPKardex.InsertAction(aCPKardexAction: ICPKardexAction);
var
  i: integer;
begin
  try
    DBConnection.BeginTransaction;

    with aCPKardexAction do
      DBConnection.ExecCommand('AddKardexAction',
        [getID, getCareActionID, getScheduleID, getPatient.getDFN, getStartDateTime,
        getStopDateTime, getScheduleType, getSchedulePRNType, getStatus,
          getSpecialInstructions, getPRNNTE, getScheduleTimes]);

    fActions.Add(aCPKardexAction);

    {Now we jam in the new set of pendings :)}
    for i := 0 to aCPKardexAction.getEventCount - 1 do
      with aCPKardexAction.getEvent(i) do
        if getStatus = esPending then
          DBConnection.ExecCommand('AddKardexEvent',
            [getEventID, getActionID, getScheduledDateTime, esPending]);

    DBConnection.CommitTransaction;
  except
    DBConnection.RollbackTransaction;
    raise
  end;
end;

procedure TDBController_CPKardex.InsertEvent(aCPKardexEvent: ICPKardexEvent);
begin
  with aCPKardexEvent do
    try
      DBConnection.ExecCommand('AddKardexEvent',
        [getEventID, getActionID, getScheduledDateTime, getStatus]);
    except
    end;
end;

procedure TDBController_CPKardex.UpdateAction(aCPKardexAction: ICPKardexAction);
var
  aNewAction: boolean;
//  aCommandName: string;
//  aXMLDoc: IXMLDocument;
  i: integer;
begin
  aNewAction := False;
  for i := 0 to fActions.Count - 1 do
    if (fActions[i] as ICPKardexAction).getID = aCPKardexAction.getID then
      begin
        aNewAction := False;
        Break;
      end;

  with aCPKardexAction do
    try
      DBConnection.BeginTransaction;

      if aNewAction then
        begin
          DBConnection.ExecCommand('AddKardexAction',
            [getID, getCareActionID, getScheduleID, getPatient.getDFN, getStartDateTime,
            getStopDateTime, getScheduleType, getSchedulePRNType, getStatus,
              getSpecialInstructions, getPRNNTE, getScheduleTimes]);
          fActions.Add(aCPKardexAction);
        end
      else
        DBConnection.ExecCommand('UpdateKardexAction',
          [getID, getCareActionID, getScheduleID, getPatient.getDFN, getStartDateTime,
          getStopDateTime, getScheduleType, getSchedulePRNType, getStatus,
            getSpecialInstructions, getPRNNTE, getScheduleTimes]);

      {Here we get rid of every pending order for this task on the DB}

      DBConnection.ExecCommand('DeleteKardexEvents', [aCPKardexAction.getID, esPending]);

      {Now we jam in the new set of pendings :)}
      for i := 0 to aCPKardexAction.getEventCount - 1 do
        with aCPKardexAction.getEvent(i) do
          if getStatus = esPending then
            DBConnection.ExecCommand('AddKardexEvent',
              [NewGUID, getActionID, getScheduledDateTime, esPending]);

      DBConnection.CommitTransaction;
    except
      DBConnection.RollbackTransaction;
      raise
    end;
end;

procedure TDBController_CPKardex.Load(aPatient: IPatient; aStartDateTime, aStopDateTime: TDateTime);
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aCPKardexAction: ICPKardexAction;
//  aCPKardexEvent: ICPKardexEvent;
  i: integer;
begin
  fActions.Clear;
  fErrors.Clear;

  try
    fPatient := aPatient;
    aXMLDoc := NewXMLDocument;
    DBConnection.ExecQuery('GetKardexActionList', [aPatient.getDFN, aStartDateTime, aStopDateTime], aXMLDoc);

    aXMLDoc.Active := True;
    aXMLNode := XMLDocumentFirstRecord(aXMLDoc);

    while Assigned(aXMLNode) do
      try
        TCPKardexAction.Create(aXMLNode, aPatient).GetInterface(ICPKardexAction, aCPKardexAction);
        aCPKardexAction.setTask(fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'TASK_ID')));
        aCPKardexAction.setSchedule(fCPService.CPSchedule(XMLValueAsStr(aXMLNode, 'SCHEDULE_ID')));
        fActions.Add(aCPKardexAction);
      finally
        aXMLNode := XMLDocumentNextRecord(aXMLNode);
      end;

    for i := 0 to fActions.Count - 1 do
      begin
        fActions[i].QueryInterface(ICPKardexAction, aCPKardexAction);

        { Check to see if it is expired :) }
        if (aCPKardexAction.getStopDateTime < Now) and (aCPKardexAction.getStatus = asActive) then
          begin {Delete all pendings and then change the status to asExpired}
            DBConnection.ExecCommand('DeleteKardexEvents', [aCPKardexAction.getID, esPending]);
            DBConnection.ExecCommand('UpdateKardexActionStatus', [aCPKardexAction.getID, asExpired]);
            aCPKardexAction.setStatus(asExpired);
          end;

        LoadEvents(aCPKardexAction);
      end;

    if fErrors.Count > 0 then
      raise ECPLoadControllerFail.Create('Errors found');
  except
    raise ECPLoadControllerFail.CreateFmt(
      'Error loading Kardex, %d error(s) encountered.', [fErrors.Count]);
  end;
end;

procedure TDBController_CPKardex.LoadActions(aListView: TListView);
var
  i: integer;
  aItem: ICPGUILoader;
begin
  aListView.Items.Clear;
  for i := 0 to fActions.Count - 1 do
    if fActions[i].QueryInterface(ICPGUILoader, aItem) = 0 then
      aItem.AddToListView(aListView);
end;

procedure TDBController_CPKardex.LoadActions(aList: TStrings; aActiveOnly: boolean);
var
  i: integer;
  aAction: ICPKardexAction;
begin
  aList.Clear;
  for i := 0 to fActions.Count - 1 do
    if fActions[i].QueryInterface(ICPKardexAction, aAction) = 0 then
      if not aActiveOnly then
        aAction.AddToList(aList)
      else if aAction.getActive then
        aAction.AddToList(aList);
end;

procedure TDBController_CPKardex.LoadAudits(aAction: ICPKardexAction; aList: IInterfaceList);
begin

end;

procedure TDBController_CPKardex.LoadAudits(aEvent: ICPKardexEvent; aList: IInterfaceList);
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aCPKardexAudit: ICPKardexAudit;
begin
  aList.Clear;

  aXMLDoc := NewXMLDocument;

  DBConnection.ExecQuery('GetKardexEventAudits',
    [aEvent.getEventID], aXMLDoc);

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);

  while Assigned(aXMLNode) do
    try
      with TCPKardexAudit.Create do
        if LoadFromXML(aXMLNode) then
          if GetInterface(ICPKardexAudit, aCPKardexAudit) then
            aList.Add(aCPKardexAudit);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;
end;

procedure TDBController_CPKardex.LoadEvents(aCPKardexAction: ICPKardexAction);
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aCPKardexEvent: ICPKardexEvent;
begin
  aCPKardexAction.ClearEvents;

  aXMLDoc := NewXMLDocument;

  DBConnection.ExecQuery('GetKardexEventList',
    [aCPKardexAction.getID], aXMLDoc);

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);

  while Assigned(aXMLNode) do
    try
      with TCPKardexEvent.Create do
        if LoadFromXML(aXMLNode) then
          if GetInterface(ICPKardexEvent, aCPKardexEvent) then
            aCPKardexAction.AddEvent(aCPKardexEvent);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;
end;

procedure TDBController_CPKardex.RenewAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aNewStopDateTime: TDateTime);
//var
//  aOriginalStopDateTime: TDateTime;
//  i: integer;
begin
  aCPKardexAction.DebugDisplay;
//  aOriginalStopDateTime := aCPKardexAction.getStopDateTime;
  aCPKardexAction.ExtendSchedule(aNewStopDateTime);
  aCPKardexAction.DebugDisplay;
  UpdateAction(aCPKardexAction);
end;

procedure TDBController_CPKardex.UpdateAudit(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDescription, aComment: string);
begin
  DBConnection.ExecCommand('AddKardexAudit',
    [aCPKardexEvent.getActionID, aCPKardexEvent.getEventID, Now, aUser.getDUZ, aDescription, aComment]);
end;

procedure TDBController_CPKardex.UpdateAudit(aCPKardexAction: ICPKardexAction; aUser: IUser; aDescription, aComment: string);
begin
  DBConnection.ExecCommand('AddKardexAudit',
    [aCPKardexAction.getID, '', Now, aUser.getDUZ, aDescription, aComment]);
end;

procedure TDBController_CPKardex.UpdateActionStatus(aCPKardexAction: ICPKardexAction; aNewStatus: TCPKardexActionStatus);
begin
  DBConnection.ExecCommand('UpdateKardexActionStatus', [aCPKardexAction.getID, aNewStatus]);
end;

end.

