unit oDBController_Patient;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 17 $  $Modtime: 11/02/11 10:16a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default controller for managing patients in VistA
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oDBController_Patient.pas $
================================================================================
}
interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  ComCtrls,
  XMLDoc,
  oCPTypes,
  oCP_Exceptions,
  oDBController,
  oInterfaces,
  oCPPatient,
  XMLIntf;

type
  TDBController_Patient = class(TDBController)
  private
    fLocations: IInterfaceList;
    function getDFNfromICN(aICN: string): string;
  public
    constructor Create;
    destructor Destroy; override;
    function ExecuteLookupByName(aLookupValue: string; aList: IInterfaceList): Boolean; stdcall;
    function ExecuteLookupByWard(aLookupValue: string; aList: IInterfaceList): Boolean; stdcall;
    function getPatient(aID: string): IPatient;
    function Load: Boolean;
    function GetLocationByName(aLocationName: string): ICPLocation;
    function GetLocationIndexByID(aList: TStrings; aLocationID: string): integer;
    function GetLocationIndexByName(aList: TStrings; aLocationName: string): integer;
    procedure LoadLocations(aList: TStrings; aType: TCPLocationType = ltUnknown);
    procedure UnLockSensitivePatient(aPatient: IPatient; aUser: IUser); stdcall;
    function CanResendADT(aPatient: IPatient): boolean;
    //function GetPersonName(aID: string):string; stdcall;
    //function GetPersonList(aValue: string; aList: TStrings): integer; stdcall;
  end;

function NewDBController_Patient: TDBController_Patient;

function NewCPPatient(aXMLNode: IXMLNode): TCPPatient;

implementation

uses
  Variants,
  uCPXML,
  Forms;

function NewDBController_Patient: TDBController_Patient;
begin
  Result := TDBController_Patient.Create;
end;

function NewCPPatient(aXMLNode: IXMLNode): TCPPatient;
begin
  Result := TCPPatient.Create;
  try
    Result.LoadFromXML(aXMLNode);
  except
    FreeAndNil(Result);
  end;
end;

{************************* TDBController_Patient ******************************}

function TDBController_Patient.CanResendADT(aPatient: IPatient): boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
begin
  aXMLDoc := NewXMLDocument;
  aXMLDoc.Active := true;

  result := false; // 2FA

  if not DBConnection.ExecQuery('CanResendADT', [aPatient.getDFN], aXMLDoc) then
    result := false
  else
  begin
    aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
    if Assigned(aXMLNode) then
    begin
      result := XMLValueAsInt(aXMLNode, 'STATUS', 0) = 1;
      if result then
      begin
        aPatient.setLastMessageID(XMLValueAsInt(aXMLNode,'MESSAGE',0));
      end;
    end;
  end;
end;

constructor TDBController_Patient.Create;
begin
  inherited Create;
  TInterfaceList.Create.GetInterface(IInterfaceList, fLocations);
end;

destructor TDBController_Patient.Destroy;
begin
  fLocations.Clear;
  inherited;
end;

function TDBController_Patient.ExecuteLookupByName(aLookupValue: string; aList: IInterfaceList): Boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aPatient: IPatient;
begin
  try
    aList.Clear;
    aXMLDoc := NewXMLDocument;
    if not DBConnection.ExecQuery('GetPatientList', [aLookupValue], aXMLDoc) then
      Result := False
    else
      begin
        aXMLDoc.Active := True;
        aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
        while aXMLNode <> nil do
          try
            NewCPPatient(aXMLNode).GetInterface(IPatient, aPatient);
            aList.Add(aPatient);
          finally
            aXMLNode := XMLDocumentNextRecord(aXMLNode);
          end;
        Result := True;
      end;
  except
    Result := False;
  end;
end;

function TDBController_Patient.ExecuteLookupByWard(aLookupValue: string; aList: IInterfaceList): Boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aPatient: IPatient;
begin
  try
    aList.Clear;
    aXMLDoc := NewXMLDocument;
    if not DBConnection.ExecQuery('GetPatientListByWard', [aLookupValue], aXMLDoc) then
      Result := False
    else
      begin
        aXMLDoc.Active := True;
        aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
        while aXMLNode <> nil do
          try
            NewCPPatient(aXMLNode).GetInterface(IPatient, aPatient);
            aList.Add(aPatient);
          finally
            aXMLNode := XMLDocumentNextRecord(aXMLNode);
          end;
        Result := True;
      end;
  except
    Result := False;
  end;
end;

function TDBController_Patient.getDFNfromICN(aICN: string): string;
//var
//  aXMLNode: IXMLNode;
begin
  (*
    with NewXMLDocument do
      try
        XML.Text := DBConnection.ExecQuery('GetDFNfromICN', [aICN]);
        Active := True;
        aXMLNode := DocumentElement.ChildNodes.FindNode('RECORD');
        if aXMLNode <> nil then
          Result := aXMLNode.ChildValues['DFN']
        else
          Result := '';
      except
        Result := '';
      end;
  *)
end;

function TDBController_Patient.getPatient(aID: string): IPatient;
var
  aNode: IXMLNode;
  aPT: TCPPatient;
  aXMLDoc: IXMLDocument;
begin
  Result := nil;

  {Does it even have data?}
  if aID = '' then
    Exit;

  {Is it an ICN?}
  if Pos('V', aID) > 0 then
    aID := getDFNfromICN(aID);

  {Is it a canonic dfn?}
  if StrToInt64Def(aID, -1) > 0 then // note DFN could be Int64
    aID := '`' + aID
  else
    Exit; {Can't find valid dfn} // should log the wrong value

  {OK, we are assuming that we have a valid DFN in aID}
  aXMLDoc := NewXMLDocument;
  if not DBConnection.ExecQuery('GetPatientList', [aID], aXMLDoc) then
    raise ECPLoadControllerFail.CreateFmt('Error loading patient %s.', [aID]);

  aXMLDoc.Active := True;
  aNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('RECORD');
  if Assigned(aNode) then
    begin
      aPt := NewCPPatient(aNode);
      if aPT <> nil then
        aPt.GetInterface(IPatient, Result);
    end
  else
    raise ECPLoadControllerFail.CreateFmt('Error, patient %s not found.', [aID]);
end;

function TDBController_Patient.GetLocationByName(aLocationName: string): ICPLocation;
var
  i: integer;
begin
  for i := 0 to fLocations.Count - 1 do
    if fLocations[i].QueryInterface(ICPLocation, Result) = 0 then
      if (Result.getName = aLocationName) then
        Exit;
  Result := nil;
end;

function TDBController_Patient.GetLocationIndexByID(aList: TStrings; aLocationID: string): integer;
var
  i: integer;
  aCPLocation: ICPLocation;
begin
  for i := 0 to aList.Count - 1 do
    if aList.Objects[i] <> nil then
      if aList.Objects[i].GetInterface(ICPLocation, aCPLocation) then
        if (aCPLocation.getID = aLocationID) then
          begin
            Result := i;
            Exit;
          end;
  Result := -1;
end;

function TDBController_Patient.GetLocationIndexByName(aList: TStrings; aLocationName: string): integer;
var
  i: integer;
  aCPLocation: ICPLocation;
begin
  for i := 0 to aList.Count - 1 do
    if aList.Objects[i] <> nil then
      if aList.Objects[i].GetInterface(ICPLocation, aCPLocation) then
        if (aCPLocation.getName = aLocationName) then
          begin
            Result := i;
            Exit;
          end;
  Result := -1;
end;

function TDBController_Patient.Load: Boolean;
var
  aXMLDoc: IXMLDocument;
  aNode: IXMLNode;
  aLocation: ICPLocation;
begin
  result := false;

  aXMLDoc := NewXMLDocument;
  if not DBConnection.ExecQuery('GetFacilityLocations', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error loading hospital locations');

  aXMLDoc.Active := True;
  aNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('RECORD');
  while aNode <> nil do
    try
      aLocation := NewCPLocation(aNode);
      if Assigned(aLocation) then
        fLocations.Add(aLocation);
    finally
      aNode := aNode.NextSibling;
    end;
end;

procedure TDBController_Patient.LoadLocations(aList: TStrings; aType: TCPLocationType = ltUnknown);
var
  i: integer;
  aCPLocation: ICPLocation;
begin
  for i := 0 to fLocations.Count - 1 do
    if fLocations[i].QueryInterface(ICPLocation, aCPLocation) = 0 then
      if (aType = aCPLocation.getType) or (aType = ltUnknown) then
        aCPLocation.AddToList(aList);
end;

procedure TDBController_Patient.UnLockSensitivePatient(aPatient: IPatient; aUser: IUser);
var
  aSensitivePatient: ISensitivePatient;
begin
  try
    if aPatient.QueryInterface(ISensitivePatient, aSensitivePatient) = 0 then
      begin
        DBConnection.ExecCommand('LogSecurityAccess', [aPatient.getDFN]);
        aSensitivePatient.Unlock;
      end
    else
      raise Exception.Create('Unable to clear patient sensitivity flag.');
  except
    raise;
  end;
end;

end.

