unit oDBController_Person;

interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  ComCtrls,
  XMLDoc,
  XMLIntf,
  uCPXML,
  oCPTypes,
  oCP_Exceptions,
  oInterfaces;

type
  TDBController_Person = class(TInterfacedObject, IPersonService)
  private
    fDBConnection: IDBConnection;
    fCache: IInterfaceList;
    function ExecuteLookup(aLookupValue: string; aList: IInterfaceList): integer; stdcall;
    function getPerson(aID: string): IPerson; stdcall;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

uses
  oCPPerson;

const
  MAX_CACHE = 100;

  { TDBController_Person }

constructor TDBController_Person.Create(aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  TInterfaceList.Create.GetInterface(IInterfaceList, fCache);
end;

destructor TDBController_Person.Destroy;
begin
  fCache.Clear;
  fCache := nil;
  fDBConnection := nil;
  inherited;
end;

function TDBController_Person.ExecuteLookup(aLookupValue: string; aList: IInterfaceList): integer;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aPerson: IPerson;
begin
  aList.Clear;
  aXMLDoc := NewXMLDocument;
  if fDBConnection.ExecQuery('GetPersonList', [aLookupValue], aXMLDoc) then
    try
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
      while Assigned(aXMLRec) do
        try
          TCPPerson.Create(aXMLRec).GetInterface(IPerson, aPerson);
          aList.Add(aPerson);
        finally
          aXMLRec := XMLDocumentNextRecord(aXMLRec);
        end;
    except
      aList.Clear;
    end;
  Result := aList.Count;
end;

function TDBController_Person.getPerson(aID: string): IPerson;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
//  aPerson: IPerson;
  i: integer;
begin
  for i := 0 to fCache.Count - 1 do
    if (fCache[i] as IPerson).getID = aID then
      begin
        fCache[i].QueryInterface(IPerson, Result);
        if i > 0 then
          fCache.Exchange(0, i);
        Exit;
      end;

  aXMLDoc := NewXMLDocument;
  try
    fDBConnection.ExecQuery('GetPerson', [aID], aXMLDoc);
    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    TCPPerson.Create(aXMLRec).GetInterface(IPerson, Result);
    fCache.Insert(0, Result);
    while fCache.Count > MAX_CACHE do
      fCache.Remove(fCache.Last);
  except
    Result := nil;
  end;
end;

end.

