unit oFileManField;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 2 $  $Modtime: 4/08/08 9:58a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Manages a single fileman field.
*
*       Notes:        HISTORICAL PURPOSES ONLY, NOT USED IN PRODUCTION/DRP
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oFileManField.pas $
*
* $History: oFileManField.pas $
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 4/08/08    Time: 8:13p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:08p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Classes,
  Controls,
  StdCtrls,
  ExtCtrls,
  SysUtils,
  oInterfaces;

type
  TFileManFieldType = (ftText, ftPointer, ftSet);

  TFileManControlType = (fmUnknown, fmTEdit, fmTLabel, fmTComboBox, fmTRadioGroup, fmTCheckBox);

  TFileManField = class (TInterfacedObject)
  private
    fChangeMade: Boolean;
    fControl: TControl;
    fControlType: TFileManControlType;
    fDDNumber: string;
    fExternalValue: string;
    fFieldNumber: string;
    fFieldType: TFileManFieldType;
    fIEN: string;
    fInternalValue: string;
  public
    function getChangeMade: Boolean; stdcall;
    function getControl: TControl; stdcall;
    function getControlType: TFileManControlType; stdcall;
    function getDDNumber: string; stdcall;
    function getDescription: string; stdcall;
    function getExternalValue: string; stdcall;
    function getFieldNumber: string; stdcall;
    function getFieldType: TFileManFieldType; stdcall;
    function getIEN: string; stdcall;
    function getIENs: string; stdcall;
    function getInternalValue: string; stdcall;
    procedure Init(aDBConnection: IDBConnection); stdcall;
    function IsValid(aDBConnection: IDBConnection; aValue: string): Boolean; stdcall;
    procedure Load(aDBConnection: IDBConnection); stdcall;
    procedure setChangeMade(const aValue: boolean); stdcall;
    procedure setDisplayValue(const aValue: string); stdcall;
    procedure setExternalValue(const aValue: string); stdcall;
    procedure setIEN(const aValue: string); stdcall;
    procedure setInternalValue(const aValue: string); stdcall;
  end;
  
type
  TLookupValue = class (TObject)
    ExternalValue: string;
    InternalValue: string;
  end;
  
const
  RPC = 'ZZDRP FMAN';

function NewFileManField(aControl: TControl; aDDNumber, aFieldNumber: string; aFieldType: TFileManFieldType): TFileManField;

implementation

uses
  Dialogs;

const
  FM_CONTROL_TYPE: array[TFileManControlType] of string = ('TUnknown', 'TEdit', 'TLabel', 'TComboBox', 'TRadioGroup', 'TCheckBox');

function NewFileManField(aControl: TControl; aDDNumber, aFieldNumber: string; aFieldType: TFileManFieldType): TFileManField;

  function GetControlType(aControl: TControl): TFileManControlType;
  begin
    for Result := Low(TFileManControlType) to High(TFileManControlType) do
      if aControl.ClassNameIs(FM_CONTROL_TYPE[Result]) then
        Exit;
    Result := fmUnknown;
  end;

begin
  Result := TFileManField.Create;
  Result.fDDNumber := aDDNumber;
  Result.fFieldNumber := aFieldNumber;
  Result.fFieldType := aFieldType;
  Result.fControl := aControl;
  Result.fControlType := GetControlType(aControl);
end;

{ TFileManField }

{
**************************************************** TFileManField *****************************************************
}
function TFileManField.getChangeMade: Boolean;
begin
  Result := fChangeMade;
end;

function TFileManField.getControl: TControl;
begin
  Result := fControl;
end;

function TFileManField.getControlType: TFileManControlType;
begin
  Result := fControlType;
end;

function TFileManField.getDDNumber: string;
begin
  Result := fDDNumber;
end;

function TFileManField.getDescription: string;
begin
  Result := Format('DD: %s, FieldNumber: %s', [fDDNumber, fFieldNumber]);
end;

function TFileManField.getExternalValue: string;
begin
  Result := fExternalValue;
end;

function TFileManField.getFieldNumber: string;
begin
  Result := fFieldNumber;
end;

function TFileManField.getFieldType: TFileManFieldType;
begin
  Result := fFieldType;
end;

function TFileManField.getIEN: string;
begin
  if fIEN <> '' then
    Result := Copy(fIEN, 2, Length(fIEN))
  else
    Result := '';
end;

function TFileManField.getIENs: string;
begin
  if fIEN <> '' then
    Result := Format('%s,', [Copy(fIEN, 2, Length(fIEN))])
  else
    Result := '';
end;

function TFileManField.getInternalValue: string;
begin
  Result := fInternalValue;
end;

procedure TFileManField.Init(aDBConnection: IDBConnection);
var
  i: Integer;
  LkupVal: TLookupValue;
begin
  case fControlType of
    fmTComboBox:
      with aDBConnection do
        begin
          ExecStoredProc('ZZDRP FMAN', ['GETCODES', fDDNumber, '', fFieldNumber], nil);
          for i := 1 to ExecResults.Count - 1 do
            begin
              LkupVal := TLookupValue.Create;
              LkupVal.InternalValue := Copy(ExecResults[i], 1, Pos(':', ExecResults[i]) - 1);
              LkupVal.ExternalValue := Copy(ExecResults[i], Pos(':', ExecResults[i]) + 1, Length(ExecResults[i]));
              TComboBox(fControl).Items.AddObject(LkupVal.ExternalValue, LkupVal);
            end
        end;
    fmTRadioGroup:
      with aDBConnection do
        begin
          ExecStoredProc('ZZDRP FMAN', ['GETCODES', fDDNumber, '', fFieldNumber], nil);
          for i := 1 to ExecResults.Count - 1 do
            begin
              LkupVal := TLookupValue.Create;
              LkupVal.InternalValue := Copy(ExecResults[i], 1, Pos(':', ExecResults[i]) - 1);
              LkupVal.ExternalValue := Copy(ExecResults[i], Pos(':', ExecResults[i]) + 1, Length(ExecResults[i]));
              TRadioGroup(fControl).Items.AddObject(LkupVal.ExternalValue, LkupVal);
            end
        end;
  end;
end;

function TFileManField.IsValid(aDBConnection: IDBConnection; aValue: string): Boolean;
  
  {Check the validity of the value entered into a control}
  
begin
  with aDBConnection do
    try
      if ExecStoredProc(RPC, ['ISVALID', getDDNumber, getIENs, getFieldNumber, aValue], nil) then
        begin
          fInternalValue := ExecResults[1];
          fExternalValue := ExecResults[2];
          Result := True;
        end
      else
        begin
          if fFieldType <> ftPointer then
            begin
              ExecResults.Delete(0); {flag node}
              MessageDlg(ExecResults.Text, mtError, [mbOk], 0);
            end;
          Result := False;
        end
    except
      Result := False;
    end;
end;

procedure TFileManField.Load(aDBConnection: IDBConnection);
begin
  with aDBConnection do
    try
      ExecStoredProc(RPC, ['GETVAL', getDDNumber, getIENS, getFieldNumber], nil);
      fInternalValue := ExecResults[1];
      fExternalValue := ExecResults[2];
      setDisplayValue(fExternalValue);
    except
      //
    end;
end;

procedure TFileManField.setChangeMade(const aValue: boolean);
begin
  fChangeMade := aValue;
end;

procedure TFileManField.setDisplayValue(const aValue: string);
begin
  case getControlType of
    fmTEdit: TEdit(getControl).text := aValue;
    fmTLabel: TLabel(getControl).Caption := aValue;
    fmTComboBox: TComboBox(getControl).ItemIndex := TComboBox(getControl).Items.IndexOf(aValue);
    fmTRadioGroup: TRadioGroup(getControl).ItemIndex := TRadioGroup(getControl).Items.IndexOf(aValue);
    fmTCheckBox: TCheckBox(getControl).Checked := (fInternalValue = '1');
  end;
end;

procedure TFileManField.setExternalValue(const aValue: string);
begin
  fExternalValue := aValue;
end;

procedure TFileManField.setIEN(const aValue: string);
begin
  fIEN := aValue;
end;

procedure TFileManField.setInternalValue(const aValue: string);
begin
  fInternalValue := aValue;
end;

end.

