{-----------------------------------------------------------------------------
 Package:        MD - Clinical Procedures / CliO
 Developer(s):   Brian Juergensmeyer
 Date Created:   13-Oct-2006
 Site Name:      Hines OIFO
 Description:
 History:
   $Header: /CP Modernization/CliO/source/Common/objects/oHL7Message.pas 2     9/05/08 10:08p Zzzzzzpetitd $
   $History: oHL7Message.pas $
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 9/05/08    Time: 10:08p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Gateway work!
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:17p
 * Created in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 8  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 12:20p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 7  *****************
 * User: Zzzzzzjuergb Date: 10/24/07   Time: 9:26a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 *
 * *****************  Version 6  *****************
 * User: Zzzzzzjuergb Date: 9/14/07    Time: 2:24p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Build 257
 *
 * *****************  Version 5  *****************
 * User: Zzzzzzjuergb Date: 6/19/07    Time: 2:24p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * After upgrade to new settings repository
 *
 * *****************  Version 4  *****************
 * User: Zzzzzzjuergb Date: 4/21/07    Time: 11:29p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Pre Spacelabs test.
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 3/26/07    Time: 9:00a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Problems with multiple messages.
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzjuergb Date: 10/13/06   Time: 1:40p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Renamed uHL7Message.pas to oHL7Message.pas per naming conventions.
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzjuergb Date: 10/13/06   Time: 10:27a
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Checkin.
-----------------------------------------------------------------------------}

unit oHL7Message;

interface

uses
  Classes,
  SysUtils,
  IniFiles,
  oGateway,
  System.Types;

type
  EHL7MessageError = class(Exception);

  THL7Message = class;
  THL7SegmentList = class;

  THL7Segment = class(TPersistent)
  private
    FParent: THL7SegmentList;
    FSegment: TStringList;
    procedure Initialize(AParent: THL7SegmentList);
  protected
    function GetField(Index: Integer): string;
    procedure SetField(Index: Integer; const Value: string);
  public
    constructor Create(AParent: THL7SegmentList); overload;
    constructor Create(AParent: THL7SegmentList; const HL7Segment: string); overload;
    destructor Destroy; override;
    function ToString: string;
    function Index: integer;
    property Field[Index: Integer]: string read GetField write SetField; default;
  published
  end;

  THL7SegmentList = class(TList)
  private
    FParent: THL7Message;
    FSegmentType: string;
  protected
    function Get(Index: Integer): THL7Segment;
    procedure Put(Index: Integer; Item: THL7Segment);
    function GetText: string;
    procedure SetText(const Value: string);
  public
    constructor Create(AParent: THL7Message; ASegmentType: string);
    procedure Clear; override;
    function Add(Item: THL7Segment): Integer; overload;
    function Add(Item: string): Integer; overload;
    function Extract(Item: THL7Segment): THL7Segment;
    function First: THL7Segment;
    function IndexOf(Item: THL7Segment): Integer;
    procedure Insert(Index: Integer; Item: THL7Segment);
    function Last: THL7Segment;
    function Remove(Item: THL7Segment): Integer;
    property Items[Index: Integer]: THL7Segment read Get write Put; default;
  published
    property Text: string read GetText write SetText;
    property SegmentType: string read FSegmentType write FSegmentType;
  end;

  THL7Message = class(TPersistent)
  private
    FLogger: ICPGatewayLogger;
    FDFN: string;
    FFacilityID: string;
    FMessage: THashedStringList;
    FDelimiter: char;
    FText: string;
    procedure ClearMessage; // Do NOT call FMessage.Clear.  You'll leak memory like a sieve.
  protected
    function GetSendingApplication: string;
    function GetSendingFacility: string;
    function GetDateTimeOfMessage: TDateTime;
    function GetVersion: string;
    function GetText: string;
    function GetDelimiter: char; { | }
    function GetEncodingChars: string; { ^~\& }
    procedure SetText(const Value: string);
    function GetSegment(Index: string): THL7SegmentList;
    procedure SetSegment(Index: string; const Value: THL7SegmentList);
    function GetSegmentText(Index: string): string;
    procedure SetSegmentText(Index: string; const Value: string);
  public
    constructor Create(Logger: ICPGatewayLogger); overload;
    constructor Create(const HL7Text: string; Logger: ICPGatewayLogger); overload;
    destructor Destroy; override;
    property Segment[Index: string]: THL7SegmentList read GetSegment write SetSegment;
    property SegmentText[Index: string]: string read GetSegmentText write SetSegmentText;
    function ToXML: string;
  published
    property Text: string read FText write SetText;
    property DFN: string read FDFN;
    property FacilityID: string read FFacilityID;
    property Delimiter: char read GetDelimiter;
    property EncodingChars: string read GetEncodingChars;
    property SendingApplication: string read GetSendingApplication;
    property SendingFacility: string read GetSendingFacility;
    property DateTimeOfMessage: TDateTime read GetDateTimeOfMessage;
    property Version: string read GetVersion;
  end;

function ConvertHL7DateTime(AHL7DateTime: string): TDateTime;

implementation

uses
  MFunStr,
  DateUtils, {sCPGateway,}
  Windows,
  ActiveX;

const
  // Constants used to index into the MSH segment.
  ENCODING_CHARS = 1;
  SENDING_APPLICATION = 3;
  SENDING_FACILITY = 4;
  DATE_TIME_OF_MESSAGE = 7;
  VERSION_ID = 12;

function ConvertHL7DateTime(AHL7DateTime: string): TDateTime;
begin
  try
    result := EncodeDateTime
      (StrToInt(Copy(AHL7DateTime, 1, 4)), //Year
      StrToInt(StringReplace(Copy(AHL7DateTime, 5, 2), '0', '', [rfReplaceAll])), //Month
      StrToInt(StringReplace(Copy(AHL7DateTime, 7, 2), '0', '', [rfReplaceAll])), //Day
      StrToIntDef(StringReplace(Copy(AHL7DateTime, 9, 2), '00', '0', [rfReplaceAll]), 0), //Hour
      StrToIntDef(StringReplace(Copy(AHL7DateTime, 11, 2), '00', '0', [rfReplaceAll]), 0), //Minute
      StrToIntDef(StringReplace(Copy(AHL7DateTime, 13, 2), '00', '0', [rfReplaceAll]), 0),
      0);
  except
    result := 0
  end;
end;

{ THL7Segment }

constructor THL7Segment.Create(AParent: THL7SegmentList);
begin
  initialize(AParent);
end;

constructor THL7Segment.Create(AParent: THL7SegmentList;
  const HL7Segment: string);
begin
  Initialize(AParent);
  FSegment.DelimitedText := HL7Segment;
end;

destructor THL7Segment.Destroy;
begin
  FreeAndNil(FSegment);
  inherited;
end;

function THL7Segment.GetField(Index: Integer): string;
begin
  if Index < FSegment.Count then
    result := FSegment[Index]
  else
    result := '';
end;

function THL7Segment.Index: integer;
begin
  result := FParent.IndexOf(Self);
end;

procedure THL7Segment.Initialize(AParent: THL7SegmentList);
begin
  FParent := AParent;
  FSegment := TStringList.Create;
  FSegment.Delimiter := FParent.FParent.FDelimiter;
end;

procedure THL7Segment.SetField(Index: Integer; const Value: string);
begin
  while FSegment.Count < Index do // Avoids those nasty index-out-of-bounds errors
    FSegment.Add('');
  FSegment[Index] := Value;
end;

function THL7Segment.ToString: string;
begin
  result := FSegment.Text;
end;

{ THL7Message }

constructor THL7Message.Create(Logger: ICPGatewayLogger);
begin
  CoInitialize(nil);
  FLogger := Logger;
  FMessage := THashedStringList.Create;
  CoUninitialize;
end;

procedure THL7Message.ClearMessage;
var
  I: Integer;
begin
  for I := 0 to FMessage.Count - 1 do
    THL7SegmentList(FMessage.Objects[I]).Free;

  FMessage.Clear;
end;

constructor THL7Message.Create(const HL7Text: string; Logger: ICPGatewayLogger);
begin
  CoInitialize(nil);
  FLogger := Logger;
  FMessage := THashedStringList.Create;
  SetText(HL7Text);
  CoUninitialize;
end;

destructor THL7Message.Destroy;
begin
  ClearMessage;
  FreeAndNil(FMessage);
  inherited;
end;

function THL7Message.GetSegment(Index: string): THL7SegmentList;
begin
  result := THL7SegmentList(FMessage.Objects[FMessage.IndexOf(Index)]);
end;

function THL7Message.GetSegmentText(Index: string): string;
var
  List: THL7SegmentList;
begin
  List := THL7SegmentList(FMessage.Objects[FMessage.IndexOf(Index)]);
  if not assigned(List) then
    result := ''
  else
    result := List.Text;
end;

function THL7Message.GetVersion: string;
var
  List: THL7SegmentList;
begin
  List := THL7SegmentList(FMessage.Objects[FMessage.IndexOf('MSH')]);
  if not assigned(List) then
    begin
      result := '';
      exit;
    end;

  // Note: for an MSH segment, we should have exactly one segment.
  result := List[0].Field[VERSION_ID];
end;

procedure THL7Message.SetSegment(Index: string; const Value: THL7SegmentList);
begin
  if FMessage.IndexOf(Index) = -1 then
    FMessage.Add(Index);

  FMessage.Objects[FMessage.IndexOf(Index)] := Value;
end;

procedure THL7Message.SetSegmentText(Index: string; const Value: string);
begin
  if FMessage.IndexOf(Index) = -1 then
    FMessage.Add(Index);

  THL7SegmentList(FMessage.Objects[FMessage.IndexOf(Index)]).Text := Value;
end;

function THL7Message.ToXML: string;
begin

end;

function THL7Message.GetText: string;
var
  I: Integer;
begin
  result := '';
  for I := 0 to FMessage.Count - 1 do
    result := result + THL7SegmentList(FMessage.Objects[I]).Text;
end;

procedure THL7Message.SetText(const Value: string);
var
  Msg: TStringList;
  SegmentName: string;
  SegmentList: THL7SegmentList;
  I: Integer;
begin
  FText := Value;

  FLogger.Log(Format('Setting text of HL7 message: %s', [Value]), llDetail);

  FLogger.Log('Clearing message', llDetail);
  try
    ClearMessage;

    Msg := TStringList.Create;
    try
      Msg.Text := Value; // This should pretty much break the message into segments.
      {
        The first line of the message is the DFN^Facility ID
      }
      FDFN := piece(Msg[0], '^', 2);
      FLogger.Log(Format('Patient DFN is %s', [FDFN]), llDetail);

      FFacilityID := piece(Msg[0], '^', 3);
      FLogger.Log(Format('Facility ID is %s', [FFacilityID]), llDetail);

      for I := 1 to Msg.Count - 1 do
        begin
          if Trim(Msg[I]) = '' then
            continue;
          SegmentName := copy(Msg[I], 1, 3);
          FLogger.Log(Format('Found %s segment', [SegmentName]), llDetail);
          if SegmentName = 'MSH' then
            begin
              FDelimiter := copy(Msg[I], 4, 1)[1];
              FLogger.Log(Format('Delimiter is %s', [FDelimiter]), llDetail);
            end;
          if FMessage.IndexOf(SegmentName) = -1 then
            begin
              FMessage.AddObject(SegmentName, THL7SegmentList.Create(Self, SegmentName));
            end;

          SegmentList := THL7SegmentList(FMessage.Objects[FMessage.IndexOf(SegmentName)]);
          SegmentList.Add(Msg[I]);
        end;
    finally
      FreeAndNil(Msg);
    end;
  except on E: Exception do
      FLogger.Log(Format('Error occurred in generating HL7 Message: %s ', [E.Message]), llError);
  end;
end;

function THL7Message.GetSendingApplication: string;
var
  List: THL7SegmentList;
begin
  List := THL7SegmentList(FMessage.Objects[FMessage.IndexOf('MSH')]);
  if not assigned(List) then
    begin
      result := '';
      exit;
    end;

  // Note: for an MSH segment, we should have exactly one segment.
  result := List[0].Field[SENDING_APPLICATION];
end;

function THL7Message.GetSendingFacility: string;
var
  List: THL7SegmentList;
begin
  List := THL7SegmentList(FMessage.Objects[FMessage.IndexOf('MSH')]);
  if not assigned(List) then
    begin
      result := '';
      exit;
    end;

  // Note: for an MSH segment, we should have exactly one segment.
  result := List[0].Field[SENDING_FACILITY];
end;

function THL7Message.GetDelimiter: char;
begin
  result := FDelimiter;
end;

function THL7Message.GetEncodingChars: string;
var
  List: THL7SegmentList;
begin
  List := THL7SegmentList(FMessage.Objects[FMessage.IndexOf('MSH')]);
  if not assigned(List) then
    begin
      result := '';
      exit;
    end;

  // Note: for an MSH segment, we should have exactly one segment.
  result := List[0].Field[ENCODING_CHARS];
end;

function THL7Message.GetDateTimeOfMessage: TDateTime;
var
  List: THL7SegmentList;
  dttm: string;
begin
  List := THL7SegmentList(FMessage.Objects[FMessage.IndexOf('MSH')]);
  if not assigned(List) then
    begin
      result := 0;
      exit;
    end;

  // Note: for an MSH segment, we should have exactly one segment.
  dttm := List[0].Field[DATE_TIME_OF_MESSAGE];
  result := ConvertHL7DateTime(dttm);
end;

{ THL7SegmentList }

function THL7SegmentList.Add(Item: THL7Segment): Integer;
begin
  result := inherited Add(Item);
end;

function THL7SegmentList.Add(Item: string): Integer;
var
  Segment: THL7Segment;
begin
  Segment := THL7Segment.Create(Self, Item);
  result := Add(Segment);
end;

procedure THL7SegmentList.Clear;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    Items[I].Free;
  inherited;
end;

constructor THL7SegmentList.Create(AParent: THL7Message;
  ASegmentType: string);
begin
  FParent := AParent;
  FSegmentType := ASegmentType;
end;

function THL7SegmentList.Extract(Item: THL7Segment): THL7Segment;
begin
  result := THL7Segment(inherited Extract(Item));
end;

function THL7SegmentList.First: THL7Segment;
begin
  result := THL7Segment(inherited First);
end;

function THL7SegmentList.Get(Index: Integer): THL7Segment;
begin
  result := THL7Segment(inherited Get(Index));
end;

function THL7SegmentList.GetText: string;
var
  I: Integer;
begin
  result := '';
  for I := 0 to Count - 1 do
    result := result + Items[I].ToString;
end;

function THL7SegmentList.IndexOf(Item: THL7Segment): Integer;
begin
  result := inherited IndexOf(Item);
end;

procedure THL7SegmentList.Insert(Index: Integer; Item: THL7Segment);
begin
  inherited Insert(Index, Item);
end;

function THL7SegmentList.Last: THL7Segment;
begin
  result := THL7Segment(inherited Last);
end;

procedure THL7SegmentList.Put(Index: Integer; Item: THL7Segment);
begin
  inherited Put(Index, Item);
end;

function THL7SegmentList.Remove(Item: THL7Segment): Integer;
begin
  result := inherited Remove(Item);
end;

procedure THL7SegmentList.SetText(const Value: string);
var
  Text: TStringList;
  I: Integer;
  SegmentName: string;
begin
  Clear;

  Text := TStringList.Create;
  try
    Text.Text := Value;
    for I := 0 to Text.Count - 1 do
      begin
        // Need to check to see if this segment id is valid
        SegmentName := copy(Text[I], 1, 3);
        if SegmentName = FSegmentType then
          Add(Text[I])
        else
          begin
            Clear;
            raise EHL7MessageError.CreateFmt('Invalid HL7 segment type.  Expected %s but found %s.',
              [FSegmentType, SegmentName]);
          end;
      end;
  finally
    FreeAndNil(Text);
  end;
end;

end.

