unit oObservation_Battery;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 3 $  $Modtime: 5/22/09 10:57a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Collection of observations.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oObservation_Battery.pas $
*
* $History: oObservation_Battery.pas $
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 5/22/09    Time: 2:17p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Build Pre-275:  
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 4/08/08    Time: 8:14p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:10p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  Forms,
  oObservation,
  oInterfaces,
  oTerm,
  XMLDoc,
  XMLIntf;

type
  TObservation_Battery = class (TInterfacedObject, IDocumentor)
  private
    fComments: string;
    fEntered_By_ID: string;
    fEntered_Date_Time: TDateTime;
    fID: string;
    fLocation_ID: string;
    fObservations: TStringList;
    fObserved_By_ID: string;
    fObserved_Date_Time: TDateTime;
    fPatient_ID: string;
    fStation_ID: string;
  public
    constructor Create;
    destructor Destroy; override;
    function zzaddObservation(aObservation: TObservation): Boolean;
    function FirstObservation: TObservation;
    function getHTML(aXMLDocument: IXMLDocument): Boolean; stdcall;
    function getText: string; stdcall;
    function getXML: string; stdcall;
    function NextObservation(aObservation: TObservation): TObservation;
    function Observation(aIndex: integer): TObservation; overload;
    function Observation(aID: string): TObservation; overload;
    function Observation(aTerm: TTerm): TObservation; overload;
    function ObservationCount: Integer;
    property Comments: string read fComments write fComments;
    property Entered_By_ID: string read fEntered_By_ID write fEntered_By_ID;
    property Entered_Date_Time: TDateTime read fEntered_Date_Time write fEntered_Date_Time;
    property ID: string read fID;
    property Location_ID: string read fLocation_ID write fLocation_ID;
    property Observed_By_ID: string read fObserved_By_ID write fObserved_By_ID;
    property Observed_Date_Time: TDateTime read fObserved_Date_Time write fObserved_Date_Time;
    property Patient_ID: string read fPatient_ID;
    property Station_ID: string read fStation_ID write fStation_ID;
  end;
  
function NewObservationBattery(aPatientID: string): TObservation_Battery;

implementation

function NewObservationBattery(aPatientID: string): TObservation_Battery;
begin
  Result := TObservation_Battery.Create;
  Result.fID := NewGUID;
  Result.fPatient_ID := aPatientID;
  Result.fObserved_Date_Time := Now;
end;


{
************************************************* TObservation_Battery *************************************************
}
constructor TObservation_Battery.Create;
begin
  inherited Create;
  fObservations := TStringList.Create;
end;

destructor TObservation_Battery.Destroy;
begin
  FreeAndNil(fObservations);
  inherited;
end;

function TObservation_Battery.zzaddObservation(aObservation: TObservation): Boolean;
begin
  {
  aObservation.Battery_ID := fID;
  aObservation.Observed_Date_Time := fObserved_Date_Time;
  aObservation.Observed_By_ID := fObserved_By_ID;

  fObservations.AddObject(aObservation.ID, aObservation);
  }
  result := false; // DeLinting
end;

function TObservation_Battery.FirstObservation: TObservation;
begin
  if fObservations.Count > 0 then
    Result := TObservation(fObservations.Objects[0])
  else
    Result := nil;
end;

function TObservation_Battery.getHTML(aXMLDocument: IXMLDocument): Boolean;
begin
  (*
  with aXMLDocument.DocumentElement.AddChild('TABLE') do
    begin
      Attributes['BORDER'] := 1;
      Attributes['WIDTH'] := 200;
      Attributes['ID'] := getID;
      for i := 0 to fObservations.Count - 1 do
        begin
          aObs := TObservation(fObservations.Objects[i]);
          with AddChild('TR') do
            begin
              with AddChild('TD') do
                NodeValue := aObs.Term.getTerm;
              with AddChild('TD') do
                NodeValue := aObs.Display_Value;
            end;
        end;
    end;
  *)
  result := false;
end;

function TObservation_Battery.getText: string;
begin
  with TStringList.Create do
    try
      Add(fComments);
      Add(Format('Date Time Observed: %s', [FormatDateTime('yyyy-mm-dd hh:nn:ss', fObserved_Date_Time)]));
      Add('--------------------');
      Add('');
      Result := Text;
    finally
      Free;
    end;
end;

function TObservation_Battery.getXML: string;
begin
  with NewXMLDocument do
    begin
      DocumentElement := AddChild('CLIO_CARNIVORE');
      with DocumentElement.AddChild('BATTERY') do
        begin
          ChildValues['BATTERY_ID'] := fID;
          ChildValues['PATIENT_ID'] := fPatient_ID;
          ChildValues['STATION_ID'] := fStation_ID;
        end;
      Result := XML.Text;
    end;
end;

function TObservation_Battery.NextObservation(aObservation: TObservation): TObservation;
var
  i: Integer;
begin
  i := fObservations.IndexOfObject(aObservation) + 1;
  if (i > 0) and (i < fObservations.Count) then
    Result := TObservation(fObservations.Objects[i])
  else
    Result := nil;
end;

function TObservation_Battery.Observation(aIndex: integer): TObservation;
begin
  if (aIndex > -1) and (aIndex < fObservations.Count) then
    Result := TObservation(fObservations.Objects[aIndex])
  else
    Result := nil;
end;

function TObservation_Battery.Observation(aID: string): TObservation;
var
  i: Integer;
begin
  i := fObservations.IndexOf(aID);
  if (i > -1) and (i < fObservations.Count) then
    Result := TObservation(fObservations.Objects[i])
  else
    Result := nil;
end;

function TObservation_Battery.Observation(aTerm: TTerm): TObservation;
begin
  Result := nil;
  {
  for i := 0 to fObservations.Count - 1 do
    begin
      if TObservation(fObservations.Objects[i]).Term = aTerm then
        begin
          Result := TObservation(fObservations.Objects[i]);
          Exit;
        end;
    end;
  }
end;

function TObservation_Battery.ObservationCount: Integer;
begin
  Result := fObservations.Count;
end;

end.

