unit oObservation_View;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 7 $  $Modtime: 4/14/10 8:45a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Defines the logic for getting and displaying collections
*                     of observations
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oObservation_View.pas $
================================================================================
}
interface

uses
  Windows,
  Classes,
  ComCtrls,
  SysUtils,
  Graphics,
  oCliO,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  Variants,
  XMLIntf,
  XMLDoc;

type
  TObservationView = class(TInterfacedObject, ICPConsoleItem, IObservationView, ICliOReportItem)
  private
    fDBConnection: IDBConnection;
    fIsLoaded: boolean;
    fCPService: ICPService;
    fTerminology: ITerminology;
    fViewTerms: IInterfaceList;
    fID: string;
    fActive: boolean;
    fComment: string;
    fName: string;
    fDisplayName: string;
    fInterval: integer;
    fViewStyle: TObservationViewStyle;
    fAllowPivot: boolean;
    fNationalTitle: boolean;

    fIncludeTotals: boolean;
    fIncludeCounts: boolean;
    fIncludeAverages: boolean;

    {Supplemental Page Items}
    fDefaultSetID: string;
    fDefaultMethod: ITerm;
    fDefaultPosition: ITerm;
    fDefaultLocation: ITerm;
    fDefaultProduct: ITerm;
    fPageType: TCPFlowsheetPageType;
    fActivatedDateTime: TDateTime;
    fActivatedByID: string;
    fActivatedByName: string;
    fActivatedComment: string;
    fDeactivatedDateTime: TDateTime;
    fDeactivatedByID: string;
    fDeactivatedByName: string;
    fDeactivatedComment: string;

    {ICPConsoleItem interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;
    function getCanDelete: boolean; stdcall;
    function Delete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function Rename(const aNewName: string): boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setService(const aCPService: ICPService); stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;
    procedure Load; stdcall;
    procedure Save; stdcall;

    {ICliOReportItem}
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;

    {IObservationView interface}
    function getDefaultMethodTerm: string; stdcall;
    function getDefaultPositionTerm: string; stdcall;
    function getDefaultLocationTerm: string; stdcall;
    function getDefaultProductTerm: string; stdcall;

    function getDisplayInterval: integer; stdcall;
    function getViewStyle: TObservationViewStyle; stdcall;
    function getAllowPivot: boolean; stdcall;
    function getIncludeTotals: boolean; stdcall;
    function getIncludeCounts: boolean; stdcall;
    function getIncludeAverages: boolean; stdcall;
    function getIsNationalTitle: boolean; stdcall;

    {Supplemental Page Stuff}
    function getDefaultSetID: string; stdcall;
    function getDefaultMethod: ITerm; stdcall;
    function getDefaultPosition: ITerm; stdcall;
    function getDefaultLocation: ITerm; stdcall;
    function getDefaultProduct: ITerm; stdcall;
    function getPageType: TCPFlowsheetPageType; stdcall;
    function getActivatedDateTime: TDateTime; stdcall;
    function getActivatedByID: string; stdcall;
    function getActivatedByName: string; stdcall;
    function getActivatedComment: string; stdcall;
    function getDeactivatedDateTime: TDateTime; stdcall;
    function getDeactivatedByID: string; stdcall;
    function getDeactivatedByName: string; stdcall;
    function getDeactivatedComment: string; stdcall;

    function getIndex(aList: TStrings): integer; stdcall;

    function getViewTermByIndex(aIndex: integer): IObservationViewTerm; stdcall;
    function getViewTermByOrder(aTermOrder: integer): IObservationViewTerm; stdcall;
    function getViewTermCount: integer; stdcall;
    function getViewTermIndex(aItem: IInterface): integer; stdcall;

    procedure setDisplayInterval(const aValue: integer); stdcall;
    procedure setViewStyle(const aValue: TObservationViewStyle); stdcall;
    procedure setAllowPivot(const aValue: boolean); stdcall;
    procedure setIncludeTotals(const aValue: boolean); stdcall;
    procedure setIncludeCounts(const aValue: boolean); stdcall;
    procedure setIncludeAverages(const aValue: boolean); stdcall;

    procedure InsertViewTerm(aViewTerm: IObservationViewTerm); stdcall;
    procedure DeleteViewTerm(aViewTerm: IObservationViewTerm); stdcall;

    function LoadFromXML(aXMLNode: IXMLNode): boolean;
    procedure DumpToFile(aFilename: string); stdcall;
  public
    constructor Create(aXMLNode: IXMLNode; aDBConnection: IDBConnection; aTerminology: ITerminology);
    destructor Destroy; override;
  end;

implementation

uses
  uCPXML,
  oCP_Exceptions;

constructor TObservationView.Create(aXMLNode: IXMLNode; aDBConnection: IDBConnection; aTerminology: ITerminology);
begin
  inherited Create;
  fID := NewGUID;
  fActive := False;
  fViewTerms := TInterfaceList.Create;
  fDBConnection := aDBConnection;
  fTerminology := aTerminology;
  fName := 'NEW OBSERVATION VIEW';
  fDisplayName := 'New Observation View';
  fComment := 'New View';
  fInterval := 1440;
  fViewStyle := vsTerminology;
  fPageType := ptMandatory;
  fNationalTitle := False;

  if Assigned(aXMLNode) then
    begin
      fID := aXMLNode.ChildValues['VIEW_ID'];
      fName := aXMLNode.ChildValues['NAME'];
      fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
      fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
      fNationalTitle := (XMLValueAsInt(aXMLNode, 'NATIONAL_TITLE') = 1);
      fDefaultSetID := XMLValueAsStr(aXMLNode, 'SET_ID');
      fPageType := TCPFlowsheetPageType(XMLValueAsInt(aXMLNode, 'ACTIVATED_AS_TYPE', 0));
    end
  else
    fDBConnection.ExecCommand('AddView', [fID, fName, fDisplayName, fInterval, fViewStyle, fComment, 0]);
end;

function TObservationView.Delete: boolean;
begin
  Result := False;
end;

procedure TObservationView.DeleteViewTerm(aViewTerm: IObservationViewTerm);
begin
  fViewTerms.Remove(aViewTerm);
end;

destructor TObservationView.Destroy;
begin
  fViewTerms.Clear;
  fViewTerms := nil;
  inherited;
end;

procedure TObservationView.DumpToFile(aFilename: string);
var
  i: integer;
  tt: TTermType;
  s: string;
  aTerm: ITerm;
  aTermName: string;
const
  afmt = '    %-20s %-10s %-25s %-10s %-10s';
  afmt2 = '    %-0.20s %-0.10s %-0.25s %-0.10s %-0.10s';
  aLine = '-------------------------------------------------------------------------------------------------';
  aLine2 = '=================================================================================================';
begin
  with TStringList.Create do
    try
      Add('TObservationView Display');
      Add(Format('%0.84s', [aLine2]));
      Add(Format('ID:...............%s', [fID]));
      Add(Format('Comment:..........%s', [fComment]));
      Add(Format('Name:.............%s', [fName]));
      Add(Format('Display Name:.....%s', [fDisplayName]));
      Add(Format('Interval:.........%d', [fInterval]));
      Add(Format('View Style:.......%s', [OBSERVATION_VIEW_STYLE[fViewStyle]]));
      Add(Format('Allow Pivot:......%s', [CP_YN_TEXT[fAllowPivot]]));
      Add(Format('Term Count:.......%d', [getViewTermCount]));
      Add(' ');

      for i := 0 to getViewTermCount - 1 do
        with getViewTermByIndex(i) do
          begin
            Add(Format('    %-0.80s', [aLine2]));
            Add(Format('    Display Order:    %d (Index: %d)', [getDisplayOrder, i]));
            Add(Format('    Display Name:     %s', [getDisplayName]));
            Add(Format('    Display Width:    %d', [getDisplayWidth]));
            Add(Format('    Diaplay Only:     %s', [CP_YN_TEXT[getDisplayOnly]]));
            Add(Format('    Include Average:  %s', [CP_YN_TEXT[getDisplayAverage]]));
            Add(Format('    Include Count:    %s', [CP_YN_TEXT[getDisplayCount]]));
            Add(Format('    Include Total:    %s', [CP_YN_TEXT[getDisplayTotal]]));
            Add(' ');
            Add(Format(aFmt, ['Attribute', 'ID', 'Term', 'Allowed', 'Mandatory']));
            Add(Format(aFmt2, [aLine, aLine, aline, aLine, aLine]));
            for tt := Low(TTermType) to High(TTermType) do
              begin
                aTerm := fTerminology.GetTerm(getAttributeID(tt));
                if Assigned(aTerm) then
                  aTermName := aTerm.getTerm
                else
                  aTermName := '';
                Add(
                  Format(aFmt,
                  [QUALNAME[tt], getAttributeID(tt), aTermName, CP_YN_TEXT[getAttributeAllowed(tt)], CP_YN_TEXT[getAttributeMandatory(tt)]]));
              end;
          end;
      if fViewTerms.Count > 0 then
        Add(Format('    %-0.80s', [aLine2]));
      //SaveToFile(aFilename);
    finally
      Clear;
      Free;
    end;
end;

{ TObservation_View }

procedure TObservationView.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, self);
end;

procedure TObservationView.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

procedure TObservationView.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  j: integer;
  aTerm: ITerm;
  aViewTerm: IObservationViewTerm;
  aTotal: Double;
  aCount: Integer;
  aDefaultUnit: ITerm;
  aObs: IObservation;
  aReportItem: ICliOReportItem;
  aValue: string;
begin
  aReportText.Add('');
  aReportText.Add(StringOfChar('-', aMargin));
  aReportText.Add(StringOfChar(' ', (aMargin - Length(getDisplayName)) div 2) + getDisplayName);
  if getDefaultSetID <> '' then
    begin
      aReportText.Add('Date Opened:.....' + FormatDateTime('mmm d, yyyy', getActivatedDateTime));
      aReportText.Add('  Method:........' + getDefaultMethodTerm);
      aReportText.Add('  Position:......' + getDefaultPositionTerm);
      aReportText.Add('  Location:......' + getDefaultLocationTerm);
      aReportText.Add('  Product:.......' + getDefaultProductTerm);
      if aIncludeDetail then
        begin {We can add the audit log here later}
          aReportText.Add('  Comment:.......' + getActivatedComment);
        end;
    end;
  aReportText.Add(StringOfChar('-', aMargin));

  for j := 0 to getViewTermCount - 1 do
    begin
      aViewTerm := getViewTermByIndex(j);
      aTerm := fTerminology.GetTerm(aViewTerm.getAttributeID(ttObservation));
      aTotal := 0;
      aCount := 0;

      if aViewTerm.getDisplayOnly then
        aReportText.Add(Format('[%s (Totals Only)]', [aViewTerm.getDisplayName]))
      else
        begin
          aReportText.Add(Format('[%s]', [aViewTerm.getDisplayName]));
          if aTerm.getDataType = dtNumeric then
            aReportText.Add(
              Format('  %-20s%-8s%-15s%' + IntToStr(aMargin - 52) + 's',
              ['Observed Date/Time', 'Initial', 'Qualifiers', 'Value']))
          else
            aReportText.Add(
              Format('  %-20s%-8s%-15s%-' + IntToStr(aMargin - 52) + 's',
              ['Observed Date/Time', 'Initial', 'Qualifiers', 'Value']));

          {
          aReportText.Add(Format('[%s]', [aViewTerm.getDisplayName]));
          aReportText.Add(
            Format('%-20s', ['Observed Date/Time']) +
            Format('%-8s  ', ['Int']) +
            Format('%-65s', ['Value']));
          }
        end;

      //aReportText.Add(StringOfChar('-', aMargin));

      aDefaultUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
      aObs := aList.FirstObservation;
      while Assigned(aObs) do
        begin
          if aObs.getStatus = osVerified then
            if aViewTerm.ContainsObservation(aObs) then
              begin
                if not aViewTerm.getDisplayOnly then
                  if aObs.QueryInterface(ICliOReportItem, aReportItem) = 0 then
                    aReportItem.AddToReport(aReportText, aMargin, aIncludeDetail);

                {This is where I need to return to use the picklist ranking value}
                if aObs.getObsTerm.getDataType in [dtNumeric] then
                  begin
                    aTotal := aTotal + fTerminology.GetConvertedValue(
                      aObs.getNumericValue,
                      aObs.getObsUnit,
                      aDefaultUnit);
                    inc(aCount);
                  end
                else
                  inc(aCount);
              end;
          aObs := aList.NextObservation(aObs);
        end;

      if aViewTerm.getDisplayTotal or aViewTerm.getDisplayCount or aViewTerm.getDisplayAverage then
        aReportText.Add(Format('%45s%' + IntToStr(aMargin - 52) + 's', ['', StringOfChar('-', 12)]));

      if aViewTerm.getDisplayTotal then
        begin
          aValue := Format('%12.2f', [aTotal]);
          if Assigned(aDefaultUnit) then
            aReportText.Add(Format('%65s%' + IntToStr(aMargin - 72) + 's%-8s', ['Total:', aValue, ' (' + aDefaultUnit.getAbbreviation + ')']))
          else
            aReportText.Add(Format('%65s%' + IntToStr(aMargin - 72) + 's', ['Total:', aValue]));
        end;

      if aViewTerm.getDisplayCount then
        aReportText.Add(Format('%65s%' + IntToStr(aMargin - 72) + 's', ['Count:', IntToStr(aCount)]));

      if aViewTerm.getDisplayAverage then
        begin
          if aCount > 0 then
            aValue := Format('%12.2f', [aTotal / aCount])
          else
            aValue := '***';

          if Assigned(aDefaultUnit) then
            aReportText.Add(Format('%65s%' + IntToStr(aMargin - 72) + 's%-8s', ['Average:', aValue, ' (' + aDefaultUnit.getAbbreviation + ')']))
          else
            aReportText.Add(Format('%65s%' + IntToStr(aMargin - 72) + 's', ['Average:', aValue]));
        end;
      aReportText.Add('');
    end;
end;

function TObservationView.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TObservationView.getViewTermByIndex(aIndex: integer): IObservationViewTerm;
begin
  try
    fViewTerms[aIndex].QueryInterface(IObservationViewTerm, Result);
  except
    Result := nil;
  end;
end;

function TObservationView.getViewTermByOrder(aTermOrder: integer): IObservationViewTerm;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to fViewTerms.Count - 1 do
    if fViewTerms[i].QueryInterface(IObservationViewTerm, Result) = 0 then
      if Result.getDisplayOrder = aTermOrder then
        Exit;
  Result := nil;
end;

function TObservationView.getViewTermCount: integer;
begin
  Result := fViewTerms.Count;
end;

procedure TObservationView.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aViewTerm: IObservationViewTerm;
begin
  fViewTerms.Clear;
  aXMLDoc := NewXMLDocument;

  if fDefaultSetID <> '' then
    begin
      if not fDBConnection.ExecQuery('GetFlowsheetSupplementalPage', [fDefaultSetID], aXMLDoc) then
        raise ECPLoadFailure.CreateFmt('Error loading supplemental page ''%s'' defintion.', [fID]);
    end
  else if not fDBConnection.ExecQuery('GetView', [fID], aXMLDoc) then
    raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' defintion.', [fID]);

  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  if Assigned(aXMLRec) then
    LoadFromXML(aXMLRec)
  else
    raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' defintion.', [fID]);

  {Load the terms}
  if not fDBConnection.ExecQuery('GetViewTerms', [fID], aXMLDoc) then
    raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' terms.', [fID]);
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aViewTerm := NewObservationViewTerm(aXMLRec, fDefaultSetID);
      fViewTerms.Add(aViewTerm);
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    except
      raise ECPLoadFailure.Create('Error loading view term');
    end;

  if fDBConnection.ExecQuery('GetViewFilters', [fID], aXMLDoc) then
    begin
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
      while Assigned(aXMLRec) do
        try
          aViewTerm := getViewTermByOrder(XMLValueAsInt(aXMLRec, 'TERM_ORDER'));
          aViewTerm.setAttribute(
            TTermType(XMLValueAsInt(aXMLRec, 'FILTER_TYPE')),
            True,
            XMLValueAsInt(aXMLRec, 'FILTER_USAGE', 0) = 1,
            XMLValueAsStr(aXMLRec, 'FILTER_TERM'));
        finally
          aXMLRec := XMLDocumentNextRecord(aXMLRec);
        end;
    end;
end;

function TObservationView.LoadFromXML(aXMLNode: IXMLNode): boolean;
begin
  try
    {Common items for views and supp pages}
    fInterval := XMLValueAsInt(aXMLNode, 'TIME_INTERVAL');
    fViewStyle := TObservationViewStyle(XMLValueAsInt(aXMLNode, 'X_AXIS', 0));
    fAllowPivot := (XMLValueAsInt(aXMLNode, 'ALLOW_PIVOT') = 1);
    fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
    fComment := XMLValueAsStr(aXMLNode, 'COMMENT');

    if fDefaultSetID <> '' then
      begin
        fDefaultMethod := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_METHOD_ID'));
        fDefaultPosition := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_POSTITION_ID'));
        fDefaultLocation := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_LOCATION_ID'));
        fDefaultProduct := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_PRODUCT_ID'));
        fPageType := TCPFlowsheetPageType(XMLValueAsInt(aXMLNode, 'PAGE_TYPE', 0));
        fActivatedDateTime := XMLValueAsDateTime(aXMLNode, 'ACTIVATED_DATE_TIME');
        fActivatedByID := XMLValueAsStr(aXMLNode, 'ACTIVATED_BY_ID');
        fActivatedByName := XMLValueAsStr(aXMLNode, 'ACTIVATED_BY_NAME');
        fActivatedComment := XMLValueAsStr(aXMLNode, 'ACTIVATED_COMMENT');
        fDeactivatedDateTime := XMLValueAsDateTime(aXMLNode, 'DEACTIVATED_DATE_TIME');
        fDeactivatedByID := XMLValueAsStr(aXMLNode, 'DEACTIVATED_BY_ID');
        fDeactivatedByName := XMLValueAsStr(aXMLNode, 'DEACTIVATED_BY_NAME');
        fDeactivatedComment := XMLValueAsStr(aXMLNode, 'DEACTIVATED_COMMENT');
      end;

    Result := True;
  except
    Result := False;
  end;
end;

function TObservationView.Rename(const aNewName: string): boolean;
begin
  Result := fDBConnection.ExecCommand('UpdateViewName', [fID, aNewName]);
  if Result then
    fName := aNewName;
end;

procedure TObservationView.Save;
var
  i: integer;
  aViewTerm: IObservationViewTerm;
  tt: TTermType;
begin
  if not fDBConnection.ExecCommand('UpdateView',
    [fID, fDisplayName, fComment, fInterval, fViewStyle, fAllowPivot, fActive, fNationalTitle]) then
    raise ECPSaveFailure.CreateFmt('Error updating view ID ''%s''', [fID]);

  if not fDBConnection.ExecCommand('DeleteViewTerms', [fID]) then
    raise ECPSaveFailure.CreateFmt('Error deleting terms for view ID ''%s''', [fID]);

  if not fDBConnection.ExecCommand('DeleteViewFilters', [fID]) then
    raise ECPSaveFailure.CreateFmt('Error deleting filters for view ID ''%s''', [fID]);

  for i := 0 to fViewTerms.Count - 1 do
    begin
      aViewTerm := getViewTermByIndex(i);
      with aViewTerm do
        begin
          if not fDBConnection.ExecCommand('AddViewTerm',
            [fID, getDisplayOrder, getAttributeID(ttObservation), getDisplayName, getDisplayWidth,
            getDisplayTotal, getDisplayCount, getDisplayAverage, getDisplayOnly, getDisplayColumns, getDisplayInCellTotal]) then
            raise ECPSaveFailure.CreateFmt('Error adding term %s to view.', [aViewTerm.getDisplayName]);

          for tt := Low(TTermType) to High(TTermType) do
            if aViewTerm.getAttributeAllowed(tt) then
              if not fDBConnection.ExecCommand('AddViewFilter',
                [fID, getDisplayOrder, tt, getAttributeID(tt),
                CP_YN[getAttributeMandatory(tt)]]) then
                raise ECPSaveFailure.CreateFmt('Error adding Filter %s to view.', [TERMTYPE[tt]]);
        end;
    end;
end;

function TObservationView.getViewTermIndex(aItem: IInterface): integer;
var
  aObservation: IObservation;
  aViewTerm: IObservationViewTerm;
begin
  if aItem.QueryInterface(IObservation, aObservation) = 0 then {It is a valid observation}
    {Check the observation against the view primary and secondary items}
    //if aObservation.getHasAttribute(fPrimaryIDMatch) and aObservation.getHasAttribute(fSecondaryIDMatch) then
    for Result := 0 to fViewTerms.Count - 1 do {Walk down the IViewTerms}
      begin
        if fViewTerms[Result].QueryInterface(IObservationViewTerm, aViewTerm) = 0 then
          if aViewTerm.ContainsObservation(aObservation) then
            Exit; {if you are here, everything matches up, bail with the index}
      end;
  Result := -1; {Something went south, no index to return}
end;

{
function TObservationView.getViewType: TObservationViewType;
begin
  Result := fViewType;
end;
}

procedure TObservationView.InsertViewTerm(aViewTerm: IObservationViewTerm);
begin
  fViewTerms.Add(aViewTerm);
end;

{ Getters and Setters }

function TObservationView.getActivatedByID: string;
begin
  Result := fActivatedByID;
end;

function TObservationView.getActivatedByName: string;
begin
  Result := fActivatedByName;
end;

function TObservationView.getActivatedComment: string;
begin
  Result := fActivatedComment;
end;

function TObservationView.getActivatedDateTime: TDateTime;
begin
  Result := fActivatedDateTime;
end;

function TObservationView.getActive: Boolean;
begin
  Result := (fActive or fNationalTitle);
end;

function TObservationView.getCanDelete: boolean;
begin
  Result := False;
end;

function TObservationView.getCanRename: boolean;
begin
  Result := (not fNationalTitle);
end;

function TObservationView.getComment: string;
begin
  Result := fComment;
end;

function TObservationView.getName: string;
begin
  Result := fName
end;

function TObservationView.getIsLoaded: boolean;
begin
  Result := False;
end;

function TObservationView.getIsNationalTitle: boolean;
begin
  Result := fNationalTitle;
end;

function TObservationView.getDisplayName: string;
begin
  if fDisplayName = '' then
    Result := fName
  else
    Result := fDisplayName;
end;

function TObservationView.getID: string;
begin
  Result := fID;
end;

function TObservationView.getItemType: TCPItemType;
begin
  Result := ciFlowsheetView;
  {
  case fViewType of
    vtGrid: Result := ciView;
    vtReport: Result := ciReportView;
  else
    Result := ciView;
  end;
  }
end;

function TObservationView.getDeactivatedByID: string;
begin
  Result := fDeactivatedByID;
end;

function TObservationView.getDeactivatedByName: string;
begin
  Result := fDeactivatedByName;
end;

function TObservationView.getDeactivatedComment: string;
begin
  Result := fDeactivatedComment;
end;

function TObservationView.getDeactivatedDateTime: TDateTime;
begin
  Result := fDeactivatedDateTime;
end;

function TObservationView.getDefaultLocation: ITerm;
begin
  Result := fDefaultLocation;
end;

function TObservationView.getDefaultLocationTerm: string;
begin
  if Assigned(fDefaultLocation) then
    Result := fDefaultLocation.getTerm
  else
    Result := '<None>';
end;

function TObservationView.getDefaultMethod: ITerm;
begin
  Result := fDefaultMethod;
end;

function TObservationView.getDefaultMethodTerm: string;
begin
  if Assigned(fDefaultMethod) then
    Result := fDefaultMethod.getTerm
  else
    Result := '<None>';
end;

function TObservationView.getDefaultPosition: ITerm;
begin
  Result := fDefaultPosition;
end;

function TObservationView.getDefaultPositionTerm: string;
begin
  if Assigned(fDefaultPosition) then
    Result := fDefaultPosition.getTerm
  else
    Result := '<None>';
end;

function TObservationView.getDefaultProduct: ITerm;
begin
  Result := fDefaultProduct;
end;

function TObservationView.getDefaultProductTerm: string;
begin
  if Assigned(fDefaultProduct) then
    Result := fDefaultProduct.getTerm
  else
    Result := '<None>';
end;

function TObservationView.getDefaultSetID: string;
begin
  Result := fDefaultSetID;
end;

function TObservationView.getDisplayInterval: integer;
begin
  Result := fInterval;
end;

function TObservationView.getAllowPivot: boolean;
begin
  Result := fAllowPivot;
end;

function TObservationView.getViewStyle: TObservationViewStyle;
begin
  Result := fViewStyle;
end;

function TObservationView.getIncludeAverages: boolean;
begin
  Result := fIncludeAverages;
end;

function TObservationView.getIncludeCounts: boolean;
begin
  Result := fIncludeCounts;
end;

function TObservationView.getIncludeTotals: boolean;
begin
  Result := fIncludeTotals;
end;

function TObservationView.getIndex(aList: TStrings): integer;
begin
  Result := aList.IndexOfObject(Self);
end;

function TObservationView.getPageType: TCPFlowsheetPageType;
begin
  Result := fPageType;
end;

procedure TObservationView.setIncludeAverages(const aValue: boolean);
begin
  fIncludeAverages := aValue;
end;

procedure TObservationView.setIncludeCounts(const aValue: boolean);
begin
  fIncludeCounts := aValue;
end;

procedure TObservationView.setIncludeTotals(const aValue: boolean);
begin
  fIncludeTotals := aValue;
end;

procedure TObservationView.setName(const aValue: string);
begin
  fName := aValue;
end;

procedure TObservationView.setService(const aCPService: ICPService);
begin
  fCPService := aCPService;
end;

procedure TObservationView.setViewStyle(const aValue: TObservationViewStyle);
begin
  fViewStyle := aValue;
end;

procedure TObservationView.setAllowPivot(const aValue: boolean);
begin
  fAllowPivot := aValue;
end;

procedure TObservationView.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TObservationView.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TObservationView.setDisplayInterval(const aValue: integer);
begin
  fInterval := aValue;
end;

procedure TObservationView.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

end.

