unit oObservation_View_Term;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 4 $  $Modtime: 4/14/10 8:45a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Defines a terms methods within a view.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oObservation_View_Term.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  SysUtils,
  oCliOTypes,
  oInterfaces,
  Variants,
  XMLIntf,
  XMLDoc,
  iCliOGrid; {Used in the display frame}

type
  TObservationViewTerm = class(TInterfacedObject, IObservationViewTerm, IGridTerm)
  private
    fAttributesAllowed: array[TTermType] of boolean;
    fAttributesMandatory: array[TTermType] of boolean;
    fAttributesID: array[TTermType] of string;

    fDisplayOrder: integer;
    fDisplayName: string;
    fDisplayWidth: Integer;
    fDisplayTotal: boolean;
    fDisplayCount: boolean;
    fDisplayAverage: boolean;
    fDisplayOnly: boolean;
    fDisplayColumns: integer;
    fDefaultSetID: string;
    fDisplayInCellTotal: boolean;

    function getAttributeAllowed(Index: TTermType): boolean; stdcall;
    function getAttributeMandatory(Index: TTermType): boolean; stdcall;
    function getAttributeID(Index: TTermType): string; stdcall;

    function getDisplayOrder: integer; stdcall;
    function getDisplayName: string; stdcall;
    function getDisplayWidth: integer; stdcall;
    function getDisplayColumns: integer; stdcall;
    function getDisplayTotal: boolean; stdcall;
    function getDisplayCount: boolean; stdcall;
    function getDisplayAverage: boolean; stdcall;
    function getDisplayOnly: boolean; stdcall;
    function getDefaultSetID: string; stdcall;
    function getDisplayInCellTotal: boolean; stdcall;

    procedure setAttribute(aTermType: TTermType; aAllowed: boolean; aMandatory: boolean; aID: string); stdcall;
    procedure setDisplayOrder(const aValue: integer); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setDisplayWidth(const aValue: integer); stdcall;
    procedure setDisplayColumns(const aValue: integer); stdcall;
    procedure setDisplayTotal(const aValue: boolean); stdcall;
    procedure setDisplayCount(const aValue: boolean); stdcall;
    procedure setDisplayAverage(const aValue: boolean); stdcall;
    procedure setDisplayOnly(const aValue: boolean); stdcall;
    procedure setDisplayInCellTotal(const aValue: boolean); stdcall;

    procedure AddToListView(aListView: TListView); stdcall;

    function LoadFromXML(aXMLNode: IXMLNode): boolean;
    function ContainsObservation(aObservation: IObservation): boolean; stdcall;
    function GetAllAttributes: string; stdcall;
  public
    constructor Create(aXMLNode: IXMLNode; aDefaultSetID: string);
    destructor Destroy; override;
  end;

implementation

uses
  uCPXML;

{ TObservation_View_Term }

constructor TObservationViewTerm.Create(aXMLNode: IXMLNode; aDefaultSetID: string);
var
  aTermType: TTermType;
begin
  inherited Create;
  for aTermType := Low(TTermType) to High(TTermType) do
    begin
      fAttributesAllowed[aTermType] := False;
      fAttributesMandatory[aTermType] := False;
      fAttributesID[aTermType] := '';
    end;

  fDefaultSetID := aDefaultSetID;
  fDisplayInCellTotal := False;

  if Assigned(aXMLNode) then
    LoadFromXML(aXMLNode);
end;

destructor TObservationViewTerm.Destroy;
begin
  inherited;
end;

function TObservationViewTerm.LoadFromXML(aXMLNode: IXMLNode): boolean;
begin
  try
    fDisplayOrder := XMLValueAsInt(aXMLNode, 'TERM_ORDER');
    fDisplayWidth := XMLValueAsInt(aXMLNode, 'DISPLAY_WIDTH');
    fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
    fDisplayTotal := (XMLValueAsInt(aXMLNode, 'DISPLAY_TOTAL', 0) = 1);
    fDisplayCount := (XMLValueAsInt(aXMLNode, 'DISPLAY_COUNT', 0) = 1);
    fDisplayAverage := (XMLValueAsInt(aXMLNode, 'DISPLAY_AVERAGE', 0) = 1);
    fDisplayOnly := (XMLValueAsInt(aXMLNode, 'DISPLAY_ONLY', 0) = 1);
    fDisplayColumns := XMLValueAsInt(aXMLNode, 'DISPLAY_COLUMNS', 0);
    fDisplayInCellTotal := (XMLValueAsInt(aXMLNode, 'DISPLAY_IN_CELL_TOTAL', 0) = 1);
    setAttribute(ttObservation, True, True, XMLValueAsStr(aXMLNode, 'TERM_ID'));
    Result := True;
  except
    Result := False;
  end;
end;

procedure TObservationViewTerm.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

function TObservationViewTerm.ContainsObservation(aObservation: IObservation): boolean;
begin
  Result := False;
  if getAttributeID(ttObservation) = aObservation.getTermID(ttObservation) then
    begin {You are on the right track, you at least have a matching observation}
      if (not getAttributeMandatory(ttMethod)) or (getAttributeID(ttMethod) = aObservation.getTermID(ttMethod)) then
        if (not getAttributeMandatory(ttPosition)) or (getAttributeID(ttPosition) = aObservation.getTermID(ttPosition)) then
          if (not getAttributeMandatory(ttLocation)) or (getAttributeID(ttLocation) = aObservation.getTermID(ttLocation)) then
            if (not getAttributeMandatory(ttQuality)) or (getAttributeID(ttQuality) = aObservation.getTermID(ttQuality)) then
              if (not getAttributeMandatory(ttProduct)) or (getAttributeID(ttProduct) = aObservation.getTermID(ttProduct)) then
                if (not getAttributeMandatory(ttNullFlavor)) or (getAttributeID(ttNullFlavor) = aObservation.getTermID(ttNullFlavor)) then
                  Result := True; {if you are here, everything matches up -- Woo Hoo!}
    end;
end;

{ Getters and Setters }

function TObservationViewTerm.getAttributeAllowed(Index: TTermType): boolean;
begin
  Result := fAttributesAllowed[Index];
end;

function TObservationViewTerm.getAttributeMandatory(Index: TTermType): boolean;
begin
  Result := fAttributesMandatory[Index];
end;

function TObservationViewTerm.getAttributeID(Index: TTermType): string;
begin
  Result := fAttributesID[Index];
end;

function TObservationViewTerm.getDisplayAverage: boolean;
begin
  Result := fDisplayAverage;
end;

function TObservationViewTerm.getDisplayColumns: integer;
begin
  Result := fDisplayColumns;
end;

function TObservationViewTerm.getDisplayCount: boolean;
begin
  Result := fDisplayCount;
end;

function TObservationViewTerm.getDisplayInCellTotal: boolean;
begin
  Result := fDisplayInCellTotal;
end;

function TObservationViewTerm.getDisplayTotal: boolean;
begin
  Result := fDisplayTotal;
end;

function TObservationViewTerm.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TObservationViewTerm.getDisplayOnly: boolean;
begin
  Result := fDisplayOnly;
end;

function TObservationViewTerm.getDisplayOrder: integer;
begin
  Result := fDisplayOrder;
end;

function TObservationViewTerm.getDefaultSetID: string;
begin
  Result := fDefaultSetID;
end;

function TObservationViewTerm.getDisplayWidth: integer;
begin
  Result := fDisplayWidth;
end;

procedure TObservationViewTerm.setAttribute(aTermType: TTermType; aAllowed, aMandatory: boolean; aID: string);
begin
  fAttributesAllowed[aTermType] := aAllowed;
  fAttributesMandatory[aTermType] := aMandatory;
  fAttributesID[aTermType] := aID;
end;

procedure TObservationViewTerm.setDisplayAverage(const aValue: boolean);
begin
  fDisplayAverage := aValue;
end;

procedure TObservationViewTerm.setDisplayColumns(const aValue: integer);
begin
  fDisplayColumns := aValue;
end;

procedure TObservationViewTerm.setDisplayCount(const aValue: boolean);
begin
  fDisplayCount := aValue;
end;

procedure TObservationViewTerm.setDisplayInCellTotal(const aValue: boolean);
begin
  fDisplayInCellTotal := aValue;
end;

procedure TObservationViewTerm.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TObservationViewTerm.setDisplayOnly(const aValue: boolean);
begin
  fDisplayOnly := aValue;
end;

procedure TObservationViewTerm.setDisplayOrder(const aValue: integer);
begin
  fDisplayOrder := aValue;
end;

procedure TObservationViewTerm.setDisplayTotal(const aValue: boolean);
begin
  fDisplayTotal := aValue;
end;

procedure TObservationViewTerm.setDisplayWidth(const aValue: integer);
begin
  fDisplayWidth := aValue;
end;

{------------------------------- DEBUG METHOD ---------------------------------}

function TObservationViewTerm.GetAllAttributes: string;
var
  tt: TTermType;
const
  B2STR: array[boolean] of string = ('False', 'True');
begin
  with TStringList.Create do
    try
      for tt := Low(TTermType) to High(TTermType) do
        begin
          Add(Format('Attribute: %s', [TERMTYPE[tt]]));
          Add(Format('    Allowed: %s', [B2STR[fAttributesAllowed[tt]]]));
          Add(Format('    Term: %s', [fAttributesID[tt]]));
          Add(Format('    Mandatory: %s', [B2STR[fAttributesMandatory[tt]]]));
          Add('-----------------------------------------------');
          Add('');
        end;
      Result := Text;
    finally
      Free;
    end;
end;

end.

