unit oTerm;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 10 $  $Modtime: 10/06/11 11:05a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default terminology object.
*
*       Notes:        NEVER ACCESS DIRECTLY, ALWAYS USE ITERM.
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oTerm.pas $
================================================================================
}
interface

uses
  SysUtils,
  Classes,
  ComCtrls,
  oCliOTypes,
  oInterfaces,
  oTermChild,
  Variants,
  XMLIntf;

type
  ETermUpdateValue = Exception;

  TTerm = class(TInterfacedObject, ITerm, ITermLoader, ITermUpdate)
  protected
    fID: string;
    fAbbreviation: string;
    fActive: Boolean;
    fDataType: TTermDataType;
    fValueType: TTermValueType;
    fDescription: string;
    fDisplayName: string;
    fHelpText: string;
    fTerm: string;
    fTermType: TTermType;
    fValueFormat: string;
    fBooleanValueTrue: string;
    fBooleanValueFalse: string;
    fMultiSelectPicklist: boolean;

    [weak] fOwnerList: IInterfaceList;
    fChildTerms: IInterfaceList;
    fUnits: IInterfaceList;
    fPairs: array[TTermType] of IInterfaceList;

    {ITermUpdate interface}
    procedure setTerm(aValue: string); stdcall;
    procedure setAbbreviation(aValue: string); stdcall;
    procedure setActive(aValue: Boolean); stdcall;
    procedure setDataType(aValue: TTermDataType); stdcall;
    procedure setValueType(aValue: TTermValueType); stdcall;
    procedure setDescription(aValue: string); stdcall;
    procedure setDisplayName(aValue: string); stdcall;
    procedure setHelpText(aValue: string); stdcall;
    procedure setTermType(aValue: TTermType); stdcall;
    procedure setValueFormat(aValueFormat: string); stdcall;
    procedure setBooleanValueTrue(aValue: string); stdcall;
    procedure setBooleanValueFalse(aValue: string); stdcall;
    procedure setMultiSelectPickList(aValue: boolean); stdcall;

    {ITermLoader interface}
    procedure ClearClinicalDataModel; stdcall;
    procedure AddTermPair(aTermPair: ITermPair); stdcall;
    procedure AddTermUnit(aTermUnit: ITermUnit); stdcall;
    procedure AddTermChild(aTermChild: ITermChild); stdcall;

    procedure SetOwnerList(aList: IInterfaceList); stdcall;
    function GetObject: TObject; stdcall;

  public
    constructor Create;
    destructor Destroy; override;

    {ITerm Interface}
    function getTermType: TTermType; stdcall;
    function getDataType: TTermDataType; stdcall;
    function getValueType: TTermValueType; stdcall;
    function getDisplayName: string; stdcall;
    function getID: string; stdcall;
    function getUnitCount: integer; stdcall;
    function getIsValidUnit(aUnit: ITerm): boolean; stdcall;
    function getIndex(aList: TStrings): integer; stdcall;

    {Keep up with the kids :)}
    function getChildCount: integer; stdcall;
    function getChildTerm(aIndex: integer): ITerm; stdcall;
    function getChildValue(aIndex: integer; aValue: string): string; stdcall;
    function getChildUnit(aIndex: integer): ITerm; stdcall;

    function getPickListValue(aIndex: integer): string; stdcall;
    function getHasQualifiers(aTermType: TTermType): boolean; stdcall;

    function LoadListOfPairs(aTermType: TTermType; aList: TStrings; aDisplayFormat: TTermDisplay = tdTerm): Boolean; stdcall;

    function AddToList(aList: TStrings; aDisplayFormat: TTermDisplay): integer; stdcall;
    function AddToListView(aListView: TListView; aDisplayFormat: TTermDisplay): TListItem; stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aDisplayFormat: TTermDisplay): TTreeNode; stdcall;

    function Validate(aUnit: ITerm; aValue: Double): TObservationRange; overload; stdcall;
    function Validate(aUnit: ITerm; aValue: string): TObservationRange; overload; stdcall;
    {End ITerm Interface}

    function LoadFromXML(aXMLNode: IXMLNode): boolean;

    function getAbbreviation: string; stdcall;
    function getActive: Boolean; stdcall;
    function getDescription: string; stdcall;
    function getFirstTermPair(aTermType: TTermType): ITerm;
    function getHelpText: string; stdcall;
    function getNextTermPair(aTermType: TTermType; aTerm: ITerm): ITerm;
    function getTerm: string; stdcall;
    function getTermPairCount(aTermType: TTermType): Integer;
    function getTermPairTerm(aTermType: TTermType; aIndex: integer): ITerm;
    function getTermPairValue(aTermType: TTermType; aIndex: integer): string; overload;
    function getTermPairValue(aTermType: TTermType; aTerm: ITerm): string; overload;
    function getValueFormat: string; stdcall;
    function getSiblingTerm(aID: string): ITerm; stdcall;
    function getBooleanValueTrue: string; stdcall;
    function getBooleanValueFalse: string; stdcall;
    function getMultiSelectPickList: boolean; stdcall;

    procedure DebugMe(aList: TStringList); stdcall;
  end;

function NewTerm(aXMLNode: IXMLNode): TTerm; overload;
function NewTerm(aTermType: TTermType; aDataType: TTermDataType; aID: string = ''): TTerm; overload;

implementation

uses
  MFunStr,
  uCPXML,
  Math,
  oTermPair;

function NewTerm(aXMLNode: IXMLNode): TTerm;
begin
  Result := TTerm.Create;
  if not Result.LoadFromXML(aXMLNode) then
    FreeAndNil(Result);
end;

function NewTerm(aTermType: TTermType; aDataType: TTermDataType; aID: string = ''): TTerm; overload;
begin
  Result := TTerm.Create;
  with Result do
    begin
      if aID <> '' then
        fID := aID
      else
        fID := NewGUID;
      fTerm := 'NEW TERM';
      fAbbreviation := 'NT';
      fDisplayName := 'New Term';
      fTermType := aTermType;
      fDataType := aDataType;
      fActive := False;
      fDescription := 'No Description';
      fHelpText := 'No Help Text';
      fValueFormat := '';
      fBooleanValueTrue := '';
      fBooleanValueFalse := '';
    end;
end;

{------------------------------------------------------------------------------}
{-------------------------------- TTerm ---------------------------------------}
{------------------------------------------------------------------------------}

procedure TTerm.ClearClinicalDataModel;
var
  tt: TTermType;
begin
  fChildTerms.Clear;
  fUnits.Clear;
  for tt := Low(TTermType) to High(TTermType) do
    fPairs[tt].Clear;
end;

constructor TTerm.Create;
var
  tt: TTermType;
begin
  inherited;

  for tt := Low(TTermType) to High(TTermType) do
    TInterfaceList.Create.GetInterface(IInterfaceList, fPairs[tt]);

  TInterfaceList.Create.GetInterface(IInterfaceList, fChildTerms);
  TInterfaceList.Create.GetInterface(IInterfaceList, fUnits);
end;

procedure TTerm.DebugMe(aList: TStringList);
var
  i: integer;
  tt: TTermType;
begin
  if fChildTerms.Count > 0 then
    begin
      aList.Add(' ');
      aList.Add('  Child Terms:');
      aList.Add('  ------------------------------------------------------------');
      for i := 0 to fChildTerms.Count - 1 do
        with (fChildTerms[i] as ITermChild) do
          aList.Add(Format('  %d: %s, (ID: %s)', [i, getChildTerm.getTerm, getChildTerm.getID]));
    end;

  for tt := Low(TTermType) to High(TTermType) do
    if fPairs[tt].Count > 0 then
      begin
        aList.Add(' ');
        aList.Add(Format('  Qualifier Type [%s]:', [TERMTYPE[tt]]));
        aList.Add('  ------------------------------------------------------------');
        for i := 0 to fPairs[tt].Count - 1 do
          with (fPairs[tt][i] as ITerm) do
            aList.Add(Format('  %d: %s (ID: %s)', [i, getTerm, getID]));
      end;

  if fUnits.Count > 0 then
    begin
      aList.Add(' ');
      aList.Add('  Available Units:');
      aList.Add('  ------------------------------------------------------------');
      for i := 0 to fUnits.Count - 1 do
        with (fUnits[i] as ITermUnit) do
          begin
            aList.Add(Format('  %d: %s (ID: %s)', [i, getUnit.getTerm, getUnit.getID]));
            aList.Add(Format('    - Min: %0.4f  Max: %0.4f  Dec: %d', [getValidMin, getValidMax, getValidDec]));
          end;
    end;

  aList.Add(' ');
end;

destructor TTerm.Destroy;
var
  tt: TTermType;
begin
  for tt := Low(TTermType) to High(TTermType) do
  begin
    fPairs[tt].Clear;
    fPairs[tt] := nil;
  end;

  fChildTerms.Clear;
  fChildTerms := nil;

  fUnits.Clear;
  fUnits := nil;

  fOwnerList := nil;

  inherited;
end;

{------------------------------------------------------------------------------}
{-------------------------- private methods -----------------------------------}
{------------------------------------------------------------------------------}

function TTerm.getAbbreviation: string;
begin
  Result := fAbbreviation;
end;

function TTerm.getActive: Boolean;
begin
  Result := fActive;
end;

function TTerm.getBooleanValueFalse: string;
begin
  if fBooleanValueFalse <> '' then
    Result := fBooleanValueFalse
  else
    Result := 'False';
end;

function TTerm.getBooleanValueTrue: string;
begin
  if fBooleanValueTrue <> '' then
    Result := fBooleanValueTrue
  else
    Result := 'True';
end;

function TTerm.getDataType: TTermDataType;
begin
  Result := fDataType;
end;

function TTerm.getDescription: string;
begin
  Result := fDescription;
end;

function TTerm.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TTerm.getTerm: string;
begin
  Result := fTerm;
end;

function TTerm.getFirstTermPair(aTermType: TTermType): ITerm;
begin
  if fPairs[aTermType].Count = 0 then
    Result := nil
  else
    fPairs[aTermType][0].QueryInterface(ITerm, Result);
end;

function TTerm.getHelpText: string;
var
  aTermUnit: ITermUnit;
  aPlural:string;
  i: integer;
  aFmt: string;
begin
  Result := fHelpText;

  for i := 0 to fUnits.Count - 1 do
    if fUnits[i].QueryInterface(ITermUnit, aTermUnit) = 0 then
      begin
        aFmt := Format('%%0.%df', [aTermUnit.getValidDec]);
        if aTermUnit.getValidDec = 1 then
          aPlural := ''
        else
          aPlural := 's';
        Result := Result + #13 +
          Format('    - values in %s must be between %s and %s with %d decimal%s.',
          [aTermUnit.getUnit.getTerm, Format(aFmt, [aTermUnit.getValidMin]), Format(aFmt, [aTermUnit.getValidMax]), aTermUnit.getValidDec, aPlural]);
      end;
end;

function TTerm.getNextTermPair(aTermType: TTermType; aTerm: ITerm): ITerm;
var
  i: Integer;
begin
  i := fPairs[aTermType].IndexOf(aTerm) + 1;
  if (i > 0) and (i < fPairs[aTermType].Count) then
    fPairs[aTermType][i].QueryInterface(ITerm, Result)
  else
    Result := nil;
end;

function TTerm.GetObject: TObject;
begin
  Result := Self;
end;

function TTerm.getPickListValue(aIndex: integer): string;
begin

end;

function TTerm.getTermPairCount(aTermType: TTermType): Integer;
begin
  Result := fPairs[aTermType].Count;
end;

function TTerm.getTermPairTerm(aTermType: TTermType; aIndex: integer): ITerm;
begin
  try
    fPairs[aTermType][aIndex].QueryInterface(ITerm, Result)
  except
    Result := nil;
  end;
end;

function TTerm.getTermPairValue(aTermType: TTermType; aIndex: integer): string;
begin
  try
    Result := (fPairs[aTermType][aIndex] as ITerm).getID;
  except
    Result := '';
  end;
end;

function TTerm.getTermPairValue(aTermType: TTermType; aTerm: ITerm): string;
var
  i: Integer;
begin
  i := fPairs[aTermType].IndexOf(aTerm);
  if i > -1 then
    Result := (fPairs[aTermType][i] as ITerm).getID
  else
    Result := '';
end;

function TTerm.getTermType: TTermType;
begin
  Result := fTermType;
end;

function TTerm.getUnitCount: integer;
begin
  Result := fPairs[ttUnit].Count;
end;

function TTerm.getValueFormat: string;
begin
  Result := fValueFormat;
end;

function TTerm.getValueType: TTermValueType;
begin
  Result := fValueType;
end;

function TTerm.getID: string;
begin
  Result := fID;
end;

procedure TTerm.setAbbreviation(aValue: string);
begin
  fAbbreviation := aValue;
end;

procedure TTerm.setActive(aValue: Boolean);
begin
  fActive := aValue;
end;

procedure TTerm.setBooleanValueFalse(aValue: string);
begin
  fBooleanValueFalse := aValue;
end;

procedure TTerm.setBooleanValueTrue(aValue: string);
begin
  fBooleanValueTrue := aValue;
end;

procedure TTerm.setDataType(aValue: TTermDataType);
begin
  fDataType := aValue;
end;

procedure TTerm.setDescription(aValue: string);
begin
  fDescription := aValue;
end;

procedure TTerm.setDisplayName(aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TTerm.setHelpText(aValue: string);
begin
  fHelpText := aValue;
end;

procedure TTerm.setMultiSelectPickList(aValue: boolean);
begin
  fMultiSelectPicklist := aValue;
end;

procedure TTerm.setTerm(aValue: string);
begin
  fTerm := aValue;
end;

procedure TTerm.setTermType(aValue: TTermType);
begin
  fTermType := aValue;
end;

procedure TTerm.setValueFormat(aValueFormat: string);
begin
  fValueFormat := aValueFormat;
end;

procedure TTerm.setValueType(aValue: TTermValueType);
begin
  fValueType := aValue;
end;

{------------------------------------------------------------------------------}
{--------------------------- public methods -----------------------------------}
{------------------------------------------------------------------------------}

procedure TTerm.AddTermChild(aTermChild: ITermChild);
begin
  fChildTerms.Add(aTermChild);
end;

procedure TTerm.AddTermPair(aTermPair: ITermPair);
begin
  fPairs[aTermPair.getQualifier.getTermType].Add(aTermPair);
end;

procedure TTerm.AddTermUnit(aTermUnit: ITermUnit);
begin
  fUnits.Add(aTermUnit);
end;

function TTerm.LoadListOfPairs(aTermType: TTermType; aList: TStrings; aDisplayFormat: TTermDisplay = tdTerm): Boolean;
var
  i: Integer;
begin
  result := false; //DeLinting

  aList.Clear;
  case aTermType of
    ttUnit:
      begin
        for i := 0 to fUnits.Count - 1 do
          (fUnits[i] as ITermUnit).getUnit.AddToList(aList, aDisplayFormat);
      end;
  else
    for i := 0 to fPairs[aTermType].Count - 1 do
      (fPairs[aTermType][i] as ITermPair).getQualifier.AddToList(aList, aDisplayFormat);
  end;
end;

function TTerm.Validate(aUnit: ITerm; aValue: string): TObservationRange;
var
  i: integer;
  aChildTerm: ITerm;
  aChildUnit: ITerm;
  aChildValue: string;
  aChildRange: TObservationRange;
  aDbl: Double;
begin
  case fDataType of
    dtComplex:
      begin
        Result := orNormal; {Assume we have all the parts valid, they will be checked one by one below}
        for i := 0 to getChildCount - 1 do
          begin
            aChildTerm := getChildTerm(i);
            aChildUnit := getChildUnit(i);
            aChildValue := getChildValue(i, aValue);
            aChildRange := aChildTerm.Validate(aChildUnit, aChildValue);
            if aChildRange <> orNormal then
              Result := aChildRange;
          end;
      end;
    dtNumeric:
      try
        Val(aValue, aDbl, i);
        if i = 0 then
          Result := Validate(aUnit, aDbl)
        else
          raise EConvertError.CreateFmt('Values for ''%s'' must be numeric.',[fTerm]);
      except
        on EConvertError do
          raise ETermValidationError.CreateFmt('Values for ''%s'' must be numeric.',[fTerm]);
      else
        raise;
      end;
  else
    Result := orUnknown;
  end;
end;

function TTerm.Validate(aUnit: ITerm; aValue: Double): TObservationRange;
var
  i: integer;
  aTermUnit: ITermUnit;
begin
  Result := orUnknown;

  if (aUnit = nil) and (fUnits.Count > 0) then {If a term has units, it must be filed with a unit}
    raise ETermValidationError.CreateFmt('Term %s must have a unit assigned.', [fTerm]);

  if aUnit = nil then {No unit sent and none assigned, assume it is correct}
    Result := orNormal
  else
    for i := 0 to fUnits.Count - 1 do
      if fUnits[i].QueryInterface(ITermUnit, aTermUnit) = 0 then
        if aTermUnit.getUnit.getID = aUnit.getID then
          begin
            {Check valid range}
            if (aValue < aTermUnit.getValidMin) or (aValue > aTermUnit.getValidMax) then
              raise ETermValidationError.CreateFmt('Value %f for %s is out of range.', [aValue,fTerm]);

            {Check decimal places}
            //aCmp := aValue * Power(10, aTermUnit.getValidDec);
            //if not (aCmp = Trunc(aCmp)) then
            //  raise ETermValidationError.CreateFmt('Value %f can only have %d decimals', [aValue, aTermUnit.getValidDec]);

            if (aValue < aTermUnit.getReferenceMin) then
              Result := orLow
            else if (aValue > aTermUnit.getReferenceMax) then
              Result := orHigh
            else
              Result := orNormal;
          end;
end;

function TTerm.LoadFromXML(aXMLNode: IXMLNode): boolean;
begin
  try
    fID := aXMLNode.ChildValues['ID'];
    fTerm := aXMLNode.ChildValues['TERM'];
    fAbbreviation := XMLValueAsStr(aXMLNode, 'ABBREVIATION');
    fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
    fTermType := TTermType(XMLValueAsInt(aXMLNode, 'TERM_TYPE'));
    fDataType := TTermDataType(XMLValueAsInt(aXMLNode, 'DATA_TYPE'));
    fValueType := TTermValueType(XMLValueAsInt(aXMLNode, 'VALUE_TYPE', 0));
    fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE', 0) = 1);
    fDescription := XMLValueAsStr(aXMLNode, 'DESCRIPTION', 'No Description');
    fHelpText := XMLValueAsStr(aXMLNode, 'HELP_TEXT', 'No Help Text');
    fValueFormat := XMLValueAsStr(aXMLNode, 'VALUE_FORMAT');
    fBooleanValueTrue := XMLValueAsStr(aXMLNode, 'BOOLEAN_VALUE_TRUE', 'True');
    fBooleanValueFalse := XMLValueAsStr(aXMLNode, 'BOOLEAN_VALUE_FALSE', 'False');
    fMultiSelectPicklist := (XMLValueAsInt(aXMLNode, 'MULTI_SELECT_PICKLIST', 0) = 1);
    Result := True;
  except
    Result := False;
  end;
end;

function TTerm.getChildCount: integer;
begin
  Result := fChildTerms.Count; // fPairs[ttObservation].Count;
end;

function TTerm.getChildTerm(aIndex: integer): ITerm;
var
  aTermChild: ITermChild;
begin
  try
    if fChildTerms[aIndex].QueryInterface(ITermChild, aTermChild) = 0 then
      Result := aTermChild.getChildTerm
    else
      Result := nil;
  except
    Result := nil;
  end;
  (*
  if aIndex < fChildTerms.Count then
    Result := TTerm_Child(fChildTerms[aIndex]).getChildTerm
  else
    Result := nil;
  *)
end;

function TTerm.getChildUnit(aIndex: integer): ITerm;
var
  aTermChild: ITermChild;
begin
  try
    if fChildTerms[aIndex].QueryInterface(ITermChild, aTermChild) = 0 then
      Result := aTermChild.getValueUnit
    else
      Result := nil;
  except
    Result := nil;
  end;
end;

function TTerm.getChildValue(aIndex: integer; aValue: string): string;
begin
  try
    with ITermChild(fChildTerms[aIndex]) do
      case getValueType of
        cvExtract: Result := Copy(aValue, getValueStartPosition, getValueStopPosition - getValueStartPosition + 1);
        cvDelimited: Result := Piece(aValue, getValueDelimiter, getValueStartPosition, getValueStopPosition);
      end
  except
    Result := '';
  end;
end;

procedure TTerm.SetOwnerList(aList: IInterfaceList);
begin
  fOwnerList := aList;
end;

function TTerm.getHasQualifiers(aTermType: TTermType): Boolean;
begin
  Result := (fPairs[aTermType].Count > 0);
end;

function TTerm.getIndex(aList: TStrings): integer;
{Assumes that the list was loaded properly by ITerminology}
begin
  Result := aList.IndexOfObject(Self);
end;

function TTerm.getIsValidUnit(aUnit: ITerm): boolean;
var
  i: integer;
  aTermUnit: ITermUnit;
begin
  for i := 0 to fUnits.Count - 1 do
    if fUnits[i].QueryInterface(ITermUnit, aTermUnit) = 0 then
      begin
        if aTermUnit.getUnit.getID = aUnit.getID then
          begin
            Result := True;
            Exit;
          end;
      end;
  Result := False;
end;

function TTerm.getMultiSelectPickList: boolean;
begin
  Result := fMultiSelectPicklist;
end;

function TTerm.getSiblingTerm(aID: string): ITerm;
var
  i: integer;
begin
  for i := 0 to fOwnerList.Count - 1 do
    if (fOwnerList[i] as ITerm).getID = aID then
      begin
        fOwnerList[i].QueryInterface(ITerm, Result);
        Exit;
      end;
  Result := nil;
end;

function TTerm.AddToList(aList: TStrings; aDisplayFormat: TTermDisplay): integer;
var
  Active: string;
begin
  if fActive then
    Active := ''
  else
    Active := ' (*)';

  case aDisplayFormat of
    tdTerm: Result := aList.AddObject(fTerm + Active, Self);
    tdAbbv: Result := aList.AddObject(fAbbreviation + Active, Self);
    tdDisplayName: Result := aList.AddObject(fDisplayName + Active, Self);
    tdID: Result := aList.AddObject(fID, Self);
  else
    Result := aList.AddObject(fID, Self);
  end;
end;

function TTerm.AddToListView(aListView: TListView; aDisplayFormat: TTermDisplay): TListItem;
begin
  //
  result := nil;
end;

function TTerm.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aDisplayFormat: TTermDisplay): TTreeNode;
var
  Active: string;
begin
  if fActive then
    Active := ''
  else
    Active := ' (*)';

  case aDisplayFormat of
    tdTerm: Result := aTreeView.Items.AddChildObject(aTreeNode, fTerm + Active, Self); //aList.AddObject(fTerm, Self);
    tdAbbv: Result := aTreeView.Items.AddChildObject(aTreeNode, fAbbreviation + Active, Self); //aList.AddObject(fAbbreviation, Self);
    tdDisplayName: Result := aTreeView.Items.AddChildObject(aTreeNode, fDisplayName + Active, Self); //aList.AddObject(fDisplayName, Self);
    tdID: Result := aTreeView.Items.AddChildObject(aTreeNode, fID, Self); //aList.AddObject(fID, Self);
  else
    Result := aTreeView.Items.AddChildObject(aTreeNode, fID, Self); //aList.AddObject(fID, Self);
  end;
end;

end.

