unit oTermChild;

{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 4 $  $Modtime: 4/08/08 9:52a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Default terminology object.
*
*       Notes:        NEVER ACCESS DIRECTLY, ALWAYS USE ITermChild.
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oTermChild.pas $
*
* $History: oTermChild.pas $
 * 
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 4/08/08    Time: 8:14p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 1/14/08    Time: 4:55p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 12/03/07   Time: 3:01p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:18p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 7/06/07    Time: 10:12a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 5/21/07    Time: 2:56p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 4/10/07    Time: 3:46p
 * Created in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 3/21/07    Time: 1:55p
 * Created in $/CP Modernization/CliO/source/Common/objects
 *
*
================================================================================
}
interface

uses
  SysUtils,
  Classes,
  oCliOTypes,
  oInterfaces,
  Variants,
  XMLIntf;

type
  TTermChild = class(TInterfacedObject, ITermChild)
  private
    fTerm: ITerm;
    fChildOrder: integer;
    fChildTerm: ITerm;
    fValueType: TTermChildValueType;
    fValueDelimiter: char;
    fValueStartPosition: integer;
    fValueStopPosition: integer;
    fValueUnit: ITerm;
    fDescription: string;
  public
    constructor Create;
    destructor Destroy; override;

    function getParentTerm: ITerm; stdcall;
    function getChildTerm: ITerm; stdcall;
    function getChildOrder: integer; stdcall;
    function getValue(aParentValue: string): string; stdcall;
    function getValueType: TTermChildValueType; stdcall;
    function getValueDelimiter: char; stdcall;
    function getValueStartPosition: integer; stdcall;
    function getValueStopPosition: integer; stdcall;
    function getValueUnit: ITerm; stdcall;
    function getEvaluation: string; stdcall;
    function getDescription: string; stdcall;
    function getParameters: string; stdcall;

    procedure setChildOrder(aValue: integer);
    procedure setChildTerm(aValue: ITerm);
    procedure setValueType(aValue: TTermChildValueType);
    procedure setValueDelimiter(aValue: char);
    procedure setValueStartPosition(aValue: integer);
    procedure setValueStopPosition(aValue: integer);
    procedure setValueUnit(aValue: ITerm);
    procedure setDescription(aValue: string);
  end;

function NewTermChild(aTerm: ITerm; aXMLNode: IXMLNode): TTermChild;

implementation

uses
  uCPXML,
  MFunStr;

function NewTermChild(aTerm: ITerm; aXMLNode: IXMLNode): TTermChild;
begin
  Result := TTermChild.Create;
  with Result do
    begin
      fTerm := aTerm;
      if Assigned(aXMLNode) then
        begin
          fChildOrder := XMLValueAsInt(aXMLNode, 'CHILD_ORDER');
          fChildTerm := aTerm.getSiblingTerm(XMLValueAsStr(aXMLNode, 'CHILD_ID'));
          fValueType := TTermChildValueType(XMLValueAsInt(aXMLNode, 'VALUE_TYPE'));
          fValueDelimiter := Chr(XMLValueAsInt(aXMLNode, 'VALUE_DELIMITER', 32));
          fValueStartPosition := XMLValueAsInt(aXMLNode, 'VALUE_START_POSITION');
          fValueStopPosition := XMLValueAsInt(aXMLNode, 'VALUE_STOP_POSITION');
          fValueUnit := aTerm.getSiblingTerm(XMLValueAsStr(aXMLNode, 'CHILD_UNIT_ID'));
          fDescription := XMLValueAsStr(aXMLNode, 'DESCRIPTION');
        end;
    end;
end;

{ TTerm_Child }

constructor TTermChild.Create;
begin
  inherited;
  fTerm := nil;
  fChildOrder := 1;
  fChildTerm := nil;
  fValueType := cvExtract;
  fValueDelimiter := ' ';
  fValueStartPosition := 1;
  fValueStopPosition := 1;
  fValueUnit := nil;
  fDescription := 'Description';
end;

destructor TTermChild.Destroy;
begin
  inherited;
end;

function TTermChild.getChildOrder: integer;
begin
  Result := fChildOrder;
end;

function TTermChild.getChildTerm: ITerm;
begin
  Result := fChildTerm;
end;

function TTermChild.getDescription: string;
begin
  Result := fDescription;
end;

function TTermChild.getEvaluation: string;
begin
  case fValueType of
    cvDelimited:
      case (fValueStartPosition = fValueStopPosition) of
        True: Result := Format('Delimited value %d delimited by ''%s''.',
            [fValueStartPosition, fValueDelimiter]);
        False: Result := Format('Delimited value(s) %d - %d delimited by ''%s''.',
            [fValueStartPosition, fValueStopPosition, fValueDelimiter]);
      end;
    cvExtract:
      Result := Format('Extracted value from postion %d for length of %d.',
        [fValueStartPosition, fValueStopPosition - fValueStartPosition + 1]);
  else
    Result := '';
  end;
end;

function TTermChild.getParameters: string;
begin
  case fValueType of
    cvExtract: Result := Format('E:%d, %d', [fValueStartPosition, fValueStopPosition]);
    cvDelimited: Result := Format('D:%d, %d, %s', [fValueStartPosition, fValueStopPosition, fValueDelimiter]);
    cvCalculation: Result := 'Param';
  end;
  if Assigned(fValueUnit) then
    Result := Result + ', ' + fValueUnit.getAbbreviation;
end;

function TTermChild.getParentTerm: ITerm;
begin
  Result := fTerm;
end;

function TTermChild.getValue(aParentValue: string): string;
begin
  case fValueType of
    cvDelimited: Result := Piece(aParentValue, fValueDelimiter, fValueStartPosition, fValueStopPosition);
    cvExtract: Result := Copy(aParentValue, fValueStartPosition, fValueStopPosition - fValueStartPosition);
  else
    Result := '';
  end;
end;

function TTermChild.getValueDelimiter: char;
begin
  Result := fValueDelimiter;
end;

function TTermChild.getValueStartPosition: integer;
begin
  Result := fValueStartPosition;
end;

function TTermChild.getValueStopPosition: integer;
begin
  Result := fValueStopPosition
end;

function TTermChild.getValueType: TTermChildValueType;
begin
  Result := fValueType;
end;

function TTermChild.getValueUnit: ITerm;
begin
  Result := fValueUnit;
end;

procedure TTermChild.setChildOrder(aValue: integer);
begin
  fChildOrder := aValue;
end;

procedure TTermChild.setChildTerm(aValue: ITerm);
begin
  fChildTerm := aValue;
end;

procedure TTermChild.setDescription(aValue: string);
begin
  fDescription := aValue;
end;

procedure TTermChild.setValueDelimiter(aValue: char);
begin
  fValueDelimiter := aValue;
end;

procedure TTermChild.setValueStartPosition(aValue: integer);
begin
  fValueStartPosition := aValue;
end;

procedure TTermChild.setValueStopPosition(aValue: integer);
begin
  fValueStopPosition := aValue;
end;

procedure TTermChild.setValueType(aValue: TTermChildValueType);
begin
  fValueType := aValue;
end;

procedure TTermChild.setValueUnit(aValue: ITerm);
begin
  fValueUnit := aValue;
end;

end.

