unit oTermConversion;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 3 $  $Modtime: 4/08/08 9:52a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Contains logic derived from db for managing unit
*                     conversions on the fly.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oTermConversion.pas $
================================================================================
}
interface

uses
  XMLIntf,
  oCliOTypes,
  oInterfaces,
  SysUtils,
  Classes;

type
  TTermConversion = class(TInterfacedObject, ITermConversion)
  private
    fTermPre: ITerm;
    fTermPost: ITerm;
    fConversionUnitPre: string;
    fConversionUnitPost: string;
    fConversionOffsetPre: Double;
    fConversionOffsetPost: Double;
    fConversionFactor: Double;
    fConversionDecimalPrec: integer;
    fConversionDescription: string;

  public
    constructor Create;
    destructor Destroy; override;

    procedure setTermPre(aValue: ITerm);
    procedure setTermPost(aValue: ITerm);
    procedure setOffsetPre(aValue: Double);
    procedure setOffsetPost(aValue: Double);
    procedure setFactor(aValue: Double);
    procedure setDecimal(aValue: integer);
    procedure setDescription(aValue: string);

    {ITermConversion}
    function getID: string; stdcall;
    function getTermPre: ITerm; stdcall;
    function getTermPost: ITerm; stdcall;
    function getOffsetPre: Double; stdcall;
    function getOffsetPost: Double; stdcall;
    function getFactor: Double; stdcall;
    function getDecimal: integer; stdcall;
    function getDescription: string; stdcall;
    function Execute(aValue: Double): Double; stdcall;
  end;

function NewTermConversion(aTerm: ITerm; aXMLNode: IXMLNode): TTermConversion;

implementation

uses
  Math,
  uCPXML,
  Variants;

function NewTermConversion(aTerm: ITerm; aXMLNode: IXMLNode): TTermConversion;
begin
  Result := TTermConversion.Create;
  Result.fTermPre := aTerm;

  if aXMLNode <> nil then
    with Result do
      try
        fConversionUnitPre := XMLValueAsStr(aXMLNode, 'UNIT_ID_PRE');
        fConversionUnitPost := XMLValueAsStr(aXMLNode, 'UNIT_ID_POST');

        fTermPost := aTerm.getSiblingTerm(fConversionUnitPost);

        fConversionOffsetPre := XMLValueAsDouble(aXMLNode, 'CONVERSION_OFFSET_PRE', 0);
        fConversionOffsetPost := XMLValueAsDouble(aXMLNode, 'CONVERSION_OFFSET_POST', 0);

        fConversionFactor := XMLValueAsDouble(aXMLNode, 'CONVERSION_FACTOR', 1);

        fConversionDecimalPrec := XMLValueAsInt(aXMLNode, 'DECIMAL_PRECISION', 0);
        fConversionDescription := XMLValueAsStr(aXMLNode, 'DESCRIPTION');
      except
        Result := nil;
      end;
end;

(******************************************************************************)
(*************************** TUnitConversion **********************************)
(******************************************************************************)

constructor TTermConversion.Create;
begin
  inherited;
  fTermPre := nil;
  fTermPost := nil;
  fConversionUnitPre := '';
  fConversionUnitPost := '';
  fConversionOffsetPre := 0.0;
  fConversionOffsetPost := 0.0;
  fConversionFactor := 1;
  fConversionDecimalPrec := 0;
  fConversionDescription := 'No Description';
end;

destructor TTermConversion.Destroy;
begin
  inherited;
end;

function TTermConversion.Execute(aValue: Double): Double;
begin
  try
    Result := RoundTo(
      (aValue + fConversionOffsetPre) *
      fConversionFactor +
      fConversionOffsetPost,
      fConversionDecimalPrec * -1);
  except
    Result := aValue;
  end;
end;

function TTermConversion.getDecimal: integer;
begin
  Result := fConversionDecimalPrec;
end;

function TTermConversion.getDescription: string;
begin
  Result := fConversionDescription;
end;

function TTermConversion.getFactor: Double;
begin
  Result := fConversionFactor;
end;

function TTermConversion.getID: string;
begin
  Result := Format('%s:%s', [fConversionUnitPre, fConversionUnitPost]);
end;

function TTermConversion.getOffsetPost: Double;
begin
  Result := fConversionOffsetPost;
end;

function TTermConversion.getOffsetPre: Double;
begin
  Result := fConversionOffsetPre;
end;

function TTermConversion.getTermPost: ITerm;
begin
  Result := fTermPost;
end;

function TTermConversion.getTermPre: ITerm;
begin
  Result := fTermPre;
end;

procedure TTermConversion.setDecimal(aValue: integer);
begin
  fConversionDecimalPrec := aValue;
end;

procedure TTermConversion.setDescription(aValue: string);
begin
  fConversionDescription := aValue;
end;

procedure TTermConversion.setFactor(aValue: Double);
begin
  fConversionFactor := aValue;
end;

procedure TTermConversion.setOffsetPost(aValue: Double);
begin
  fConversionOffsetPost := aValue;
end;

procedure TTermConversion.setOffsetPre(aValue: Double);
begin
  fConversionOffsetPre := aValue;
end;

procedure TTermConversion.setTermPost(aValue: ITerm);
begin
  fTermPost := aValue;
end;

procedure TTermConversion.setTermPre(aValue: ITerm);
begin
  fTermPre := aValue;
end;

end.

