unit oTerm_Range_Check;

interface

uses
  Classes,
  SysUtils,
  XMLIntf,
  XMLDoc,
  oInterfaces;

type
  TTerm_Range_Check = class(TInterfacedObject, IRangeCheck, ITermRangeCheck)
  private
    fTerm: ITerm;
    fUnit: ITerm;
    fSexSpecific: boolean;
    fAgeSpecific: boolean;
    fSex: string;
    fAgeMin: integer;
    fAgeMax: integer;
    fNormalMin: Double;
    fNormalMax: Double;
    fAbnormalMin: Double;
    fAbnormalMax: Double;
    fCriticalMin: Double;
    fCriticalMax: Double;
    fDescription: string;
  public
    constructor Create;

    function getTerm: ITerm; stdcall;
    function getUnit: ITerm; stdcall;
    function getAgeSpecific: boolean; stdcall;
    function getSexSpecific: boolean; stdcall;
    function getSex: string; stdcall;
    function getAgeMin: integer; stdcall;
    function getAgeMax: integer; stdcall;
    function getNormalMin: Double; stdcall;
    function getNormalMax: Double; stdcall;
    function getAbnormalMin: Double; stdcall;
    function getAbnormalMax: Double; stdcall;
    function getCriticalMin: Double; stdcall;
    function getCriticalMax: Double; stdcall;
    function getDescription: string; stdcall;
    function getAllRanges: string; stdcall;

    procedure setUnit(aValue: ITerm);
    procedure setAgeSpecific(aValue: boolean);
    procedure setSex(aValue: string);
    procedure setAgeMin(aValue: integer);
    procedure setAgeMax(aValue: integer);
    procedure setNormalMin(aValue: Double);
    procedure setNormalMax(aValue: Double);
    procedure setAbnormalMin(aValue: Double);
    procedure setAbnormalMax(aValue: Double);
    procedure setCriticalMin(aValue: Double);
    procedure setCriticalMax(aValue: Double);
    procedure setDescription(aValue: string);

    function Execute(aValue: Double; aUnit: ITerm): boolean; stdcall;
  end;

function NewTermRangeCheck(aTerm: ITerm; aXMLNode: IXMLNode): TTerm_Range_Check;

implementation

uses
  uCPXML;

function NewTermRangeCheck(aTerm: ITerm; aXMLNode: IXMLNode): TTerm_Range_Check;
begin
  Result := TTerm_Range_Check.Create;
  with Result do
    try
      fTerm := aTerm;
      fUnit := fTerm.getSiblingTerm(XMLValueAsStr(aXMLNode, 'UNIT_ID'));

      fSex := XMLValueAsStr(aXMLNode, 'SEX');
      fSexSpecific := (fSex <> 'N');

      fAgeMin := XMLValueAsInt(aXMLNode, 'AGE_MINIMUM');
      fAgeMax := XMLValueAsInt(aXMLNode, 'AGE_MAXIMUM');
      fAgeSpecific := (fAgeMin > -1) and (fAgeMax > -1);

      fNormalMin := XMLValueAsDouble(aXMLNode, 'NORMAL_LOW');
      fNormalMax := XMLValueAsDouble(aXMLNode, 'NORMAL_HIGH');

      fAbnormalMin := XMLValueAsDouble(aXMLNode, 'ABNORMAL_LOW');
      fAbnormalMax := XMLValueAsDouble(aXMLNode, 'ABNORMAL_HIGH');

      fCriticalMin := XMLValueAsDouble(aXMLNode, 'CRITICAL_LOW');
      fCriticalMax := XMLValueAsDouble(aXMLNode, 'CRITICAL_HIGH');

      fDescription := XMLValueAsStr(aXMLNode, 'DESCRIPTION');
    except
      FreeAndNil(Result);
    end;
end;

{ TTerm_Range_Check }

constructor TTerm_Range_Check.Create;
begin
  inherited Create;
  fTerm := nil;
  fUnit := nil;
  fSexSpecific := false;
  fAgeSpecific := false;
  fSex := 'N';
  fAgeMin := -1;
  fAgeMax := -1;
  fNormalMin := 0;
  fNormalMax := 0;
  fAbnormalMin := 0;
  fAbnormalMax := 0;
  fCriticalMin := 0;
  fCriticalMax := 0;
  fDescription := 'New Range Check';
end;

function TTerm_Range_Check.Execute(aValue: Double; aUnit: ITerm): boolean;
begin
  Result := False;
end;

function TTerm_Range_Check.getAbnormalMax: Double;
begin
  Result := fAbnormalMax;
end;

function TTerm_Range_Check.getAbnormalMin: Double;
begin
  Result := fAbnormalMin;
end;

function TTerm_Range_Check.getAgeMax: integer;
begin
  Result := fAgeMax;
end;

function TTerm_Range_Check.getAgeMin: integer;
begin
  Result := fAgeMin;
end;

function TTerm_Range_Check.getAgeSpecific: boolean;
begin
  Result := fAgeSpecific;
end;

function TTerm_Range_Check.getAllRanges: string;
begin
  Result := Format(
    '%0.1f<%0.1f<%0.1f<Normal>%0.1f>%0.1f>%0.1f',
    [fCriticalMin, fAbnormalMin, fNormalMin, fNormalMax, fAbnormalMax, fCriticalMax]);
end;

function TTerm_Range_Check.getCriticalMax: Double;
begin
  Result := fCriticalMax;
end;

function TTerm_Range_Check.getCriticalMin: Double;
begin
  Result := fCriticalMin;
end;

function TTerm_Range_Check.getDescription: string;
begin
  Result := fDescription;
end;

function TTerm_Range_Check.getNormalMax: Double;
begin
  Result := fNormalMax;
end;

function TTerm_Range_Check.getNormalMin: Double;
begin
  Result := fNormalMin;
end;

function TTerm_Range_Check.getSex: string;
begin
  Result := fSex;
end;

function TTerm_Range_Check.getSexSpecific: boolean;
begin
  Result := fSexSpecific;
end;

function TTerm_Range_Check.getTerm: ITerm;
begin
  Result := fTerm;
end;

function TTerm_Range_Check.getUnit: ITerm;
begin
  Result := fUnit;
end;

procedure TTerm_Range_Check.setAbnormalMax(aValue: Double);
begin
  fAbnormalMax := aValue;
end;

procedure TTerm_Range_Check.setAbnormalMin(aValue: Double);
begin
  fAbnormalMin := aValue;
end;

procedure TTerm_Range_Check.setAgeMax(aValue: integer);
begin
  fAgeMax := aValue;
end;

procedure TTerm_Range_Check.setAgeMin(aValue: integer);
begin
  fAgeMin := aValue;
end;

procedure TTerm_Range_Check.setAgeSpecific(aValue: boolean);
begin
  fAgeSpecific := aValue;
end;

procedure TTerm_Range_Check.setCriticalMax(aValue: Double);
begin
  fCriticalMax := aValue;
end;

procedure TTerm_Range_Check.setCriticalMin(aValue: Double);
begin
  fCriticalMin := aValue;
end;

procedure TTerm_Range_Check.setDescription(aValue: string);
begin
  fDescription := aValue;
end;

procedure TTerm_Range_Check.setNormalMax(aValue: Double);
begin
  fNormalMax := aValue;
end;

procedure TTerm_Range_Check.setNormalMin(aValue: Double);
begin
  fNormalMin := aValue;
end;

procedure TTerm_Range_Check.setSex(aValue: string);
begin
  fSex := aValue;
  fSexSpecific := (fSex <> 'N');
end;

procedure TTerm_Range_Check.setUnit(aValue: ITerm);
begin
  fUnit := aValue;
end;

end.

