unit oTrackableInterfacedObject;

interface

type
  TTrackableInterfacedObject = class(TInterfacedObject)
  private
    fTrackingID: string;
  public
    constructor Create;
    destructor Destroy; override;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  end;

implementation

uses
  oInterfaces,
  Dialogs,
  SysUtils,
  IniFiles,
  Windows; // Added for DeleteFile() procedure to be inlined.

var
  TrackingFile: TMemIniFile;

constructor TTrackableInterfacedObject.Create;
var
  i: integer;
begin
  inherited;
  if not Assigned(TrackingFile) then
    begin
      DeleteFile('.\ObjectTracking.ini');
      TrackingFile := TMemIniFile.Create('.\ObjectTracking.ini');
      TrackingFile.WriteDate('INITIALIZATION', 'DATE/TIME', Now);
      MessageDlg(
        'Object tracking initiated. Severe performance penalty will be seen',
        mtInformation, [mbOk], 0);
    end;

  fTrackingID := NewGUID;

  with TrackingFile do
    begin
      i := ReadInteger('OBJECTS_' + Self.ClassName, 'ACTIVE_OBJECT_COUNT', 0);
      inc(i);
      WriteInteger('OBJECTS_' + Self.ClassName, 'ACTIVE_OBJECT_COUNT', i);
      WriteString('OBJECTS_' + Self.Classname, fTrackingID, 'Created ' + DateTimeToStr(Now));
      UpdateFile;
    end;
end;

destructor TTrackableInterfacedObject.Destroy;
var
  i: integer;
begin
  with TrackingFile do
    begin
      DeleteKey('OBJECTS_' + Self.Classname, fTrackingID);
      i := ReadInteger('OBJECTS_' + Self.ClassName, 'ACTIVE_OBJECT_COUNT', 0);
      dec(i);
      WriteInteger('OBJECTS_' + Self.ClassName, 'ACTIVE_OBJECT_COUNT', i);
      UpdateFile;
    end;
  inherited;
end;

function TTrackableInterfacedObject._AddRef: Integer;
var
  i: integer;
begin
  result := inherited _AddRef;
  with TrackingFile do
    begin
      i := ReadInteger(fTrackingID, 'REFCOUNT', 0);
      inc(i);
      WriteString(fTrackingID, 'CLASSNAME', Self.ClassName);
      WriteInteger(fTrackingID, 'REFCOUNT', i);
      UpdateFile;
    end;
end;

function TTrackableInterfacedObject._Release: Integer;
var
  i: integer;
begin
  with TrackingFile do
    begin
      i := ReadInteger(fTrackingID, 'REFCOUNT', 0);
      dec(i);
      WriteInteger(fTrackingID, 'REFCOUNT', i);
      if i = 0 then
        EraseSection(fTrackingID);
      UpdateFile;
    end;
  result := inherited _Release;
end;

end.

