unit u508Dialogs;

interface

uses
  Windows, Dialogs, Consts, Controls, Forms, Classes;


function MessageDlg508(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;

var
  Captions: array[TMsgDlgType] of Pointer = (@SMsgDlgWarning, @SMsgDlgError,
    @SMsgDlgInformation, @SMsgDlgConfirm, nil);

implementation

function MessageDlg508(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
var
  Caption: string;
  Flags: Word;
  WinResult: word;
begin
  if DlgType <> mtCustom then
    Caption := LoadResString(Captions[DlgType])
  else
    Caption := Application.Title;

  Flags := MB_APPLMODAL;

  case DlgType of
    mtWarning: Flags := Flags + MB_ICONWARNING;
    mtError: Flags := Flags + MB_ICONERROR;
    mtInformation: Flags := Flags + MB_ICONINFORMATION;
    mtConfirmation: Flags := Flags + MB_ICONQUESTION;
  else
    Flags := 0;
  end;

  if Application.BiDiMode <> bdLeftToRight then
    Flags := Flags + MB_RTLREADING;

  {
  The Delphi MessageDlg() API is a bit more flexible in that individual buttons can
    be selected for display, while the Win32 API only allows sets of
    buttons to be selected.  We'll be working with the intersection of the
    sets down below.  Specifically,  mbYesAllNoAllCancel and mbAbortIgnore are
    not available as a set from the Win32 API.
  }

  if Buttons = mbYesNo then
    Flags := Flags + MB_YESNO;
  if Buttons = mbYesNoCancel then
    Flags := Flags + MB_YESNOCANCEL;
  if Buttons = mbOKCancel then
    Flags := Flags + MB_OKCANCEL;
  if Buttons = mbAbortRetryIgnore then
    Flags := Flags + MB_ABORTRETRYIGNORE;

  WinResult := MessageBox(Application.ActiveFormHandle, PChar(Msg), PChar(Caption), Flags);

  {
  Note: thew ID_ id's and the ModalResult array are not in the same order.  Which means
  that, unfortunately, we can just return WinResult and be done with it.
  }
  case WinResult of
    IDOK: result := mrOK;
    IDCANCEL: result := mrCancel;
    IDABORT: result := mrAbort;
    IDRETRY: result := mrRetry;
    IDIGNORE: result := mrIgnore;
    IDYES: result := mrYes;
    IDNO: result := mrNo;
    {
    Note: These don't map directly.  So, we'll ignore them for now.
    IDCLOSE: ;
    IDHELP: ;
    IDTRYAGAIN: ;
    IDCONTINUE: ;
    }
    else
      result := 0;
  end;
end;

end.
