unit uCCOW;
{
================================================================================
*
*       Application:  Plug and Play Unit
*       Revision:     $Revision: 4 $  $Modtime: 5/26/09 12:44p $
*       Developer:    Team 2
*       Site:         Hines OIFO
*
*       Description:  Sentillion CCOW implementation
*
*       Notes:        Requires Sentillion SDK installation
*                     for ActiveX TContextorControl as well
*                     as the desktop locator.
*
*                     Developed with Sentillion SDK v 3.3
*                     Build 3.3.1.8907
================================================================================
}

interface

uses
  Sysutils,
  Controls,
  Dialogs,
  Classes,
  ActiveX,
  ImgList,
  OleCtrls,
  VERGENCECONTEXTORLib_TLB,
  TRPCB,
  CCOWRPCBroker,
  Forms;

type
  TVACCOW_RejoinMode = (rmGet, rmSet);
  TVACCOW_LinkStatus = (lsNoCCOW, lsJoined, lsBroken, lsChanging);
  TVACCOW_AbleToChangeStatus = (acYes, acNo, acCannot);

  TNotifyAbleToChange = procedure(var OkayToChange: TVACCOW_AbleToChangeStatus; var MsgText: string) of object;
  TNotifyNewPatientID = procedure(const PtID: string) of object;
  TNotifyNewLinkStatus = procedure(const Status: TVACCOW_LinkStatus) of object;

  TVACCOW = class(Tobject)
  private
    fContextManager: TContextorControl;
    FLinkStatus: TVACCOW_LinkStatus;
    FAbleToChangeStatus: TVACCOW_AbleToChangeStatus;
    FCCOWEnabled: Boolean;
    FPatientContextItem: string;
    fBroker: TCCOWRPCBroker;

    procedure NotifyNewPatientIDDefault(const aPtId: string);
    procedure NotifyNewLinkStatusDefault(const Status: TVACCOW_LinkStatus);
    procedure NotifyAbleToChangeDefault(var OkayToChange: TVACCOW_AbleToChangeStatus; var MsgText: string);

    procedure SetLinkStatus(const NewStatus: TVACCOW_LinkStatus);
    procedure CleanupBeforeChange;
  public
    NotifyNewPatientID: TNotifyNewPatientID;
    NotifyNewLinkStatus: TNotifyNewLinkStatus;
    NotifyAbleToChange: TNotifyAbleToChange;

    constructor Create(AOwner: TComponent);
    destructor Destroy; override;

    {pointers to methods of fContextManager}
    procedure OnPendingExecute(Sender: TObject; const aContextItemCollection: IDispatch);
    procedure OnCommitExecute(Sender: TObject);
    procedure OnCancelExecute(Sender: TObject);

    function JoinContext(ApplicationName: WideString; Passcode: WideString = ''; Survey: Boolean = True; InitialNotificationFilter: string = '*'): Boolean;
    function SuspendContext: Boolean;
    function ResumeContext(const RejoinMode: TVACCOW_RejoinMode): Boolean;

    function SetNewPatientContext(const PtId: string): boolean;
    function RefreshContext: boolean;
  published
    property CCOWEnabled: Boolean read FCCOWEnabled;
    property LinkStatus: TVACCOW_LinkStatus read FLinkStatus;
    property PatientContextItem: string read fPatientContextItem write fPatientContextItem;
  end;

const
  CCOW_PATIENT_ID_MRN = 'patient.id.MRN.CCOW';
  CCOW_PATIENT_ID_ALT = 'patient.id.ALTERNATE.CCOW';
  CCOW_PATIENT_ID_MPI = 'patient.id.MPI';
  CCOW_PATIENT_ID_NID = 'patient.id.NATIONALIDNUMBER';

  CCOW_PATIENT_CO_NAME = 'patient.co.PATIENTNAME';
  CCOW_PATIENT_CO_ALIAS = 'patient.co.ALIASNAME';
  CCOW_PATIENT_CO_DOB = 'patient.co.DATETIMEOFBIRTH';
  CCOW_PATIENT_CO_SEX = 'patient.co.SEX';
  CCOW_PATIENT_CO_DLN = 'patient.co.DLN';
  CCOW_PATIENT_CO_SSN = 'patient.co.SSN';

  CCOW_OBSERVATION_ID_PON = 'observation.id.PLACER_ORDER_NUMBER.CCOW';
  CCOW_OBSERVATION_ID_FON = 'observation.id.FILLER_ORDER_NUMBER.CCOW';

  CCOW_OBSERVATION_CO_SYSID = 'observation.co.SYSTEM_ID.CCOW';
  CCOW_OBSERVATION_CO_SVCID = 'observation.co.SERVICE_ID.CCOW';
  CCOW_OBSERVATION_CO_DATETIME = 'observation.co.DATE_TIME.CCOW';
  CCOW_OBSERVATION_CO_SVCDESC = 'observation.co.SERVICE_DESC.CCOW';

  CCOW_LINK_STATUS: array[TVACCOW_LinkStatus] of string = ('No CCOW', 'Joined', 'Broken', 'Changing');

function NewCCOWContexter(aOwner: TComponent; aCCOWBroker: TCCOWRPCBroker): TVACCOW;

implementation

/////{$R uCCOW.res}

{ TVACCOW }

{ Procedures to instatiate new TVACCOW object}

function NewCCOWContexter(aOwner: TComponent; aCCOWBroker: TCCOWRPCBroker): TVACCOW;
begin
  try {If this fails, we have a non-ccow state}
    Result := TVACCOW.Create(aOwner);
    {If we pass in a broker, assign the real contextor to it}
    if Assigned(aCCOWBroker) then
      begin
        Result.fBroker := aCCOWBroker;
        Result.fContextManager.Enabled := True;
        aCCOWBroker.Contextor := Result.fContextManager;
      end;
  except
    Result := nil;
  end;
end;

constructor TVACCOW.Create(AOwner: TComponent);
begin
  fContextManager := TContextorControl.Create(AOwner);
  fContextManager.OnCanceled := OnCancelExecute;
  fContextManager.OnCommitted := OnCommitExecute;
  fContextManager.OnPending := OnPendingExecute;

  FCCOWEnabled := False; //to enable CCOW call Join Context
  FLinkStatus := lsBroken;

  NotifyNewPatientID := NotifyNewPatientIDDefault;
  NotifyNewLinkStatus := NotifyNewLinkStatusDefault;
  NotifyAbleToChange := NotifyAbleToChangeDefault;
end;

destructor TVACCOW.Destroy;
begin
  if Assigned(fBroker) then
    fBroker.Contextor := nil;
  FreeAndNil(fContextManager);
  inherited;
end;

function TVACCOW.JoinContext(ApplicationName: WideString; Passcode: WideString = ''; Survey: Boolean = True; InitialNotificationFilter: string = '*'): Boolean;
begin
  try
    fContextManager.Run(ApplicationName + '#', Passcode, Survey, InitialNotificationFilter);
    FCCOWEnabled := True;
    Result := True;
    SetLinkStatus(lsJoined);
  except
    FCCOWEnabled := False;
    Result := False;
    SetLinkStatus(lsNoCCOW);
  end;
end;

function TVACCOW.RefreshContext: boolean;
{Simple method to fire the setpt and change of status}
begin
  OnCommitExecute(nil);
  NotifyNewLinkStatus(FLinkStatus);
  result := true;
end;

{procedures methods for responding to contextor}

procedure TVACCOW.OnCancelExecute(Sender: TObject);
begin
  // Do nothing, it was cancelled
end;

procedure TVACCOW.OnPendingExecute(Sender: TObject; const aContextItemCollection: IDispatch);
var
  OkayToChange: TVACCOW_AbleToChangeStatus;
  MsgText: string;
begin
  NotifyAbleToChange(OkayToChange, MsgText);
  FAbleToChangeStatus := OkayToChange;
  case FAbleToChangeStatus of
    acNo: fContextManager.SetSurveyResponse(MsgText);
    acCannot: fContextManager.SetSurveyResponse(MsgText);
  end;
end;

procedure TVACCOW.OnCommitExecute(Sender: TObject);
var
  ContextItemCollection: IContextItemCollection;
  ContextItem: IContextItem;
  NewID: string;
begin
  if not CCOWEnabled then
    exit;

  if FAbleToChangeStatus = acCannot then
    SuspendContext // Status set in OnPendingExecute
  else
    begin
      CleanupBeforeChange;
      ContextItemCollection := fContextManager.CurrentContext;
      ContextItem := ContextItemCollection.Present(FPatientContextItem);

      if ContextItem <> nil then
        NewID := ContextItem.Value
      else
        NewID := '';

      NotifyNewPatientID(NewID);
    end;
end;

procedure TVACCOW.CleanupBeforeChange;
var
  index: integer;
begin
  for index := 0 to Screen.FormCount - 1 do
    begin
      if fsModal in Screen.Forms[index].FormState then
        Screen.Forms[index].ModalResult := mrCancel
      else
        Break;
    end;
end;

{ Public functions to manage the connection and context }

function TVACCOW.SuspendContext: Boolean;
begin
  try
    if fContextManager.State = csParticipating then
      fContextManager.Suspend;
    Result := True;
    SetLinkStatus(lsBroken);
  except
    Result := False;
  end;
end;

function TVACCOW.ResumeContext(const RejoinMode: TVACCOW_RejoinMode): Boolean;
begin
  if fContextManager.State <> csParticipating then
    try
      FAbleToChangeStatus := acYes;
      fContextManager.Resume;
      case RejoinMode of
        rmGet:
          begin
            SetLinkStatus(lsJoined);
            OnCommitExecute(nil);
          end;
        rmSet:
          begin
            SetLinkStatus(lsJoined);
            OnCommitExecute(nil);
          end;
      end;
      Result := True;
    except
      SetLinkStatus(lsBroken);
      Result := False;
    end
  else
    Result := False;
end;

function TVACCOW.SetNewPatientContext(const PtId: string): Boolean;
var
  NewContextData: IContextItemCollection;
  NewContextItem: IContextItem;
  Response: UserResponse;
begin
  if not CCOWEnabled then
    begin
      Result := False;
      Exit;
    end;

  if fContextManager.State = csParticipating then
    begin
      fContextManager.StartContextChange;

      NewContextData := CoContextItemCollection.Create;
      NewContextItem := CoContextItem.Create;
      NewContextItem.Name := FPatientContextItem;
      NewContextItem.Value := PtId;

      NewContextData.Add(NewContextItem);

      Response := fContextManager.EndContextChange(True, NewContextData);

      case Response of
        urCommit:
          begin
            NotifyNewPatientID(PtId);
            Result := True;
          end;
        urCancel:
          begin
            Result := False;
          end;
        urBreak:
          begin
            SuspendContext;
            Result := True;
          end;
      else
        Result := False;
      end;
    end
  else
    begin
      NotifyNewPatientID(PtId);
      Result := True;
    end;
end;

procedure TVACCOW.SetLinkStatus(const NewStatus: TVACCOW_LinkStatus);
begin
  if NewStatus = FLinkStatus then
    Exit;
  FLinkStatus := NewStatus;
  NotifyNewLinkStatus(NewStatus);
end;

{ Default procedures for methods }

procedure TVACCOW.NotifyNewPatientIDDefault(const aPtId: string);
begin
  //
end;

procedure TVACCOW.NotifyNewLinkStatusDefault(const Status: TVACCOW_LinkStatus);
begin
  //
end;

procedure TVACCOW.NotifyAbleToChangeDefault(var OkayToChange: TVACCOW_AbleToChangeStatus; var MsgText: string);
begin
  OkayToChange := acYes;
end;

end.

