unit uCP_GUI;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 13 $  $Modtime: 7/24/09 9:07a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Generic Graphical User Interface (GUI) utilities.
*
*       Notes:        This file contains generic calls that add to the common
*                     look and feel across the CP Suite of applications.
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/utilities/uCP_GUI.pas $
*
* $History: uCP_GUI.pas $
 * 
 * *****************  Version 13  *****************
 * User: Zzzzzzjuergb Date: 7/24/09    Time: 10:10a
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Added drop-down widener.
 * 
 * *****************  Version 12  *****************
 * User: Zzzzzzjuergb Date: 5/22/09    Time: 2:17p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Build Pre-275:  
 * 
 * *****************  Version 11  *****************
 * User: Zzzzzzpetitd Date: 3/23/09    Time: 11:24a
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Added SetApplicationHelpFile procedure / DanP
 *
 * *****************  Version 10  *****************
 * User: Zzzzzzpetitd Date: 9/11/08    Time: 8:54p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 *
 * *****************  Version 9  *****************
 * User: Zzzzzzpetitd Date: 12/05/07   Time: 4:08p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 *
 * *****************  Version 8  *****************
 * User: Zzzzzzpetitd Date: 9/27/07    Time: 5:58p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 *
 * *****************  Version 7  *****************
 * User: Zzzzzzpetitd Date: 7/25/07    Time: 12:32p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Working on the recursive load call so that we can respond back on the
 * progress of the observation loader.
 *
 * *****************  Version 6  *****************
 * User: Zzzzzzpetitd Date: 7/06/07    Time: 10:16a
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 *
 * *****************  Version 5  *****************
 * User: Zzzzzzpetitd Date: 5/24/07    Time: 3:47p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Implementation of new ICliOSource
 *
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 2/23/07    Time: 1:55p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Post CMO conference
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 11/22/06   Time: 3:31p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Added the role name to the Hint added to an action when a user doens't
 * meet the requirements to use that action.
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 11/09/06   Time: 3:02p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Weekly update of all changes.
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/27/06   Time: 1:43p
 * Created in $/CP Modernization/CliO/source/Common/utilities
 *
 *
*
================================================================================
}
interface

uses
  ActnList,
  Forms,
  Dialogs,
  Types,
  oInterfaces,
  Classes,
  ComCtrls,
  Controls,
  WinInet,
  oVersionInformation,
  XMLDoc,
  XMLIntf,
  SysUtils,
  StdCtrls,
  Graphics;

procedure InitStatusBar(aStatusBar: TStatusBar; aUser: IUser; aDBConnection: IDBConnection);
procedure EnableActionByRole(aAction: TAction; aUser: IUser; aRole: string);
procedure SetApplicationHelpFile;
function IsThisVersionCompatible(aDBConnection: IDBConnection): boolean;
function NodeNameExists(aRootNode: TTreeNode; aName: string): boolean;
function NewNodeName(aRootNode: TTreeNode; aPrefix: string): string;
function GetCommandLineValue(aCommandLineParameter: string; aDefault: string = ''): string;
//function GetUpdateFile(const fileURL, FileName: string): boolean;
function GetUpdateFile(const fileURL: string; aStream: TStream): boolean;
procedure SetDropDownWidth(AComboBox: TComboBox);
function GetMaxCanvasWidth(ACanvas: TCanvas; AStrings: TStrings): Integer;


implementation

uses
  uCPXML,
  Messages,
  Math
  , System.UITypes // 2FA
  ;

const
  fmts: array[0..2] of string = ('/%s=', '-%s=', '%s=');

procedure InitStatusBar(aStatusBar: TStatusBar; aUser: IUser; aDBConnection: IDBConnection);
begin
  with aStatusBar do
    begin
      Panels.Clear;
      Panels.Add;
      Panels.Add;
      Panels.Add;
      Panels.Add;
      Panels.Add;
      Height := Canvas.TextHeight('X') + 10;
      {User Information}
      Panels[0].Text := Format(' %s (%s)  ', [aUser.getName, aUser.getInternalID]);
      Panels[0].Width := trunc(Canvas.TextWidth(Panels[0].Text) + 5);
      {User Station Information}
      Panels[1].Text := Format(' %s (%s)  ', [aUser.getStation_Name, aUser.getStation_ID]);
      Panels[1].Width := trunc(Canvas.TextWidth(Panels[1].Text) + 5);
      {User DB Connection Information}
      Panels[2].Text := Format(' %s  ', [aDBConnection.getDescription]);
      Panels[2].Width := trunc(Canvas.TextWidth(Panels[2].Text) + 5);
      {Reserved for CCOW indicators}
      Panels[3].Text := '';
      Panels[3].Width := 25 + Canvas.TextWidth(' WWWWWWWWWWWW ');
      Panels[3].Style := psOwnerDraw;
      {DBController work space}
      Panels[4].Style := psOwnerDraw;
    end;
end;

procedure EnableActionByRole(aAction: TAction; aUser: IUser; aRole: string);
begin
  aAction.Enabled := aUser.HasRole(aRole);
  if not aAction.Enabled then
    aAction.Hint := Format('This function is restricted to users authorized to act in the ''%s'' role.', [aRole]);
end;

procedure SetApplicationHelpFile;
{Set application helpfile to appdir\help\appname.hlp}
begin
  Application.HelpFile :=
    IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)) +
    'help\' +
    ChangeFileExt(ExtractFileName(Application.ExeName), '.chm');
end;

function IsThisVersionCompatible(aDBConnection: IDBConnection): boolean;
var
  aVerInfo: TVersionInformation;
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
begin
  Result := False;
  aVerInfo := TVersionInformation.Create(Application.ExeName, 'MD');
  aXMLDoc := NewXMLDocument;
  try
    if aDBConnection.ExecQuery('GetVersionInformation', [aVerInfo.getFileNameAndVersion], aXMLDoc) then
      begin
        aXMLNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('RECORD');
        if Assigned(aXMLNode) then
          Result := (XMLValueAsInt(aXMLNode, 'COMPATIBLE') = 1) and
            (aVerInfo.getCRC32 = XMLValueAsStr(aXMLNode, 'CRC32'));
        if not Result and (ExtractFileName(Application.ExeName) = 'CPConsole.exe') then
          Result := MessageDlg('This version of CPConsole is not listed as a compatible version.' + #13 +
            'Do you want to continue using it?', mtWarning, [mbYes, mbNo, mbCancel], 0) = mrYes;
      end;
  finally
    FreeAndNil(aVerInfo);
  end;
end;

function NodeNameExists(aRootNode: TTreeNode; aName: string): boolean;
var
  aNode: TTreeNode;
begin
  aNode := aRootNode.getFirstChild;

  while Assigned(aNode) do
    if aNode.Text = aName then
      Break
    else
      aNode := aNode.getNextSibling;

  Result := Assigned(aNode);
end;

function NewNodeName(aRootNode: TTreeNode; aPrefix: string): string;
var
  i: integer;
begin
  if NodeNameExists(aRootNode, aPrefix) then
    begin
      i := 1;
      while NodeNameExists(aRootNode, Format('%s (%d)', [aPrefix, i])) do
        inc(i);
      Result := Format('%s (%d)', [aPrefix, i]);
    end
  else
    Result := aPrefix;
end;

function GetCommandLineValue(aCommandLineParameter: string; aDefault: string = ''): string;
var
  i: integer;
  j: integer;
begin
  aCommandLineParameter := LowerCase(aCommandLineParameter);

  for i := 1 to ParamCount do
    for j := Low(fmts) to High(fmts) do
      if Pos(Format(fmts[j], [aCommandLineParameter]), LowerCase(ParamStr(i))) = 1 then
        begin
          Result := Copy(ParamStr(i), Pos('=', ParamStr(i)) + 1, Length(ParamStr(i)));
          Exit;
        end;

  aCommandLineParameter := Copy(aCommandLineParameter, 1, 1);

  for i := 1 to ParamCount do
    for j := Low(fmts) to High(fmts) do
      if Pos(Format(fmts[j], [aCommandLineParameter]), LowerCase(ParamStr(i))) = 1 then
        begin
          Result := Copy(ParamStr(i), Pos('=', ParamStr(i)) + 1, Length(ParamStr(i)));
          Exit;
        end;

  Result := aDefault;
end;

function GetUpdateFile(const fileURL: string; aStream: TStream): boolean;
const
  BufferSize = 1024;
var
  hSession, hURL: HInternet;
  Buffer: array[1..BufferSize] of Byte;
  BufferLen: DWORD;
  sAppName: string;
begin
//  Result := false;
  sAppName := ExtractFileName(Application.ExeName);
  hSession := InternetOpen(PChar(sAppName), INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  try
    hURL := InternetOpenURL(hSession, PChar(fileURL), nil, 0, 0, 0);
    try
      repeat
        InternetReadFile(hURL, @Buffer, SizeOf(Buffer), BufferLen);
        aStream.Write(Buffer, BufferLen);
      until BufferLen = 0;
      Result := True;
    finally
      InternetCloseHandle(hURL)
    end
  finally
    InternetCloseHandle(hSession)
  end
end;

{
function GetUpdateFile(const fileURL, FileName: string): boolean;
const
  BufferSize = 1024;
var
  hSession, hURL: HInternet;
  Buffer: array[1..BufferSize] of Byte;
  BufferLen: DWORD;
  f: file;
  sAppName: string;
begin
  result := false;
  sAppName := ExtractFileName(Application.ExeName);
  hSession := InternetOpen(PChar(sAppName), INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  try
    hURL := InternetOpenURL(hSession, PChar(fileURL), nil, 0, 0, 0);
    try
      AssignFile(f, FileName);
      Rewrite(f, 1);
      repeat
        InternetReadFile(hURL, @Buffer, SizeOf(Buffer), BufferLen);
        BlockWrite(f, Buffer, BufferLen)
      until BufferLen = 0;
      CloseFile(f);
      result := True;
    finally
      InternetCloseHandle(hURL)
    end
  finally
    InternetCloseHandle(hSession)
  end
end;
}

procedure SetDropDownWidth(AComboBox: TComboBox);
var
  MaxLength: Integer;
begin
  if AComboBox.Items.Count = 0 then
    exit;

  MaxLength := GetMaxCanvasWidth(AComboBox.Canvas, AComboBox.Items);

  AComboBox.Perform(CB_SETDROPPEDWIDTH, Max(MaxLength, AComboBox.Width), 0)
end;


function GetMaxCanvasWidth(ACanvas: TCanvas; AStrings: TStrings):Integer;
var
  I: Integer;
  MaxLength: Integer;
begin
  if (not assigned(ACanvas)) or (not assigned(AStrings)) then
    raise Exception.Create('Parameter null in GetMaxCanvasWidth');

  MaxLength := 0;

  for I := 0 to AStrings.Count - 1 do
    if ACanvas.TextWidth(AStrings[I]) > ACanvas.TextWidth(AStrings[MaxLength]) then
      MaxLength := I;

  Result := ACanvas.TextWidth(AStrings[MaxLength]) + 25; //Take width of scrollbar into account;
end;

end.

