unit uConversions;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 1 $  $Modtime: 10/05/06 10:10a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Manages complex unit conversion such as temp c to f.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/utilities/uConversions.pas $
*
* $History: uConversions.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:35p
 * Created in $/CP Modernization/CliO/source/Common/utilities
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Classes;

type
  TConversionType = (ctFahrenheit2Celsius, ctFahrenheight2Kelvin, ctCelsius2Fahrenheit, ctCelsius2Kelvin, ctKelvin2Fahrenheit, ctKelvin2Celsius);

const
  CONVERSION_TYPE_NAME: array[TConversionType] of string = (
    'Fahrenheit to Celsius',
    'Fahrenheit to Kelvin',
    'Celsius to Fahrenheit',
    'Celsius to Kelvin',
    'Kelvin to Fahrenheight',
    'Kelvin to Celsius');

function Convert(aValue: Double; aConversionType: TConversionType): Double;
procedure LoadConversionTypes(aList: TStrings);

implementation

function Convert(aValue: Double; aConversionType: TConversionType): Double;
begin
  case aConversionType of
    ctCelsius2Fahrenheit: Result := (aValue * 1.8) + 32;
    ctCelsius2Kelvin: Result := (aValue + 273.15);
    ctFahrenheit2Celsius: Result := (aValue - 32) / 1.8;
    ctFahrenheight2Kelvin: Result := Convert(Convert(aValue, ctFahrenheit2Celsius), ctCelsius2Kelvin);
    ctKelvin2Celsius: Result := aValue - 273.5;
    ctKelvin2Fahrenheit: Result := Convert(Convert(aValue, ctKelvin2Celsius), ctCelsius2Fahrenheit);
  else
    Result := aValue;
  end;
end;

procedure LoadConversionTypes(aList: TStrings);
var
  i: TConversionType;
begin
  for i := Low(TConversionType) to High(TConversionType) do
    aList.AddObject(CONVERSION_TYPE_NAME[i], Ptr(Ord(i)));
end;

end.

