unit uMDK_Utils;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 7/17/08 2:58p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   dddddddddomain.user@domain.ext
*       Description:  this unit contains the Hemodialysis application
*                     Common routines
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/uMDK_Utils.pas $
*
*       $ History: uMDK_Utils.pas $
}
interface

uses
  Forms
  , Classes
  , Controls
  , SysUtils
  , Windows
  , ComCtrls
  , Dialogs
  ;

type
  TFMDateTimeMode = set of (
    fmdtDateOnly, // Convert date only
    fmdtShortDate, // Short date (digits only)
    fmdtYear2, // 2 digit year
    fmdtTimeOnly, // Convert time only
    fmdtShortTime, // Do not convert seconds
    fmdtTime24 // Military time format
    );

procedure setFormParent(aForm: TForm; aTab: TWinControl);
procedure setUpForm(aForm: TForm; aTab: TWinControl; aVisible:Boolean=False);
function getWorkstationName: string;

function Piece(Value, Delimiter: string;
  StartPiece: Integer = 1): string; overload;
function Piece(Value, Delimiter: string;
  StartPiece: Integer; EndPiece: Integer): string; overload;

function CmdLineSwitch(swlst: array of string): Boolean; overload;
function CmdLineSwitch(swlst: array of string; var swval: string): Boolean;
overload;

function CheckOption(aName, aValue, aMin, aMax: string; var Reason: string):
  Boolean;

function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean =
  True): Boolean;
//function IsValidInteger(aString: string): Boolean;
function IsValidDateString(aString: string): Boolean;

function FMDateTimeStr(DateTime: string; Mode: TFMDateTimeMode = [];
  DfltValue: string = ''): string;

function FormatSSN(aSSN: string): string;

function FMDateTimeToWindowsDateTime(FMDateTime: Double): TDateTime;
function FMDateToWindowsDate(FMDate: Double): TDate;
function WindowsDateTimeToFMDateTime(WinDate: TDateTime): Double;

procedure CloseApplicationForms(aCaller: TForm);
procedure UpdateSBSize(StatusBar: TStatusBar);


var
  FMDateFormat: array[1..2, 1..3] of string;

implementation
(*
function IsValidInteger(aString: string): Boolean;
var
  E, aValue: Integer;
begin
  Val(aString, aValue, E);
  Result := E = 0;
end;
*)
function IntVal(str: string): Integer;
begin
  if str <> '' then
    try
      Result := StrToInt(str);
    except
      on EConvertError do
        Result := 0;
    else
      raise;
    end
  else
    Result := 0;
end;


function IsValidDateString(aString: string): Boolean;
begin
  Result := False;
  try
    StrToDateTime(aString);
    Result := True;
  except
  end;
end;

procedure setFormParent(aForm: TForm; aTab: TWinControl);
begin
  if aForm.Parent <> aTab then
    begin
      aForm.Parent := aTab;
      aForm.Align := alClient;
    end;
end;

procedure setUpForm(aForm: TForm; aTab: TWinControl; aVisible:Boolean=False);
begin
  if not Assigned(aForm) then
    Exit; 
  if aForm.Parent <> aTab then
    begin
      aForm.BorderStyle := bsNone;
      aForm.Parent := aTab;
      aForm.Align := alClient;
      if aVisible then
        aForm.Show;
    end;
end;

function getWorkstationName: string;
var
  Buffer: array[0..200] of char;
  iBuffer: DWord;
begin
  iBuffer := 200;
  Result := 'unknown';
  try
    GetComputerName(Buffer, iBuffer);
    Result := copy(Buffer, 1, iBuffer);
  except
  end;
end;

function Piece(Value, Delimiter: string; StartPiece: Integer = 1): string;
begin
  Result := Piece(Value, Delimiter, StartPiece, StartPiece);
end;

function Piece(Value, Delimiter: string; StartPiece: Integer; EndPiece:
  Integer): string;
var
  dlen, i, pnum: Integer;
  buf: string;
begin
  Result := '';
  if (Value <> '') and (StartPiece > 0) and (EndPiece >= StartPiece) then
    begin
      dlen := Length(Delimiter);
      i := Pos(Delimiter, Value) - 1;
      if i >= 0 then
        begin
          buf := Value;
          pnum := 1;
          repeat
            if pnum > EndPiece then
              break;
            if i < 0 then
              i := Length(buf);
            if pnum = StartPiece then
              Result := Copy(buf, 1, i)
            else if pnum > StartPiece then
              Result := Result + Delimiter + Copy(buf, 1, i);
            Delete(buf, 1, i + dlen);
            i := Pos(Delimiter, buf) - 1;
            Inc(pnum);
          until (i < 0) and (buf = '');
        end
      else if StartPiece = 1 then
        Result := Value;
    end;
end;

function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean =
  True): Boolean;
var
  i: integer;
begin
  i := 0;
  Result := False;
  while (i <= High(SubStrs)) and (Result = False) do
    begin
      if CaseSensitive then
        begin
          if Pos(SubStrs[i], S) > 0 then
            Result := True
          else
            Inc(i)
        end
      else
        begin
          if Pos(LowerCase(SubStrs[i]), LowerCase(S)) > 0 then
            Result := True
          else
            Inc(i)
        end;
    end
end;

function CmdLineSwitch(swlst: array of string): Boolean;
var
  swval: string;
begin
  Result := CmdLineSwitch(swlst, swval);
end;

function CmdLineSwitch(swlst: array of string; var swval: string): Boolean;
var
  i: Integer;
begin
  Result := False;
  swval := '';
  for i := 1 to ParamCount do
    begin
      if InString(ParamStr(i), swlst, False) then
        begin
          swval := Piece(ParamStr(i), '=', 2);
          Result := True;
          break;
        end;
    end;
end;

function CheckOption(aName, aValue, aMin, aMax: string; var Reason: string):
  Boolean;
var
  iCode,
  iMin, iMax, iVal: Integer;
  sV, sMin, sMax,
    upValue,
    upName: string;

const
  ValidPainLevel = '0,1,2,3,4,5,6,7,8,9,99,';

begin
  Result := True;
  sV := aValue;
  upValue := Uppercase(sV);
  upName := Uppercase(aName);
  if (upName = 'PAIN LEVEL') and (pos(upValue + ',', ValidPainLevel) = 0) then
    begin
      Reason := 'Invalid Pain Level value of "' + aValue + '"' + #13#10 +
        'Select one of: 0,1,2,3,4,5,6,7,8,9,99';
      Result := False;
    end
  else if (
    (pos('ALLOW', upName) = 1) or
    (pos('ALWAYS', upName) = 1) or
    (pos('AUTOSAVE', upName) = 1) or
    (pos('DISABLE', upName) = 1) or
    (pos('FALLS ASSESSMENT', upName) = 1) or
    (pos('IGNORE', upName) = 1) or
    (pos('KEEP', upName) = 1) or
    (pos('NOTIFY', upName) = 1) or
    (pos('REGISTER', upName) = 1) or
    (pos('REVERSE', upName) = 1) or
    (pos('SAVE', upName) = 1) or
    (pos('SHOW', upName) = 1) or
    (pos('SMART', upName) = 1) or
    (pos('STUDY LIST AUTOREFRESH', upName) = 1)
    ) and (
    ('TRUE' <> upValue) and
    ('FALSE' <> upValue)
    ) then
    begin
      Result := False;
      Reason :=
        'Option ' + aName + ':' + #13#10 +
        'Value "' + sV + '" is not valid Boolean' + #13#10 +
        'Select TRUE or FALSE';
    end
  else if (
    (pos('(SEC)', upName) <> 0) or
    (pos('(MIN)', upName) <> 0) or
    (pos('QUANTITY', upName) <> 0) or
    (pos('LENGTH', upName) <> 0) or
    (pos('LIMIT', upName) <> 0)
    //    or (pos('IEN', upName) <> 0)
    ) then
    begin
      Val(sV,iVal,iCode);
      if iCode <> 0 then
        begin
          Result := False;
          Reason :=
            'Option ' + aName + ':' + #13#10 +
            'Value "' + sV + '" is not valid integer';
        end
      else
        begin
          sMin := aMin; //PreferenceComments.sMinValue(aName);
          sMax := aMax; //PreferenceComments.sMaxValue(aName);
          if (sMax <> '') and (sMin <> '') then
            try
              iMin := StrToInt(sMin);
              iMax := StrToInt(sMax);
              if (iVal < iMin) or (iMax < iVal) then
                begin
                  Result := False;
                  Reason :=
                    'Option ' + aName + ':' + #13#10 +
                    'Value "' + sV + '" is outside of the range ' + sMin + '..'
                    + sMax;

                end
              else
                Result := True;
            except
            end;
        end;
    end
  else if pos('COLOR', upName) <> 0 then
    begin
      val(sV,iVal,iCode);
      if iCode <> 0 then
        begin
          Result := False;
          Reason :=
            'Option ' + aName + ':' + #13#10 +
            'Value "' + sV + '" is not valid integer';
        end
      else
        begin
          Result := True;
          iCode := iVal;
        end;
    end
  else
    Result := True;
end;

// copy from uROR_CommonUtils
function FMDateTimeStr(DateTime: string; Mode: TFMDateTimeMode;
  DfltValue: string): string;
var
  buf, format: string;
  day, month, year: Word;
  hour, min, sec: Word;
  date, time: TDate;
  dateType, datePart: Integer;
begin
  if DateTime = '' then
    begin
      Result := DfltValue;
      Exit;
    end;
  Result := '';

  buf := Piece(DateTime, '.', 1);
  if (not (fmdtTimeOnly in Mode)) and (buf <> '') then
    begin
      year := IntVal(Copy(buf, 1, 3)) + 1700;
      month := IntVal(Copy(buf, 4, 2));
      day := IntVal(Copy(buf, 6, 2));

      if fmdtShortDate in Mode then
        dateType := 2
      else
        dateType := 1;

      datePart := 1;
      if day = 0 then
        begin
          day := 1;
          datePart := 2;
          if month = 0 then
            begin
              month := 1;
              datePart := 3;
            end;
        end;

      format := FMDateFormat[dateType][datePart];
      if fmdtYear2 in Mode then
        format := StringReplace(format, 'YYYY', 'YY', []);

      date := EncodeDate(year, month, day);
      Result := FormatDateTime(format, date);
    end;

  buf := Piece(DateTime, '.', 2);
  if (not (fmdtDateOnly in Mode)) and (buf <> '') then
    begin
      buf := Copy(buf + '000000', 1, 6);

      hour := IntVal(Copy(buf, 1, 2));
      min := IntVal(Copy(buf, 3, 2));
      sec := IntVal(Copy(buf, 5, 2));

      if hour >= 24 then
        begin
          hour := 23;
          min := 59;
          sec := 59;
        end
      else if min >= 60 then
        begin
          min := 59;
          sec := 59;
        end
      else if sec >= 60 then
        sec := 59;

      time := EncodeTime(hour, min, sec, 0);

      if fmdtTime24 in Mode then
        if fmdtShortTime in Mode then
          format := 'HH:NN'
        else
          format := 'HH:NN:SS'
      else if fmdtShortTime in Mode then
        format := 'T'
      else
        format := 'TT';

      if Result <> '' then
        Result := Result + '  ' + FormatDateTime(format, time)
      else
        Result := FormatDateTime(format, time);
    end;
end;

procedure FMInitFormatArray;
var
  format: string;
begin
  format := UpperCase(ShortDateFormat);
  if Pos('M', format) > Pos('D', format) then
    begin
      FMDateFormat[1][1] := 'DD MMM YYYY';
      FMDateFormat[1][2] := 'MMM YYYY';
      FMDateFormat[1][3] := 'YYYY';

      FMDateFormat[2][1] := 'DD/MM/YYYY';
      FMDateFormat[2][2] := 'MM/YYYY';
      FMDateFormat[2][3] := 'YYYY';
    end
  else
    begin
      FMDateFormat[1][1] := 'MMM DD, YYYY';
      FMDateFormat[1][2] := 'MMM YYYY';
      FMDateFormat[1][3] := 'YYYY';

      FMDateFormat[2][1] := 'MM/DD/YYYY';
      FMDateFormat[2][2] := 'MM/YYYY';
      FMDateFormat[2][3] := 'YYYY';
    end;
end;

function FormatSSN(aSSN: string): string;
begin
  try
    Result := Copy(aSSN, 1, 3) + '-' + Copy(aSSN, 4, 2) + '-' + Copy(aSSN, 6,
      99);
  except
    Result := aSSN
  end;
end;

function ReFormatSSN(aSSN: string): string;
var
  i: Integer;
  sResult: string;
begin
  i := pos('-', aSSN);
  sResult := aSSN;
  while i <> 0 do
    begin
      sResult := copy(sResult, 1, i - 1) + copy(sResult, i + 1, 99);
      i := pos('-', sResult);
    end;
  Result := sResult;
end;

function WindowsDateToFMDate(WinDate: TDate): Double;
var
  Year, Month, Day: Word;
begin
  DecodeDate(WinDate, Year, Month, Day);
  Result := ((Year - 1700) * 10000) + (Month * 100) + Day;
end;

function FMDateToWindowsDate(FMDate: Double): TDate;
var
  FMString: string;
  Year, Month, Day: Word;
begin
  FMString := FloatToStr(FMDate);
  try
    Year := StrToInt(Copy(FMString, 1, 3)) + 1700;
    Month := StrToInt(Copy(FMString, 4, 2));
    Day := StrToInt(Copy(FMString, 6, 2));

    if Month = 0 then
      Month := 1; // zzzzzzandria 050414
    if Day = 0 then
      Day := 1; // zzzzzzandria 050414
    Result := EncodeDate(Year, Month, Day);
  except
    on E: EConvertError do
      begin
{$IFDEF AANTEST}
        ShowMessage('Error Converting Date ' + FMString);
{$ENDIF}
        Result := Now;
      end;
  else
    raise;
  end;
end;

function WindowsDateTimeToFMDateTime(WinDate: TDateTime): Double;
var
  Hour, Min, Sec, Msec: Word;
begin
  Result := WindowsDateToFMDate(WinDate);
  DecodeTime(WinDate, Hour, Min, Sec, Msec);
  Result := Result + (Hour * 0.01) + (Min * 0.0001) + (Sec * 0.000001);
end;

function FMDateTimeToWindowsDateTime(FMDateTime: Double): TDateTime;
var
  FMString: string;
  Hour, Min, Sec: Word;
begin
  FMString := FLoatToStr(FMDateTime);
  Result := FMDateToWindowsDate(FMDateTime);
  Hour := StrToIntDef(Copy(FMString, 9, 2), 0);
  Min := StrToIntDef(Copy(FMString, 11, 2), 0);
  Sec := StrToIntDef(Copy(FMString, 13, 2), 0);
  Result := Result + EncodeTime(Hour, Min, Sec, 0);
end;

procedure CloseApplicationForms(aCaller: TForm);
var
  i: Integer;
begin
  if aCaller = nil then
    aCaller := Application.MainForm;
  for i := 0 to Screen.CustomFormCount - 1 do
    begin // Close all modal forms here
      if Screen.CustomForms[i] = aCaller then
        continue;
      if fsModal in Screen.CustomForms[i].FormState then
        begin
          Screen.CustomForms[i].ModalResult := mrCancel;
          Screen.CustomForms[i].Close;
        end;
    end;
end;

procedure UpdateSBSize(StatusBar: TStatusBar);
var
  i, wd: Integer;
begin
  wd := StatusBar.Width;
  for i := StatusBar.Panels.Count - 1 downto 1 do
    begin
      if StatusBar.Panels[i].Text <> '' then
        StatusBar.Panels[i].Width :=
          StatusBar.Canvas.TextWidth(StatusBar.Panels[i].Text) + 10;
      wd := wd - StatusBar.Panels[i].Width;
    end;
  StatusBar.Panels[0].Width := wd;
  StatusBar.Invalidate;
end;

initialization
  FMInitFormatArray;
end.
