unit uParser;

interface

uses
  SysUtils, Classes, StrUtils;

procedure Parse(AFilename: string; AMask: string; Results: TStrings);

implementation

uses
  Masks, MFunStr;

const
  OPEN = '<%';
  CLOSE = '%>';
  VALID_CHAR='[1-9a-zA-Z]';
  RULE_PATTERN_1 = VALID_CHAR + VALID_CHAR + VALID_CHAR + ':?,?,*';
  RULE_PATTERN_2 = '*(*)';

function ReadyToCollect(Mask: string): boolean;
begin
  result := (Pos(OPEN, Mask) = 1) and (Pos(CLOSE, Mask) > 1);
end;

function GetTagname(Mask: string): string;
var
  OpenPos: Integer;
  ClosePos: Integer;
begin
  result := '';
  OpenPos := Pos(OPEN, Mask) + 2;
  ClosePos := Pos(CLOSE, Mask);
  if OpenPos > ClosePos then
    exit;

  result := copy(Mask, OpenPos, ClosePos - OpenPos);
end;


function GetRulename(ARule: string): string;
begin
  result := '';
end;

function FindMatchingParen(OpenParenPos: Integer; AValue: string): Integer;
var
  Level: Integer;
  I: Integer;
begin
  Level := 0;
  Result := 0;
  for I := OpenParenPos to Length(AValue) do
  begin
    if AValue[I] = '(' then
      Inc(Level);
    if AValue[I] = ')' then
      Dec(Level);

    if Level = 0 then
    begin
      result := I;
      exit;
    end;
  end;
end;

procedure ApplyRules(ARule: string; AValue: string; Results: TStrings);
var
  RuleBegin: Integer;
  RuleEnd: Integer;
  ColonPos: Integer;
  Name: string;
  Value: string;
  Delim: string;
  Start: Integer;
  Finish: Integer;
begin
  if MatchesMask(ARule, RULE_PATTERN_1) then
  begin
    Name := Piece(ARule,':',1);
    Value := Piece(ARule,':',2);

    if Piece(Value,',',1) = 'P' then
    begin
      Delim := Piece(Value,',',2);
      Start := StrToIntDef(Piece(Value,',',3), 1);
      Results.Add(Name + '=' + Piece(AValue, Delim, Start));
      Exit;
    end;
    if Piece(Value,',',1) = 'E' then
    begin
      Start := StrToIntDef(Piece(Value,',',2),1);
      Finish := StrToIntDef(Piece(Value,',',3),2);
      Results.Add(Name + '=' + AnsiMidStr(AValue, Start, Finish));
      Exit;
    end;

    raise Exception.CreateFmt('Invalid instruction in pattern field: %s', [Piece(Value,',',1)]);
  end;

  if MatchesMask(ARule, RULE_PATTERN_2) then
  begin
    ColonPos := Pos(':', ARule);
    RuleBegin := Pos('(',ARule);
    if RuleBegin > 0 then
    begin
      if (ColonPos > 1) and (ColonPos < RuleBegin) then
      begin
        Results.Add(Copy(ARule, 1, ColonPos - 1) + '=' + AValue);
      end;
      if RuleBegin > 0 then
      begin
        RuleEnd := FindMatchingParen(RuleBegin, ARule);
        ApplyRules(copy(ARule, RuleBegin + 1, RuleEnd - RuleBegin - 1), AValue, Results);
        ApplyRules(copy(ARule, RuleEnd + 1, Length(ARule)), AValue, Results);
      end;
    end;
    exit;
  end;

  if Trim(ARule) <> '' then
    Results.Add(ARule + '=' + AValue);
end;


procedure Internal_Parse(AFilename: string; AMask: string; Results: TStrings);
var
  Name: string;
  Value: string;
  Filename: string;
  ParseStrStop: Integer;
  //--
  NextTag: Integer;
  ConstStr: string;
  IgnoreStrStart: Integer;
  IgnoreStrEnd: Integer;
begin
  if ReadyToCollect(AMask) then
  begin
    Name := GetTagName(AMask);
    AMask := copy(AMask, Length(Name) + 5, Length(AMask));
    if AMask = '' then
    begin
      Value := AFilename;
      ApplyRules(Name, Value, Results);
      exit;
    end
    else
    begin
      ConstStr := copy(AMask, 1, Pos(OPEN, AMask) - 1);
      ParseStrStop := pos(ConstStr, AFilename) - 1;
      if ParseStrStop < 1 then
      begin
        Results.Clear;
        exit;
      end;

      Value := copy(AFilename, 1, ParseStrStop);
      ApplyRules(Name, Value, Results);
      Filename := copy(AFilename, ParseStrStop, Length(AFilename));
      Internal_Parse(Filename, AMask, Results);
    end;
  end
  else
  begin
    NextTag := Pos(OPEN, AMask);
    if NextTag < 1 then
      exit;
    ConstStr := Copy(AMask, 1, NextTag - 1);
    IgnoreStrStart :=  Pos(ConstStr, AFilename);
    if IgnoreStrStart < 1 then
    begin
      Results.Clear;
      exit;
    end;
    IgnoreStrEnd := Length(ConstStr);
    AFilename := copy(AFilename, IgnoreStrStart + IgnoreStrEnd, Length(AFilename));
    AMask := copy(AMask, IgnoreStrEnd + 1, Length(AMask));
    Internal_Parse(AFilename, AMask, Results);
  end;
end;

procedure Parse(AFilename: string; AMask: string; Results: TStrings);
begin
  if not assigned(Results) then
    raise Exception.Create('Results must be instantiated before parsing.');

  Results.Clear;

  Internal_Parse(AFilename, AMask, Results);
end;

end.
