unit uRegistryFunc;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 2 $  $Modtime: 5/22/09 1:23p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Manages registry setting for the TCPGatewayConfig object.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/utilities/uRegistryFunc.pas $
*
* $History: uRegistryFunc.pas $
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzjuergb Date: 5/22/09    Time: 2:17p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Build Pre-275:  
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:24p
 * Created in $/CP Modernization/CliO/source/Common/utilities
 * 
 * *****************  Version 4  *****************
 * User: Zzzzzzjuergb Date: 6/19/07    Time: 8:37a
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * commented out LoadConfig and SaveConfig.  No longer used.
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 11/08/06   Time: 2:51p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Added VistAPort (for VistA message service) and LogLevel (for verbosity
 * of messages in system log).
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzjuergb Date: 11/01/06   Time: 1:33p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Changed LoadConfig() to only open the registry key read-only.  Also
 * wrapped the allocation and deallocation of the Reg object in
 * LoadConfig() and SaveConfig() in a Try...Finally block to ensure that
 * the resources are released.
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:35p
 * Created in $/CP Modernization/CliO/source/Common/utilities
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  oCPGatewayConfig;

function getConfigurationFile: string;
procedure setConfigurationFile(const aFilename: string);

function LoadConfig(aCPGatewayConfiguration: TCPGatewayConfig): boolean;
function SaveConfig(aCPGatewayConfiguration: TCPGatewayConfig): boolean;
function RenameConfig(const aValue: string): boolean;

function LoadConnection(aConnectionName: string; var aConnectionParams: array of Variant): boolean;
function SaveConnection(aConnectionName: string; aConnectionParams: array of Variant): boolean;

implementation

uses
  Registry,
  SysUtils,
  Variants,
  Windows;

const
  REG_KEY_ROOT = '\Software\VistA\Clinical Procedures\CPGateway';
  REG_KEY_CONN = '\Software\VistA\Clinical Procedures\Connections';
  REG_VALUE_CONFIG = 'Configuration File';
  REG_VALUE_NAME = 'Configuration Name';
  REG_VALUE_SERVER = 'Server';
  REG_VALUE_PORT = 'Port';
  REG_VALUE_ACCESS = 'Access Code';
  REG_VALUE_VERIFY = 'Verify Code';
  //REG_VALUE_POLLING = 'Polling Interval';
  {
    blj 8 Nov. 2006: Added 'VistA Port' and removed 'Polling Interval'.
    CP Gateway no longer polls for messages.  It now is notified over 'VistA Port'
    that a message is ready to be processed.
  }
  REG_VALUE_VISTAPORT = 'VistA Port';
  REG_VALUE_LOGLEVEL = 'Log Level';
  REG_VALUE_INSTDATA = 'Days to keep instrument data';
  REG_VALUE_SHARED_DIR = 'CP Imaging shared directory';
  REG_VALUE_LOGFILE_DIR = 'CP Gateway logfiles directory';

function LoadConfig(aCPGatewayConfiguration: TCPGatewayConfig): boolean;
//var
//  reg: TRegistry;
begin
//  reg := TRegistry.Create;
//  try
//    reg.RootKey := HKEY_LOCAL_MACHINE;
//    reg.OpenKeyReadOnly(REG_KEY_ROOT);
//
//    try
//      aCPGatewayConfiguration.setDisplayName(reg.ReadString(REG_VALUE_NAME));
//      aCPGatewayConfiguration.setServer(reg.ReadString(REG_VALUE_SERVER));
//      aCPGatewayConfiguration.setPort(StrToIntDef(reg.ReadString(REG_VALUE_PORT), 9200));
//      aCPGatewayConfiguration.EncryptedAccess := reg.ReadString(REG_VALUE_ACCESS);
//      aCPGatewayConfiguration.EncryptedVerify := reg.ReadString(REG_VALUE_VERIFY);
//      aCPGatewayConfiguration.setVistAPort(StrToIntDef(reg.ReadString(REG_VALUE_VISTAPORT), 8888));
//      aCPGatewayConfiguration.setLogLevel(StrToIntDef(reg.ReadString(REG_VALUE_LOGLEVEL), 1));
//      aCPGatewayConfiguration.setDaysToKeepData(StrToIntDef(reg.ReadString(REG_VALUE_INSTDATA), 0));
//      aCPGatewayConfiguration.setSharedDirectory(reg.ReadString(REG_VALUE_SHARED_DIR));
//      aCPGatewayConfiguration.setLogFileDirectory(reg.ReadString(REG_VALUE_LOGFILE_DIR));
//      Result := True;
//    except
//      Result := False;
//    end;
//  finally
//    reg.CloseKey;
//    FreeAndNil(reg);
//  end;
  result := false;
end;

function SaveConfig(aCPGatewayConfiguration: TCPGatewayConfig): boolean;
//var
//  reg: TRegistry;
begin
//  reg := TRegistry.Create;
//  try
//    reg.RootKey := HKEY_LOCAL_MACHINE;
//    reg.OpenKey(REG_KEY_ROOT, True);
//
//    try
//      reg.WriteString(REG_VALUE_NAME, aCPGatewayConfiguration.getDisplayName);
//      reg.WriteString(REG_VALUE_SERVER, aCPGatewayConfiguration.getServer);
//      reg.WriteString(REG_VALUE_PORT, IntToStr(aCPGatewayConfiguration.getPort));
//      reg.WriteString(REG_VALUE_ACCESS, aCPGatewayConfiguration.EncryptedAccess);
//      reg.WriteString(REG_VALUE_VERIFY, aCPGatewayConfiguration.EncryptedVerify);
//      reg.WriteString(REG_VALUE_VISTAPORT, IntToStr(aCPGatewayConfiguration.getVistAPort));
//      reg.WriteString(REG_VALUE_LOGLEVEL, IntToStr(aCPGatewayConfiguration.getLogLevel));
//      reg.WriteString(REG_VALUE_INSTDATA, IntToStr(aCPGatewayConfiguration.getDaysToKeepData));
//      reg.WriteString(REG_VALUE_SHARED_DIR, aCPGatewayConfiguration.getSharedDirectory);
//      reg.WriteString(REG_VALUE_LOGFILE_DIR, aCPGatewayConfiguration.getLogFileDirectory);
//      Result := True;
//    except
//      Result := False;
//    end;
//
//  finally
//    reg.CloseKey;
//    FreeAndNil(reg);
//  end;

  result := false;
end;

function RenameConfig(const aValue: string): boolean;
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKey(REG_KEY_ROOT, True);

  try
    reg.WriteString(REG_VALUE_NAME, aValue);
    Result := True;
  except
    Result := False;
  end;

  reg.CloseKey;
  FreeAndNil(reg);
end;

function getConfigurationFile: string;
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKey(REG_KEY_ROOT, True);

  try
    Result := reg.ReadString(REG_VALUE_CONFIG);
  except
    Result := '';
  end;

  reg.CloseKey;
  FreeAndNil(reg);
end;

procedure setConfigurationFile(const aFilename: string);
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKey(REG_KEY_ROOT, True);

  try
    reg.WriteString(REG_VALUE_CONFIG, aFilename);
  except
    //
  end;

  reg.CloseKey;
  FreeAndNil(reg);
end;

function LoadConnection(aConnectionName: string; var aConnectionParams: array of Variant): boolean;
var
  reg: TRegistry;
  par: string;
  i: integer;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;
    reg.OpenKey(REG_KEY_CONN, True);

    try
      par := reg.ReadString(aConnectionName);
    except
      par := '';
    end;

    reg.CloseKey;
  finally
    FreeAndNil(reg);
  end;

  if par = '' then
    par := '-1';

  for i := Low(aConnectionParams) to High(aConnectionParams) do
    if Pos(';', par) > 0 then
      begin
        aConnectionParams[i] := Copy(par, 1, Pos(';', par) - 1);
        par := Copy(par, Pos(';', par) + 1, Length(par));
      end
    else
      begin
        aConnectionParams[i] := par;
        par := '';
      end;

  result := true;
  //SetLength(aConnectionParams, 10);
  //aConnectionParams[0] := 'piece1';
  //aConnectionParams[1] := 'piece2';
  //aConnectionParams[2] := 'piece3';
  //aConnectionParams[3] := 'piece4';
  //aConnectionParams[4] := 'piece5';

end;

function SaveConnection(aConnectionName: string; aConnectionParams: array of Variant): boolean;
var
  reg: TRegistry;
  regValue: string;
  i: integer;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;
    reg.OpenKey(REG_KEY_CONN, True);

    regValue := '';

    for i := Low(aConnectionParams) to High(aConnectionParams) do
      if Length(regValue) > 0 then
        regValue := Format('%s;%s', [regValue, VarToStr(aConnectionParams[i])])
      else
        regValue := VarToStr(aConnectionParams[i]);

    try
      //reg.WriteString(REG_VALUE_CONFIG, aFilename);
      reg.WriteString(aConnectionName, regValue);
      result := true;
    except
      //
      result := false;
    end;

    reg.CloseKey;
  finally
    FreeAndNil(reg);
  end;
end;

end.

