unit uServiceFunc;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 2 $  $Modtime: 5/22/09 1:11p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Manages calls to the service control manager
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/utilities/uServiceFunc.pas $
*
* $History: uServiceFunc.pas $
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzjuergb Date: 5/22/09    Time: 2:17p
 * Updated in $/CP Modernization/CliO/source/Common/utilities
 * Build Pre-275:  
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:24p
 * Created in $/CP Modernization/CliO/source/Common/utilities
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:35p
 * Created in $/CP Modernization/CliO/source/Common/utilities
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  SysUtils,
  Types,
  WinSvc;

function GatewayInstalled: boolean;
function GatewayStart: boolean;
function GatewayStop: boolean;
function GatewayStatus: DWord;

implementation

const
  SVC_NAME = 'CPGatewaySvc';

function GatewayInstalled: boolean;
var
  schm: SC_Handle; {Service Control Manager}
  schs: SC_Handle; {Service Handle}
//  ss: TServiceStatus; {Service Status}
//  psTemp: PChar; {Temp char pointer}
//  dwChkP: DWord; {Check Point}
begin
  Result := False;
  {connect to the service control manager}
  schm := OpenSCManager(PChar(''), nil, SC_MANAGER_CONNECT);
  {if successful}
  if (schm > 0) then
    begin
      {open a handle to the specified service}
      schs := OpenService(schm, PChar(SVC_NAME), SERVICE_START or SERVICE_QUERY_STATUS);
      {if successful...}
      if (schs > 0) then
        begin
          Result := True;
          CloseServiceHandle(schs);
        end;
      CloseServiceHandle(schm);
    end;
end;

function GatewayStart: boolean;
var
  schm: SC_Handle; {Service Control Manager}
  schs: SC_Handle; {Service Handle}
  ss: TServiceStatus; {Service Status}
  psTemp: PChar; {Temp char pointer}
  dwChkP: DWord; {Check Point}
begin
  ss.dwCurrentState := 0;

  {connect to the service control manager}
  schm := OpenSCManager(PChar(''), nil, SC_MANAGER_CONNECT);

  {if successful}
  if (schm > 0) then
    begin
      {open a handle to the specified service}
      schs := OpenService(schm, PChar(SVC_NAME), SERVICE_START or SERVICE_QUERY_STATUS);

      {if successful...}
      if (schs > 0) then
        begin
          psTemp := nil;
          if (StartService(schs, 0, psTemp)) then
            if (QueryServiceStatus(schs, ss)) then
              while (SERVICE_RUNNING <> ss.dwCurrentState) do
                begin
                  dwChkP := ss.dwCheckPoint;
                  Sleep(ss.dwWaitHint);

                  if (not QueryServiceStatus(schs, ss)) then
                    break;

                  if (ss.dwCheckPoint < dwChkP) then
                    break;
                end;
          CloseServiceHandle(schs);
        end;
      CloseServiceHandle(schm);
    end;
  Result := True;
end;

function GatewayStop: boolean;
var
  schm: SC_Handle; {Service Control Manager}
  schs: SC_Handle; {Service Handle}
  ss: TServiceStatus; {Service Status}
//  psTemp: PChar; {Temp char pointer}
  dwChkP: DWord; {Check Point}
begin
  result := false;

  ss.dwCurrentState := 0;

  {connect to the service control manager}
  schm := OpenSCManager(PChar(''), nil, SC_MANAGER_CONNECT);

  {if successful}
  if (schm > 0) then
    begin
      {open a handle to the specified service}
      schs := OpenService(schm, PChar(SVC_NAME), SERVICE_STOP or SERVICE_QUERY_STATUS);

      {if successful...}
      if (schs > 0) then
        begin
//          psTemp := nil;
          if (ControlService(schs, SERVICE_CONTROL_STOP, ss)) then
            if (QueryServiceStatus(schs, ss)) then
              begin
                while (SERVICE_STOPPED <> ss.dwCurrentState) do
                  begin
                    dwChkP := ss.dwCheckPoint;
                    Sleep(ss.dwWaitHint);

                    if (not QueryServiceStatus(schs, ss)) then
                      break;

                    if (ss.dwCheckPoint < dwChkP) then
                      break;
                  end;
                CloseServiceHandle(schs);
              end;
          CloseServiceHandle(schm);
        end;
      Result := SERVICE_STOPPED = ss.dwCurrentState;
    end;
end;

function GatewayStatus: DWord;
var
  schm, schs: SC_Handle;
  ss: TServiceStatus;
  dwStat: DWord;
begin
  dwStat := 0;

  {connect to the service control manager}
  schm := OpenSCManager(PChar(''), nil, SC_MANAGER_CONNECT);

  if (schm > 0) then
    try
      {open the service}
      schs := OpenService(schm, PChar(SVC_NAME), SERVICE_QUERY_STATUS);

      if (schs > 0) then
        try
          if (QueryServiceStatus(schs, ss)) then
            dwStat := ss.dwCurrentState;
        finally
          CloseServiceHandle(schs);
        end;

    finally
      CloseServiceHandle(schm);
    end;
  Result := dwStat;
  {
  case dwStat of
    0: MessageDlg('CP Gateway service is not installed on the machine.', mtInformation, [mbOK], 0);
    SERVICE_STOPPED: MessageDlg('CP Gateway service is stopped', mtInformation, [mbOK], 0);
    SERVICE_RUNNING: MessageDlg('CP Gateway service is running', mtInformation, [mbOK], 0);
    SERVICE_PAUSED: MessageDlg('CP Gateway service is paused', mtInformation, [mbOK], 0);
  else
    MessageDlg('CP Gateway service is not responding.', mtInformation, [mbOK], 0);
  end;
  }
end;

end.

