unit uWordDoc;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  WordXP,
  ComObj,
  OleServer;

type
  TWordAlign = (waLeft, waRight, waCenter);

procedure InitWordDoc;
procedure PageFeed;
procedure LineFeed;
procedure SaveFile(aFileName: OleVariant);
procedure DrawHeader(aText: TStrings);
procedure DrawSubHeaderA(aText: string);
procedure ClearHangingIndent;
procedure SetHangingIndent(aTabPosition: Single; aDotLeader: boolean = False);
procedure ClearTabs;
procedure SetTabs(aTabPosition: Single; aAlignment: TAlignment; aDotLeader: boolean = False);
procedure DefaultList(aList: TStrings);
procedure NumberedList(aList: TStrings);
procedure AddIndex(aValue: string);

var
  WordDoc: TWordDocument;
  WordApp: TWordApplication;
  WordSel: WordSelection;
  OleVar01, OleVar02, OleVar03: OleVariant;
  OleInt, OleFileName, OleBoolean, OleTabAlign, OleTabChar: OleVariant;

implementation

procedure InitWordDoc;
begin
  WordApp := TWordApplication.Create(Application);
  WordDoc := TWordDocument.Create(Application);
  WordApp.Connect;
  WordDoc.ConnectTo(WordApp.Documents.AddOld(EmptyParam, EmptyParam));
  WordSel := WordApp.Selection;
  WordSel.Font.Name := 'Arial';
  WordSel.Font.Size := 12;

  WordSel.PageSetup.LeftMargin := WordApp.InchesToPoints(0.75);
  WordSel.PageSetup.RightMargin := WordApp.InchesToPoints(0.75);

  OleInt := 1;
  WordSel.ParagraphFormat.TabStops.ClearAll;
  WordDoc.DefaultTabStop := WordApp.InchesToPoints(0.5); //InchesToPoints(0.5);

  OleTabAlign := wdAlignTabLeft;
  OleTabChar := wdTabLeaderSpaces;
  WordSel.ParagraphFormat.TabStops.Add(WordApp.InchesToPoints(0.75), OleTabAlign, OleTabChar); //wdAlignTabRight, wdTabLeaderDots);

  OleTabAlign := wdAlignTabRight;
  OleTabChar := wdTabLeaderDots;
  WordSel.ParagraphFormat.TabStops.Add(WordApp.InchesToPoints(6.5), OleTabAlign, OleTabChar); //wdAlignTabRight, wdTabLeaderDots);

  with WordApp.ListGalleries.Item(wdNumberGallery).ListTemplates.Item(OleInt).ListLevels.Item(1) do
    begin
      NumberFormat := '%1.';
      TrailingCharacter := wdTrailingTab;
      NumberStyle := wdListNumberStyleArabic;
      NumberPosition := WordApp.InchesToPoints(0.25);
      Alignment := wdListLevelAlignLeft;
      TextPosition := WordApp.InchesToPoints(0.75);
      TabPosition := WordApp.InchesToPoints(0.75);
      ResetOnHigher := 0;
      StartAt := 1;
      {
      with Font do
        begin
          Bold := wdUndefined;
          Italic := wdUndefined;
          StrikeThrough := wdUndefined;
          Subscript := wdUndefined;
          Superscript := wdUndefined;
          Shadow := wdUndefined;
          Outline := wdUndefined;
          Emboss := wdUndefined;
          Engrave := wdUndefined;
          AllCaps := wdUndefined;
          Hidden := wdUndefined;
          Underline := wdUndefined;
          Color := wdUndefined;
          Size := 10;
          Animation := wdUndefined;
          DoubleStrikeThrough := wdUndefined;
          Name := '';
        end;
      }
      LinkedStyle := '';
    end;

  (*
  OleBoolean := False;
  OleVar01 := wdListApplyToWholeList;
  OleVar02 := wdWord10ListBehavior;
  WordApp.ListGalleries.Item(wdNumberGallery).ListTemplates.Item(OleInt).Name := '';
  WordSel.Range.ListFormat.ApplyListTemplate(
    WordApp.ListGalleries.Item(wdNumberGallery).ListTemplates.Item(OleInt),
    OleBoolean, OleVar01, OleVar02);

  WordSel.Font.Size := 10;
  WordSel.TypeText('hello' + #9 + 'there');
  WordSel.TypeParagraph;
  WordSel.TypeText('everybody' + #9 + 'It''s');
  WordSel.TypeParagraph;
  WordSel.TypeText('time' + #9 + 'to play');
  WordSel.TypeParagraph;
  OleVar03 := wdNumberParagraph;
  WordSel.Range.ListFormat.RemoveNumbers(OleVar03);
  WordSel.TypeParagraph;
  *)
end;

procedure PageFeed;
begin
  OleVar01 := wdPageBreak;
  WordSel.InsertBreak(OleVar01);
end;

procedure LineFeed;
begin
  WordSel.TypeParagraph;
end;

procedure SaveFile(aFileName: OleVariant);
begin
  WordDoc.SaveAs(aFileName);
end;

procedure AddIndex(aValue: string);
var
  OleBool01,OleBool02: OleVariant;
  OleText01,OleText02: OleVariant;
  aRange: WordRange; // 2FA Range replaced with WordRange
begin
  OleText01 := aValue;
  OleText02 := aValue;
  OleBool01 := False;
  OleBool02 := False;
  aRange := WordSel.Range;

  WordSel.TypeText(' ');

  WordDoc.Indexes.MarkEntry(
    aRange, OleText01, OleText02,
    EmptyParam, EmptyParam, EmptyParam,
    EmptyParam, OleBool01, OleBool02);

  WordSel.TypeText(' ');
end;

procedure DrawSubHeaderA(aText: string);
begin
  WordSel.TypeParagraph;
  OleVar01 := wdLine;
  OleInt := 1;
  WordSel.MoveUp(OleVar01, OleInt, EmptyParam);

  with WordSel.Borders.Item(wdBorderBottom) do
    begin
      LineStyle := WordApp.Options.DefaultBorderLineStyle;
      LineWidth := WordApp.Options.DefaultBorderLineWidth;
      Color := WordApp.Options.DefaultBorderColor;
    end;

  WordSel.Font.Bold := 1;
  WordSel.Font.Size := 10;
  WordSel.TypeText(aText);
  WordSel.Font.Bold := 0;
  WordSel.Font.Size := 10;
  OleVar01 := wdLine;
  OleInt := 1;
  WordSel.MoveDown(OleVar01, OleInt, EmptyParam);
end;

procedure DrawHeader(aText: TStrings);
var
  i: integer;
begin
  WordSel.TypeParagraph;
  OleVar01 := wdLine;
  OleInt := 1;
  WordSel.MoveUp(OleVar01, OleInt, EmptyParam); //Unit:=wdLine, Count:=1

  with WordSel.Borders.Item(wdBorderTop) do
    begin
      LineStyle := WordApp.Options.DefaultBorderLineStyle;
      LineWidth := WordApp.Options.DefaultBorderLineWidth;
      Color := WordApp.Options.DefaultBorderColor;
    end;

  with WordSel.Borders.Item(wdBorderLeft) do
    begin
      LineStyle := WordApp.Options.DefaultBorderLineStyle;
      LineWidth := WordApp.Options.DefaultBorderLineWidth;
      Color := WordApp.Options.DefaultBorderColor;
    end;

  with WordSel.Borders.Item(wdBorderBottom) do
    begin
      LineStyle := WordApp.Options.DefaultBorderLineStyle;
      LineWidth := WordApp.Options.DefaultBorderLineWidth;
      Color := WordApp.Options.DefaultBorderColor;
    end;

  with WordSel.Borders.Item(wdBorderRight) do
    begin
      LineStyle := WordApp.Options.DefaultBorderLineStyle;
      LineWidth := WordApp.Options.DefaultBorderLineWidth;
      Color := WordApp.Options.DefaultBorderColor;
    end;

  WordSel.Font.Bold := 1;
  WordSel.Font.Size := 12;
  for i := 0 to aText.Count - 1 do
    begin
      WordSel.TypeText(aText[i]);
      if i < aText.Count - 1 then
        WordSel.TypeParagraph;
    end;
  WordSel.Font.Bold := 0;
  WordSel.Font.Size := 10;

  OleVar01 := wdLine;
  OleInt := 1;
  WordSel.MoveDown(OleVar01, OleInt, EmptyParam);
end;

procedure ClearTabs;
begin
  WordSel.ParagraphFormat.TabStops.ClearAll;
  WordDoc.DefaultTabStop := WordApp.InchesToPoints(0.5);
end;

procedure ClearHangingIndent;
begin
  with WordSel.ParagraphFormat do
    begin
      LeftIndent := WordApp.InchesToPoints(0.75);
      SpaceBeforeAuto := 0;
      SpaceAfterAuto := 0;
    end;

  with WordSel.ParagraphFormat do
    begin
      SpaceBeforeAuto := 0;
      SpaceAfterAuto := 0;
      FirstLineIndent := WordApp.InchesToPoints(-0.75);
    end;

  WordSel.ParagraphFormat.TabStops.ClearAll;
  WordDoc.DefaultTabStop := WordApp.InchesToPoints(0.5);
end;

procedure SetHangingIndent(aTabPosition: Single; aDotLeader: boolean = False);
begin
  with WordSel.ParagraphFormat do
    begin
      LeftIndent := WordApp.InchesToPoints(aTabPosition);
      SpaceBeforeAuto := 0;
      SpaceAfterAuto := 0;
    end;

  with WordSel.ParagraphFormat do
    begin
      SpaceBeforeAuto := 0;
      SpaceAfterAuto := 0;
      FirstLineIndent := WordApp.InchesToPoints(aTabPosition * -1);
    end;

  OleTabAlign := wdAlignTabLeft;
  if aDotLeader then
    OleTabChar := wdTabLeaderDots
  else
    OleTabChar := wdTabLeaderSpaces;

  WordSel.ParagraphFormat.TabStops.ClearAll;
  WordDoc.DefaultTabStop := WordApp.InchesToPoints(0.5);
  WordSel.ParagraphFormat.TabStops.Add(WordApp.InchesToPoints(aTabPosition), OleTabAlign, OleTabChar);

  {
      With Selection.ParagraphFormat
        .LeftIndent = InchesToPoints(3)
        .SpaceBeforeAuto = False
        .SpaceAfterAuto = False
    End With
    With Selection.ParagraphFormat
        .SpaceBeforeAuto = False
        .SpaceAfterAuto = False
        .FirstLineIndent = InchesToPoints(-3)
    End With
    Selection.ParagraphFormat.TabStops.ClearAll
    ActiveDocument.DefaultTabStop = InchesToPoints(0.5)
    Selection.ParagraphFormat.TabStops.Add Position:=InchesToPoints(3), _
        Alignment:=wdAlignTabLeft, Leader:=wdTabLeaderDots
    }
end;

procedure SetTabs(aTabPosition: Single; aAlignment: TAlignment; aDotLeader: boolean = False);
begin
  case aAlignment of
    taLeftJustify: OleTabAlign := wdAlignTabLeft;
    taRightJustify: OleTabAlign := wdAlignTabRight;
    taCenter: OleTabAlign := wdAlignTabCenter;
  else
    OleTabAlign := wdAlignTabLeft;
  end;

  if aDotLeader then
    OleTabChar := wdTabLeaderDots
  else
    OleTabChar := wdTabLeaderSpaces;

  WordSel.ParagraphFormat.TabStops.Add(WordApp.InchesToPoints(aTabPosition), OleTabAlign, OleTabChar); //wdAlignTabRight, wdTabLeaderDots);
end;

procedure DefaultList(aList: TStrings);
var
  i: integer;
begin
  WordSel.Font.Bold := 0;
  WordSel.Font.Size := 10;
  for i := 0 to aList.Count - 1 do
    begin
      WordSel.TypeText(aList[i]);
      WordSel.TypeParagraph;
    end;
end;

procedure NumberedList(aList: TStrings);
var
  i: integer;
begin
  OleTabAlign := wdAlignTabLeft;
  OleTabChar := wdTabLeaderSpaces;
  WordSel.ParagraphFormat.TabStops.Add(WordApp.InchesToPoints(0.8), OleTabAlign, OleTabChar); //wdAlignTabRight, wdTabLeaderDots);

  OleTabAlign := wdAlignTabRight;
  OleTabChar := wdTabLeaderDots;
  WordSel.ParagraphFormat.TabStops.Add(WordApp.InchesToPoints(6.5), OleTabAlign, OleTabChar); //wdAlignTabRight, wdTabLeaderDots);

  OleBoolean := False;
  OleVar01 := wdListApplyToWholeList;
  OleVar02 := wdWord10ListBehavior;
  WordApp.ListGalleries.Item(wdNumberGallery).ListTemplates.Item(OleInt).Name := '';
  WordSel.Range.ListFormat.ApplyListTemplate(
    WordApp.ListGalleries.Item(wdNumberGallery).ListTemplates.Item(OleInt),
    OleBoolean, OleVar01, OleVar02);

  WordSel.Font.Bold := 0;
  WordSel.Font.Size := 10;
  for i := 0 to aList.Count - 1 do
    begin
      WordSel.TypeText(aList[i]);
      WordSel.TypeParagraph;
    end;

  OleVar03 := wdNumberParagraph;
  WordSel.Range.ListFormat.RemoveNumbers(OleVar03);
  WordSel.TypeParagraph;
end;

end.

